unit OrderF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Mask, ToolEdit, CurrEdit, StdCtrls, SoCtrls, ComCtrls, SohoBtns, Grids,
  SoDBGrid, SoGrids, SohoRgst, SoCtmFld, SoFolder, Db, DBTables,
  SoCtmRgs, SoBckgrd, SoFrmStorage, DBGrids, SoClass;

type
  TOrderForm = class(TForm)
    sohoLabel1: TsohoLabel;
    sohoLabel2: TsohoLabel;
    sohoLabel3: TsohoLabel;
    NumberE: TsohoEdit;
    OrdDateE: TsohoEdit;
    SummE: TCurrencyEdit;
    Pages: TPageControl;
    OkB: TsohoBitBtn;
    CancelB: TsohoBitBtn;
    IncludePage: TTabSheet;
    PositionsPage: TTabSheet;
    sohoLabel4: TsohoLabel;
    sohoLabel5: TsohoLabel;
    KnoGrid: TsohoDBGrid;
    KnoPosGrid: TsohoDBGrid;
    sohoLabel6: TsohoLabel;
    FactoryGrid: TsohoDBGrid;
    PosGrid: TsohoStringGrid;
    Ini: TsohoFormStorage;
    SubReg: TsohoSubRegister;
    KnoQ: TQuery;
    KnoSource: TDataSource;
    KnoFolder: TsohoFolder;
    KnoPosFolder: TsohoFolder;
    KnoPosSource: TDataSource;
    KnoPosQ: TQuery;
    FactoryFolder: TsohoFolder;
    FactorySource: TDataSource;
    FactoryQ: TQuery;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure KnoGridKeyPress(Sender: TObject; var Key: Char);
    procedure KnoPosGridKeyPress(Sender: TObject; var Key: Char);
    procedure FactoryGridKeyPress(Sender: TObject; var Key: Char);
    procedure PosGridCalcCellColors(Sender: TObject; Col, Row: Integer;
      aFont: TFont; var aColor: TColor; State: TGridDrawState);
    procedure PosGridAfterCellEdit(Sender: TObject; Col, Row: Integer);
    procedure PosGridBeforeCellEdit(Sender: TObject; Col, Row: Integer);
    procedure PagesChanging(Sender: TObject;
      var AllowChange: Boolean);
    procedure KnoPosQFilterRecord(DataSet: TDataSet; var Accept: Boolean);
  private
    { Private declarations }

    procedure SaveOrder;
    procedure PrepareOrder;

    procedure Recalc; //   

    function  NewOrder : boolean;
    function  EditOrder (IdOrder : LongInt) : boolean;

    procedure CreateCrosses; //  cross-cells  
                             //   ""

    procedure AddPosition (Id, IdUnits : LongInt);
    procedure RemovePosition (Id, IdUnits : LongInt);

    procedure AddFactory (Id : LongInt; FactoryName : string);
    procedure RemoveFactory (Id : LongInt);

    procedure CheckEnable;

    function GetKnoNumbers : string;

  public
    { Public declarations }
    FIdOrder : LongInt;
    EditMode : boolean;
    UnitsIds, UnitsIncIds : TsohoIntegerList;
    UnitsNames : TStringList;
    DelPosIds, AddPosIds : TsohoIntegerList;
    DelFacIds, AddFacIds : TsohoIntegerList;
    DelKnoIds, AddKnoIds : TsohoIntegerList;

    PosRemIds : TsohoFloatList;
    PosCost : TsohoFloatList;
    UnitsQuantity : TsohoFloatList;

    LastValue : Double;

    RecalcEnabled : boolean;
  end;

var OrderForm: TOrderForm;

function NewOrder : boolean;
function EditOrder (IdOrder : LongInt) : boolean;

implementation
uses SoUtils, SoDBRtn, Global, SoUnit, Main, SoDate, Clipbrd,
    SoExlCst, SohoOLE, SoExRepD, SoBDERtn, SoVCLUtils, SoStrUtils,
    SoAppUtils;

{$R *.DFM}

function NewOrder : boolean;
begin
  if OrderForm = nil then OrderForm := TOrderForm.Create(Application);
  Result := OrderForm.NewOrder;
end;

function EditOrder (IdOrder : LongInt) : boolean;
begin
  if OrderForm = nil then OrderForm := TOrderForm.Create(Application);
  Result := OrderForm.EditOrder(IdOrder);
end;

procedure TOrderForm.CheckEnable;
begin
  OkB.Enabled := (KnoPosGrid.Results.Count>0) and
     (FactoryGrid.Results.Count>0);
  // PositionsPage.Visible := OkB.Enabled;
end;

procedure TOrderForm.SaveOrder;
var Index, ColIndex, RowIndex, IdOrderInclude,
    IdFactory, IdKnoPos : LongInt;
    OrdIncQuantity : Double;
    Error : boolean;
begin
  //      
  try
    MainForm.Database.StartTransaction;
    MainForm.Gauge.Progress := 0;
    MainForm.Gauge.MaxValue := KnoPosGrid.Results.Count *
      FactoryGrid.Results.Count + KnoGrid.Results.Count + 30;
    MainForm.Gauge.Caption := '';
  //     
    if not ExecuteSQL(SubReg.PathToSQL+'Delete_exem_for_order.sql', BaseDir, [
      IntToStr(FIdOrder)
    ]) then begin
      MainForm.Database.Rollback;
      exit;
    end;
    MainForm.Gauge.AddProgress(10);
  //   
    if not ExecuteSQL(SubReg.PathToSQL+'Delete_order_contents.sql', BaseDir, [
      IntToStr(FIdOrder)
    ]) then begin
      MainForm.Database.Rollback;
      exit;
    end;
    MainForm.Gauge.AddProgress(10);
   //  
   if EditMode then begin
    if not ExecuteSQL(SubReg.PathToSQL+'Update_order.sql', BaseDir, [
      NumberE.AsString,
      DateToSQLDate(OrdDateE.AsDate),
      FLoatToSQLFLoat(SummE.Value),
      IntToStr(FIdOrder)
    ]) then begin
      MainForm.Database.Rollback;
      exit;
    end;
   end
   else
    if not ExecuteSQL(SubReg.PathToSQL+'Add_order.sql', BaseDir, [
      IntToStr(FIdOrder),
      NumberE.AsString,
      DateToSQLDate(OrdDateE.AsDate),
      FLoatToSQLFLoat(SummE.Value)
    ]) then begin
      MainForm.Database.Rollback;
      exit;
    end;
   MainForm.Gauge.AddProgress(10);
  //   
    for Index := 0 to pred(KnoGrid.Results.Count) do begin
      if not ExecuteSQL(SubReg.PathToSQL+'Add_exem_to_order.sql', BaseDir, [
        IntToStr(GetNewId('IBKno.New.OrderKno.')),
        IntToStr(KnoGrid.Results[Index]),
        IntToStr(FIdOrder)
      ]) then begin
        MainForm.Database.Rollback;
        exit;
      end;
      MainForm.Gauge.AddProgress(1);
    end;
  //   
    for ColIndex := 0 to pred(KnoPosGrid.Results.Count) do
      for RowIndex := 0 to pred(FactoryGrid.Results.Count) do begin
        IdOrderInclude := GetNewId('IBKno.New.OrderInclude.');
        OrdIncQuantity := WStrToFloat(PosGrid.Cells[ColIndex*2+2, RowIndex+2], Error);
        IdFactory      := FactoryGrid.Results[RowIndex];
        IdKnoPos       := KnoPosGrid.Results[ColIndex];
        if not ExecuteSQL(SubReg.PathToSQL+'Add_position_to_order.sql', BaseDir, [
          IntToStr(IdOrderInclude),
          FloatToSQLFloat(OrdIncQuantity),
          IntToStr(FIdOrder),
          IntToStr(IdFactory),
          IntToStr(IdKnoPos)
        ]) then begin
          MainForm.Database.Rollback;
          exit;
        end;
        MainForm.Gauge.AddProgress(1);
      end;
    MainForm.Gauge.Progress := MainForm.Gauge.MaxValue;
    MainForm.Gauge.Caption := '';
    MainForm.Database.Commit;
  except
    MainForm.Database.Rollback;
  end;
end;

procedure TOrderForm.PrepareOrder;
var TmpQ : TQuery;
    Index,
    ColIndex,
    RowIndex : LongInt;
    CellValue : Double;
begin
  KnoPosGrid.Results.Clear;
  KnoGrid.Results.Clear;
  FactoryGrid.Results.Clear;
  DelPosIds.Clear;
  AddPosIds.Clear;
  DelFacIds.Clear;
  AddFacIds.Clear;
  DelKnoIds.Clear;
  AddKnoIds.Clear;
  UnitsIds.Clear;
  UnitsQuantity.Clear;
  UnitsNames.Clear;
  UnitsIncIds.Clear;
  PosRemIds.Clear;
  PosCost.Clear;


  RefreshDataSet(KnoQ);
  RefreshDataSet(KnoPosQ);
  RefreshDataSet(FactoryQ);


  NumberE.AsString := '';
  OrdDateE.AsDate := Date;
  SummE.Value := 0;

  with PosGrid do begin
    ColCount := 3;
    RowCount := 4;
    for Index := 1 to 3 do Cells[0, Index] := '';
    Cells[1, 2] := 'Exemption remainder';
    Cells[1, 3] := 'Total';
    Cells[2, 0] := Cells[1, 3];
  end;

  if EditMode then begin
    RecalcEnabled := false;
    //   ,   
    TmpQ := GetQuery(SubReg.PathToSQL+'Exem_list_for_order.sql',
       BaseDir, [IntToStr(FIdOrder)]);
    if TmpQ = nil then exit;
    with TmpQ do begin
      while not Eof do begin
        KnoGrid.Results.Add(FieldByName('IdKno').AsInteger);
        Next;
      end;
      Free;
    end;

    TmpQ := GetQuery(SubReg.PathToSQL+'Positions_for_order.sql',
       BaseDir, [IntToStr(FIdOrder)]);
    if TmpQ = nil then exit;
    with TmpQ do begin
      while not Eof do begin
        LocaleById(KnoPosQ, 'Id', FieldByName('IdKnoPos').AsInteger);
        AddPosition(FieldByName('IdKnoPos').AsInteger,
           KnoPosQ.FieldByName('IdUnits').AsInteger);
        Next;
      end;
      Free;
    end;

    TmpQ := GetQuery(SubReg.PathToSQL+'Factories_for_order.sql',
       BaseDir, [IntToStr(FIdOrder)]);
    if TmpQ = nil then exit;
    with TmpQ do begin
      while not Eof do begin
        AddFactory(FieldByName('IdFactory').AsInteger,
                   GetFactoryName(FieldByName('IdFactory').AsInteger));
        Next;
      end;
      Free;
    end;

    RefreshDataSetFilter(KnoPosQ);
    // 
    TmpQ := GetQuery(SubReg.PathToSQL+'Select_order_by_Id.sql',
       BaseDir, [IntToStr(FIdOrder)]);
    if TmpQ = nil then exit;
    with TmpQ do begin
      NumberE.AsString := FieldByName('OrdNumber').AsString;
      OrdDateE.AsDate := FieldByName('OrdDate').AsDateTime;
      // SummE.Value := FieldByName('OrdSumm').AsFloat;
      Free;
    end;

    // ,  ,     
    //  ,   ...

    //    !
    //  OrderInclude   

    TmpQ := GetQuery(SubReg.PathToSQL+'Select_order_contents_by_Id.sql',
       BaseDir, [IntToStr(FIdOrder)]);
    if TmpQ = nil then exit;
    with TmpQ do begin
      while not Eof do begin
        ColIndex  := 2 + 2*KnoPosGrid.Results.IndexOf(FieldByName('IdKnoPos').AsInteger);
        RowIndex  := 2 + FactoryGrid.Results.IndexOf(FieldByName('IdFactory').AsInteger);
        CellValue := FieldByName('OrdIncQuantity').AsFloat;
        PosGrid.Cells[ColIndex, RowIndex] := FloatToStrF(CellValue, 15, 3);
        Next;
      end;
      Free;
    end;
  end;
  RecalcEnabled := true;
  Recalc;
  CheckEnable;
end;

function TOrderForm.NewOrder : boolean;
begin
  FIdOrder := GetNewId('IBKno.New.Order.');
  EditMode := false;
  PrepareOrder;
  Result := ShowModal = mrOk;
  if not Result then exit;
  SaveOrder;
end;

function TOrderForm.EditOrder (IdOrder : LongInt) : boolean;
begin
  FIdOrder := IdOrder;
  EditMode := true;
  PrepareOrder;
  Result := ShowModal = mrOk;
  if not Result then exit;
  SaveOrder;
end;

procedure TOrderForm.FormCreate(Sender: TObject);
begin
  DelPosIds   := TsohoIntegerList.Create;
  AddPosIds   := TsohoIntegerList.Create;
  DelFacIds   := TsohoIntegerList.Create;
  AddFacIds   := TsohoIntegerList.Create;
  DelKnoIds   := TsohoIntegerList.Create;
  AddKnoIds   := TsohoIntegerList.Create;
  UnitsIds    := TsohoIntegerList.Create;
  UnitsQuantity := TsohoFloatList.Create;
  UnitsIncIds := TsohoIntegerList.Create;
  PosRemIds   := TsohoFloatList.Create;
  PosCost     := TsohoFloatList.Create;
  UnitsNames  := TStringList.Create;

  KnoQ.SQL.LoadFromFile(SubReg.PathToSQL+'Exemptions_list.sql');
  KnoPosQ.SQL.LoadFromFile(SubReg.PathToSQL+'Select_positions_from_diff_exemptions.sql');
  FactoryQ.SQL.LoadFromFile(SubReg.PathToSQL+'Factories.sql');
end;

procedure TOrderForm.FormDestroy(Sender: TObject);
begin
  DelPosIds.Free;
  AddPosIds.Free;
  DelFacIds.Free;
  AddFacIds.Free;
  DelKnoIds.Free;
  AddKnoIds.Free;
  UnitsIds.Free;
  UnitsQuantity.Free;
  UnitsNames.Free;
  UnitsIncIds.Free;
  PosRemIds.Free;
  PosCost.Free;
end;

procedure TOrderForm.FormActivate(Sender: TObject);
begin
  Pages.ActivePage := IncludePage;
  IncludePage.Repaint;
  RefreshDataSetFilter(KnoPosQ);
end;

procedure TOrderForm.KnoGridKeyPress(Sender: TObject; var Key: Char);
var Id : LongInt;
begin
  if Key<>#32 then exit;
  Key := #00;
  Id := KnoQ.FieldByName('Id').AsInteger;
  with KnoGrid do begin
    if Results.IndexOf(Id)<>-1 then begin
      if not YesNoMsg('If you unmark this record all positions from exemption' +
      #13 + KnoQ.FieldByName('KnoNumber').AsString +
      ' are deleted.'#13'Are you sure you want to unmark this exemption?') then exit
      else Results.Remove(Id);
    end
    else Results.Add(Id);
  end;
  RefreshDataSetFilter(KnoPosQ);
  CheckEnable;
end;

procedure TOrderForm.CreateCrosses;
var Index, SummCount : integer;
begin
  with PosGrid do begin
    for Index := pred(CrossCount) downto 0 do DeleteCrossCells(Index);
    for Index := 0 to pred(KnoPosGrid.Results.Count) do
       AddCrossCells(GetGridRect( Index*2 + 2, 0, Index*2 + 3, 0));
    SummCount := UnitsIds.Count;
    if SummCount > 0 then begin
      AddCrossCells( GetGridRect(ColCount - SummCount - 1, 0, ColCount-1, 0));
      for Index := ColCount - SummCount - 1 to ColCount - 1 do
        Cells[Index, 0] := '';
      Cells[ColCount - SummCount - 1, 0] := 'Total';
    end;
    Cells[ColCount - 1, 1] := 'Sum';
  end;
end;

procedure TOrderForm.AddPosition (Id, IdUnits : LongInt);
var ColIndex : LongInt;
    Tmp : TQuery;
    UnitsIndex : LongInt;
begin
  //   . .   "".
  //   .,    ,    
  //  
  // !----!--------!---!---!---!---!
  // ! NN !Factory !       !       !
  // !
  UnitsIncIds.Add(IdUnits);
  UnitsIndex := UnitsIds.IndexOf(IdUnits);
  if UnitsIndex = -1 then begin
    //   ,    
    UnitsIds.Add(IdUnits);
    UnitsNames.Add(KnoPosQ.FieldByName('UnitsName').AsString);
    PosGrid.InsertColumn(PosGrid.ColCount-1);
    PosGrid.Cells[PosGrid.ColCount-2, 1] := UnitsNames[UnitsNames.Count-1];
    UnitsQuantity.Add(0);
  end;

  KnoPosGrid.Results.Add(Id);
  with PosGrid do begin
    ColIndex := ColCount - UnitsIds.Count - 1;
    InsertColumn(ColIndex);
    InsertColumn(ColIndex);
    Cells[ColIndex, 0] := KnoPosQ.FieldByName('Name').AsString;

    if UnitsIndex = -1 then Cells[ColIndex, 1] := UnitsNames[UnitsNames.Count-1]
    else Cells[ColIndex, 1] := UnitsNames[UnitsIndex];

    Cells[ColIndex+1, 1] := '';
  end;
  Tmp := GetQuery(SubReg.PathToSQL+'Remainder_by_exem_position.sql',
                 'IBKno_DataBase', [
                 IntToStr(FIdOrder),
                 IntToStr(Id),
                 IntToStr(Id)
                 ]);
  if Tmp = nil then exit;
  with Tmp do begin
    PosRemIds.Add(FieldByName('KnoIncQuantity').AsFloat-
                  FieldByName('OutQuantity').AsFloat);
    PosCost.Add(FieldByName('KnoIncCost').AsFloat);
    PosGrid.Cells[ColIndex, PosGrid.RowCount - 2] := FloatToStrF(
      PosRemIds[PosRemIds.Count-1], 15, 3);
    PosGrid.Cells[ColIndex+1, PosGrid.RowCount - 2] := FloatToStrF(
       FieldByName('KnoIncCost').AsFloat, 15, 3);
    Free;
  end;
  if DelPosIds.IndexOf(Id) <> -1 then DelPosIds.Remove(Id);
  CreateCrosses;
  Recalc;
end;

procedure TOrderForm.RemovePosition (Id, IdUnits : LongInt);
var PosIndex, ColIndex : LongInt;
begin
  PosIndex := KnoPosGrid.Results.IndexOf(Id);
  PosRemIds.Delete(PosIndex);
  PosCost.Delete(PosIndex);
  KnoPosGrid.Results.Delete(PosIndex);
  //  ..   "".  
  //    ,   
  //   
  UnitsIncIds.Remove(IdUnits);
  if UnitsIncIds.IndexOf(IdUnits) = -1 then begin
    // . .  ,    
    //   
    ColIndex := UnitsIds.IndexOf(IdUnits);
    UnitsQuantity.Delete(ColIndex);
    UnitsIds.Remove(IdUnits);
    UnitsNames.Delete(ColIndex);
    ColIndex := PosGrid.ColCount - 1 - ColIndex;
    PosGrid.DeleteColumn(ColIndex);
  end;
  //     ,     
  // !  0 !    1   ! 2 ! 3 ! 4 ! 5 !
  // !----!--------!---!---!---!---!
  // ! NN !Factory ! Goods1! Goods2!
  // !
  ColIndex := 2 + PosIndex*2;
  PosGrid.DeleteColumn(ColIndex);
  PosGrid.DeleteColumn(ColIndex);
  DelPosIds.Add(Id);
  CreateCrosses;
  Recalc;
end;

procedure TOrderForm.KnoPosGridKeyPress(Sender: TObject; var Key: Char);
var Id, IdUnits : LongInt;
    Index : integer;
begin
  if Key<>#32 then exit;
  Key := #00;
  Id := KnoPosQ.FieldByName('Id').AsInteger;
  IdUnits := KnoPosQ.FieldByName('IdUnits').AsInteger;
  Index := KnoPosGrid.Results.IndexOf(Id);
  if Index<>-1 then begin
    if not YesNoMsg('If you unmark this record the goods is deleted from order.'#13+
      'Are you sure you want to unmark?') then exit
    else RemovePosition(Id, IdUnits);
  end
  else AddPosition(Id, IdUnits);
  CheckEnable;
end;

procedure TOrderForm.Recalc; //   
var ColIndex, RowIndex : LongInt;
    S1 : Double;
    PosSumm, CellValue : Double;
    Error : boolean;
    UnitsIndex : Longint;
begin
  if not RecalcEnabled then exit;

  //   
  with PosGrid do begin
    for ColIndex := 0 to pred(KnoPosGrid.Results.Count) do begin
      S1 := 0; //-
      for RowIndex := 2 to (RowCount-3) do begin
        CellValue := WStrToFloat(Cells[2*ColIndex+2, RowIndex], Error);
        S1 := S1 + CellValue;
        Cells[2*ColIndex+3, RowIndex] := FloatToStrF(CellValue * PosCost[ColIndex],
          15, 3);
      end;

      Cells[2*ColIndex+2, RowCount - 1] := FloatToStrF(S1, 15, 3);
      Cells[2*ColIndex+3, RowCount - 1] := FloatToStrF(S1 * PosCost[ColIndex], 15, 3);

      Cells[2*ColIndex+2, RowCount - 2] := FloatToStrF(PosRemIds[ColIndex] - S1, 15, 3);
      Cells[2*ColIndex+3, RowCount - 2] := FloatToStrF((PosRemIds[ColIndex] - S1)
        * PosCost[ColIndex], 15, 3);

    end;
    for RowIndex := 2 to RowCount - 1 do begin
      S1 := 0;
      for ColIndex := 0 to pred(UnitsQuantity.Count) do
        UnitsQuantity[ColIndex] := 0;

      for ColIndex := 0 to pred(KnoPosGrid.Results.Count) do begin
        CellValue  :=  WStrToFloat(Cells[2*ColIndex+2, RowIndex], Error);
        UnitsIndex := UnitsIds.IndexOf(UnitsIncIds[ColIndex]);
        PosSumm    := CellValue*PosCost[ColIndex];
        S1         := S1 + PosSumm;
        UnitsQuantity[UnitsIndex] := UnitsQuantity[UnitsIndex] + CellValue;
      end;
      Cells[ColCount-1, RowIndex] := FloatToStrF(S1, 15, 3);

      for ColIndex := 0 to pred(UnitsQuantity.Count) do
        Cells[ColCount - 1 - UnitsQuantity.Count + ColIndex, RowIndex] :=
          FloatToStrF(UnitsQuantity[ColIndex], 15, 3);
    end;
    SummE.Value := WStrToFloat(Cells[ColCount-1, RowCount -1], Error);
  end;
end;

procedure TOrderForm.RemoveFactory (Id : LongInt);
var RowIndex, Index : integer;
begin
  Index := FactoryGrid.Results.IndexOf(Id);
  FactoryGrid.Results.Delete(Index);
  with PosGrid do begin
    RowIndex := Index + 2;
    DeleteRow(RowIndex);
  end;
  Recalc;
end;

procedure TOrderForm.AddFactory (Id : LongInt; FactoryName : string);
begin
  FactoryGrid.Results.Add(Id);
  with PosGrid do begin
    InsertRow(RowCount - 2);
    Cells[0, RowCount - 3] := IntToStr(FactoryGrid.Results.Count);
    Cells[1, RowCount - 3] := FactoryName;
  end;
  Recalc;
end;

procedure TOrderForm.FactoryGridKeyPress(Sender: TObject; var Key: Char);
var Id, Index : LongInt;
begin
  if Key<>#32 then exit;
  Key := #00;
  Id := FactoryQ.FieldByName('Id').AsInteger;
  Index := FactoryGrid.Results.IndexOf(Id);
  if Index<>-1 then begin
    if not YesNoMsg('If you unmark this record the factory is deleted from order.'#13+
      'Are you sure you want to unmark?') then exit
    else RemoveFactory(Id);
  end
  else AddFactory(Id, FactoryQ.FieldByName('Name').AsString);
  CheckEnable;
end;

procedure TOrderForm.PosGridCalcCellColors(Sender: TObject; Col,
  Row: Integer; aFont: TFont; var aColor: TColor; State: TGridDrawState);
var Error : boolean;
    Value : Double;
begin
  if (Row>=2) and (Col>=2) then begin
    Value := WStrToFloat(PosGrid.Cells[Col, Row], Error);
    if Value < 0 then begin
      aColor := clRed;
      aFont.Color := clYellow;
      exit;
    end;
    if (Row = PosGrid.RowCount - 2) then aFont.Color := clBlue;
    if (Row = PosGrid.RowCount - 1) then aFont.Style := [fsBold];
  end;
end;

procedure TOrderForm.PosGridAfterCellEdit(Sender: TObject; Col,
  Row: Integer);
var Value : Double;
    Error : boolean;
    DoRecalc : boolean;
begin
  {if (Row >= PosGrid.RowCount - 2) or
     (Col >= PosGrid.ColCount - UnitsIds.Count - 1) then
     Value := LastValue;}
  Value := WStrToFloat(PosGrid.Cells[Col, Row], Error);
  DoRecalc := Value <> LastValue;
  PosGrid.Cells[Col, Row] := FLoatToStrF(Value, 15, 3);
  if DoRecalc then Recalc;
end;

procedure TOrderForm.PosGridBeforeCellEdit(Sender: TObject; Col,
  Row: Integer);
var Error : boolean;
begin
  LastValue := WStrToFloat(PosGrid.Cells[Col, Row], Error);
end;

procedure TOrderForm.PagesChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  AllowChange := (Pages.ActivePage = PositionsPage) or
    ((KnoPosGrid.Results.Count>0) and (FactoryGrid.Results.Count>0));
end;

function TOrderForm.GetKnoNumbers : string;
begin
  Result := '';
  with KnoQ do begin
    First;
    while not Eof do begin
      if KnoGrid.Results.IndexOf(FieldByName('Id').AsInteger)<>-1 then
        Result := Result + FieldByName('KnoNumber').AsString+', ';
      Next;
    end;
  end;
  if Result <> '' then Result := DecLength(Result, 2);
end;

procedure TOrderForm.KnoPosQFilterRecord(DataSet: TDataSet;
  var Accept: Boolean);
var Id, IdKNO, IdUnits : LongInt;
begin
  IdKno   := DataSet.FieldByName('IdKNO').AsInteger;
  Id      := DataSet.FieldByName('Id').AsInteger;
  IdUnits := DataSet.FieldByName('IdUnits').AsInteger;
  Accept := KnoGrid.Results.IndexOf(IdKNO)<>-1;
  if (not Accept) and (KnoPosGrid.Results.IndexOf(Id)<>-1) then
    RemovePosition(Id, IdUnits);
end;

initialization
  OrderForm := nil;
end.
