{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
    TsohoAboutBox -   
 " ..."
}
unit SoAbout;

interface
uses Windows, SysUtils, Classes, Controls, Forms, SoCtrls, SoVrsInf,
     Graphics, StdCtrls, ExtCtrls, SohoBtns;

type

  TsohoAboutBox = class;

  TsohoItemKind = (ikSeparator, ikLabel, ikURL, ikMailTo,
    ikFileVersion, ikLegalCopyright, ikLegalTrademarks,
    ikFreeMemory, ikOperationSystem, ikSohoLibPro, ikSohoIBLibPro);
  {     TsohoAboutItem   TsohoAboutBox }

  TsohoAboutItem = class(TCollectionItem)
  private
    FText: string;
    FContents: string;
    FComment: string;
    FKind: TsohoItemKind;
    FBitmap: TBitmap;
    FHeight: Integer;
    FOffset: Integer;
    FFont: TFont;
  protected
    function GetDisplayName: string; override;
    procedure SetBitmap(Value: TBitmap);
    procedure SetFont(Value: TFont);
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Text: string read FText write FText;
    {   ,     }
    property Kind: TsohoItemKind read FKind write FKind default ikLabel;
    {   ,     }
    property Bitmap: TBitmap read FBitmap write SetBitmap;
    {   ,        }
    property Height: Integer read FHeight write FHeight default -1;
    {   ,     . 
         -1,      }
    property Font: TFont read FFont write SetFont;
    {   ,       }
    property Contents: string read FContents write FContents;
    {   ,      .
         ,   Kind  ikURL 
      ikMailTo.      url   e-mail  
        Contents.       ,
             }
    property Comment: string read FComment write FComment;
    {   ,     design-time.  
      run-time    }
    property Offset: Integer read FOffset write FOffset default 0;
    {   ,      
          }
  end;

  TsohoAboutItems = class(TCollection)
  private
    FAboutBox: TsohoAboutBox;
    function GetItem(Index: Integer): TsohoAboutItem;
    procedure SetItem(Index: Integer; Value: TsohoAboutItem);
  protected
    function GetOwner: TPersistent; override;
  public
    constructor Create(AboutBox: TsohoAboutBox);
    function Add: TsohoAboutItem;
    {     }
    property Items[Index: Integer]: TsohoAboutItem read GetItem write SetItem; default;
    {       }
  end;

  TsohoSeperatorStyle = (ssFullSize, ssSpace, ssSolid);
  {        TsohoAboutBox.
       :
    ssFullSize -        
    ssSpace -   ,    
    ssSolid -      
    }
  TsohoSeperatorStyles = set of TsohoSeperatorStyle;
  {       }

  TsohoBitmapOption = (boCenter, boByItems, boUseMainIcon);
  {        
    TsohoABoutBox.    :
    boCenter -      
                 
    boByItems -        
    boUseMainIcon -      
        
    }
  TsohoBitmapOptions = set of TsohoBitmapOption;
  {   ,    
       TsohoAboutBox }

  TsohoAboutBox = class(TComponent)
  private
    { Private declarations }
    FItems: TsohoAboutItems;
    FCaption: string;
    FWidth: LongInt;
    FSepStyle: TsohoSeperatorStyles;
    FImagesAreaColor: TColor;
    FImagesOptions: TsohoBitmapOptions;
    procedure SetItems(const Value: TsohoAboutItems);
    procedure LinkClicked(Sender: TObject);
  protected
    { Protected declarations }
    procedure CreateDefaultItems; virtual;
    function GetMaxBitmapWidth: LongInt;
    function PlaceBitmaps(Frm: TForm; BmpWidth: LongInt): LongInt;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute;
    {   }
  published
    property Caption: string read FCaption write FCaption;
    {   ,     }
    property Items: TsohoAboutItems read FItems write SetItems;
    {   ,   
             }
    property Width: LongInt read FWidth write FWidth;
    {   ,    . 
              ,
             , 
              Widht 
          }
    property SeparatorStyle: TsohoSeperatorStyles read FSepStyle write FSepStyle default [ssFullSize];
    {   ,    .  ,
          SeparatorStyle   ssSolid, 
       ssSpace  }
    property ImagesAreaColor: TColor read FImagesAreaColor write FImagesAreaColor default clBtnFace;
    {   ,       , 
        ,     }
    property ImagesOptions: TsohoBitmapOptions read FImagesOptions write FImagesOptions default [boByItems, boUseMainIcon];
    {   ,       .
          boCenter,   
            
      .      boByItems, 
          v   , 
               
        }
  end;

implementation
uses SoStrUtils, SoAppUtils, SoCmnCns;

{$R SOABOUT.R32}

const

  Offset = 8;

{ TsohoAboutBox }
constructor TsohoAboutBox.Create(aOwner: TComponent);
begin
  inherited;
  FItems := TsohoAboutItems.Create(Self);
  FCaption := EmptyStr;
  FWidth := 200;
  FSepStyle := [ssFullSize];
  FImagesAreaColor := clBtnFace;
  FImagesOptions := [boByItems, boUseMainIcon];
  CreateDefaultItems;
end;

procedure TsohoAboutBox.CreateDefaultItems;

    function AddItem(const aText, aContents, aComment, aResBitmap: string;
      aKind: TsohoItemKind): TsohoAboutItem;
    begin
      Result := Items.Add;
      with Result do begin
        Text := aText;
        Kind := aKind;
        Contents := aContents; 
        Comment := aComment;
        if aResBitmap <> EmptyStr then
          Bitmap.LoadFromResourceName(hInstance, aResBitmap);
      end;
    end;

begin
  with AddItem('Program name', EmptyStr, 'Program name', EmptyStr, ikLabel) do
    Offset := 5;
  Items[0].Font.Size := 18;
  Items[0].Font.Style := [fsBold];
  with AddItem('Separator', EmptyStr, 'Separator', EmptyStr, ikSeparator) do
    Offset := 5;
  AddItem('2000, Second Dream', EmptyStr, 'Copyright', EmptyStr, ikLegalCopyright);
  with AddItem('2000, Second Dream', EmptyStr, 'Trademark',
    EmptyStr, ikLegalTrademarks) do Offset := 5;
  with AddItem(EmptyStr, EmptyStr, 'File version',
    EmptyStr, ikFileVersion) do Offset := 5;
  with AddItem(EmptyStr, EmptyStr, 'Separator', EmptyStr, ikSeparator) do
    Offset := 5;
  AddItem('SohoLib Logo', EmptyStr, 'SohoLib', 'SOHOLIBLOGO', ikSohoLibPro);
  AddItem('http://www.second-dream.ru', 'http://www.second-dream.ru',
    'SohoLib URL', EmptyStr, ikURL);
  with AddItem('soholib@sdream.san.ru', 'soholib@sdream.san.ru',
    'SohoLib e-mail', EmptyStr, ikMailTo) do Offset := 15;
  with AddItem('Separator', EmptyStr, 'Separator', EmptyStr, ikSeparator) do
    Offset := 5;
  AddItem('Operation System', EmptyStr, 'Operation System', EmptyStr, ikOperationSystem);
  with AddItem('Free memory item', EmptyStr, 'Free memory item', EmptyStr,
    ikFreeMemory) do Offset := 5;
  with AddItem('Separator', EmptyStr, 'Separator', EmptyStr, ikSeparator) do
    Offset := 15;
end;

destructor TsohoAboutBox.Destroy;
begin
  FItems.Free;
  inherited;
end;

function TsohoAboutBox.PlaceBitmaps(Frm: TForm; BmpWidth: LongInt): LongInt;
var Index, NextTop: LongInt;
    Image: TImage;
begin
  NextTop := Offset;
  for Index := 0 to pred(FItems.Count) do begin
    if (not FItems[Index].Bitmap.Empty) or
       ((boUseMainIcon in FImagesOptions) and (Index = 0)) then begin
      Image := TImage.Create(Frm);
      with Image do begin
        Parent := Frm;
        Left := Offset;
        Top := NextTop;
        if (boUseMainIcon in FImagesOptions) and (Index = 0) then
          Picture.Assign(Application.Icon)
        else
          Picture.Assign(FItems[Index].Bitmap);
        AutoSize := true;
        if boCenter in FImagesOptions then begin
          AutoSize := false;
          Width := BmpWidth;
          Center := true;
        end;
      end;
      NextTop := NextTop + Image.Height;
    end;
  end;
  Result := NextTop;
end;

procedure TsohoAboutBox.Execute;
var Frm: TForm;
    CurHeight, NextTop, ImagesHeight: LongInt;
    TextLeft, BmpWidth: LongInt;
    Index: LongInt;
    MemStat: TMemoryStatus;
    Shape: TShape;

    procedure CreateLabel(const aText: string; Index: LongInt);
    var Lbl: TLabel;
    begin
      Lbl := TLabel.Create(Frm);
      with Lbl do begin
        Parent := Frm;
        AutoSize := false;
        Top := NextTop;
        Font.Assign(FItems[Index].Font);
        Left := TextLeft;
        WordWrap := true;
        Caption := aText;
        Width := FWidth - 3 * Offset;
        if FItems[Index].Height = -1 then begin
          AutoSize := false;
          AutoSize := true;
        end
        else Height := FItems[Index].Height;
      end;
      CurHeight := Lbl.Height + 2 + FItems[Index].Offset;
    end;

    procedure CreateSohoLabel(const aText: string; Index: LongInt);
    var Lbl: TsohoLabel;
    begin
      Lbl := TsohoLabel.Create(Frm);
      with Lbl do begin
        Bevel := false;
        AutoSize := true;
{$IFDEF Delphi4_or_5}       
        Layout := tlTop;
{$ENDIF}
        Parent := Frm;
        Top := NextTop;
        Font.Assign(FItems[Index].Font);
        FontActive.Assign(FItems[Index].Font);
        Left := TextLeft;
        WordWrap := false;
        Caption := aText;
        Width := FWidth - 3 * Offset;
        Alive := true;
        Font.Color := clBlue;
        FontActive.Color := clBlue;
        FontActive.Style := [fsUnderline];
        Cursor := crHandPoint;
        Tag := Index;
        OnClick := LinkClicked;
        if FItems[Index].Height <> -1 then
          Height := FItems[Index].Height;
      end;
      CurHeight := Lbl.Height + FItems[Index].Offset;
    end;

    procedure CreateSeparator;
    var Separator: TGraphicControl;
    begin
      if ssSolid in FSepStyle then
        Separator := TShape.Create(Frm)
      else
        Separator := TBevel.Create(Frm);
      with Separator do begin
        Parent := Frm;
        Top := NextTop + 2;
        if ssFullSize in FSepStyle then begin
          Width := FWidth + BmpWidth - Offset;
          Left := 4;
        end
        else begin
          Width := FWidth - 3 * Offset;
          Left := TextLeft;
        end;
        if ssSolid in FSepStyle then begin
          TShape(Separator).Brush.Color := clBlack;
          Height := 1;
        end
        else begin
          Height := 3;
          TBevel(Separator).Style := bsLowered;
          if ssSpace in FSepStyle then
{$IFDEF Delphi4_or_5}
            TBevel(Separator).Shape := bsSpacer
{$ELSE}
            TBevel(Separator).Shape := bsBox
{$ENDIF}
          else
            TBevel(Separator).Shape := bsTopLine;
        end;
      end;
      if FItems[Index].Height <> -1 then
        CurHeight := FItems[Index].Height + 4 + FItems[Index].Offset
      else
        CurHeight := Separator.Height + 4 + FItems[Index].Offset;
    end;

    procedure CreateImage(Index: LongInt);
    var Image: TImage;
    begin
      Image := TImage.Create(Frm);
      with Image do begin
        Parent := Frm;
        Left := Offset;
        Top := NextTop;
        if (boUseMainIcon in FImagesOptions) and (Index = 0) then
          Picture.Assign(Application.Icon)
        else
          Picture.Assign(FItems[Index].Bitmap);
        AutoSize := true;
        if boCenter in FImagesOptions then begin
          AutoSize := false;
          Width := BmpWidth;
          Center := true;
        end;
      end;
      if Image.Height > CurHeight then
        CurHeight := Image.Height;
    end;

begin
  Frm := TForm.Create(Application);
  NextTop := Offset;
  BmpWidth := GetMaxBitmapWidth;
  TextLeft := BmpWidth + Offset * 2;

  Shape := nil;
  if FImagesAreaColor <> clBtnFace then begin
    Shape := TShape.Create(Frm);
    with Shape do begin
      Parent := Frm;
      Left := Offset;
      Width := BmpWidth;
      Top := Offset;
      Brush.Color := FImagesAreaColor;
      Pen.Color := FImagesAreaColor;
    end;
  end;

  with Frm do begin
    BorderStyle := bsToolWindow;
    Caption := FCaption; // - 
    Position := poScreenCenter;
    ClientWidth := FWidth + BmpWidth;

    for Index := 0 to pred(FItems.Count) do begin
      CurHeight := 0;
      case FItems[Index].Kind of
        ikLabel: CreateLabel(FItems[Index].Text, Index);
        ikURL: CreateSohoLabel('url: ' + FItems[Index].Text, Index);
        ikMailTo: CreateSohoLabel('e-mail: ' + FItems[Index].Text, Index);
        ikFileVersion: CreateLabel(sohoAboutFileVersion + GetFileVersion, Index);
        ikLegalCopyright: CreateLabel('Copyright  ' +
          GetLegalCopyright, Index);
        ikLegalTrademarks: CreateLabel('Trademark  ' +
          GetLegalTrademarks, Index);
        ikFreeMemory: begin
          MemStat.dwLength := SizeOf(TMemoryStatus);
          GlobalMemoryStatus(MemStat);
          CreateLabel(sohoAboutMemory +
            FloatToStrF(trunc(MemStat.dwTotalPhys / 1024), ffNumber, 7, 0)
             + ' KB ', Index);
        end;
        ikOperationSystem: CreateLabel(WindowsVerStr, Index);
        ikSohoLibPro: CreateLabel(sohoAboutSohoLib, Index);
        ikSohoIBLibPro: CreateLabel(sohoAboutSohoIBLib, Index);
        else CreateSeparator; // Separator
      end;
      if (boByItems in FImagesOptions) and
        (
           (not FItems[Index].Bitmap.Empty)
            or
           ((boUseMainIcon in FImagesOptions) and (Index = 0))
          ) then
        CreateImage(Index);
      NextTop := NextTop + CurHeight;
    end;
    ImagesHeight := 0;

    if not (boByItems in FImagesOptions) then
      ImagesHeight := PlaceBitmaps(Frm, BmpWidth);
    if NextTop + 40 < ImagesHeight then
      ClientHeight := ImagesHeight + Offset
    else
      ClientHeight := NextTop + 40;

    if Shape <> nil then
      Shape.Height := ClientHeight - 2 * Offset;

    with TsohoBitBtn.Create(Frm) do begin
      Parent := Frm;
      Caption := 'OK';
      Default := true;
      Cancel := true;
      Top := Frm.ClientHeight - Height - Offset;
      Left := FWidth + BmpWidth - Width - Offset;
      ModalResult := mrOk;
    end;
    ShowModal;
  end;
end;

function TsohoAboutBox.GetMaxBitmapWidth: LongInt;
var Index: LongInt;
begin
  Result := 0;
  for Index := 0 to pred(FItems.Count) do
    if FItems[Index].Bitmap.Width > Result then
      Result := FItems[Index].Bitmap.Width;
end;

procedure TsohoAboutBox.LinkClicked(Sender: TObject);
var Item: TsohoAboutItem;
    Run: string;
begin
  Item := FItems[(Sender as TsohoLabel).Tag];
  case Item.Kind of
    ikURL: begin
      Run := Item.Contents;
      if AnsiLowerCase(Copy(Run, 1, 7)) <> 'http://'
       then Run := 'http://' + Run;
      Exec(Run, EmptyStr, sw_shownormal, false);
    end;
    ikMailTo: begin
      Run := Item.Contents;
      if AnsiLowerCase(Copy(Run, 1, 7)) <> 'mailto:'
       then Run := 'mailto:' + Run;
      Exec(Run, EmptyStr, sw_shownormal, false);
    end;
    else ;
  end;
end;

procedure TsohoAboutBox.SetItems(const Value: TsohoAboutItems);
begin
  FItems.Assign(Value);
end;

{ TsohoAboutItem }

procedure TsohoAboutItem.Assign(Source: TPersistent);
begin
  if not (Source is TsohoAboutItem) then exit; //   
  FKind := TsohoAboutItem(Source).Kind;
  FBitmap := TsohoAboutItem(Source).Bitmap;
  FText := TsohoAboutItem(Source).Text;
  FFont := TsohoAboutItem(Source).Font;
  FHeight := TsohoAboutItem(Source).Height;
end;

constructor TsohoAboutItem.Create(Collection: TCollection);
begin
  FKind := ikLabel;
  FBitmap := TBitmap.Create;
  FFont := TFont.Create;
  FHeight := -1;
  FContents := EmptyStr;
  inherited Create(Collection);
end;

destructor TsohoAboutItem.Destroy;
begin
  FBitmap.Free;
  FFont.Free;
  inherited;
end;

function TsohoAboutItem.GetDisplayName: string;
begin
  Result := Text;
  if Result = EmptyStr then Result := inherited GetDisplayName;
end;

procedure TsohoAboutItem.SetBitmap(Value: TBitmap);
begin
  FBitmap.Assign(Value);
end;

procedure TsohoAboutItem.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
end;

{ TsohoAboutItems }

function TsohoAboutItems.Add: TsohoAboutItem;
begin
  Result := TsohoAboutItem(inherited Add);
end;

constructor TsohoAboutItems.Create(AboutBox: TsohoAboutBox);
begin
  inherited Create(TsohoAboutItem);
  FAboutBox := AboutBox;
end;

function TsohoAboutItems.GetItem(Index: Integer): TsohoAboutItem;
begin
  Result := TsohoAboutItem(inherited GetItem(Index));
end;

function TsohoAboutItems.GetOwner: TPersistent;
begin
  Result := FAboutBox;
end;

procedure TsohoAboutItems.SetItem(Index: Integer; Value: TsohoAboutItem);
begin
  inherited SetItem(Index, Value);
end;

end.
