{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{           Partial copyright (c) AllexSoft             }
{           Written by Alexander Govyadinov,            }
{                   Serg Vostrikov                      }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoCalcF;

{$I SOHOLIB.INC}

interface

uses Windows, Messages, Classes, Graphics, Controls, Forms, SysUtils,
     StdCtrls, SohoBtns, SoCtrls, Buttons, ExtCtrls, Menus;


const

   NumberOfOperations = 20;

type

   TLongDouble = array[1..NumberOfOperations] of Double;
   PLongDouble = ^TLongDouble;
   TShortChar = string[3];
   TLongChar = array[1..NumberOfOperations] of TShortChar;
   PLongChar = ^TLongChar;
   TShortProcedure = procedure of object;
   TLongProcedure = array[1..NumberOfOperations] of TShortProcedure;
   PLongProcedure = ^TLongProcedure;
   TLongByte = array[1..NumberOfOperations] of byte;
   PLongByte = ^TLongByte;
   TLongBoolean = array[1..NumberOfOperations] of boolean;
   PLongBoolean = ^TLongBoolean;

var
   Operations : TStringList;

type

   TCalcOperation = class(TObject)
     Proc: procedure of object;
     Immediatly: boolean;
     Priority: byte;
   end;

   TsohoCalcHistory = record
     Value: Double;
     Decimals: LongInt;
     DigitalInput: boolean;
   end;

   TsohoCalculating = class
   private
     fOperationsNumber: LongInt;
     fOnChange: TNotifyEvent;
     fAutoCalc: boolean;
     fAutoPrecision: boolean;
     fStatement: string;
     fStack: array[1..NumberOfOperations] of double;
     fOperationStack: array[1..NumberOfOperations] of char;
     fFormat     : TFloatFormat;
     fPrecision  : LongInt;
     fDigits     : LongInt;
     SP          : LongInt;
     OP          : LongInt;
     fMemory     : Double;
     fError      : boolean;
     fErrorText  : string;
     fDecimals   : LongInt;
     fHistory    : array[1..100] of TsohoCalcHistory;
     FDisplayFormat: string;
     OperationName: PLongChar;
     OperationMaker: PLongProcedure;
     OperateImmediatly: PLongBoolean;
     OperationPriority: PLongByte;
     function GetValueAsString: string;
   protected
     // function  GetValueByName(Name: string): double; virtual; abstract;
     procedure DoChange; virtual;
     procedure Push; virtual;
     procedure Pop; virtual;
     procedure PushHistory; virtual;
     procedure PopHistory; virtual;
     procedure PushOperations; virtual;
     procedure Popoperations; virtual;
     procedure AppendDigit; virtual;
     procedure Add; virtual;
     procedure SubTract; virtual;
     procedure Multiply; virtual;
     procedure Divide; virtual;
     procedure SquareRoot; virtual;
     procedure Minus; virtual;
     procedure Clear; virtual;
     procedure BackSpace; virtual;
     procedure Percent; virtual;
     procedure Invert; virtual;
     procedure Store; virtual;
     procedure Restore; virtual;
     procedure MemoryClear; virtual;
     procedure MemoryAdd; virtual;
     procedure LeftPar; virtual;
     procedure RightPar; virtual;
     procedure EqualSign; virtual;
   public
     DigitSet: set of char;
     PointSet: set of char;
     DigitalInput: boolean;
     constructor Create; virtual;
     destructor Destroy; override;
     procedure NextChar(Value: char);
     procedure NextValue(Value: Double);
     procedure NextStatement(Value: string);
     procedure Calculate;
     procedure CalculateStatement;
     property AutoCalc      :boolean      read fAutoCalc write fAutoCalc;
     property AutoPrecision :boolean      read fAutoPrecision write fAutoPrecision;
     property InputChar     :char         write NextChar;
     property InputValue    :Double       write NextValue;
     property InputStatement:string       read fStatement write NextStatement;
     property AsFloat       :Double       read fStack[1];
     property Asstring      :string       read GetValueAsString;
     property Format        :TFloatFormat read fFormat write fFormat;
     property Precision     :LongInt      read fPrecision write fPrecision;
     property Digits        :LongInt      read fDigits write fDigits;
     property Error         :boolean      read fError;
     property ErrorText     :string       read fErrorText;
     property DisplayFormat: string read FDisplayFormat write FDisplayFormat;
     property OnChange: TNotifyEvent read fOnChange write fOnChange;
  end;

  TsohoCalculatorForm = class(TForm)
    Enter: TsohoSpeedButton;
    Digit3: TsohoSpeedButton;
    Digit2: TsohoSpeedButton;
    Digit1: TsohoSpeedButton;
    Digit6: TsohoSpeedButton;
    Digit5: TsohoSpeedButton;
    Digit4: TsohoSpeedButton;
    CLearError: TsohoSpeedButton;
    Divide: TsohoSpeedButton;
    Digit7: TsohoSpeedButton;
    Digit8: TsohoSpeedButton;
    Multiply: TsohoSpeedButton;
    Minus: TsohoSpeedButton;
    Digit9: TsohoSpeedButton;
    Plus: TsohoSpeedButton;
    BackSpace: TsohoSpeedButton;
    Digit0: TsohoSpeedButton;
    UnMinus: TsohoSpeedButton;
    Point: TsohoSpeedButton;
    Invert: TsohoSpeedButton;
    SQRTB: TsohoSpeedButton;
    Percent: TsohoSpeedButton;
    Clear: TsohoSpeedButton;
    MemoryClear: TsohoSpeedButton;
    MemoryAdd: TsohoSpeedButton;
    MemoryStore: TsohoSpeedButton;
    MemoryResult: TsohoSpeedButton;
    SpeedButton1: TsohoSpeedButton;
    SpeedButton2: TsohoSpeedButton;
    OkB: TsohoSpeedButton;
    CancelB: TsohoSpeedButton;
    DispP: TPanel;
    DispL: TLabel;
    ClipMenu: TPopupMenu;
    N1: TMenuItem;
    N2: TMenuItem;
    N3: TMenuItem;
    procedure BtnClick(Sender: TObject);
    procedure OkBClick(Sender: TObject);
    procedure CancelBClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure N1Click(Sender: TObject);
    procedure N2Click(Sender: TObject);
    procedure N3Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    procedure CalcChanged (Sender: TObject);
    procedure ClipCut;
    procedure ClipCopy;
    procedure ClipPaste;
  public
    { Public declarations }
    Calculator: TsohoCalculating;
    procedure ButtonsFlat(Value: boolean);
  end;

var
  sohoCalculatorForm: TsohoCalculatorForm;

function GetValue(var Value: Double): boolean;
function GetValueWithPos(Sender: TControl; var Value: Double): boolean;

implementation
uses ClipBrd, SoUtils, SoVCLUtils, SoCmnCns;

{$R *.DFM}


function Power(Base:Double;Degree:LongInt):Double;
var i:LongInt;
    ADegree:longInt;
begin
 if Degree<0 then ADegree:=-Degree else ADegree:=Degree;
 Result:=1;
 for i:=1 to ADegree do Result:=Result*Base;
 if Degree<0 then Result:=1/Result;
end;

procedure TsohoCalculating.DoChange;
begin
  if Assigned(fOnChange) then fOnChange(Self);
end;

constructor TsohoCalculating.Create;
var Index: LongInt;
begin
  inherited; // ,  
  DigitSet := ['0'..'9']; //Now 10; May be extend for 2/8/16 etc // 
  PointSet := ['.', ',']+[DecimalSeparator]; // ,     
  fFormat := ffFixed; // 
  fPrecision := 6;
  fDigits := 10;
  fOperationsNumber := Operations.Count;

  GetMem(OperationName, fOperationsNumber*SizeOf(TShortChar));
  GetMem(OperateImmediatly, fOperationsNumber*SizeOf(Boolean));
  GetMem(OperationPriority, fOperationsNumber*SizeOf(Byte));
  GetMem(OperationMaker, fOperationsNumber*SizeOf(TShortprocedure));

  for Index := 1 to fOperationsNumber do
  with Operations do begin
   OperationName[Index] := Operations[Index-1];
   OperateImmediatly[Index] := TCalcOperation(Objects[Index-1]).Immediatly;
   OperationPriority[Index] := TCalcOperation(Objects[Index-1]).Priority;
   if Assigned(TCalcOperation(Objects[Index-1]).Proc) then
      OperationMaker[Index] := TCalcOperation(Objects[Index-1]).Proc
   else
      case Index of
        1: OperationMaker[Index] := Self.Add;
        2: OperationMaker[Index] := Self.SubTract;
        3: OperationMaker[Index] := Self.Multiply;
        4: OperationMaker[Index] := Self.Divide;
        5: OperationMaker[Index] := Self.Clear;
        6: OperationMaker[Index] := Self.EqualSign;
        7: OperationMaker[Index] := Self.SquareRoot;
        8: OperationMaker[Index] := Self.Minus;
        9: OperationMaker[Index] := Self.BackSpace;
       10: OperationMaker[Index] := Self.LeftPar;
       11: OperationMaker[Index] := Self.RightPar;
       12: OperationMaker[Index] := Self.Invert;
       13: OperationMaker[Index] := Self.Percent;
       14: OperationMaker[Index] := Self.MemoryClear;
       15: OperationMaker[Index] := Self.ReStore;
       16: OperationMaker[Index] := Self.Store;
       17: OperationMaker[Index] := Self.MemoryAdd;
       else ErrorMsg('SohoCalculating initialization section : Bad Programmer - replace and strike any key.');
      end;
  end;
  FDisplayFormat := '';
end;

destructor TsohoCalculating.Destroy;
begin
  FreeMem(OperationName);
  FreeMem(OperateImmediatly);
  FreeMem(OperationPriority);
  FreeMem(OperationMaker);
  inherited;
end;

procedure TSohoCalculating.NextChar(Value:Char);
var i:LongInt;
begin
 //Char may be digit or operation sign
 fError:=False; //Reset error signal
 PushHistory;
 if Value in DigitSet then begin
  if not DigitalInput then begin
   fStack[1]:=0;
   fDecimals:=0;
  end;
  i:=Ord(Value)-Ord('0');
   if fDecimals=0
    then fStack[1]:=fStack[1]*10+I
    else begin
    fStack[1]:=fStack[1]+I*Power(10,-fDecimals);
    inc(fDecimals);
   end; //if
  DigitalInput:=True;
  DoChange;
  Exit;
 end; //if digitset

 if Value in PointSet then begin
  if not DigitalInput then fStack[1]:=0;
  fDecimals:=1;
  DigitalInput:=True;
  DoChange;
  Exit;
 end; //if pointset

 for i:=1 to fOperationsNumber do
  if OperationName[i]=Value then Break;
 if (OperationName[i]=Value) and (Assigned(OperationMaker[i]))
  then if
    OperateImmediatly[i] then OperationMaker[i]
    else begin
      PushOperations;
      fOperationStack[1]:=Value;
      Push;
    end;
 DigitalInput:=False;
 DoChange;
end;

procedure TSohoCalculating.NextValue(Value:Double);
begin
 PushHistory;
 fStack[1]:=Value;
 DoChange;
end;

procedure TSohoCalculating.NextStatement(Value:String);

var
 LC,RC,I:LongInt;
begin
//Get statement, and conditionally parse and calculate it
 //   .
 LC:=0;RC:=0;
 for i:=1 to Length(Value) do
 case Value[i] of
 '(':inc(LC);
 ')':inc(RC);
 end;

 if LC<>RC then begin
  fErrorText:='"(" <> ")" !';
  fError:=True;
  Exit;
 end;

 fStateMent:=Value;
 if fAutoCalc then CalculateStatement;
 DoChange;
end;

procedure TSohoCalculating.Calculate;
var i:LongInt;
begin
 for i:=1 to fOperationsNumber do
  if OperationName[i]=fOperationStack[1] then Break;
 if (OperationName[i]=fOperationStack[1]) and (Assigned(OperationMaker[i]))
   then   OperationMaker[i];
  PopOperations;
  DoChange;
end;

function TSohoCalculating.GetValueAsString:string;
var PE:LongInt;
    Power:String;
begin
 if fAutoPrecision then begin
  Result:=SysUtils.FloatToStrF(fStack[1],fFormat,13,50);
  PE:=Pos('E',Result);
  if PE<>0 then begin
   Power:=copy(Result,PE,100);
   Delete(Result,PE,100);
  end;
  //Supress head blanks
  while Result[1]=' ' do Result:=Copy(Result,2,Length(Result));
  //Supress tail zeroes
  while Result[Length(Result)]='0' do Result:=Copy(Result,1,Length(Result)-1);
  //Supress decimal point if nessesary
  if Result[Length(Result)] in PointSet then Result:=Copy(Result,1,Length(Result)-1);
  if PE<>0 then Result:=Result+Power;
 end
 else begin
   if FDisplayFormat <> '' then Result := FormatFloat(FDisplayFormat, fStack[1])
   else
     Result := SysUtils.FloatToStrF(fStack[1], fFormat, fPrecision, fDigits);
 end;
end;

procedure TSohoCalculating.AppendDigit;
begin
 PushHistory; //what is it?
 Push;
 DoChange;
end;

procedure TSohoCalculating.Add;
var Temp:Double;
begin
 Temp:=fStack[2]+fStack[1];
 Pop; Pop; fStack[1]:=Temp;
 DoChange;
end;

procedure TSohoCalculating.SubTract;
var Temp:Double;
begin
 Temp:=fStack[2]-fStack[1];
 Pop; Pop; fStack[1]:=Temp;
 DoChange;
end;

procedure TSohoCalculating.Multiply;
var Temp:Double;
begin
 Temp:=fStack[2]*fStack[1];
 Pop; Pop; fStack[1]:=Temp;
 DoChange;
end;

procedure TSohoCalculating.Divide;
var Temp:Double;
begin
 fError:=abs(fStack[1])<1E-30;
 if fError then begin
  fErrorText:='Zero divide';
  Exit;
 end;
 Temp:=fStack[2]/fStack[1];
 Pop;Pop; fStack[1]:=Temp;
 DoChange;
end;

procedure TSohoCalculating.SquareRoot;
begin
 fStack[1]:=SQRT(fStack[1]);
 DoChange;
end;

procedure TSohoCalculating.Minus;
begin
 fStack[1]:=-fStack[1];
 DoChange;
end;

procedure TSohoCalculating.BackSpace;
begin
 PopHistory;
 PopHistory;
 DoChange;
end;

procedure TSohoCalculating.Clear;
begin
  fStack[1]:=0;
  SP:=1;
  fDecimals:=0;
  DoChange;
end;

procedure TSohoCalculating.EqualSign;
begin
 while OP>=1 do
 RightPar;
 DoChange;
end;


procedure TSohoCalculating.Percent;
var Temp:Double;
begin
 EqualSign;
 Temp:=fStack[2]*fStack[1]/100;
 Pop; Pop; fStack[1]:=Temp;
 DoChange;
end;

procedure TSohoCalculating.Invert;
begin
 EqualSign;
 fError := abs(fStack[1])<1E-30;
 if fError then begin
  fErrorText := 'Zero divide';
  DoChange;
  Exit;
 end;
 fStack[1]:=1/fStack[1];
 DoChange;
end;

procedure TSohoCalculating.Store;
begin
 fMemory:=fStack[1];
 DoChange;
end;

procedure TSohoCalculating.Restore;
begin
 fStack[1]:=fMemory;
 DoChange;
end;

procedure TSohoCalculating.MemoryClear;
begin
 fMemory:=0;
 DoChange;
end;

procedure TSohoCalculating.MemoryAdd;
begin
 fMemory:=fMemory+fStack[1];
 DoChange;
end;

procedure TSohoCalculating.LeftPar;
begin
 Push;
 DoChange;
end;

procedure TSohoCalculating.RightPar;
begin
 Calculate;
 DoChange;
end;

procedure TSohoCalculating.Push;
var
 i:LongInt;
begin
 for i:=High(fStack) DownTo Low(fStack)+1 do
  fStack[i]:=fStack[i-1];
 Inc(SP);
 DoChange;
end;

procedure TSohoCalculating.Pop;
var
 i:LongInt;
begin
 for i:=Low(fStack) To High(fStack)-1 do
  fStack[i]:=fStack[i+1];
 Dec(SP);
 DoChange;
end;

procedure TSohoCalculating.PushHistory;
var
 i:LongInt;
begin
 for i:=High(fHistory) DownTo Low(fHistory)+1 do
  fHistory[i]:=fHistory[i-1];
 fHistory[1].Value:=fStack[1];
 fHistory[1].Decimals:=fDecimals;
 fHistory[1].DigitalInput:=DigitalInput;
 DoChange;
end;

procedure TSohoCalculating.PopHistory;
var
 i:LongInt;
begin
 fStack[1]:=fHistory[1].Value;
 fDecimals:=fHistory[1].Decimals;
 DigitalInput:=fHistory[1].DigitalInput;
 for i:=Low(fHistory) to High(fHistory)-1 do
  fHistory[i]:=fHistory[i+1];
 DoChange;
end;


procedure TSohoCalculating.PushOperations;
var I:LongInt;
begin
 for i:=High(fOperationStack) DownTo Low(fOperationStack)+1 do
  fOperationStack[i]:=fOperationStack[i-1];
 Inc(Op);
 DoChange;
end;

procedure TSohoCalculating.PopOperations;
var I:LongInt;
begin
 for i:=Low(fOperationStack) To High(fOperationStack)-1 do
  fOperationStack[i]:=fOperationStack[i+1];
 Dec(Op);
 DoChange;
end;

procedure TSohoCalculating.CalculateStatement;
var
 i:LongInt;
begin
 //   -   
 //       /,  
 //  
 //  CalculateStatement       
 //.
  for i:=1 to length(fStateMent) do
   InputChar:=fStateMent[i];
end;

function GetValue(var Value: Double): boolean;
begin
  Result := GetValueWithPos(nil, Value);
end;

function GetValueWithPos(Sender: TControl; var Value: Double): boolean;
var Pnt: TPoint;

   procedure PutToCenter;
   begin
     sohoCalculatorForm.Position := poScreenCenter;
   end;

begin
  try
    sohoCalculatorForm := TsohoCalculatorForm.Create(Application);
    sohoCalculatorForm.Calculator.AutoPrecision := true;
    sohoCalculatorForm.Calculator.InputValue := Value;
    if Assigned(Sender) then begin
      Pnt := Point(Sender.Left, Sender.Top + Sender.Height + 1);
      if Assigned(Sender.Parent) then begin
         Pnt := Sender.Parent.ClientToScreen(Pnt);
         sohoCalculatorForm.Left := Pnt.X;
         sohoCalculatorForm.Top := Pnt.Y;
      end
      else PutToCenter;
    end
    else PutToCenter;
    Result := sohoCalculatorForm.ShowModal = mrOk;
    if Result then Value := sohoCalculatorForm.Calculator.AsFloat;
  finally
    sohoCalculatorForm.Free;
  end;
end;

procedure TsohoCalculatorForm.CalcChanged (Sender: TObject);
begin
  DispL.Caption := Calculator.AsString + #32;
end;

procedure TsohoCalculatorForm.BtnClick(Sender: TObject);
var RealKey: char;
begin
  RealKey := chr((Sender as TsohoSpeedButton).Tag);
  if RealKey in ['+', '-', '*', '/'] then
    Calculator.InputChar := chr(61);
  Calculator.InputChar := RealKey;
end;

procedure TsohoCalculatorForm.OkBClick(Sender: TObject);
begin
  ModalResult := mrOk;
end;

procedure TsohoCalculatorForm.CancelBClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TsohoCalculatorForm.FormKeyPress(Sender: TObject; var Key: Char);
var RealKey: char;
begin
  case Key of
    #8      : RealKey := chr(10);
    'q', 'Q': RealKey := chr(82);
    '%'     : RealKey := chr(37);
    'c','C' : RealKey := chr(32);
    'i','I' : RealKey := chr(88);
    else RealKey := Key;
  end;
  if Key = #27 then ModalResult := mrCancel;
  if Key = #13 then RealKey := chr(61);

  if RealKey in ['+', '-', '*', '/'] then
    Calculator.InputChar := chr(61);
  Calculator.InputChar := RealKey;
end;

procedure TsohoCalculatorForm.ClipCut;
begin
  Clipboard.AsText := DispL.Caption;
  DispL.Caption := '0 ';
end;

procedure TsohoCalculatorForm.ClipCopy;
begin
  Clipboard.AsText := DispL.Caption;
end;

procedure TsohoCalculatorForm.ClipPaste;
var Error: boolean;
    Tmp: Double;
begin
  if Clipboard.HasFormat(CF_TEXT) then
    try
      Tmp := WStrToFloat(Trim(Clipboard.AsText), Error);
      if not Error then Calculator.InputValue := Tmp;
    except
      Calculator.InputValue := 0;
    end;
end;

procedure TsohoCalculatorForm.N1Click(Sender: TObject);
begin
  ClipCut;
end;

procedure TsohoCalculatorForm.N2Click(Sender: TObject);
begin
  ClipCopy;
end;

procedure TsohoCalculatorForm.N3Click(Sender: TObject);
begin
  ClipPaste;
end;

procedure TsohoCalculatorForm.ButtonsFlat(Value: boolean);
var Index: LongInt;
begin
  if Value then exit;
  for Index := 0 to pred(ComponentCount) do
    if (Components[Index] is TsohoSpeedButton) then
       (Components[Index] as TsohoSpeedButton).Flat := Value;
end;

const
 //     .  
 OperationNameC    :string='+-*/ =R~'+#10+'()X%crsp';
 //    , T- ,    - 
 //         
 OperateImmediatlyC:string='FFFFTTTT'+'T'+'TTTTFTTT';
 //  (n-)   
 //        
 OperationPriorityC:string='33660000'+'0'+'00006000';
 // Add Subtract Multiply divide Clear EqualSign SquareRoot Minus
 // BackSpace Percent Invert (1/x) Store ReStore MemoryClear MemoryAdd LeftPar RightPar
var
 Maker: TCalcOperation;
 Index: LongInt;

procedure TsohoCalculatorForm.FormCreate(Sender: TObject);
begin
  Calculator := TsohoCalculating.Create;
  Calculator.OnChange := CalcChanged;

  OkB.Caption := SOKButton;
  CancelB.Caption := SCancelButton;
end;

procedure TsohoCalculatorForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = vk_Return) and (ssCtrl in Shift) then ModalResult := mrOk;
end;

initialization
  Operations := TStringList.Create; //clear!
  for Index := 1 to Length(OperationNameC) do
  with Operations do begin
   Add(OperationNameC[Index]);
   Maker := TCalcOperation.Create;
   Maker.Immediatly := OperateImmediatlyC[Index] = 'T';
   Maker.Priority := ord(OperationPriorityC[Index]);
   Maker.Proc:=nil;
   Objects[Index-1] := Maker;
  end;
finalization
  Operations.Free;
end.
