{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoCalendar;

{$I SOHOLIB.INC}

interface
uses Windows, Messages, SysUtils, Classes, ComCtrls, Controls,
     Graphics, Forms {$IFDEF Delphi3}, CalCtrls{$ENDIF};

type

  TsohoMonthCalColors = class(TPersistent)
   {   TsohoCalendar }
  private
    FBackColor: TColor;
    FTextColor: TColor;
    FTitleBackColor: TColor;
    FTitleTextColor: TColor;
    FMonthBackColor: TColor;
    FTrailingTextColor: TColor;
    procedure SetColor(Index: Integer; Value: TColor);
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
  published
    property BackColor: TColor index 0 read FBackColor write SetColor default clWindow;
{    }
    property TextColor: TColor index 1 read FTextColor write SetColor default clWindowText;
{   }
    property TitleBackColor: TColor index 2 read FTitleBackColor write SetColor default clActiveCaption;
{     }
    property TitleTextColor: TColor index 3 read FTitleTextColor write SetColor default clWhite;
{    }
    property MonthBackColor: TColor index 4 read FMonthBackColor write SetColor default clWhite;
{      }
    property TrailingTextColor: TColor index 5 read FTrailingTextColor
      write SetColor default clInactiveCaptionText;
{      }
  end;

  TsohoCalendar = class(TComponent)
    {          }
  private
    { Private declarations }
    FCalColors: TsohoMonthCalColors;
    FCaption: string;
    FDateTime: TDateTime;
    FEndDate: TDate;
    FFirstDayOfWeek: TCalDayOfWeek;
    FMaxDate: TDate;
    FMaxSelectRange: Integer;
    FMinDate: TDate;
    FMultiSelect: Boolean;
    FShowToday: Boolean;
    FShowTodayCircle: Boolean;
    FWeekNumbers: Boolean;
    FControl: TWinControl;
  protected
    { Protected declarations }
    procedure Notification (AComponent: TComponent; Operation: TOperation);override;
    procedure CheckValidDate(Value: TDate); virtual;
    function DoStoreEndDate: Boolean;
    function DoStoreMaxDate: Boolean;
    function DoStoreMinDate: Boolean;
    function  GetDate: TDate;
    function  GetTime: TTime;
    procedure SetCalColors(Value: TsohoMonthCalColors);
    procedure SetDate(Value: TDate);
    procedure SetTime(Value: TTime);
    procedure SetDateTime(Value: TDateTime);
    procedure SetEndDate(Value: TDate);
    procedure SetMaxDate(Value: TDate);
    procedure SetMinDate(Value: TDate);
    procedure SetMultiSelect(Value: Boolean);
    procedure SetRange(MinVal, MaxVal: TDate);
    procedure SetSelectedRange(Date, EndDate: TDate);
    procedure CopyToCalendar(Calendar: TMonthCalendar);
    procedure SetDialogPos(Form: TForm);
  public
    constructor Create (aOwner: TComponent); override;
    destructor Destroy; override;

    function SelectDate: boolean;
  {        }
    function SelectTime: boolean;
  {        }
    function SelectDateTime: boolean;
  {          }
    property Control: TWinControl read FControl write FControl;
  { ,     ""  .  Control = nil, 
              ,   ,    
        Control }
  published
    property Caption: string read FCaption write FCaption;
{   }
    property Date: TDate read GetDate write SetDate;
{  ,    }
    property Time: TTime read GetTime write SetTime;
{  ,    }
    property DateTime: TDateTime read FDateTime write SetDateTime;
{    ,    }
    property EndDate: TDate read FEndDate write SetEndDate stored DoStoreEndDate;
{  EndDate,      ,   . 
       Date. ,       EndDate, 
    MultiSelect = True }
    property FirstDayOfWeek: TCalDayOfWeek read FFirstDayOfWeek write FFirstDayOfWeek
      default dowLocaleDefault;
{    }
    property MaxDate: TDate read FMaxDate write SetMaxDate stored DoStoreMaxDate;
{        }
    property MaxSelectRange: Integer read FMaxSelectRange write FMaxSelectRange default 31;
{ - ,     ,   MultiSelect = true }
    property MinDate: TDate read FMinDate write SetMinDate stored DoStoreMinDate;
{        }
    property MultiSelect: Boolean read FMultiSelect write SetMultiSelect default False;
{     .       Date,
    EndDate }
    property ShowToday: Boolean read FShowToday write FShowToday default True;
{  ShowToday,          }
    property ShowTodayCircle: Boolean read FShowTodayCircle write
      FShowTodayCircle default True;
{  ShowTodayCircle,         }
    property WeekNumbers: Boolean read FWeekNumbers write FWeekNumbers default False;
{  WeekNumbers,         }
  end;

function SelectDate(Sender: TWinControl; var aDate: TDate; const DlgCaption: TCaption;
  aFirstDayOfWeek: TCalDayOfWeek): Boolean;
{     ,        }
function SelectTime(Sender: TWinControl; var aTime: TTime; const DlgCaption: TCaption): Boolean;
{     ,        }
function SelectDateTime(Sender: TWinControl; var aDateTime: TDateTime; const DlgCaption: TCaption;
  aFirstDayOfWeek: TCalDayOfWeek): Boolean;
{     ,          }

{$IFNDEF Delphi5}
{$IFNDEF Delphi3}
procedure ReplaceTime(var DateTime: TDateTime; const NewTime: TDateTime);
procedure ReplaceDate(var DateTime: TDateTime; const NewDate: TDateTime);
{$ENDIF}
{$ENDIF}

implementation
uses SoCmnCns, Consts, Commctrl, SoClndF {$IFDEF Delphi6}, RTLConsts{$ENDIF};

type

   EsohoCalendarException = class(Exception);

{ TsohoMonthCalColors }

const
  ColorIndex: array[0..5] of Integer = (MCSC_BACKGROUND, MCSC_TEXT,
    MCSC_TITLEBK, MCSC_TITLETEXT, MCSC_MONTHBK, MCSC_TRAILINGTEXT);

{$IFNDEF Delphi5}
{$IFNDEF Delphi3}
procedure ReplaceTime(var DateTime: TDateTime; const NewTime: TDateTime);
begin
  DateTime := Trunc(DateTime);
  if DateTime >= 0 then
    DateTime := DateTime + Abs(Frac(NewTime))
  else
    DateTime := DateTime - Abs(Frac(NewTime));
end;

procedure ReplaceDate(var DateTime: TDateTime; const NewDate: TDateTime);
var
  Temp: TDateTime;
begin
  Temp := NewDate;
  ReplaceTime(Temp, DateTime);
  DateTime := Temp;
end;
{$ENDIF}
{$ENDIF}

constructor TsohoMonthCalColors.Create;
begin
  FBackColor := clWindow;
  FTextColor := clWindowText;
  FTitleBackColor := clActiveCaption;
  FTitleTextColor := clWhite;
  FMonthBackColor := clWhite;
  FTrailingTextColor := clInactiveCaptionText;
end;

procedure TsohoMonthCalColors.Assign(Source: TPersistent);
var
  SourceName: string;
begin
  if Source = nil then SourceName := 'nil'
  else SourceName := Source.ClassName;
  if (Source = nil) or not (Source is TsohoMonthCalColors) then
  {$IFDEF Delphi5}
    raise EConvertError.CreateResFmt(@SAssignError, [SourceName, ClassName]);
  {$ELSE}
    raise EConvertError.CreateFmt('Assign error %s for class %s', [SourceName, ClassName]);
  {$ENDIF}
  FBackColor := TsohoMonthCalColors(Source).BackColor;
  FTextColor := TsohoMonthCalColors(Source).TextColor;
  FTitleBackColor := TsohoMonthCalColors(Source).TitleBackColor;
  FTitleTextColor := TsohoMonthCalColors(Source).TitleTextColor;
  FMonthBackColor := TsohoMonthCalColors(Source).MonthBackColor;
  FTrailingTextColor := TsohoMonthCalColors(Source).TrailingTextColor;
end;

procedure TsohoMonthCalColors.SetColor(Index: Integer; Value: TColor);
begin
  case Index of
    0: FBackColor := Value;
    1: FTextColor := Value;
    2: FTitleBackColor := Value;
    3: FTitleTextColor := Value;
    4: FMonthBackColor := Value;
    5: FTrailingTextColor := Value;
  end;
end;

{ TsohoCalendar }
function TsohoCalendar.DoStoreEndDate: Boolean;
begin
  Result := FMultiSelect;
end;

function TsohoCalendar.DoStoreMaxDate: Boolean;
begin
  Result := FMaxDate <> 0.0;
end;

function TsohoCalendar.DoStoreMinDate: Boolean;
begin
  Result := FMinDate <> 0.0;
end;

procedure TsohoCalendar.Notification (AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (AComponent = FControl) and (Operation = opRemove) then
    FControl := nil;
end;

procedure TsohoCalendar.CheckValidDate(Value: TDate);
begin
  if (FMaxDate <> 0.0) and (Value > FMaxDate) then
    raise EsohoCalendarException.CreateFmt(sohoCalWrongDate, [DateToStr(FMaxDate)]);
  if (FMinDate <> 0.0) and (Value < FMinDate) then
    raise EsohoCalendarException.CreateFmt(sohoCalWrongDate, [DateToStr(FMinDate)]);
end;

function  TsohoCalendar.GetTime: TTime;
begin
  Result := TTime(FDateTime);
end;

function TsohoCalendar.GetDate: TDate;
begin
  Result := TDate(FDateTime);
end;

procedure TsohoCalendar.SetCalColors(Value: TsohoMonthCalColors);
begin
  if FCalColors <> Value then FCalColors.Assign(Value);
end;

procedure TsohoCalendar.SetDate(Value: TDate);
begin
  ReplaceTime(TDateTime(Value), FDateTime);
  try
    CheckValidDate(Trunc(Value));
    SetDateTime(Value);
  except
    SetDateTime(FDateTime);
    raise;
  end;
end;

procedure TsohoCalendar.SetTime(Value: TTime);
begin
  ReplaceDate(TDateTime(Value), FDateTime);
  try
    CheckValidDate(Trunc(Value));
    SetDateTime(Value);
  except
    SetDateTime(FDateTime);
    raise;
  end;
end;

procedure TsohoCalendar.SetDateTime(Value: TDateTime);
var
  ST: TSystemTime;
begin
  DateTimeToSystemTime(Value, ST);
  if FMultiSelect then SetSelectedRange(Value, FEndDate)
  else FDateTime := Value;
end;

procedure TsohoCalendar.SetEndDate(Value: TDate);
var
  TruncValue: TDate;
begin
  TruncValue := Trunc(Value);
  if Trunc(FEndDate) <> TruncValue then
    SetSelectedRange(Date, TruncValue);
end;

procedure TsohoCalendar.SetMaxDate(Value: TDate);
begin
  if (FMinDate <> 0.0) and (Value < FMinDate) then
    raise EsohoCalendarException.CreateFmt(sohoCalMaxWrongDate, [DateToStr(FMinDate)]);
  if FMaxDate <> Value then begin
    SetRange(FMinDate, Value);
    FMaxDate := Value;
  end;
end;

procedure TsohoCalendar.SetMinDate(Value: TDate);
begin
  if (FMaxDate <> 0.0) and (Value > FMaxDate) then
    raise EsohoCalendarException.CreateFmt(sohoCalMinWrongDate, [DateToStr(FMaxDate)]);
  if FMinDate <> Value then begin
    SetRange(Value, FMaxDate);
    FMinDate := Value;
  end;
end;

procedure TsohoCalendar.SetMultiSelect(Value: Boolean);
begin
  if FMultiSelect <> Value then begin
    FMultiSelect := Value;
    if Value then FEndDate := FDateTime
    else FEndDate := 0.0;
  end;
end;

procedure TsohoCalendar.SetRange(MinVal, MaxVal: TDate);
var
  STA: packed array[1..2] of TSystemTime;
  {$IFDEF Delphi5}
  TruncDate, TruncEnd, TruncMin, TruncMax: Int64;
  {$ELSE} 
  TruncDate, TruncEnd, TruncMin, TruncMax: DWORD;
  {$ENDIF}
begin
  TruncMin := Trunc(MinVal);
  TruncMax := Trunc(MaxVal);
  TruncDate := Trunc(FDateTime);
  TruncEnd := Trunc(FEndDate);
  if TruncMin <> 0 then begin
    if TruncDate < TruncMin then SetDate(MinVal);
    if TruncEnd < TruncMin then SetEndDate(MinVal);
    DateTimeToSystemTime(TruncMin, STA[1]);
  end;
  if TruncMax <> 0 then begin
    if TruncDate > TruncMax then SetDate(MaxVal);
    if TruncEnd > TruncMax then SetEndDate(MaxVal);
    DateTimeToSystemTime(TruncMax, STA[2]);
  end;
end;

procedure TsohoCalendar.SetSelectedRange(Date, EndDate: TDate);
var
  DateArray: array[1..2] of TSystemTime;
begin
  if not FMultiSelect then SetDateTime(Date)
  else begin
    DateTimeToSystemTime(Date, DateArray[1]);
    DateTimeToSystemTime(EndDate, DateArray[2]);
    FDateTime := Date;
    FEndDate := EndDate;
  end;
end;

constructor TsohoCalendar.Create (aOwner: TComponent);
begin
  inherited Create(aOwner);
  FControl := nil;
  FCalColors := TsohoMonthCalColors.Create;
  FDateTime := Now;
  FEndDate := 0;
  FFirstDayOfWeek := dowLocaleDefault;
  FMaxDate := 0;
  FMaxSelectRange := 31;
  FMinDate := 0;
  FShowToday := true;
  FShowTodayCircle := true;
  FWeekNumbers := false;
  FCaption := '';
end;

destructor TsohoCalendar.Destroy;
begin
  FCalColors.Free;
  inherited;
end;

procedure TsohoCalendar.CopyToCalendar(Calendar: TMonthCalendar);
begin
  Calendar.MultiSelect := FMultiSelect;
  Calendar.FirstDayOfWeek := FFirstDayOfWeek;
  Calendar.MaxDate := FMaxDate;
  Calendar.MaxSelectRange := FMaxSelectRange;
  Calendar.MinDate := FMinDate;
  Calendar.ShowToday := FShowToday;
  Calendar.ShowTodayCircle := FShowTodayCircle;
  Calendar.WeekNumbers := FWeekNumbers;
  Calendar.Date := GetDate;
  Calendar.CalColors.BackColor := FCalColors.BackColor;
  Calendar.CalColors.TextColor := FCalColors.TextColor;
  Calendar.CalColors.TitleBackColor := FCalColors.TitleBackColor;
  Calendar.CalColors.TitleTextColor := FCalColors.TitleTextColor;
  Calendar.CalColors.MonthBackColor := FCalColors.MonthBackColor;
  Calendar.CalColors.TrailingTextColor := FCalColors.TrailingTextColor;
end;

procedure TsohoCalendar.SetDialogPos(Form: TForm);
var Pnt: TPoint;
begin
  if Assigned(Control) then begin
    Pnt := Point(Control.Left, Control.Top + Control.Height + 1);
    if Assigned(Control.Parent) then begin
       Pnt := Control.Parent.ClientToScreen(Pnt);
       if Pnt.X + Form.Width > Screen.Width then
         Pnt.X := Screen.Width - Form.Width - 1;
       if Pnt.Y + Form.Height > Screen.Height then
         Pnt.Y := Screen.Height - Form.Height - 1;
       Form.Left := Pnt.X;
       Form.Top := Pnt.Y;
    end
    else Form.Position := poScreenCenter;
  end
  else Form.Position := poScreenCenter;
end;

function TsohoCalendar.SelectDate: boolean;
begin
  try
    SoCalendarForm := TSoCalendarForm.Create(Self);
    CopyToCalendar(SoCalendarForm.MonthE);
    SoCalendarForm.TimeP.Visible := false;
    if FCaption = '' then SoCalendarForm.Caption := sohoCalMonthCaption
    else SoCalendarForm.Caption := FCaption;
    SoCalendarForm.AlignPanels;
    SetDialogPos(SoCalendarForm);
    Result := SoCalendarForm.ShowModal = mrOk;
    if Result then Date := SoCalendarForm.MonthE.Date;
  finally
    SoCalendarForm.Free;
  end;
end;

function TsohoCalendar.SelectTime: boolean;
begin
  try
    SoCalendarForm := TSoCalendarForm.Create(Self);
    SoCalendarForm.CalendarP.Visible := false;
    SoCalendarForm.TimeE.Time := Time;
    if FCaption = '' then SoCalendarForm.Caption := sohoCalTimeCaption
    else SoCalendarForm.Caption := FCaption;
    SoCalendarForm.AlignPanels;
    SetDialogPos(SoCalendarForm);
    Result := SoCalendarForm.ShowModal = mrOk;
    if Result then Time := SoCalendarForm.TimeE.Time;
  finally
    SoCalendarForm.Free;
  end;
end;

function TsohoCalendar.SelectDateTime: boolean;
begin
  try
    SoCalendarForm := TSoCalendarForm.Create(Self);
    CopyToCalendar(SoCalendarForm.MonthE);
    SoCalendarForm.TimeE.Time := Time;
    if FCaption = '' then SoCalendarForm.Caption := sohoCalMonthTimeCaption
    else SoCalendarForm.Caption := FCaption;
    SoCalendarForm.AlignPanels;
    SetDialogPos(SoCalendarForm);
    Result := SoCalendarForm.ShowModal = mrOk;
    if Result then begin
      Date := SoCalendarForm.MonthE.Date;
      Time := SoCalendarForm.TimeE.Time;
    end;
  finally
    SoCalendarForm.Free;
  end;
end;

function SelectDate(Sender: TWinControl; var aDate: TDate; const DlgCaption: TCaption;
  aFirstDayOfWeek: TCalDayOfWeek): Boolean;
var Tmp: TsohoCalendar;
begin
  Tmp := TsohoCalendar.Create(nil);
  with Tmp do begin
    Caption := DlgCaption;
    FirstDayOfWeek := aFirstDayOfWeek;
    Date := aDate;
    Control := Sender;
  end;
  Result := Tmp.SelectDate;
  if Result then aDate := Tmp.Date;
  Tmp.Free;
end;

function SelectTime(Sender: TWinControl; var aTime: TTime; const DlgCaption: TCaption): Boolean;
var Tmp: TsohoCalendar;
begin
  Tmp := TsohoCalendar.Create(nil);
  with Tmp do begin
    Caption := DlgCaption;
    Time := aTime;
    Control := Sender;
  end;
  Result := Tmp.SelectTime;
  if Result then aTime := Tmp.Time;
  Tmp.Free;
end;

function SelectDateTime(Sender: TWinControl; var aDateTime: TDateTime; const DlgCaption: TCaption;
  aFirstDayOfWeek: TCalDayOfWeek): Boolean;
var Tmp: TsohoCalendar;
begin
  Tmp := TsohoCalendar.Create(nil);
  with Tmp do begin
    Caption := DlgCaption;
    FirstDayOfWeek := aFirstDayOfWeek;
    Date := TDate(aDateTime);
    Time := TTime(aDateTime);
    Control := Sender;
  end;
  Result := Tmp.SelectDateTime;
  if Result then aDateTime := trunc(Tmp.Date) + Frac(Tmp.Time);
  Tmp.Free;
end;


end.
