{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
     TsohoCheckBox
}
unit SoChkBox;

interface
uses Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls,
     Menus, Forms;

{$I SOHOLIB.INC}

type

  TsohoCheckBoxType = (btCheckBox, btRadioButton);
{  TheckBox }

  TsohoCheckAlign = (baTop, baCenter, baBottom);
{    TheckBox:   ,       }

  TsohoCustomCheckBox = class(TCustomControl)
  private
    { Private declarations }
    FAllowGrayed: boolean;
    FAlive: boolean;
    FOldChecked: boolean;
    FCheckStyle : TsohoCheckBoxType;
    FChecked : boolean;
    FWordWrap: boolean;
    FVertAlign: TsohoCheckAlign;
    FCaption : string;
    FInRect : boolean;
    FToggleKey: TShortCut;
    FAlignment: TAlignment;
    FOnChange: TNotifyEvent;
    FOnUndo: TNotifyEvent;
    FState: TCheckBoxState;
    FActFont: TFont;
    procedure SetFActiveFont(const Value: TFont);
  protected
    { Protected declarations }
    procedure SetStyle (Value : TsohoCheckBoxType);
    procedure SetChecked (Value : boolean);
    procedure SetCaption (Value : string);
    procedure SetAlignment(Value: TAlignment);
    procedure SetState(Value: TCheckBoxState);
    function GetChecked: boolean;
    procedure SetWordWrap (Value: boolean);
    procedure SetVertAlign (Value: TsohoCheckAlign);

    procedure CMParentFontChanged(var message: TMessage); message CM_PARENTFONTCHANGED;
    procedure CMEnabledChanged (var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave (var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMEnter(var message: TCMEnter); message CM_ENTER;
    procedure CMExit(var message: TCMExit); message CM_EXIT;
    procedure CreateParams(var Params: TCreateParams);override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure KeyDown(var Key: Word; Shift: TShiftState);override;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure Changed;virtual;

    property AllowGrayed: boolean read FAllowGrayed write FAllowGrayed;
    property Alive: boolean read FAlive write FAlive default false;
    property FontActive: TFont read FActFont write SetFActiveFont;
    property Style: TsohoCheckBoxType read FCheckStyle write SetStyle;
    property Checked: boolean read GetChecked write SetChecked;
    property Caption: string read FCaption write SetCaption;
    property ToggleKey: TShortCut read FToggleKey write FToggleKey;
    property State: TCheckBoxState read FState write SetState;
    property Alignment: TAlignment read FAlignment write SetAlignment default taRightJustify;
    property WordWrap: boolean read FWordWrap write SetWordWrap;
    property VertAlign: TsohoCheckAlign read FVertAlign write SetVertAlign;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnUndo: TNotifyEvent read FOnUndo write FOnUndo;
  public
    { Public declarations }
    procedure Paint;override;
    constructor Create(aOwner : TComponent);override;
    destructor Destroy; override;
    procedure ToggleCheck;
    procedure Undo; virtual;
  published
    property TabStop default true;
    property TabOrder;
  end;

  TsohoCheckBox = class(TsohoCustomCheckBox)
{      TCheckBox,   TRadioButton.   
   Style }	
  published
{$IFDEF Delphi4_or_5}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property ImeMode;
    property ImeName;
{$ENDIF}
    property AllowGrayed default false;
{   AllowGrayed  true,       : checked, unchecked,  grayed.
 ,  AllowGrayed = false,       checked  unchecked. 
    State   Checked}
    property Alive;
{   Alive,     , 
       }
    property FontActive;
{   FontActive,    , 
     ,       }
    property WordWrap default false;
{  WordWrap,  ,      Caption,   
   ,        }
    property VertAlign default baCenter;
{         Caption }
    property Alignment;
{         Caption }
    property Caption;
{  ,   }
    property Checked;
{   Checked,     }
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property State default cbUnchecked;
{   State,     }
    property Style default btCheckBox;
{     }
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    property OnChange;
{      Checked  State }
    property OnUndo;
{       Checked  State }
  end;

function GetGroupIndex (GroupBox: TGroupBox): integer;
{  GetGroupIndex,     TsohoCheckBox  GroupBox,  
  Checked   true }
procedure SetGroupIndex (GroupBox: TGroupBox; Index: integer);
{  SetGroupIndex,     TsohoCheckBox  GroupBox, 
    hecked   true }

implementation
uses SoUtils, ExtCtrls, SoGraph, SoCtrls;

{$R SOCHKBOX.R32}

var
   CheckBmps,
   RadioBmps : TBitmap;

function GetGroupIndex (GroupBox: TGroupBox): integer;
var Index : integer;
begin
  Result := 0;
  for Index := 0 to pred(GroupBox.ControlCount) do
    if GroupBox.Controls[Index] is TsohoCustomCheckBox then
      if TsohoCustomCheckBox(GroupBox.Controls[Index]).Checked then begin
        Result := Index;
        exit;
      end;
end;

procedure SetGroupIndex (GroupBox: TGroupBox; Index: integer);
begin
    if GroupBox.Controls[Index] is TsohoCustomCheckBox then
      TsohoCustomCheckBox(GroupBox.Controls[Index]).Checked := true;
end;

{ TsohoCustomCheckBox }
function TsohoCustomCheckBox.GetChecked: boolean;
begin
  Result := FState = cbChecked;
end;

procedure TsohoCustomCheckBox.SetWordWrap (Value: boolean);
begin
  if FWordWrap = Value then exit;
  FWordWrap := Value;
  Invalidate;
end;

procedure TsohoCustomCheckBox.SetVertAlign (Value: TsohoCheckAlign);
begin
  if FVertAlign = Value then exit;
  FVertAlign := Value;
  Invalidate;
end;

procedure TsohoCustomCheckBox.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TsohoCustomCheckBox.CMDialogChar(var Message: TCMDialogChar);
begin
  if Enabled and IsAccel(Message.CharCode, FCaption) then begin
     SetFocus;
     ToggleCheck;
     Message.Result := 1;
  end;
end;

procedure TsohoCustomCheckBox.SetAlignment(Value: TAlignment);
begin
  if FAlignment = Value then exit;
  FAlignment := Value;
  Invalidate;
end;

procedure TsohoCustomCheckBox.SetState(Value: TCheckBoxState);

  procedure TurnSiblingsOff;
  var Index: LongInt;
      Sibling: TControl;
  begin
    if Parent <> nil then
      with Parent do
        for Index := 0 to pred(ControlCount) do begin
          Sibling := Controls[Index];
          if (Sibling <> Self) and (Sibling is TsohoCheckBox) and
             (TsohoCheckBox(Sibling).Style = btRadioButton) then
            TsohoCheckBox(Sibling).Checked := false;
        end;
  end;

begin
  if FState = Value then exit;
  if (FCheckStyle <> btCheckBox) and (Value = cbGrayed) then exit;
  FState := Value;
  if (FCheckStyle = btRadioButton) and (Value = cbChecked)
    then TurnSiblingsOff;
  Changed;
  Invalidate;
end;

procedure TsohoCustomCheckBox.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then begin
    if TabStop and CanFocus then SetFocus;
    if ((FCheckStyle = btRadioButton) and (not Checked)) or
       (FCheckStyle <> btRadioButton) then ToggleCheck;
  end;
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TsohoCustomCheckBox.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if Enabled and (FToggleKey = ShortCut(Key, Shift)) then begin
    ToggleCheck;
    Key := 0;
  end;
  if ((Key = ord('z')) or (Key = ord('Z'))) and (ssCtrl in Shift) then
    Undo;
  if (Key = VK_BACK) and (ssAlt in Shift) then
    Undo;
end;

procedure TsohoCustomCheckBox.ToggleCheck;
begin
  if FAllowGrayed then begin
    if State = cbUnchecked then State := cbGrayed
    else State := pred(FState);
  end
  else begin
    if State <> cbUnchecked then State := cbUnchecked
    else State := cbChecked;
  end;
end;

procedure TsohoCustomCheckBox.SetStyle (Value : TsohoCheckBoxType);
begin
  if FCheckStyle = Value then exit;
  FCheckStyle := Value;
  if (FCheckStyle = btRadioButton) and (FState = cbGrayed) then
     FState := cbUnchecked;
  Invalidate;
end;

procedure TsohoCustomCheckBox.Changed;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TsohoCustomCheckBox.SetChecked (Value : boolean);
begin
  State := TCheckBoxState(ord(Value));
end;

procedure TsohoCustomCheckBox.SetCaption (Value : string);
begin
   if FCaption = Value then exit;
   FCaption := Value;
   Invalidate;
end;

procedure TsohoCustomCheckBox.CMEnabledChanged (var Message: TMessage);
begin
   inherited;
   Invalidate;
end;

procedure TsohoCustomCheckBox.CMMouseEnter (var Message: TMessage);
begin
  inherited;
  FInRect := true;
  Invalidate;
end;

procedure TsohoCustomCheckBox.CMMouseLeave (var Message: TMessage);
begin
  inherited;
  FInRect := false;
  Invalidate;
end;

procedure TsohoCustomCheckBox.CMEnter(var message: TCMEnter);
begin
  inherited;
  Invalidate;
end;

procedure TsohoCustomCheckBox.CMExit(var message: TCMExit);
begin
  inherited;
  Invalidate;
end;

procedure TsohoCustomCheckBox.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);

  CheckPictureNumber: array[TCheckBoxState, Boolean, Boolean] of byte = (
    ((0, 4), (2, 4)),( (1, 5), (3, 5)), ((6, 5), (5, 5))
  );

var TmpR, R : TRect;
    y : LongInt;
    ImageIndex, H : integer;

  procedure DrawDisabled;
  begin
    Canvas.Brush.Style := bsClear;
    OffsetRect(R, 1, 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawText(Canvas.Handle, PChar(FCaption), Length(FCaption), R, DT_EXPANDTABS or
      WordWraps[FWordWrap] or Alignments[FAlignment]);
    OffsetRect(R, -1, -1);
    Canvas.Font.Color := clBtnShadow;
    DrawText(Canvas.Handle, PChar(FCaption), Length(FCaption), R, DT_EXPANDTABS or
      WordWraps[FWordWrap] or Alignments[FAlignment]);
    Canvas.Brush.Style := bsSolid;
  end;

begin
  //     :
  // Unchecked, Checked, Grayed +
  //       Enabled / Disabled
  //      MouseUnder / mouse   
  //   Disabled,     MouseUnder / mouse   
  //  :
  // UME - 0
  // UMD - 4
  // U-E - 2
  // U-D - 4
  // CME - 1
  // CMD - 5
  // C-E - 3
  // C-D - 5
  // GME - 6
  // GMD - 5
  // G-E - 5
  // G-D - 5

  ImageIndex := CheckPictureNumber[FState, not FInRect, not Enabled];

  Canvas.Font := Font;
  with Canvas do begin
    Brush.Color := Self.Color;
    R := ClientRect;
    FillRect(R);
    if FCheckStyle = btCheckBox then H := 11
    else H := 12;
    case FVertAlign of    //
      baTop: Y := 1;
      baCenter: Y := VertCenter(R, H);
      else Y := R.Bottom - H - 1;
    end;    // case

    if FCheckStyle = btCheckBox then begin
       TmpR := Bounds(0, y, 13, 13);
       Frame3D(Canvas, TmpR, clBtnShadow, clBtnHighlight, 1);
       CopyRect( Bounds(1, y+1, 11, 11), CheckBmps.Canvas,
                 Bounds(ImageIndex * 11, 0, 11, 11))
    end
    else
       BrushCopy(Bounds(0, y, 12, 12), RadioBmps,
                 Bounds(ImageIndex * 12, 0, 12, 12),
                 RadioBmps.Canvas.Pixels[0, 11]);
    R.Left := 18;
    if not FWordWrap then begin
      case FVertAlign of    //
        baTop: Y := 1;
        baCenter: Y := VertCenter(R, TextHeight(FCaption));
        baBottom: Y := R.Bottom - TextHeight(FCaption) - 1;
      end;    // case
      R.Top := Y;
      R.Bottom := R.Top + TextHeight(FCaption);
      R.Right := R.Left + TextWidth(FCaption);
      if Enabled then begin
        if FInRect and FAlive then Canvas.Font := FActFont
        else Canvas.Font := Self.Font;
        TextInRect(Canvas, FCaption, R, FAlignment);
      end
      else DrawDisabled;
    end
    else begin
      R.Top := 1;
      R.Bottom := R.Bottom - 1;
      R.Right := R.Right - 2;
      if Enabled then begin
        if FInRect and FAlive then Canvas.Font := FActFont
        else Canvas.Font := Self.Font;
        WrapTextInRect(Canvas, FCaption, R, FAlignment)
      end
      else DrawDisabled;
    end;

    if Screen.ActiveControl = Self then begin
      R.Left := R.Left - 4;
      R.Right := R.Right + 2;
      if R.Right > Left + Width then R.Right := Left + Width;

      R.Bottom := R.Bottom;
      DrawFocusRect(R);
    end;
  end;
end;

procedure TsohoCustomCheckBox.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
    WindowClass.Style := WindowClass.Style or CS_HREDRAW or CS_VREDRAW;
end;

constructor TsohoCustomCheckBox.Create(aOwner : TComponent);
begin
  inherited Create(aOwner);
  ControlStyle := [csClickEvents, csSetCaption, csCaptureMouse,
    csOpaque, csDoubleClicks];
  FAllowGrayed := false;
  FCheckStyle := btCheckBox;
  FChecked := false;
  FOldChecked := false;
  FState := cbUnchecked;
  FInRect := false;
  Height := 14;
  Width := 75;
  TabStop := True;
  FToggleKey := vk_Space;
  FAlignment := taLeftJustify;
  FWordWrap := false;
  FVertAlign := baCenter;
  FActFont := TFont.Create;
  FAlive := false;
end;

procedure TsohoCustomCheckBox.Undo;
var Tmp1, Tmp2: LongInt;
begin
  Tmp1 := LongInt(Checked);
  Tmp2 := LongInt(FOldChecked);
  SwapLongInt(Tmp1, Tmp2);
  FOldChecked := boolean(Tmp2);
  Checked := boolean(Tmp1);
  if Assigned(FOnUndo) then FOnUndo(Self);
end;

procedure TsohoCustomCheckBox.SetFActiveFont(const Value: TFont);
begin
  FActFont.Assign(Value);
end;

procedure TsohoCustomCheckBox.CMParentFontChanged(var message: TMessage);
begin
  inherited;
  if ParentFont then FActFont.Assign(Font);
end;

destructor TsohoCustomCheckBox.Destroy;
begin
  FActFont.Free;
  inherited;
end;

initialization
   CheckBmps := TBitmap.Create;
   CheckBmps.LoadFromResourceName(hInstance, 'FLATCHECKS');
   RadioBmps := TBitmap.Create;
   RadioBmps.LoadFromResourceName(hInstance, 'FLATRADIO');
finalization
   CheckBmps.Free;
   RadioBmps.Free;
end.
