{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
     TsohoRegionForm,   
    .
}
unit SoCmplFm;

{$I SOHOLIB.INC}

interface
uses Controls, Windows, Graphics, Classes;

type

   TsohoRegionForm = class;

  TRegionType = (rtRectangle, rtRoundRect, rtEllipse);
  {    TsohoRegionFigure:
    rtRectangle - 
    rtRoundRect -    
    rtEllipse   - 
  }

  TsohoRegionFigure = class(TGraphicControl)
  {      ,  
      .     
          }
  private
    FRegionType: TRegionType;
    FRegionForm: TsohoRegionForm;
    procedure SetRegionType(Value: TRegionType);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    {   Parent }
    property RegionType: TRegionType read FRegionType write SetRegionType default rtRectangle;
    {   }
  end;

  TCombineMode = (cmAnd, cmCopy, cmDiff, cmOr, cmXor);
  {     :
    cmAnd  -   
    cmCopy -     
    cmDiff -  ,     
    cmOr   -   
    cmXor  -   ,    
  }

  TRegionsOption = (roIncludeMenu, roIncludeCaption);
  {     :
    roIncludeMenu    -       
    roIncludeCaption -      
  }
  TRegionsOptions = set of TRegionsOption;
  {      }

  TRegionCreateEvent = procedure (Sender: TObject; RegionFigure: TsohoRegionFigure) of object;
  {   OnRegionCreate.     
    ,     TsohoRegionFigure }

  TsohoRegionForm = class(TComponent)
  {      ( TForm) 
    .        TsohoRegionFigure,
        design-time.     
      API  -   AddRegion   
    OnBeforeApply.     ,  bitmap  
     CreateRgnFromBitmap.
  }
  private
    FMainMode     : TCombineMode;
    FRegionsMode  : TCombineMode;
    FOptions      : TRegionsOptions;
    FOnFormCreate : TNotifyEvent;
    FBeforeApply  : TNotifyEvent;
    FFormRegion   : HRGN;
    FRegions      : TList;
    FFigures      : TList;
    FOnRegionCreate: TRegionCreateEvent;
    procedure DoFormCreate   (Sender : TObject);
  protected
    procedure SetFormRegions;
    procedure Loaded;override;
    procedure RegionCreate(RegionFigure: TsohoRegionFigure); virtual;
    procedure DeleteRegionFigure(Value: TsohoRegionFigure);
  public
    function CreateRgnFromBitmap (ClientX, ClientY : integer; Bitmap : TBitmap; TransparentColor : TColor) : HRGN;
    {    Bitmap. ClientX, ClientY -  
      ,  ,    Bitmap.
      TransparentColor - ,    ,  ,
      ,     }
    procedure AddRegion (Value : HRGN);virtual;
    {      ,   
         }
    function RegionsCount : LongInt;
    {     }
    function AddRegionFigure : boolean;
    {   TsohoRegionFigure   }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property MainMode    : TCombineMode    read FMainMode write FMainMode default cmAnd;
    {    .  ,   cmAnd 
           ,     .  ,
        cmXor     ,      }
    property RegionsMode : TCombineMode    read FRegionsMode write FRegionsMode default cmOr;
    {        }
    property Options     : TRegionsOptions read FOptions write FOptions default [];
    {     :     }
    property OnBeforeApply : TNotifyEvent read FBeforeApply write FBeforeApply;
    {        .
               }
    property OnRegionCreate: TRegionCreateEvent read FOnRegionCreate write FOnRegionCreate;
    {     ,    
      TsohoRegionFigure  ""  .   
         }
  end;

implementation
uses SoVCLUtils, Forms, SoShell, SoAppUtils, SoRTTI;

{ TsohoRegionFigure }
destructor TsohoRegionFigure.Destroy;
begin
  if Assigned(FRegionForm) then FRegionForm.DeleteRegionFigure(Self);
  inherited;
end;

constructor TsohoRegionFigure.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FRegionType := rtRectangle;
  FRegionForm := nil;
  Width := 65;
  Height := 65;
end;

procedure TsohoRegionFigure.Paint;
var X, Y, W, H, S: Integer;
begin
  if not (csDesigning in ComponentState) then exit;
  with Canvas do begin
    with Pen do begin
      Color := clBlue;
      Width := 1;
      Style := psDashDot;
    end;
    Brush.Style := bsClear;
    X := Pen.Width div 2;
    Y := X;
    W := Width - Pen.Width + 1;
    H := Height - Pen.Width + 1;
    if Pen.Width = 0 then begin
      Dec(W);
      Dec(H);
    end;
    if W < H then S := W else S := H;
    case FRegionType of
      rtRectangle : Rectangle(X, Y, X + W, Y + H);
      rtRoundRect : RoundRect(X, Y, X + W, Y + H, S div 4, S div 4);
      rtEllipse   : Ellipse(X, Y, X + W, Y + H);
    end;
  end;
end;

procedure TsohoRegionFigure.SetRegionType(Value: TRegionType);
begin
  if FRegionType <> Value then begin
    FRegionType := Value;
    Invalidate;
  end;
end;

{ TsohoRegionForm }
procedure TsohoRegionForm.DeleteRegionFigure(Value: TsohoRegionFigure);
begin
  FFigures.Remove(Value);
end;

procedure TsohoRegionForm.DoFormCreate   (Sender : TObject);
begin
  if Assigned(FOnFormCreate) then FOnFormCreate(Sender);
  SetFormRegions;
end;

procedure TsohoRegionForm.AddRegion (Value : HRGN);
begin
   FRegions.Add(pointer(Value));
end;

procedure TsohoRegionForm.RegionCreate(RegionFigure: TsohoRegionFigure);
begin
  if Assigned(FOnRegionCreate) then FOnRegionCreate(Self, RegionFigure);
end;

procedure TsohoRegionForm.SetFormRegions;
const
  Modes : array [TCombineMode] of integer = (RGN_AND, RGN_COPY, RGN_DIFF,
                                             RGN_OR, RGN_XOR);

var Index : LongInt;
    Figure : TsohoRegionFigure;
    NewRegion : HRGN;
    XOffset, YOffset : LongInt;
    Form : TForm;
    ResultRegion : HRGN;
    R: TRect;
begin
  Form := GetOwnerForm(Self);
  YOffset := GetCaptionHeight(Form)+GetMainMenuHeight(Form) +
             GetBorderHeight(Form);
  XOffset := trunc((Form.Width-Form.ClientWidth)/2);

  for Index := 0 to pred(Form.ControlCount) do
    if Form.Controls[Index] is TsohoRegionFigure then begin
      Figure := Form.Controls[Index] as TsohoRegionFigure;
      RegionCreate(Figure);
      with Figure do
       case RegionType of
        rtEllipse   : NewRegion := CreateEllipticRgn(XOffset+Left, YOffset+Top,
                      XOffset+Left+Width, YOffset+Top+Height);
        rtRectangle : NewRegion := CreateRectRgn(XOffset+Left, YOffset+Top,
                      XOffset+Left+Width, YOffset+Top+Height);
        else
        {rtRoundRect : }NewRegion := CreateRoundRectRgn(XOffset+Left, YOffset+Top,
                      XOffset+Left+Width, YOffset+Top+Height, 20, 20);
       end;
      AddRegion(NewRegion);
    end;
  if (roIncludeCaption in FOptions) and
     (Form.BorderStyle<>bsNone) then begin
     R:=GetCaptionRect(Form);
     NewRegion := CreateRectRgn(0, 0, Form.Width, R.Bottom);
     AddRegion(NewRegion);
  end;
  if (roIncludeMenu in FOptions) and Assigned(Form.Menu) then begin
     NewRegion := CreateRectRgn(0, GetCaptionHeight(Form)+GetBorderHeight(Form),
       Form.Width, GetCaptionHeight(Form)+GetBorderHeight(Form)+GetMainMenuHeight(Form));
     AddRegion(NewRegion);
  end;
  if Assigned(FBeforeApply) then FBeforeApply(Self);
  if FRegions.Count>0 then begin
    ResultRegion := HRGN(FRegions[0]);
    for Index := 1 to pred(FRegions.Count) do
      CombineRgn(ResultRegion, ResultRegion, HRGN(FRegions[Index]),Modes[FRegionsMode]);
    FFormRegion := CreateRectRgn(0,0,Form.Width, Form.Height);
    {ResultRegion := }CombineRgn(FFormRegion, FFormRegion, ResultRegion,
                    Modes[FMainMode]);
    SetWindowRgn(Form.Handle, FFormRegion, true);
    for Index := 0 to pred(FRegions.Count) do DeleteObject(HRGN(FRegions[Index]));
  end;
end;

procedure TsohoRegionForm.Loaded;
var //Index : LongInt;
    Form  : TForm;
begin
  inherited Loaded;
  if csDesigning in ComponentState then exit;
  Form := GetOwnerForm(Self);
  FOnFormCreate := Form.OnCreate;
  Form.OnCreate := DoFormCreate;
end;

function TsohoRegionForm.RegionsCount : LongInt;
begin
  Result := FRegions.Count;
end;

function TsohoRegionForm.AddRegionFigure : boolean;
var NewRegion : TsohoRegionFigure;
    Form : TForm;
begin
  Result := false;
  Form := GetOwnerForm(Self);
  try
    NewRegion := TsohoRegionFigure.Create(Form);
    NewRegion.Name := GetUniqueName(NewRegion, Form);
    NewRegion.Parent := Form;
    NewRegion.FRegionForm := Self;
    FFigures.Add(NewRegion);
    Result := true;
  except
  end;
end;

function TsohoRegionForm.CreateRgnFromBitmap (ClientX, ClientY : integer;
             Bitmap : TBitmap; TransparentColor : TColor) : HRGN;
var bmp: TBitmap;
    X, Y, MaxRects, X0, XOffset, YOffset: Integer;
    HData: HGlobal;
    PData: PRgnData;
    CB,CR,CG,LR,LG,LB: Byte;
    P32: Pointer;
    P: PLongInt;
    PR: PRect;
    H: Hrgn;
    Form : TForm;
begin
  Form := GetOwnerForm(Self);
  YOffset := GetCaptionHeight(Form)+GetMainMenuHeight(Form) +
             GetBorderHeight(Form) + ClientY;
  XOffset := trunc((Form.Width-Form.ClientWidth)/2) + ClientX;
  Result := 0;
  BMP := TBitmap.Create;
  BMP.Assign(Bitmap);
  BMP.HandleType := bmDIB;
  BMP.PixelFormat := pf32bit;
  MaxRects := 100;
  HData := GlobalAlloc(GMem_Moveable, SizeOf(TRgnDataHeader)+SizeOf(TRect)*MaxRects);
  PData := GlobalLock(HData);
  with PData^.RDH do begin
    dwSize := SizeOf(TRgnDataHeader);
    iType  := RDH_Rectangles;
    nCount := 0;
    nRgnSize := 0;
  end;    // with
  SetRect(PData^.RDH.rcBound, MaxInt, MaxInt, 0, 0);
  LR := GetRValue(ColorToRGB(TransparentColor));
  LG := GetGValue(ColorToRGB(TransparentColor));
  LB := GetBValue(ColorToRGB(TransparentColor));
  for Y := 0 to pred(Bmp.Height) do begin
    X := -1;
    P32 := BMP.ScanLine[Y];
    while X + 1 < Bmp.Width do begin
      Inc(X);
      X0 := X;
      P := PLongInt(P32);
      Inc(PChar(P), X*SizeOf(LongInt));
      while X < Bmp.Width do begin
        CR := GetBValue(P^);
        CG := GetGValue(P^);
        CB := GetRValue(P^);
        if ((CR=LR) and (CG=LG) and (CB=LB)) then Break;
        Inc(PChar(P), SizeOf(LongInt));
        Inc(X);
      end;
      if X > X0 then begin
        if Integer(PData^.RDH.nCount) >= MaxRects then begin
          GlobalUnlock(HData);
          Inc(MaxRects, 100);
          HData := GlobalReAlloc(HData, SizeOf(TRgnDataHeader)+SizeOf(TRect)*MaxRects,GMem_Moveable);
          PData := GlobalLock(HData)
        end;
        PR := @PData^.Buffer[PData^.RDH.nCount*SizeOf(TRect)];
        SetRect(PR^,
                X0  + XOffset,
                Y   + YOffset,
                X   + XOffset,
                Y+1 + YOffset);
        if X0 < PData^.RDH.rcBound.Left then
            PData^.RDH.rcBound.Left := X0 + XOffset;
        if Y<PData^.RDH.rcBound.Top then
            PData^.RDH.rcBound.Top := Y + YOffset;
        if X>PData^.RDH.rcBound.Right then
            PData^.RDH.rcBound.Left := X + XOffset;
        if Y+1>PData^.RDH.rcBound.Bottom then
            PData^.RDH.rcBound.Bottom := Y + 1 + YOffset;
        Inc(PData^.RDH.nCount);
        if PData^.RDH.nCount = 2000 then begin
          H := ExtCreateRegion(nil, SizeOf(TRgnDataHeader) +
               (SizeOf(TRect)*MaxRects), PData^);
          if Result <> 0 then begin
            CombineRgn(Result, Result, H, RGN_OR);
            DeleteObject(H);
          end
          else Result := H;
          PData^.RDH.nCount := 0;
          SetRect(PData^.RDH.rcBound, MaxInt, MaxInt, 0, 0)
        end
      end
    end
  end;
  H := ExtCreateRegion(nil, SizeOf(TRgnDataHeader) +
      (SizeOf(TRect)*MaxRects), PData^);
  if Result <> 0 then begin
    CombineRgn(Result, Result, H, RGN_OR);
    DeleteObject(H);
  end
  else Result := H;
  GlobalFree(HData);
  Bmp.Free;
end;

constructor TsohoRegionForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMainMode     := cmAnd;
  FRegionsMode  := cmOr;
  FOptions      := [];
  FOnFormCreate := nil;
  FRegions      := TList.Create;
  FFigures      := TList.Create;
end;

destructor TsohoRegionForm.Destroy;
begin
  if FRegions.Count > 0 then begin
    DeleteObject(FFormRegion);
    FRegions.Free;
  end;
  while FFigures.Count > 0 do TsohoRegionFigure(FFigures[0]).Free;
  FFigures.Free;
  if Assigned(FOnFormCreate) and not (csDestroying in ComponentState)
    then GetOwnerForm(Self).OnCreate := FOnFormCreate;
  inherited Destroy;
end;

initialization
  RegisterClasses([TsohoRegionFigure]);
end.
