{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
     TsohoOwnerDrawComboBox,
  TsohoColorComboBox  TsohoImageComboBox
}
unit SoCombos;

interface

{$I SOHOLIB.INC}

uses Windows, Messages, SysUtils, Classes, Graphics,
     Controls, SoCtrls, StdCtrls 
     {$IFDEF Delphi4_or_5}, ImgList {$ENDIF};

type

  TsohoOwnerDrawComboStyle = csDropDown..csDropDownList;

  TsohoOwnerDrawComboBox = class(TsohoCustomComboBox)
  private
    FStyle: TsohoOwnerDrawComboStyle;
    FItemHeightChanging: Boolean;
    procedure SetComboStyle(Value: TsohoOwnerDrawComboStyle);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMRecreateWnd(var Message: TMessage); message CM_RECREATEWND;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure ResetItemHeight; virtual;
    function MinItemHeight: Integer; virtual;
    property Style: TsohoOwnerDrawComboStyle read FStyle write SetComboStyle default csDropDownList;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property OnChange;   
  end;

  TsohoColorItem = (ciStandard, ciSystem);
  TsohoColorItems = set of TsohoColorItem;

  TsohoColorComboBox = class(TsohoOwnerDrawComboBox)
  {     
     .   
     TColorComboBox (RxLib),    
     - TsohoColorComboBox   
     ,      . 
         "". 
        ,  
       ,  
            .
       ,    
    ,     .  
      ,       
    ,       , 
         .
  }
  private
    FColorValue: TColor;
    FDisplayNames: Boolean;
    FColorNames: TStrings;
    FCustomColor: TColor;
    FOptions: TsohoColorItems;
    FCustomAdded: boolean;
    FOnChange: TNotifyEvent;
    function GetColorValue: TColor;
    procedure SetColorValue(NewValue: TColor);
    procedure SetDisplayNames(Value: Boolean);
    procedure SetColorNames(Value: TStrings);
    procedure ColorNamesChanged(Sender: TObject);
    procedure SetOptions(Value: TsohoColorItems);
  protected
    procedure CreateWnd; override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Click; override;
    procedure Change; override;
    procedure PopulateList; virtual;
    procedure DoChange; dynamic;
    procedure SetCustomColor(Value: TColor);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Text;
  published
    property Align;
    property Options: TsohoColorItems read FOptions write SetOptions default [ciStandard];
    property ColorValue: TColor read GetColorValue write SetColorValue default clBlack;
    {    }
    property ColorNames: TStrings read FColorNames write SetColorNames;
    {    .   ,  
      ,     : Black, Blue & etc }
    property DisplayNames: Boolean read FDisplayNames write SetDisplayNames default True;
    {  ,      
           }
    property CustomColor: TColor read FCustomColor write SetCustomColor default clBlack;
    {  ,      }
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property Font;
{$IFDEF Delphi4_or_5}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property ImeMode;
    property ImeName;
{$ENDIF}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Style;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    {      ColorValue }
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
{$IFDEF Delphi5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF Delphi4_or_5}
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
  end;

  TsohoGetImageEvent = procedure (Sender: TObject; ItemIndex: LongInt; var ImageIndex: LongInt) of object;
  {   OnGetImageIndex  TsohoImageComboBox.
     ItemIndex     
    TsohoImageComboBox.Items.     
    TsohoImageComboBox.Images    
      ImageIndex }

  TsohoImageDrawType = (dtLeft, dtRight, dtNoText);
  {   DrawType  TsohoImageComboBox.  
       :
    dtLeft   -   
    dtRight  -   
    dtNoText -     }

  TsohoImageComboBox = class(TsohoOwnerDrawComboBox)
  {     ,   
         ImageList ( Images).
          OnGetImageIndex,
      ,    ImageList  
      }
  private
    { Private declarations }
    FOnGetImageIndex: TsohoGetImageEvent;
    FImages: TImageList;
    FDrawType: TsohoImageDrawType;
  protected
    { Protected declarations }
    procedure SetDrawType(Value: TsohoImageDrawType);
    procedure SetImages(Value: TImageList);
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Notification(aComponent: TComponent; aOperation: TOperation); override;
    procedure GetImageIndex (ItemIndex: LongInt; var ImageIndex: LongInt); virtual;
    procedure ResetItemHeight; override;
  public
    constructor Create(aOwner: TComponent); override;
  published
    property Align;
    property Items;
    property Images: TImageList read FImages write SetImages;
    {     TImageList,   
           }
    property DrawType: TsohoImageDrawType read FDrawType write SetDrawType default dtLeft;
    {       }
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property Font;
{$IFDEF Delphi4_or_5}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property ImeMode;
    property ImeName;
{$ENDIF}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Style;
    property Sorted;
    property TabOrder;
    property Text;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
    property OnGetImageIndex: TsohoGetImageEvent read FOnGetImageIndex write FOnGetImageIndex;
    {     .   ,
           Images    }
{$IFDEF Delphi5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF Delphi4_or_5}
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
  end;

  TsohoFontDevice = (sfdScreen, sfdPrinter, sfdBoth);
  TsohoFontListOption = (sfoAnsiOnly, sfoTrueTypeOnly, sfoFixedPitchOnly,
    sfoNoOEMFonts, sfoOEMFontsOnly, sfoScalableOnly, sfoNoSymbolFonts);
  TsohoFontListOptions = set of TsohoFontListOption;

  TsohoFontComboBox = class(TsohoOwnerDrawComboBox)
  private
    FTrueTypeBMP: TBitmap;
    FDeviceBMP: TBitmap;
    FOnChange: TNotifyEvent;
    FDevice: TsohoFontDevice;
    FUpdate: Boolean;
    FUseFonts: Boolean;
    FOptions: TsohoFontListOptions;
    procedure SetFontName(const NewFontName: TFontName);
    function GetFontName: TFontName;
    procedure SetDevice(Value: TsohoFontDevice);
    procedure SetOptions(Value: TsohoFontListOptions);
    procedure SetUseFonts(Value: Boolean);
    procedure Reset;
    procedure WMFontChange(var Message: TMessage); message WM_FONTCHANGE;
  protected
    procedure PopulateList; virtual;
    procedure Change; override;
    procedure Click; override;
    procedure DoChange; dynamic;
    procedure CreateWnd; override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    function MinItemHeight: Integer; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Text;
  published
    property Device: TsohoFontDevice read FDevice write SetDevice default sfdScreen;
    property FontName: TFontName read GetFontName write SetFontName;
    property Options: TsohoFontListOptions read FOptions write SetOptions default [];
    property UseFonts: Boolean read FUseFonts write SetUseFonts default False;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property Font;
{$IFDEF Delphi4_or_5}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property ImeMode;
    property ImeName;
{$ENDIF}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Style;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
{$IFDEF Delphi5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF Delphi4_or_5}
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
  end;

implementation
uses SoCmnCns, Dialogs, SoGraph, SoUtils, Printers;

{$R SOCOMBOS.R32}

function GetItemHeight(Font: TFont): Integer;
var
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;
begin
  DC := GetDC(0);
  try
    SaveFont := SelectObject(DC, Font.Handle);
    GetTextMetrics(DC, Metrics);
    SelectObject(DC, SaveFont);
  finally
    ReleaseDC(0, DC);
  end;
  Result := Metrics.tmHeight + 1;
end;

{ TOwnerDrawComboBox }

constructor TsohoOwnerDrawComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited Style := csDropDownList;
  FStyle := csDropDownList;
end;

procedure TsohoOwnerDrawComboBox.SetComboStyle(Value: TsohoOwnerDrawComboStyle);
begin
  if FStyle <> Value then begin
    FStyle := Value;
    inherited Style := Value;
  end;
end;

function TsohoOwnerDrawComboBox.MinItemHeight: Integer;
begin
  Result := GetItemHeight(Font);
  if Result < 9 then Result := 9;
end;

procedure TsohoOwnerDrawComboBox.ResetItemHeight;
var
  H: Integer;
begin
  H := MinItemHeight;
  FItemHeightChanging := True;
  try
    inherited ItemHeight := H;
  finally
    FItemHeightChanging := False;
  end;
  if HandleAllocated then SendMessage(Handle, CB_SETITEMHEIGHT, 0, H);
end;

procedure TsohoOwnerDrawComboBox.CreateParams(var Params: TCreateParams);
const
  ComboBoxStyles: array[TsohoOwnerDrawComboStyle] of DWORD =
    (CBS_DROPDOWN, CBS_SIMPLE, CBS_DROPDOWNLIST);
begin
  inherited CreateParams(Params);
  with Params do
    Style := (Style and not CBS_DROPDOWNLIST) or CBS_OWNERDRAWFIXED or
      ComboBoxStyles[FStyle];
end;

procedure TsohoOwnerDrawComboBox.CreateWnd;
begin
  inherited CreateWnd;
  ResetItemHeight;
end;

procedure TsohoOwnerDrawComboBox.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ResetItemHeight;
  RecreateWnd;
end;

procedure TsohoOwnerDrawComboBox.CMRecreateWnd(var Message: TMessage);
begin
  if not FItemHeightChanging then inherited;
end;

{ TsohoColorComboBox }

const
  StandardColors = 16;
  SystemColors = 24;
  ColorsInList = StandardColors + SystemColors;

  ColorValues: array [0..ColorsInList - 1] of TColor = (
    clBlack, clMaroon, clGreen, clOlive, clNavy,
    clPurple, clTeal, clGray, clSilver, clRed,
    clLime, clYellow, clBlue, clFuchsia, clAqua,
    clWhite,
    //      
    clBackground, clActiveCaption, clInactiveCaption,
    clMenu, clWindow, clWindowFrame, clMenuText,
    clWindowText, clCaptionText, clActiveBorder,
    clInactiveBorder, clAppWorkSpace, clHighlight,
    clHighlightText, clBtnFace, clBtnShadow,
    clGrayText, clBtnText, clInactiveCaptionText,
    clBtnHighlight, cl3DDkShadow, cl3DLight,
    clInfoText, clInfoBk
    );

constructor TsohoColorComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FColorValue := clBlack;
  FOptions := [ciStandard];
  FColorNames := TStringList.Create;
  TStringList(FColorNames).OnChange := ColorNamesChanged;
  FDisplayNames := True;
  FCustomColor := clBlack;
  FCustomAdded := false;
end;

destructor TsohoColorComboBox.Destroy;
begin
  TStringList(FColorNames).OnChange := nil;
  FColorNames.Free;
  FColorNames := nil;
  inherited Destroy;
end;

procedure TsohoColorComboBox.SetCustomColor(Value: TColor);
var Index: LongInt;
begin
  if FCustomColor = Value then exit;
  //     
  FCustomColor := Value;
  for Index := 0 to pred(Items.Count - 1) do
    if TColor(Items.Objects[Index]) = Value then begin
      if FCustomAdded then begin
        Items.Delete(0);
        FCustomAdded := false;
      end;
      exit;
    end;
  if csDesigning in ComponentState then exit;
  if FCustomAdded then begin
    Items.Objects[0] := TObject(FCustomColor);
    Items[0] := ColorToString(FCustomColor);
  end
  else Items.InsertObject(0, ColorToString(FCustomColor),
    TObject(FCustomColor));
  FCustomAdded := true;
end;

procedure TsohoColorComboBox.CreateWnd;
begin
  inherited CreateWnd;
  PopulateList;
  SetColorValue(FColorValue);
end;

procedure TsohoColorComboBox.PopulateList;
var
  ColorName: string;
  I: Integer;           

begin
  if Parent = nil then exit;
  Items.BeginUpdate;
  try
    Clear;
    if ciStandard in FOptions then begin
      for I := 0 to Pred(StandardColors) do begin
        if (I <= Pred(FColorNames.Count)) and (FColorNames[I] <> '') then
          ColorName := FColorNames[I]
        else
          ColorName := Copy(ColorToString(ColorValues[I]), 3, MaxInt);
        Items.AddObject(ColorName, TObject(ColorValues[I]));
      end;
    end;
    if ciSystem in FOptions then begin
      for I := 0 to Pred(SystemColors) do begin
        if (I <= Pred(FColorNames.Count)) and (FColorNames[I] <> '') then
          ColorName := FColorNames[I]
        else
          ColorName := Copy(ColorToString(ColorValues[I + StandardColors]), 3, MaxInt);
        Items.AddObject(ColorName, TObject(ColorValues[I + StandardColors]));
      end;
    end;
    //      ,
    //       
    // 
    Items.AddObject(sohoCustomColor, TObject(0));
  finally
    Items.EndUpdate;
  end;
end;

procedure TsohoColorComboBox.ColorNamesChanged(Sender: TObject);
begin
  if HandleAllocated then begin
    FColorValue := ColorValue;
    RecreateWnd;
  end;
end;

procedure TsohoColorComboBox.SetColorNames(Value: TStrings);
begin
  FColorNames.Assign(Value);
end;

procedure TsohoColorComboBox.SetDisplayNames(Value: Boolean);
begin
  if DisplayNames <> Value then begin
    FDisplayNames := Value;
    Invalidate;
  end;
end;

function TsohoColorComboBox.GetColorValue: TColor;
var
  I: Integer;
begin
  Result := FColorValue;
  if (Style <> csDropDownList) and (ItemIndex < 0) then begin
    I := Items.IndexOf(inherited Text);
    if I >= 0 then Result := TColor(Items.Objects[I])
    else begin
      Val(inherited Text, Result, I);
      if I <> 0 then Result := FColorValue;
    end;
  end;
end;

procedure TsohoColorComboBox.SetColorValue(NewValue: TColor);
var
  S: string;

  function FindColorIndex (Value: TColor): boolean;
  var
    CurrentColor: TColor;
    Item: Integer;
  begin
    Result := true;
    for Item := 0 to Pred(Items.Count) do begin
      CurrentColor := TColor(Items.Objects[Item]);
      if CurrentColor = Value then begin
        FColorValue := Value;
        if ItemIndex <> Item then ItemIndex := Item;
        DoChange;
        Exit;
      end;
    end;
    Result := false;
  end;

begin
  if (ItemIndex < 0) or (NewValue <> FColorValue) then begin
    { change selected item }
    if FindColorIndex(NewValue) then exit;
    CustomColor := NewValue;
    FindColorIndex(NewValue);
    // FColorValue := NewValue;
    S := ColorToString(NewValue);
    if Pos('cl', S) = 1 then System.Delete(S, 1, 2);
    inherited Text := S;
    DoChange;
  end;
end;

procedure TsohoColorComboBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
const
  ColorWidth = 22;
var
  ARect: TRect;
  Text: array[0..255] of Char;
  Safer: TColor;
  Invert: boolean;

  procedure DrawTriangle(StartLeft: LongInt);
  begin
    with Canvas do begin
      MoveTo(StartLeft, ARect.Top);
      LineTo(StartLeft + (WidthOf(aRect)-2) div 2,
        ARect.Top + HeightOf(aRect) div 2);
      LineTo(StartLeft, ARect.Bottom);
      LineTo(StartLeft, ARect.Top);
      FloodFill(StartLeft + 3, ARect.Top + HeightOf(aRect) div 2,
        Brush.Color, fsBorder);
    end;
  end;

begin
  Invert := (odSelected in State) or (odFocused in State);
  ARect := Rect;
  Inc(ARect.Top, 2);
  Inc(ARect.Left, 2);
  Dec(ARect.Bottom, 2);
  if FDisplayNames then ARect.Right := ARect.Left + ColorWidth
  else Dec(ARect.Right, 3);
  with Canvas do begin
    FillRect(Rect);
    Safer := Brush.Color;
    if Invert then Pen.Color := clWhite
    else Pen.Color := clWindowText;
    if (Index < pred(Items.Count)) then begin
      Rectangle(ARect.Left, ARect.Top, ARect.Right, ARect.Bottom);
      Brush.Color := TColor(Items.Objects[Index]);
      try
        InflateRect(ARect, -1, -1);
        FillRect(ARect);
      finally
        Brush.Color := Safer;
      end;
    end
    else begin
      //  ... ,  ...
      if Invert then Brush.Color := clWhite
      else Brush.Color := clBlack;
      InflateRect(ARect, -4, -2);
      try
        DrawTriangle(ARect.Left);
        DrawTriangle(ARect.Left + (WidthOf(aRect)-2) div 2 + 2);
      finally
        Brush.Color := Safer;
      end;
    end;
    if FDisplayNames then begin
      StrPCopy(Text, Items[Index]);
      Rect.Left := Rect.Left + ColorWidth + 6;
      DrawText(Canvas.Handle, Text, StrLen(Text), Rect,
{$IFDEF Delphi4_or_5}
        DrawTextBiDiModeFlags(DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX));
{$ELSE}
        DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
{$ENDIF}
    end;
  end;
end;

procedure TsohoColorComboBox.Change;
var
  AColor: TColor;
begin
  inherited Change;
  AColor := GetColorValue;
  if FColorValue <> AColor then begin
    FColorValue := AColor;
    DoChange;
  end;
end;

procedure TsohoColorComboBox.Click;
var aColorValue: TColor;
    aDlg: TColorDialog;
begin
  if ItemIndex >= 0 then begin
    aColorValue := TColor(Items.Objects[ItemIndex]);
    if ItemIndex = pred(Items.Count) then begin
      //     
      aDlg := TColorDialog.Create(Self);
      if FCustomAdded then
        aDlg.CustomColors.Add('ColorA='+
          IntToHex(FCustomColor, 8))
      else
        aDlg.Color := FCustomColor;

      if not aDlg.Execute then exit;
      CustomColor := aDlg.Color;
      ColorValue := FCustomColor;
    end
    else ColorValue := aColorValue;
  end;
  inherited Click;
end;

procedure TsohoColorComboBox.DoChange;
begin
  if not (csReading in ComponentState) then
    if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TsohoColorComboBox.SetOptions(Value: TsohoColorItems);
begin
  if (FOptions = Value) or (Value = []) then exit;
  FOptions := Value;
  PopulateList;
end;

{ TsohoImageComboBox }
procedure TsohoImageComboBox.SetImages(Value: TImageList);
begin
  FImages := Value;
  ResetItemHeight;
  Invalidate;
end;

procedure TsohoImageComboBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  ARect: TRect;
  Text: array[0..255] of Char;
  Invert: boolean;
  ImageIndex: LongInt;
begin
  if not Assigned(FImages) then begin
    //     ,    
    inherited;
    exit;
  end;

  Invert := (odSelected in State) or (odFocused in State);

  ARect := Rect;
  Inc(ARect.Top, 2);
  Dec(ARect.Bottom, 2);

  case FDrawType of
    dtLeft: begin
      Inc(ARect.Left, 2);
      ARect.Right := ARect.Left + FImages.Width;
    end;
    dtRight: begin
      Dec(ARect.Right, 2);
      ARect.Left := ARect.Right - FImages.Width;
    end;
    else begin
      Inc(ARect.Left, 2);
      Dec(ARect.Right, 2);
    end;
  end;

  with Canvas do begin
    FillRect(Rect);
    if Invert then Pen.Color := clWhite
    else Pen.Color := clWindowText;

    //    ?
    GetImageIndex(Index, ImageIndex);
    if ImageIndex <> -1 then begin
      if FDrawType = dtNoText then
        FImages.Draw(Canvas, HorCenter(aRect, FImages.Width),
          VertCenter(aRect, FImages.Height), ImageIndex)
      else
        FImages.Draw(Canvas, aRect.Left,
          VertCenter(aRect, FImages.Height), ImageIndex);
    end;

    if FDrawType <> dtNoText then begin
      StrPCopy(Text, Items[Index]);
      if FDrawType = dtLeft then
        Rect.Left := Rect.Left + FImages.Width + 4
      else begin
        Rect.Right := aRect.Left - 4;
        Inc(Rect.Left, 2);
      end;
      DrawText(Canvas.Handle, Text, StrLen(Text), Rect,
      {$IFDEF Delphi4_or_5}
      DrawTextBiDiModeFlags(DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX));
      {$ELSE}
        DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
      {$ENDIF}
    end;
  end;
end;

procedure TsohoImageComboBox.Notification(aComponent: TComponent; aOperation: TOperation);
begin
  inherited;
  if (aComponent = FImages) and (aOperation = opRemove) then Images := nil;
end;

procedure TsohoImageComboBox.GetImageIndex (ItemIndex: LongInt; var ImageIndex: LongInt);
begin
  ImageIndex := -1;
  if Assigned(FOnGetImageIndex) then FOnGetImageIndex(Self, ItemIndex, ImageIndex);
end;

constructor TsohoImageComboBox.Create(aOwner: TComponent);
begin
  inherited;
  FImages := nil;
  FDrawType := dtLeft;
end;

procedure TsohoImageComboBox.ResetItemHeight;
var H: Integer;
begin
  if Assigned(FImages) then H := GetMax(FImages.Height + 2, MinItemHeight)
  else H := MinItemHeight;
  FItemHeightChanging := True;
  try
    inherited ItemHeight := H;
  finally
    FItemHeightChanging := False;
  end;
  if HandleAllocated then SendMessage(Handle, CB_SETITEMHEIGHT, 0, H);
end;

procedure TsohoImageComboBox.SetDrawType(Value: TsohoImageDrawType);
begin
  if FDrawType = Value then exit;
  FDrawType := Value;
  Invalidate;
end;

{ TsohoFontComboBox }
const
  WRITABLE_FONTTYPE = 256;

function IsValidFont(Box: TsohoFontComboBox; LogFont: TLogFont;
  FontType: Integer): Boolean;
begin
  Result := True;
  if (sfoAnsiOnly in Box.Options) then
    Result := Result and (LogFont.lfCharSet = ANSI_CHARSET);
  if (sfoTrueTypeOnly in Box.Options) then
    Result := Result and (FontType and TRUETYPE_FONTTYPE = TRUETYPE_FONTTYPE);
  if (sfoFixedPitchOnly in Box.Options) then
    Result := Result and (LogFont.lfPitchAndFamily and FIXED_PITCH = FIXED_PITCH);
  if (sfoOEMFontsOnly in Box.Options) then
    Result := Result and (LogFont.lfCharSet = OEM_CHARSET);
  if (sfoNoOEMFonts in Box.Options) then
    Result := Result and (LogFont.lfCharSet <> OEM_CHARSET);
  if (sfoNoSymbolFonts in Box.Options) then
    Result := Result and (LogFont.lfCharSet <> SYMBOL_CHARSET);
  if (sfoScalableOnly in Box.Options) then
    Result := Result and (FontType and RASTER_FONTTYPE = 0);
end;

function EnumFontsProc(var EnumLogFont: TEnumLogFont;
  var TextMetric: TNewTextMetric; FontType: Integer; Data: LPARAM): Integer;
  export; stdcall;
var
  FaceName: string;
begin
  FaceName := StrPas(EnumLogFont.elfLogFont.lfFaceName);
  with TsohoFontComboBox(Data) do
    if (Items.IndexOf(FaceName) < 0) and
      IsValidFont(TsohoFontComboBox(Data), EnumLogFont.elfLogFont, FontType) then
    begin
      if EnumLogFont.elfLogFont.lfCharSet <> SYMBOL_CHARSET then
        FontType := FontType or WRITABLE_FONTTYPE;
      Items.AddObject(FaceName, TObject(FontType));
    end;
  Result := 1;
end;

procedure TsohoFontComboBox.Change;
var
  I: Integer;
begin
  inherited;
  if Style <> csDropDownList then begin
    I := Items.IndexOf(inherited Text);
    if (I >= 0) and (I <> ItemIndex) then begin
      ItemIndex := I;
      DoChange;
    end;
  end;
end;

procedure TsohoFontComboBox.Click;
begin
  inherited;
  DoChange;
end;

constructor TsohoFontComboBox.Create(AOwner: TComponent);
begin
  inherited;
  FTrueTypeBMP := TBitmap.Create;
  FTrueTypeBMP.LoadFromResourceName(hInstance, 'SOHO_TRUETYPE_FNT');
  FDeviceBMP := TBitmap.Create;
  FDeviceBMP.LoadFromResourceName(hInstance, 'SOHO_DEVICE_FNT');
  FDevice := sfdScreen;
  Sorted := True;
  inherited ItemHeight := MinItemHeight;
end;

procedure TsohoFontComboBox.CreateWnd;
var
  OldFont: TFontName;
begin
  OldFont := FontName;
  inherited CreateWnd;
  FUpdate := True;
  try
    PopulateList;
    inherited Text := '';
    SetFontName(OldFont);
  finally
    FUpdate := False;
  end;
  if AnsiCompareText(FontName, OldFont) <> 0 then DoChange;
end;

destructor TsohoFontComboBox.Destroy;
begin
  FTrueTypeBMP.Free;
  FDeviceBMP.Free;
  inherited;
end;

procedure TsohoFontComboBox.DoChange;
begin
  if not (csReading in ComponentState) then
    if not FUpdate and Assigned(FOnChange) then FOnChange(Self);
end;

procedure TsohoFontComboBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
var
  Bitmap: TBitmap;
  BmpWidth: Integer;
  Text: array[0..255] of Char;
begin
  with Canvas do begin
    FillRect(Rect);
    BmpWidth  := 20;
    if (Integer(Items.Objects[Index]) and TRUETYPE_FONTTYPE) <> 0 then
      Bitmap := FTrueTypeBMP
    else if (Integer(Items.Objects[Index]) and DEVICE_FONTTYPE) <> 0 then
      Bitmap := FDeviceBMP
    else Bitmap := nil;
    if Bitmap <> nil then begin
      BmpWidth := Bitmap.Width;
      BrushCopy(Bounds(Rect.Left + 2, (Rect.Top + Rect.Bottom - Bitmap.Height)
        div 2, Bitmap.Width, Bitmap.Height), Bitmap, Bounds(0, 0, Bitmap.Width,
        Bitmap.Height), Bitmap.TransparentColor);
    end;
    StrPCopy(Text, Items[Index]);
    Rect.Left := Rect.Left + BmpWidth + 6;
    if FUseFonts and (Integer(Items.Objects[Index]) and WRITABLE_FONTTYPE <> 0) then
      Font.Name := Items[Index];
    DrawText(Handle, Text, StrLen(Text), Rect,
{$IFDEF Delphi4_or_5}
      DrawTextBiDiModeFlags(DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX));
{$ELSE}
      DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
{$ENDIF}
  end;
end;

function TsohoFontComboBox.GetFontName: TFontName;
begin
  Result := inherited Text;
end;

function TsohoFontComboBox.MinItemHeight: Integer;
begin
  Result := inherited MinItemHeight;
  if Result < FTrueTypeBMP.Height - 1 then
    Result := FTrueTypeBMP.Height - 1;
end;

procedure TsohoFontComboBox.PopulateList;
var
  DC: HDC;
begin
  if not HandleAllocated then Exit;
  Items.BeginUpdate;
  try
    Clear;
    DC := GetDC(0);
    try
      if (FDevice = sfdScreen) or (FDevice = sfdBoth) then
        EnumFontFamilies(DC, nil, @EnumFontsProc, Longint(Self));
      if (FDevice = sfdPrinter) or (FDevice = sfdBoth) then
      try
        EnumFontFamilies(Printer.Handle, nil, @EnumFontsProc, Longint(Self));
      except
      end;
    finally
      ReleaseDC(0, DC);
    end;
  finally
    Items.EndUpdate;
  end;
end;

procedure TsohoFontComboBox.Reset;
var
  SaveName: TFontName;
begin
  if HandleAllocated then begin
    FUpdate := True;
    try
      SaveName := FontName;
      PopulateList;
      FontName := SaveName;
    finally
      FUpdate := False;
      if FontName <> SaveName then DoChange;
    end;
  end;
end;

procedure TsohoFontComboBox.SetDevice(Value: TsohoFontDevice);
begin
  if Value <> FDevice then begin
    FDevice := Value;
    Reset;
  end;
end;

procedure TsohoFontComboBox.SetFontName(const NewFontName: TFontName);
var
  Item: Integer;
begin
  if FontName <> NewFontName then begin
    if not (csLoading in ComponentState) then begin
      HandleNeeded;
      { change selected item }
      for Item := 0 to Items.Count - 1 do
        if AnsiCompareText(Items[Item], NewFontName) = 0 then begin
          ItemIndex := Item;
          DoChange;
          Exit;
        end;
      if Style = csDropDownList then ItemIndex := -1
      else inherited Text := NewFontName;
    end
    else inherited Text := NewFontName;
    DoChange;
  end;
end;

procedure TsohoFontComboBox.SetOptions(Value: TsohoFontListOptions);
begin
  if Value <> Options then begin
    FOptions := Value;
    Reset;
  end;
end;

procedure TsohoFontComboBox.SetUseFonts(Value: Boolean);
begin
  if Value <> FUseFonts then begin
    FUseFonts := Value;
    Invalidate;
  end;
end;

procedure TsohoFontComboBox.WMFontChange(var Message: TMessage);
begin
  inherited;
  Reset;
end;

end.
