{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov,              }
{                   Igor Chernyshov                     }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoCtmCmb;

{$I SOHOLIB.INC}

interface

uses SysUtils, Windows, Messages, Classes, Graphics, Controls,
     Forms, StdCtrls, SoCtrls, Menus, SoUnit, SoTools, DB;

type

  TsohoCustomDBComboBox = class(TsohoComboBox)
  {         .  
      (SQLFile),  ,   
         (TextField),     
     (KeyField),         IdItems.
        TableName -    
        SQLFile   .   
         ItemId.  ,    
    ,        , ,
TsohoDBComboBox
  }
  private
    { Private declarations }
    FTextField: string;
    FKeyField: string;
    FAutoInsert: Boolean;
    FAutoSelect: Boolean;
    FSQLFile: TSQLFileName;
    FTableName: TFileName;
    FPrepared: Boolean;
    FBeforeOpen: TDataSetNotifyEvent;
    FItemsPreffix : string;
    FItemID: LongInt;

    procedure SetAutoInsert(Value: Boolean);
    procedure SetIdItem(index: Integer; Value: Longint);
    function GetIdItem(index: Integer): Longint;
    procedure SetItemId(Value: Longint);
    function GetItemId: Longint;
    procedure SetAutoSelect(Value: Boolean);
    procedure SetPrepared(Value: Boolean);
    procedure SetSQLFile(Value: TSQLFileName);
  protected
    { Protected declarations }
    procedure GenerateInsertSQL(Query: TStringList); virtual;
    procedure GenerateSelectSQL(Query: TStringList); virtual;
    function  ActivateSelection (SQLText : TStringList) : TDataSet;virtual; abstract;
    procedure DoInsertSQL (SQLText : TStringList);virtual; abstract;
    procedure PrepareList; virtual;
    function  IndexOfId(Value: Longint): Integer;
    procedure KeyDown (var Key: Word; Shift: TShiftState);override;
    procedure DoExit;override;
    procedure InsertNewItem; virtual;
    procedure DoBeforeOpen (DataSet : TDataSet);virtual;
    procedure Change; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure DropDown; override;
    {   }
    procedure Activate;
    {    }
    procedure Deactivate;
    {    }
    procedure AddItem(aItemId: Longint; const aItemText: string);
    {    .      }
    function  DeleteId(Value: LongInt): boolean;
    {    ,    ! }
    property IdItems[index: Integer]: Longint read GetIdItem write SetIdItem;
    {       }
    property ItemID: Longint read GetItemId write SetItemId;
    {    }
    property Active: Boolean read FPrepared write SetPrepared;
    { ,        }
    property Style;
    {    TCustomComboBox,    
      published,      ,
       Style    csDropDown }
  published
    property ItemsPreffix : string read FItemsPreffix write FItemsPreffix;
    {     . . GetNewId }
    property TextField: string read FTextField write FTextField;
    {  ,        }
    property KeyField: string read FKeyField write FKeyField;
    {  ,          }
    property AutoSelect: Boolean read FAutoSelect write SetAutoSelect default true;
    {  true,           }
    property AutoInsert: Boolean read FAutoInsert write SetAutoInsert default false;
    {       }
    property SQLFile: TSQLFileName read FSQLFile write SetSQLFile;
    {    SQL-,     }
    property TableName: TFileName read FTableName write FTableName;
    {  ,       .  
,  ,     .    SQLFile }
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    property Enabled;
    property Font;
    property ItemHeight;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnBeforeOpen: TDataSetNotifyEvent read FBeforeOpen write FBeforeOpen;
    {      }
  end;

implementation
uses SoAppUtils, SoVCLUtils, SoCtmRgs, SoDBCns, SoStrUtils;

procedure TsohoCustomDBComboBox.DoBeforeOpen (DataSet : TDataSet);
begin
 if Assigned(FBeforeOpen) then FBeforeOpen(DataSet);
end;

procedure TsohoCustomDBComboBox.SetPrepared(Value: Boolean);
begin
  if FPrepared = Value then exit;
  if Value then PrepareList
  else begin
    Items.Clear;
    FPrepared := False;
  end;
end;

procedure TsohoCustomDBComboBox.Activate;
begin
  Active := True;
end;

procedure TsohoCustomDBComboBox.Deactivate;
begin
  Active := False;
end;

procedure TsohoCustomDBComboBox.SetSQLFile(Value: TSQLFileName);
begin
  if FSQLFile = Value then exit;
  Active := False;
  FSQLFile := Value;
end;

procedure TsohoCustomDBComboBox.GenerateInsertSQL(Query: TStringList);
begin
  if ExtractFileExt(FTableName)<>'' then
    Query.Add('insert into "' + FTableName + '" (' + KeyField +
      ', ' + TextField + ')')
  else
    Query.Add('insert into ' + FTableName + ' (' + KeyField +
      ', ' + TextField + ')');
end;

procedure TsohoCustomDBComboBox.InsertNewItem;
var Query : TStringList;
    NewId : LongInt;
begin
  if (FTableName = '') or
     (Items.IndexOf(Text)<>-1) or
     (Text = '') then exit;

  if Items.IndexOf(Text) <> -1 then exit;

  Query := TStringList.Create;
  with Query do begin
    try
      GenerateInsertSQL(Query);
      repeat
        NewId := GetNewId(FItemsPreffix);
      until IndexOfId(NewId)=-1;
      Add('values (' + IntToStr(NewId) + ', ' + SingleQuotes(Self.Text) + ')');
      PushCursor(crHourGlass);
      DoInsertSQL(Query);
      AddItem(NewId, Self.Text);
      ItemId := NewId;
    finally
      RestoreCursor;
      Free;
    end;
  end;
end;

procedure TsohoCustomDBComboBox.DoExit;
begin
  inherited DoExit;
  if FAutoInsert then InsertNewItem;
end;

procedure TsohoCustomDBComboBox.KeyDown (var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if (Key = VK_INSERT) and FAutoInsert then InsertNewItem;
end;

procedure TsohoCustomDBComboBox.SetAutoInsert(Value: Boolean);
begin
  FAutoInsert := Value;
  if FAutoInsert then Style := csDropDown
  else Style := csDropDownList;
end;

function TsohoCustomDBComboBox.IndexOfId(Value: Longint): Integer;
begin
  Result := Items.IndexOfObject(TObject(Value));
end;

function  TsohoCustomDBComboBox.DeleteId(Value: LongInt): boolean;
var Index: LongInt;
begin
  Result := false;
  Index := IndexOfId(Value);
  if Index = -1 then exit;
  Items.Delete(Index);
  Result := true;
end;

procedure TsohoCustomDBComboBox.SetIdItem(index: Integer; Value: Longint);
begin
  Items.Objects[index] := TObject(Value);
end;

function TsohoCustomDBComboBox.GetIdItem(index: Integer): Longint;
begin
  Result := Longint(Items.Objects[index]);
end;

procedure TsohoCustomDBComboBox.SetItemId(Value: Longint);
var index: Integer;
begin
  index := IndexOfId(Value);
  if index = -1 then exit;
  FItemID := Value;
  ItemIndex := index;
  if Style = csDropDown then Text := Items[index];
end;

function TsohoCustomDBComboBox.GetItemId: Longint;
begin
  if Style <> csDropDown then begin
    Result := -1;
    if ItemIndex = -1 then exit;
    Result := IdItems[ItemIndex];
  end
  else
    Result := FItemID;
end;

procedure TsohoCustomDBComboBox.SetAutoSelect(Value: Boolean);
begin
  // if FAutoSelect = Value then exit;
  FAutoSelect := Value;
  {      ,    
          ! }
  if (Items.Count > 0) and (ItemIndex = -1) then ItemID := IdItems[0];
end;

procedure TsohoCustomDBComboBox.AddItem(aItemId: Longint; const aItemText: string);
begin
  Items.AddObject(aItemText, TObject(aItemId));
end;

procedure TsohoCustomDBComboBox.GenerateSelectSQL(Query: TStringList);
begin
   if ExtractFileExt(FTableName)<>'' then
     Query.Add('select ' + FKeyField + ', ' + FTextField + ' from "' + FTableName + '"')
   else
     Query.Add('select ' + FKeyField + ', ' + FTextField + ' from ' + FTableName);
end;

procedure TsohoCustomDBComboBox.PrepareList;
var Query: TStringList;
    DataSet : TDataSet;
    Reg  : TsohoCustomRegister;
    CanAdd : boolean;
begin
  DataSet := nil;
  Query := TStringList.Create;
  try
    Reg := nil;
    try
      Clear;
      with Query do begin
        Clear;
        if not FileExists(FSQLFile) then
          if SingleRegister<>nil then FSQLFile := SingleRegister.PathToSQL + FSQLFile;
        if FSQLFile <> '' then LoadFromFile(FSQLFile)
        else begin
          if FTableName = '' then begin
            ErrorMsg(Format(sohoCtmCmbActivateError, [Name]));
            Query.Free;
            exit;
          end;
          GenerateSelectSQL(Query);
        end;
        DataSet := ActivateSelection(Query);
        if DataSet = nil then begin
          Query.Free;
          exit;
        end;
        if DataSet.FindField('PROTECT')<>nil then Reg := SingleRegister;
        CanAdd := true;
        while not DataSet.EOF do begin
          {     }
          if Reg<>nil then
            CanAdd := (DataSet.FieldByName('PROTECT').AsInteger in Reg.DataReadRights);
          if CanAdd then AddItem(DataSet.FieldByName(FKeyField).AsInteger,
                                 DataSet.FieldByName(FTextField).AsString);
          DataSet.Next;
        end;
        DataSet.Close;
      end;
      FPrepared := True;
      if FAutoSelect then
        if (Items.Count > 0) and (ItemIndex = -1) then ItemID := IdItems[0];
    except
    end;
  finally
    Query.Free;
    if DataSet <> nil then DataSet.Free;
  end;
end;

constructor TsohoCustomDBComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTextField := 'Name';
  FKeyField := 'Id';
  FAutoInsert := false;
  FAutoSelect := true;
  FSQLFile := '';
  FTableName := '';
  FPrepared := False;
  Style := csDropDownList;
end;

procedure TsohoCustomDBComboBox.DropDown;
begin
  inherited DropDown;
end;

procedure TsohoCustomDBComboBox.Change;
var Index: LongInt;
begin
  inherited;
  Index := Items.IndexOf(Text);
  if Index <> -1 then begin
    ItemID := IdItems[Index];
    SelStart := Length(Text);
  end;
end;

end.

