{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{              Written by Igor Chernyshov,              }
{                    Serg Vostrikov                     }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{    TsohoCustomDBAssist,   
  sql-     ,   
    ,     . 
}
unit SoCtmDBAssist;

{$I SOHOLIB.INC}
interface
uses SysUtils, Classes, DB, SoCtrls, Controls;

type

  TGetFieldValueEvent = procedure (Sender: TObject;
    const FieldName: string; var FieldValue: string) of object;
{    OnGetValue. FieldName -   ,  
     .      FieldValue }

  TsohoCustomDBAssist = class(TComponent)
  private
    FDataSet       : TDataSet;
    FTableName     : string;
    FKeyField      : string;
    FKeyValue      : LongInt;
    FFieldsList    : TStringList;
    FEditsList     : TStringList;
    FNullIfDisable : boolean;
    FInsertKeyField: boolean;
    FOnGetValue    : TGetFieldValueEvent;
  protected
    FSQL: TStringList;

    procedure SetParams;
    procedure SetFieldsList(Value : TStringList);
    procedure SetEditsList (Value : TStringList);
    procedure SetParamByName(Control: TControl; const ParamName: string);

    // ,       
    // 
    procedure DoGetValue(const FieldName: string; var FieldValue: string); virtual;
    procedure Notification (aComponent: TComponent; aOperation: TOperation); override;
    procedure SetDataSet(Value : TDataSet); virtual;
    procedure ExecuteInsertSQL; virtual; abstract;
    procedure ExecuteUpdateSQL; virtual; abstract;
    procedure SetStrParam(const ParamName: string; const Value: string); virtual; abstract;
    procedure SetIntParam(const ParamName: string; const Value: LongInt); virtual; abstract;
    procedure SetFltParam(const ParamName: string; const Value: Double); virtual; abstract;
    function TypeRequest(Edit: TsohoCustomEdit): integer; virtual; abstract;
    procedure SetDateParam(const ParamName: string; const Value: TDate); virtual; abstract;
    procedure SetTimeParam(const ParamName: string; const Value: TTime); virtual; abstract;
    procedure SetDateTimeParam(const ParamName: string; const Value: TDateTime); virtual; abstract;
    procedure SetParamNull(const ParamName: string); virtual; abstract;
    function ParamExists(const ParamName: string): boolean; virtual; abstract;
    function GetTableName: string; virtual;
    function GetFieldName(const FieldName: string): string; virtual;
    procedure PrepareQuery; virtual; abstract;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ReadData; virtual;
    {  ,   FieldsList  DataSet   
        ,   EditsList }
    procedure InsertData; virtual;
    {         }
    procedure UpdateData; virtual;
    {         }
  published
    property DataSet    : TDataSet read FDataSet write SetDataSet;
    {   DataSet  }
    property TableName  : string         read FTableName write FTableName;
    {  .      insert  update }
    property KeyField   : string         read FKeyField write FKeyField;
    {  .     update }
    property KeyValue  : LongInt read FKeyValue write FKeyValue default -1;
    {   .    InsertData, 
       InsertKeyField = true }
    property FieldsList : TStringList    read FFieldsList write SetFieldsList;
    {  ,      }
    property EditsList  : TStringList    read FEditsList  write SetEditsList;
    {  ,     FieldsList }
    property NullIfDisable : boolean     read FNullIfDisable write FNullIfDisable default True;
    {  True,   Insert/Update   Null, 
          Disable}
    property InsertKeyField: boolean read FInsertKeyField write FInsertKeyField default false;
    {  InsertKeyField = true,    InsertData  sql-
         KeyField     KeyValue.  
      ,       , ,
              
          }
    property OnGetValue: TGetFieldValueEvent read FOnGetValue write FOnGetValue;
    {  ,      , 
           .     
               }
  end;

implementation
uses SoVCLUtils, SoDBCns, SoStrUtils, SohoSpin, SoChkBox,
     SoCtmCmb, StdCtrls, ComCtrls;

{ TsohoCustomDBAssist }

constructor TsohoCustomDBAssist.Create(AOwner: TComponent);
begin
  inherited;
  FFieldsList := TStringList.Create;
  FEditsList := TStringList.Create;
  FSQL := TStringList.Create;
  FNullIfDisable := true;
  FKeyValue := -1;
  FInsertKeyField := false;
end;

destructor TsohoCustomDBAssist.Destroy;
begin
  FFieldsList.Free;
  FEditsList.Free;
  FSQL.Free;
  inherited;
end;

procedure TsohoCustomDBAssist.DoGetValue(const FieldName: string;
  var FieldValue: string);
begin
  if Assigned(FOnGetValue) then FOnGetValue(Self, FieldName, FieldValue);
end;

function TsohoCustomDBAssist.GetTableName: string;
begin
  Result := FTableName;
end;

function TsohoCustomDBAssist.GetFieldName(const FieldName: string): string;
begin
  Result := FieldName;
end;

procedure TsohoCustomDBAssist.InsertData;
var Index: Integer;
     s, FieldName : string;
begin
  if FTableName = EmptyStr then begin
    ErrorMsg(Format(sohoDBAssistTableNameNotSet, [Self.Name]));
    Exit;
  end;
  //  
  FSql.Clear;
  FSql.Add('insert into ' + GetTableName + ' (');
  if InsertKeyField then
    FSql.Add(GetFieldName(KeyField));
  for Index := 0 to pred(FFieldsList.Count) do begin
    if trim(FFieldsList[Index]) <> EmptyStr then begin
      FieldName := GetFieldName(FFieldsList[Index]);
      if (Index > 0) or InsertKeyField then s := ',' + FieldName
      else s := FieldName;
      FSql.Add(S);
    end;
  end;
  FSql.Add(') VALUES (');

  if InsertKeyField then FSql.Add(IntToStr(KeyValue));

  for Index := 0 to pred(FFieldsList.Count) do begin
    if trim(FFieldsList[Index]) <> EmptyStr then begin
      FieldName := GetFieldName(FFieldsList[Index]);
      if (Index > 0) or InsertKeyField then s := ', ?' + FieldName
      else s := '?' + FieldName;
      FSql.Add(S);
    end;
  end;
  FSql.Add(')');

  PrepareQuery;
  SetParams;
  ExecuteInsertSQL;
end;

procedure TsohoCustomDBAssist.Notification(aComponent: TComponent;
  aOperation: TOperation);
begin
  inherited;
  if (aComponent = FDataSet) and (aOperation = opRemove) then
    FDataset := nil;
end;

procedure TsohoCustomDBAssist.ReadData;
var Index: LongInt;
    C: TComponent;
    FieldName: string;
begin
  for Index := 0 to pred(FFieldsList.Count) do begin
    if (trim(FFieldsList[Index]) <> EmptyStr) and
       (trim(FEditsList[Index]) <> EmptyStr) then begin

      C := Owner.FindComponent(trim(FEditsList[Index]));
      FieldName := trim(FFieldsList[Index]);

      if (c = nil) or (FDataSet.FieldByName(FieldName) = nil) then
        continue;

       if (c is TsohoCustomEdit) then (c as TsohoCustomEdit).Text :=
             FDataSet.FieldByName(FieldName).AsString;

       if (c is TsohoLabel) then (c as TsohoLabel).Caption :=
           FDataSet.FieldByName(FieldName).AsString;

       if (c is TsohoComboBox) then (c as TsohoComboBox).Text :=
           FDataSet.FieldByName(FieldName).AsString;

       if (c is TsohoSpinEdit) then (c as TsohoSpinEdit).Value :=
           FDataSet.FieldByName(FieldName).Value;

       if (c is TsohoCheckBox) then begin
          if FDataSet.FieldByName(FieldName).DataType = ftBoolean then
            (c as TsohoCheckBox).Checked :=
               FDataSet.FieldByName(FieldName).AsBoolean
          else
            (c as TsohoCheckBox).Checked :=
               FDataSet.FieldByName(FieldName).AsInteger <> 0;
       end;

       if (c is TsohoCustomDBComboBox) then
          (c as TsohoCustomDBComboBox).ItemID :=
           FDataSet.FieldByName(FieldName).AsInteger;

       if (c is TCustomMemo) then (c as TCustomMemo).Lines.Text :=
           FDataSet.FieldByName(FieldName).AsString;

       if (c is TCustomRichEdit) then (c as TCustomRichEdit).Lines.Text :=
           FDataSet.FieldByName(FieldName).AsString;

    end; {--- if ---}
  end;  {--- for ---}
end;

procedure TsohoCustomDBAssist.SetDataSet(Value: TDataSet);
begin
  FDataSet := Value;
end;

procedure TsohoCustomDBAssist.SetEditsList(Value: TStringList);
begin
  FEditsList.Assign(Value);
end;

procedure TsohoCustomDBAssist.SetFieldsList(Value: TStringList);
begin
  FFieldsList.Assign(Value);
end;

procedure TsohoCustomDBAssist.SetParamByName(Control: TControl;
  const ParamName: string);
var Edit: TsohoCustomEdit;
begin
  if (FNullIfDisable) and
     (not Control.Enabled) then SetParamNull(ParamName)
  else begin

    { TsohoCustomEdit handler }
    if (Control is TsohoCustomEdit) then begin
      Edit := Control as TsohoCustomEdit;
      case TypeRequest(Edit) of
        0 {String}: SetStrParam(ParamName, Edit.AsString);
        1 {Integer}: SetIntParam(ParamName, Edit.AsInteger);
        2 {Float}: SetFltParam(ParamName, Edit.AsFloat);
        3 {Date}: begin
          if Edit.AsDate = 0 then
            SetParamNull(ParamName)
          else
            SetDateParam(ParamName, Edit.AsDate);
        end;
        4 {Time}: begin
          if Edit.AsTime = 0 then
            SetParamNull(ParamName)
          else
            SetTimeParam(ParamName, Edit.AsTime);
        end;
        5 {DateTime}: begin
          if Edit.AsDateTime = 0 then
            SetParamNull(ParamName)
          else
            SetDateTimeParam(ParamName, Edit.AsDateTime);
        end;
      end;
    end;
    { TsohoCustomEdit handler }

    if (Control is TsohoComboBox) then begin
      if (Control is TsohoCustomDBComboBox) then
        SetIntParam(ParamName, (Control as TsohoCustomDBComboBox).ItemID)
      else
        SetStrParam(ParamName, (Control as TsohoComboBox).Text);
    end;

    if (Control is TsohoLabel) then
      SetStrParam(ParamName, (Control as TsohoLabel).Caption);

    if (Control is TsohoSpinEdit) then begin
      if (Control as TsohoSpinEdit).ValueType = vtInteger then
        SetIntParam(ParamName, (Control as TsohoSpinEdit).AsInteger)
      else
        SetFltParam(ParamName, (Control as TsohoSpinEdit).Value);
    end;

    if (Control is TsohoCheckBox) then begin
      if (Control as TsohoCheckBox).Checked then
        SetIntParam(ParamName, 1)
      else
        SetIntParam(ParamName, 0);
    end; {--- if ---}


    {if (Control is TsohoCustomDBComboBox) then
       SetIntParam(ParamName, (Control as TsohoCustomDBComboBox).ItemId);}

    if (Control is TCustomMemo) then
      SetStrParam(ParamName, (Control as TCustomMemo).Lines.Text);

    if (Control is TCustomRichEdit) then
      SetStrParam(ParamName, (Control as TCustomRichEdit).Lines.Text);
  end;
end;

procedure TsohoCustomDBAssist.SetParams;
var Index: LongInt;
    C: TComponent;
    FieldName, FieldValue: string;
begin
  for Index := 0 to pred(FEditsList.Count) do begin
    if trim(FEditsList[Index]) <> EmptyStr then begin
      FieldName := trim(FFieldsList[Index]);
      if (Index > pred(FEditsList.Count)) or 
         (FEditsList[Index] = sohoDBAssistSetManually) then begin
        FieldValue := EmptyStr;
        DoGetValue(FieldName, FieldValue);
        SetStrParam(FieldName, FieldValue);
      end
      else begin
        c := Owner.FindComponent(FEditsList[Index]);
        if (c = nil) or (not ParamExists(FieldName)) then Continue;
        SetParamByName(TControl(C), FieldName);
      end;
    end;
  end;
end;

procedure TsohoCustomDBAssist.UpdateData;
var Index: LongInt;
    FieldName, s: string;
begin
  if FKeyField = EmptyStr then begin
    ErrorMsg(Format(sohoDBAssistKeyFieldNotSet, [Self.name]));
    Exit;
  end;

  if FTableName = EmptyStr then begin
    ErrorMsg(Format(sohoDBAssistTableNameNotSet, [Self.name]));
    Exit;
  end;

  //  
  FSql.Clear;
  FSql.Add('update ' + GetTableName + ' set');
  for Index := 0 to pred(FFieldsList.Count) do begin
    if trim(FFieldsList[Index]) <> EmptyStr then begin
      FieldName := GetFieldName(FFieldsList[Index]);
      s := FieldName + ' = ?' + FieldName;
      if Index <> pred(FFieldsList.Count) then s := s + ',';
      FSql.Add(S);
    end;
  end;
  FSql.Add('WHERE ' +
    GetFieldName(FKeyField) + ' = ' +
      FDataSet.FieldByName(FKeyField).AsString);

  PrepareQuery;
  SetParams;
  ExecuteUpdateSQL;
end;

end.
