{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{             Written by Igor Chernyshov,               }
{                   Serg Vostrikov                      }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoCtmDBGridAddon;

{$I SOHOLIB.INC}
{.$DEFINE USE_EHLIB}
interface

uses Windows, Graphics, SysUtils, Classes, DB, DBGrids,
     {$IFNDEF USE_EHLIB} SoDBGrid{$ELSE}DBGridEh, Grids{$ENDIF},
     SoFrmStorage, Menus, Controls;

type

  PsohoFilterData = ^TsohoFilterData;
  TsohoFilterData = record
    RecordCount: LongInt;
    FilterMode: LongInt; // 0 - no filter, 1 - first 10, 2 - auto filter
    Cond1, Cond2: integer;
    IsAnd: boolean;
    Value1, Value2: string;
    FieldName: string;
    CaseSens: boolean;
  end;

  TsohoCustomDBGridAddon = class(TComponent)
{     TsohoDBGrid.    , 
  ,      .  TsohoCustomDBGridAddon
   - DB Engine,        .  TsohoCustomDBGridAddon,
  DBE,  TsohoDBGridAddon
}
  private
    FDataset        : TDataSet;
    {$IFNDEF USE_EHLIB}
    FGrid           : TsohoDBGrid;
    {$ELSE}
    FGrid           : TDBGridEh;
    {$ENDIF}
    FOldFieldName   : String;
    FOldFieldNo     : LongInt;
    {$IFNDEF USE_EHLIB}
    FSortMarker     : TSortMarker;
    {$ELSE}
    FSortMarker     : TSortMarkerEh;
    {$ENDIF}
    AscSortOrder    : boolean;
    FAllowSorting   : boolean;
    FAllowFiltering : boolean;
    FActive         : boolean;
    FSortByFieldNo  : boolean;
    FFieldNames     : TStringList;
    FPopup          : TPopupMenu;
    FCurField       : TField;
    FTurnOffFilter  : boolean;
    {  }
    FFilterFields   : TStringList;
    FOldFilterRecord: TFilterRecordEvent;
    FBeforeOperation,
    FAfterOperation: TNotifyEvent;

    procedure SetFieldNames(Value : TStringList);
    procedure ClearFilterFields;
    procedure SetAllowSorting(const Value: Boolean);
    procedure SetAllowFiltering(const Value: boolean);
    procedure SetActive(const Value: boolean);
  protected
    procedure MenuItemClicked(Sender: TObject);
    procedure CreateMenu; virtual;
    procedure MenuPopup(Sender: TObject);
    procedure DoFilterRecord (DataSet: TDataSet; var Accept: Boolean);
    function  CheckFilter(DataSet: TDataSet; Index: LongInt): boolean;
    procedure RestoreEvents;
    procedure ResetRecordCounts;
    procedure FullRefreshDataSetFilter; virtual;

    procedure DoBeforeOperation; virtual;
    procedure DoAfterOperation; virtual;

    {$IFNDEF USE_EHLIB}
    procedure SetGrid(Value: TsohoDBGrid);
    procedure DBGridTitleBtnClick(Sender: TObject; ACol: Integer; Field: TField);
    procedure DBGridGetBtnParams (Sender: TObject; Field: TField;
              AFont: TFont; var Background: TColor; var SortMarker: TSortMarker;
              IsDown: Boolean);
    {$ELSE}
    procedure SetGrid(Value: TDBGridEh);
    procedure DBGridTitleBtnClick(Sender: TObject; ACol: Integer;
              Column: TColumnEh);
    procedure DBGridGetBtnParams (Sender: TObject;
              Column: TColumnEh; AFont: TFont; var Background: TColor;
              var SortMarker: TSortMarkerEh; IsDown: Boolean);
    procedure SetTitleButtons(Value: boolean);
    {$ENDIF}
    procedure Notification (aComponent: TComponent; aOperation: TOperation); override;
    { ,        }
    function  ValidDataSet : boolean; virtual; abstract;
    {  ORDER BY }
    procedure ChangeOrderBy (FieldNo : integer; const FieldName : string; AscSortOrder : Boolean); virtual; abstract;
    {   }
    procedure NoSort; virtual;
    {       (TField.FieldName)  
       (TField.FieldNo).     FieldNames.  , 
      select    ,       .
    ,    :
    ---
    SELECT MAIN_PROV.NAME, SPR_PCH.NAME, MEROPR_GRP.NAME
    FROM   MAIN_PROV, SPR_PCH, MEROPR_GRP
    WHERE  MAIN_PROV.KOD_PCH     = SPR_PCH.ID AND
           MAIN_PROV.IDMEROPRGRP = MEROPR_GRP.ID
    ---
          ,  SortByFieldNo = true
      FieldNames       :
    MAIN_PROV.NAME
    SPR_PCH.NAME
    MEROPR_GRP.NAME
    }
    property  SortByFieldNo : boolean read FSortByFieldNo write FSortByFieldNo default False;
    {     .   . SortByFieldNo }
    property  FieldNames    : TStringList read FFieldNames write SetFieldNames;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure Loaded; override;
    procedure SetEvents;
    procedure SetDataSet;

    procedure SaveStatement(Storage: TsohoFormStorage);
    {     ini- }
    procedure LoadStatement(Storage: TsohoFormStorage);
    {     ini- }

    property  AllowSorting  : Boolean read FAllowSorting write SetAllowSorting default true;
    {   }
    property AllowFiltering : boolean read FAllowFiltering write SetAllowFiltering default true;
    {       }
    property Active: boolean read FActive write SetActive default false;
    {   Active,    }

    procedure DoSorting(FieldNumber: integer; const FieldName: string;
      AscSort: boolean); virtual;
    {      (FieldNumber)    FieldName, 
               ( AscSort = true) }
    property SortField: string read FOldFieldName;
    {  ,      }
    property SortFieldNo: LongInt read FOldFieldNo;
    {  ,      }
    property TurnOffFilter : boolean read FTurnOffFilter write FTurnOffFilter default true;
    {  -        TurnOffFilter = true
        Filtering  DataSet    false }
    property BeforeOperation: TNotifyEvent read FBeforeOperation write FBeforeOperation;
    {        }
    property AfterOperation: TNotifyEvent read FAfterOperation write FAfterOperation;
    {         }
  published
    {$IFNDEF USE_EHLIB}
    property Grid: TsohoDBGrid read FGrid write SetGrid;
    {$ELSE}
    property Grid: TDBGridEh read FGrid write SetGrid;
    {$ENDIF}
    {   ,      }
  end;

implementation
uses SoDBCns, SoDBRtn, SoFltForm, SoUtils, SoDate, SoStrUtils,
     SoWldStr{$IFDEF Delphi4_or_5}, ImgList{$ENDIF};

{$R SoCtmDBGridAddon.R32}

{$IFDEF Delphi4_or_5}
var Images: TImageList;
{$ENDIF}

procedure ClearFilterData(var Value: TsohoFilterData);
begin
  with Value do begin
    RecordCount := 0;
    FilterMode := 0;
    Cond1 := 0;
    Cond2 := 0;
    IsAnd := true;
    Value1 := EmptyStr;
    Value2 := EmptyStr;
    CaseSens := false;
  end;
end;

//------------------------------------------------------------------------------
// PRIVATE
//------------------------------------------------------------------------------
procedure TsohoCustomDBGridAddon.SetDataSet;
begin
  //      DataSet
  if (FGrid <> nil) and (FGrid.DataSource <> nil) then
    FDataSet := FGrid.DataSource.DataSet
  else FDataSet := nil;
end;

procedure TsohoCustomDBGridAddon.SetFieldNames(Value : TStringList);
begin
  FFieldNames.Assign(Value);
end;

{$IFNDEF USE_EHLIB}
procedure TsohoCustomDBGridAddon.SetGrid(Value: TsohoDBGrid);
{$ELSE}
procedure TsohoCustomDBGridAddon.SetGrid(Value: TDBGridEh);
{$ENDIF}
begin
  FGrid := Value;
  FGrid.OnTitleBtnClick := DBGridTitleBtnClick;
  FGrid.OnGetBtnParams := DBGridGetBtnParams;
  SetDataSet;
end;

{$IFDEF USE_EHLIB}
procedure TsohoCustomDBGridAddon.SetTitleButtons(Value: boolean);
var Index: LongInt;
begin
  for Index := 0 to pred(FGrid.Columns.Count) do
    FGrid.Columns[Index].Title.TitleButton := Value;
end;
{$ENDIF}


procedure TsohoCustomDBGridAddon.DoSorting(FieldNumber: integer; const FieldName: string;
  AscSort: boolean);
var s: string;
    i: integer;
begin
  if not (
     FAllowSorting and
     ValidDataSet and
     (FieldName <> EmptyStr) and
     (FieldNumber <> -1) and
     (FDataSet <> nil) and
     (FDataSet.FindField(FieldName) <> nil)) then exit;

  AscSortOrder := AscSort;

  {$IFNDEF USE_EHLIB}
  if AscSortOrder then FSortMarker := smDown
  else FSortMarker := smUp;
  {$ELSE}
  if AscSortOrder then FSortMarker := smDownEh
  else FSortMarker := smUpEh;
  {$ENDIF}

  FOldFieldName := FieldName;
  FOldFieldNo := FieldNumber;

  //   ORDER BY
  try
    if FSortByFieldNo then S := FFieldNames[FieldNumber - 1]
    else S := FieldName;

    I := FGrid.LeftCol;
    DoBeforeOperation;

    FGrid.DataSource.DataSet.DisableControls;
    ChangeOrderBy(FieldNumber, S, AscSortOrder);
    FGrid.LeftCol := i;
    FGrid.DataSource.DataSet.EnableControls;

    DoAfterOperation;
  except
  end;
end;

{$IFNDEF USE_EHLIB}
procedure TsohoCustomDBGridAddon.DBGridTitleBtnClick(Sender: TObject;
  ACol: Integer; Field: TField);
{$ELSE}
procedure TsohoCustomDBGridAddon.DBGridTitleBtnClick(Sender: TObject;
  ACol: Integer; Column: TColumnEh);
{$ENDIF}
var Pos: TPoint;
    R: TRect;
   {$IFDEF USE_EHLIB}
    Coord: TGridCoord;
   {$ELSE}
    tCol, tRow: LongInt;
   {$ENDIF}
begin
  {$IFNDEF USE_EHLIB}
  FCurField := Field;
  {$ELSE}
  FCurField := Column.Field;
  {$ENDIF}

  inherited;
  if not Assigned(FGrid) then exit;

  GetCursorPos(Pos);
  Pos := FGrid.ScreenToClient(Pos);
  {$IFNDEF USE_EHLIB}
  FGrid.MouseToCell(Pos.X, Pos.Y, tCol, tRow);
  R := FGrid.GetCellRect(tCol, tRow);
  {$ELSE}
  Coord := FGrid.MouseCoord(Pos.X, Pos.Y);
  R := FGrid.CellRect(Coord.X, Coord.Y);
  {$ENDIF}
  Pos := FGrid.ClientToScreen(Point(R.Left, R.Bottom + 1));
  FPopup.Popup(Pos.X, Pos.Y);
end;

{$IFNDEF USE_EHLIB}
procedure TsohoCustomDBGridAddon.DBGridGetBtnParams (Sender: TObject; Field: TField;
   AFont: TFont; var Background: TColor; var SortMarker: TSortMarker;
   IsDown: Boolean);
{$ELSE}
procedure TsohoCustomDBGridAddon.DBGridGetBtnParams (Sender: TObject;
   Column: TColumnEh; AFont: TFont; var Background: TColor;
   var SortMarker: TSortMarkerEh; IsDown: Boolean);
{$ENDIF}
begin
  {$IFNDEF USE_EHLIB}
  if Field.FieldName = FOldFieldName then SortMarker := FSortMarker
  else SortMarker := smNone;
  {$ELSE}
  if Column.Field.FieldName = FOldFieldName then SortMarker := FSortMarker
  else SortMarker := smNoneEh;
  {$ENDIF}

  {$IFNDEF USE_EHLIB}
  if FFilterFields.IndexOf(Field.FieldName) <> -1 then begin
  {$ELSE}
  if FFilterFields.IndexOf(Column.Field.FieldName) <> -1 then begin
  {$ENDIF}
    AFont.Color := clWIndow;
    Background := clBtnShadow;
  end;
  inherited;
end;

procedure TsohoCustomDBGridAddon.Notification (aComponent: TComponent; aOperation: TOperation);
begin
  inherited;
  if (aComponent = FGrid) and (aOperation = opRemove) then begin
    FGrid := nil;
    FDataset := nil;
  End; {--- if ---}
end;

//------------------------------------------------------------------------------
// PUBLIC
//------------------------------------------------------------------------------
constructor TsohoCustomDBGridAddon.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPopup         := TPopupMenu.Create(Self);
  FFieldNames    := TStringList.Create;
  FFilterFields  := TStringList.Create;
  FOldFieldName  := EmptyStr;
  AscSortOrder   := True;
  FAllowSorting  := true;
  FActive        := true;
  FAllowFiltering := true;
  FSortByFieldNo := False;
  FTurnOffFilter := true;
  CreateMenu;
end;

procedure TsohoCustomDBGridAddon.ClearFilterFields;
var Index: LongInt;
begin
  for Index := 0 to pred(FFilterFields.Count) do
    FreeMem(PsohoFilterData(FFilterFields.Objects[Index]));
  FFilterFields.Clear;
end;

destructor TsohoCustomDBGridAddon.Destroy;
begin
  RestoreEvents;
  FFieldNames.Free;
  FPopup.Free;
  ClearFilterFields;
  FFilterFields.Free;
  inherited;
end;

procedure TsohoCustomDBGridAddon.LoadStatement(Storage: TsohoFormStorage);
begin
  with Storage do begin
    DoSorting(
      ReadInteger(Owner.ClassName + Name, 'Sort Field No', -1),
      ReadString(Owner.ClassName + Name, 'Sort Field', ''),
      ReadBool(Owner.ClassName + Name, 'Asc Sort', true)
      );
  end;    // with
end;

procedure TsohoCustomDBGridAddon.SaveStatement(Storage: TsohoFormStorage);
begin
  with Storage do begin
    WriteInteger(Owner.ClassName + Name, 'Sort Field No', SortFieldNo);
    WriteString(Owner.ClassName + Name, 'Sort Field', SortField);
    WriteBool(Owner.ClassName + Name, 'Asc Sort', AscSortOrder);
  end;    // with
end;

procedure TsohoCustomDBGridAddon.FullRefreshDataSetFilter;
begin
  with FGrid.DataSource.DataSet do begin
    DoBeforeOperation;
    DisableControls;
    try
      Filtered := false;
      RefreshDataSet(FGrid.DataSource.DataSet);
      Filtered := true;
    except
    end;
    EnableControls;
    DoAfterOperation;
  end;
end;

procedure TsohoCustomDBGridAddon.MenuItemClicked(Sender: TObject);
var I: LongInt;
    Found: PsohoFilterData;
    Tmp: TsohoFilterData;

    function CheckFilterData(const aFieldName: string): PsohoFilterData;
    var Index: LongInt;
    begin
      Index := FFilterFields.IndexOf(aFieldName);
      if Index = -1 then begin
        New(Result);
        ClearFilterData(Result^);
        Result^.FieldName := aFieldName;
        FFilterFields.AddObject(aFieldName, TObject(Result));
      end
      else Result := PsohoFilterData(FFilterFields.Objects[Index]);
    end;
begin
  if FCurField = nil then exit;
  case (Sender as TMenuItem).Tag of
    0: DoSorting(FCurField.Index, FCurField.FieldName, true);
    1: DoSorting(FCurField.Index, FCurField.FieldName, false);
    2: begin
      I := FGrid.LeftCol;
      DoBeforeOperation;

      FGrid.DataSource.DataSet.DisableControls;
      NoSort;
      FGrid.LeftCol := i;
      FGrid.DataSource.DataSet.EnableControls;

      DoAfterOperation;
    end;
    3: begin
      I := FFilterFields.IndexOf(FCurField.FieldName);
      if I = -1 then exit;
      FreeMem(PsohoFilterData(FFilterFields.Objects[I]));
      FFilterFields.Delete(I);
      FullRefreshDataSetFilter;
      if (FFilterFields.Count = 0) and
         FTurnOffFilter then FDataSet.Filtered := false;
    end; //   
    4: begin
      Found := CheckFilterData(FCurField.FieldName);
      Found.FilterMode := 1;
      Found.RecordCount := 0;
      FullRefreshDataSetFilter;
    end;
    5: begin
      I := FFilterFields.IndexOf(FCurField.FieldName);
      ClearFilterData(Tmp);
      Found := nil;
      if I <> -1 then begin
        Found := CheckFilterData(FCurField.FieldName);
        Tmp := Found^;
      end;

      with Tmp do
        if GetFilterConditions (Cond1, Cond2, IsAnd, CaseSens, Value1,
          Value2) then begin
            if I <> -1 then Found^ := Tmp
            else begin
              Found := CheckFilterData(FCurField.FieldName);
              Found^ := Tmp;
            end;
          Found^.FilterMode := 2;
          Found^.FieldName := FCurField.FieldName;
        end
        else exit;
      FullRefreshDataSetFilter;
    end
    else ;
  end;
end;

procedure TsohoCustomDBGridAddon.CreateMenu;

  function AddItem(const aCaption: string; aTag: LongInt;
    aGroupIndex: LongInt; aRadioItem: boolean): TMenuItem;
  begin
    Result := NewItem(aCaption, 0, false, true, MenuItemClicked, 0, '');
    Result.Tag := aTag;
    Result.RadioItem := aRadioItem;
    FPopup.Items.Add(Result);
  end;

begin
  AddItem(sohoDBAddonAscending, 0, 1, true);
  AddItem(sohoDBAddonDescending, 1, 1, true);
  AddItem(sohoDBAddonNoSort, 2, 1, true);
  AddItem('-', -1, 0, false);
  AddItem(sohoDBAddonAll, 3, 0, false);
  AddItem(sohoDBAddon10, 4, 0, false);
  AddItem(sohoDBAddonFilter, 5, 0, false);
  {$IFDEF Delphi4_or_5}
  FPopup.Images := Images;
  FPopup.Items[4].ImageIndex := 0;
  FPopup.Items[6].ImageIndex := 1;
  {$ENDIF}

  FPopup.OnPopup := MenuPopup;
end;

procedure TsohoCustomDBGridAddon.MenuPopup(Sender: TObject);
var SortEnabled: boolean;
    Index: LongInt;
    CorrectType: boolean;
begin
  for Index := 0 to pred(FPopup.Items.Count) do
    FPopup.Items[Index].Enabled := false;

  if (not Assigned(FGrid)) or
     {$IFNDEF USE_EHLIB}
     (not FGrid.DataLink.Active)
     {$ELSE}
     (FGrid.DataSource = nil) or
     (FGrid.DataSource.DataSet = nil) or
     (not FGrid.DataSource.DataSet.Active)
     {$ENDIF}
     or (FCurField = nil) then exit;


  if FOldFieldName = FCurField.FieldName then begin
    if AscSortOrder then
      FPopup.Items[0].Checked := true
    else
      FPopup.Items[1].Checked := true
  end
  else FPopup.Items[2].Checked := true;
  SortEnabled := FAllowSorting and ValidDataSet;
  
  FPopup.Items[0].Enabled := SortEnabled;
  FPopup.Items[1].Enabled := SortEnabled;
  FPopup.Items[2].Enabled := SortEnabled;

  Index := FFilterFields.IndexOf(FCurField.FieldName);
  CorrectType := FAllowFiltering and (FCurField.DataType in
    [ftString, ftSmallint, ftInteger, ftWord, ftBoolean,
     ftFloat, ftCurrency, ftDate, ftTime, ftDateTime 
{$IFDEF Delphi4_or_5}
          ,ftWideString, ftLargeint
{$ENDIF}
]);

  FPopup.Items[4].Enabled := CorrectType and (Index <> -1);
  FPopup.Items[5].Enabled := CorrectType and (Index = -1);
  FPopup.Items[6].Enabled := CorrectType and ((Index = -1) or
    (PsohoFilterData(FFilterFields.Objects[Index])^.FilterMode = 2));
end;

procedure TsohoCustomDBGridAddon.NoSort;
begin
  {$IFNDEF USE_EHLIB}
  FSortMarker := smNone;
  {$ELSE}
  FSortMarker := smNoneEh;
  {$ENDIF}
  FOldFieldName := EmptyStr;
end;

procedure TsohoCustomDBGridAddon.SetEvents;
begin
  if csDesigning in ComponentState then exit;
  if Assigned(FGrid) and
     Assigned(FGrid.DataSource) and
     Assigned(FGrid.DataSource.DataSet) then begin
     FOldFilterRecord := FGrid.DataSource.DataSet.OnFilterRecord;
     FGrid.DataSource.DataSet.OnFilterRecord := DoFilterRecord;
  end;
end;

procedure TsohoCustomDBGridAddon.Loaded;
begin
  inherited;
  SetEvents;
  SetDataSet;
end;

function TsohoCustomDBGridAddon.CheckFilter(DataSet: TDataSet; Index: LongInt): boolean;

  function CheckValue(const aFieldName, Value: string;
    Operation: longInt; CaseSens: boolean): boolean;
  var Error: boolean;
      aValue, aString: string;
  begin
    aValue := Value;
    aString := DataSet.FieldByName(aFieldName).AsString;
    if not CaseSens then begin
      aString := AnsiUpperCase(DataSet.FieldByName(aFieldName).AsString);
      aValue := AnsiUpperCase(aValue);
    end;
    with DataSet.FieldByName(aFieldName) do
      case Operation of
        1: Result := WildStringCompare(aValue, aString);
        2: Result := not WildStringCompare(aValue, aString);
        3: case DataType of
              ftSmallint, ftInteger, ftWord{$IFDEF Delphi4_or_5}, ftLargeint{$ENDIF}:
                Result := AsInteger > StrToIntDef(aValue, 0);
              ftDate, ftTime, ftDateTime:
                Result := AsDateTime > WStrToDateTime(aValue);
              ftFloat, ftCurrency:
                Result := AsFloat > WStrToFloat(aValue, Error);
              else Result := aString > aValue;
           end;
        4: case DataType of
              ftSmallint, ftInteger, ftWord{$IFDEF Delphi4_or_5}, ftLargeint{$ENDIF}:
                 Result := AsInteger >= StrToIntDef(aValue, 0);
              ftDate, ftTime, ftDateTime:
                Result := AsDateTime >= WStrToDateTime(aValue);
              ftFloat, ftCurrency:
                Result := AsFloat >= WStrToFloat(aValue, Error);
              else Result := aString >= aValue;
           end;
        5: case DataType of
              ftSmallint, ftInteger, ftWord{$IFDEF Delphi4_or_5}, ftLargeint{$ENDIF}:
                Result := AsInteger < StrToIntDef(aValue, 0);
              ftDate, ftTime, ftDateTime:
                Result := AsDateTime < WStrToDateTime(aValue);
              ftFloat, ftCurrency:
                Result := AsFloat < WStrToFloat(aValue, Error);
              else Result := aString < aValue;
           end;
        6: case DataType of
              ftSmallint, ftInteger, ftWord{$IFDEF Delphi4_or_5}, ftLargeint{$ENDIF}:
                Result := AsInteger <= StrToIntDef(aValue, 0);
              ftDate, ftTime, ftDateTime:
                Result := AsDateTime <= WStrToDateTime(aValue);
              ftFloat, ftCurrency:
                Result := AsFloat <= WStrToFloat(aValue, Error);
              else Result := aString <= aValue;
            end;
        7: Result := Pos(aValue, aString) = 1;
        8: Result := Pos(aValue, aString) <> 1;
        9: Result := Copy(aString, length(aString)-
          length(aValue) + 1, length(aValue)) = aValue;
        10: Result := Copy(aString, length(aString)-
          length(aValue) + 1, length(aValue)) <> aValue;
        11: Result := Pos(aValue, aString) <> 0;
        12: Result := Pos(aValue, aString) = 0;
        else Result := true;
      end;
  end;

begin
  with PsohoFilterData(FFilterFields.Objects[Index])^ do begin
    case FilterMode of
      // 0 - no filter, 1 - first 10, 2 - auto filter
      0: Result := true;
      1: begin
        inc(RecordCount);
        Result := RecordCount <= 10;
      end;
      else begin
        Result := CheckValue(FieldName, Value1, Cond1, CaseSens);
        if Cond2 <> 0 then begin
          if IsAnd then
            Result := Result and CheckValue(FieldName, Value2, Cond2, CaseSens)
          else
            Result := Result or CheckValue(FieldName, Value2, Cond2, CaseSens);
        end;
      end;
    end;
  end;
end;

procedure TsohoCustomDBGridAddon.DoFilterRecord(DataSet: TDataSet;
  var Accept: Boolean);
var Index: LongInt;
begin
  if Assigned(FOldFilterRecord) then FOldFilterRecord(DataSet, Accept);
  if not Accept then exit;

  for Index := 0 to pred(FFilterFields.Count) do begin
    Accept := CheckFilter(DataSet, Index);
    if not Accept then exit;
  end;
end;

procedure TsohoCustomDBGridAddon.RestoreEvents;
begin
  if Assigned(FGrid) and
     Assigned(FGrid.DataSource) and
     Assigned(FGrid.DataSource.DataSet) then
     FGrid.DataSource.DataSet.OnFilterRecord := FOldFilterRecord;
end;

procedure TsohoCustomDBGridAddon.SetAllowSorting(const Value: Boolean);
begin
  FAllowSorting := Value;
end;

procedure TsohoCustomDBGridAddon.ResetRecordCounts;
var Index: LongInt;
begin
  for Index := 0 to pred(FFilterFields.Count) do
    PsohoFilterData(FFilterFields.Objects[Index])^.RecordCount := 0;
end;

procedure TsohoCustomDBGridAddon.SetAllowFiltering(const Value: boolean);
begin
  FAllowFiltering := Value;
  if FFilterFields.Count > 0 then begin
    ClearFilterFields;
    FullRefreshDataSetFilter;
  end;
end;

procedure TsohoCustomDBGridAddon.DoAfterOperation;
begin
  {$IFDEF USE_EHLIB}
  SetTitleButtons(FActive);
  {$ENDIF}
  if Assigned(FAfterOperation) then FAfterOperation(Self);
end;

procedure TsohoCustomDBGridAddon.DoBeforeOperation;
begin
  if Assigned(FBeforeOperation) then FBeforeOperation(Self);
end;

procedure TsohoCustomDBGridAddon.SetActive(const Value: boolean);
begin
  FActive := Value;
  {$IFNDEF USE_EHLIB}
  if Assigned(FGrid) then FGrid.TitleButtons := Value;
  {$ELSE}
  if Assigned(FGrid) then SetTitleButtons(Value);
  {$ENDIF}
end;

{$IFDEF Delphi4_or_5}
initialization
 Images := TImageList.Create(nil);
 Images.ResourceLoad(rtBitmap, 'SOHOFILTER', clAqua);
finalization
 Images.Free;
{$ENDIF}
end.
