{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{           Partial copyright (c) AllexSoft             }
{           Written by Alexander Govyadinov,            }
{                   Serg Vostrikov                      }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
    TsohoCustomQuickSearch
}
unit SoCtmQSrch;

{$I SOHOLIB.INC}

interface

uses SysUtils, Windows, Messages, Classes, Graphics, Controls,
     Forms, DB, SoCtrls;

type

  PIndex = ^TIndex;
  TIndex = array[0..100] of PString; {   }

  PIDIndex = ^TIDIndex;
  TIDIndex = array[0..100] of LongInt;{   }

  EQuickSearch = class(Exception);{  }

  TsohoCustomQuickSearch = class(TsohoEdit)
  {         DataSet. 
     "", ,        .     
,        DataSet.  ,   
  " "  ,     ,  . 
   (   FieldList  ..)     Activate.  
. ,     ,      DataSet.  
     :
      =     
      =  .  
      =    
     =   
             
    ,           , 
        . ,   ,  
     DataSet.
  }
  private
    { Private declarations }
    Locked: Boolean; { , }
    BookMark:TBookMark; {   }
    IndexMark:LongInt;  {  }
    FActive       : boolean;
    FAutoSearch   : boolean;
    FControl      : TWinControl;
    FDataSet      : TDataSet;
    FFieldsList   : TStrings;
    FAfterOpen    : TDataSetNotifyEvent;
    FAfterPost    : TDataSetNotifyEvent;
    FAfterClose   : TDataSetNotifyEvent;
    FBeforeDelete : TDataSetNotifyEvent;
    FKeyField     : string;
  protected
    { Protected declarations }
    SearchSuccess : boolean;
    FIndex     : PIndex;
    FIDIndex   : PIDIndex;
    FIndexSize : LongInt;
    FIndexNumb : LongInt;
    procedure EraseLastSymbol;
    procedure Search(Direct : ShortInt);
    procedure SetActive(Value : boolean);
    procedure SetFieldsList(Value: TStrings);
    procedure DoAfterOpen(DataSet:TDataSet);
    procedure DoAfterPost(DataSet:TDataSet);
    procedure DoAfterClose(DataSet:TDataSet);
    procedure DoBeforeDelete(DataSet:TDataSet);
    procedure ActivateEvents;
    procedure DeActivateEvents;
    procedure InternalSearch(IndexNumb: LongInt); virtual;
    procedure LoadData;
    procedure FreeData;
    procedure Test;
    function FindID(DataSet : TDataSet) : LongInt;
    function GetIndex : string;
  public
    procedure Change; override;
    procedure KeyDown (var Key: Word; Shift: TShiftState); override;
    procedure MouseUp (Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

    procedure Activate;
    {       }
    procedure DeActivate;
    {  ""  DataSet. ,  ,     DataSet }
    procedure GotoNext;
    {    ,    }
    procedure GotoPrior;
    {    ,    }
    procedure GotoFirst;
    {    ,    }
    procedure GotoLast;
    {    ,    }
    procedure SavePosition;
    {     DataSet }
    procedure RestorePosition;
    {    DataSet }
  published
    property Active     : boolean read FActive write SetActive;
    { / . .  Activate/Deactivate }
    property AutoSearch : boolean read FAutoSearch write FAutoSearch default true;
    {   ,    
         ,  
       }
    property Control    : TWinControl read FControl write FControl;
    { ,     
          Enter  Esc }
    property DataSet    : TDataSet read FDataSet write FDataSet;
    {   }
    property FieldsList : TStrings read FFieldsList write SetFieldsList;
    {  ,      }
    property KeyField : string read FKeyField write FKeyField;
    {  ,     .   Id }
  end;

implementation
uses SoStrUtils, SoDBCns;

constructor TsohoCustomQuickSearch.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FFieldsList := TStringList.Create;
  FIndexSize := 0;
  FIndex := nil;
  FIDIndex := nil;
  Locked := False;
  BookMark := nil;
  FKeyField := 'Id';
  FAutoSearch := true;
end;

procedure TsohoCustomQuickSearch.SavePosition;
begin
  if Assigned(BookMark) and
     Assigned(FDataSet) then FDataSet.FreeBookMark(BookMark);
  if Assigned(FDataSet) then BookMark := FDataSet.GetBookMark;
  IndexMark := FIndexNumb;
end;

procedure TsohoCustomQuickSearch.RestorePosition;
begin
 if Assigned(BookMark) then
    try
     FDataSet.GotoBookMark(BookMArk);
     FIndexNumb := IndexMark;
    except
     GotoFirst;
    end
  else GotoFirst;
end;

destructor TsohoCustomQuickSearch.Destroy;
begin
  FFieldsList.Free;
  If Assigned(BookMark) then FDataSet.FreeBookMark(BookMark);
  if FActive then DeActivate;
  inherited Destroy;
end;

procedure TsohoCustomQuickSearch.Activate;
begin
  if FActive then Exit;
  if not Assigned(FDataSet) then
    raise EQuickSearch.Create(sohoQSActivateError);
  ActivateEvents;
  if FDataSet.Active then LoadData;
  FActive := True;
end;

procedure TsohoCustomQuickSearch.DeActivate;
begin
  if not FActive then exit;
  DeActivateEvents;
  FreeData;
  FActive := False;
  Text := EmptyStr;
end;

procedure TsohoCustomQuickSearch.GotoNext;
begin {        }
  Search(1);
end;

procedure TsohoCustomQuickSearch.GotoPrior;
begin
  Search(-1);
end;

procedure TsohoCustomQuickSearch.GotoFirst;
begin
  FIndexNumb := -1;
  Search(1);
end;

procedure TsohoCustomQuickSearch.GotoLast;
begin
  FIndexNumb := FIndexSize;
  Search(-1);
end;

procedure TsohoCustomQuickSearch.Change;
begin
  if Locked then Exit;
  inherited Change;
  if not AutoSearch then Exit;
  SavePosition;
  SearchSuccess := True;
  GotoFirst;
  if not SearchSuccess then begin
   RestorePosition;
   EraseLastSymbol;
  end;
end;

procedure TsohoCustomQuickSearch.KeyDown(var Key: Word; Shift: TShiftState);
var wKey: word;
begin
  wKey := Key;
  inherited KeyDown(Key, Shift);
  Key := 0;
  SearchSuccess := True;
  case wKey of
    vk_Down: begin
       GotoNext;
       if not SearchSuccess then GotoFirst;
    end;
    vk_Up: begin
      GotoPrior;
      if not SearchSuccess then GotoLast;
    end;
    vk_Left: Text := EmptyStr;
    vk_Home: GotoFirst;
    vk_End: GotoLast;
    vk_Return,vk_Escape: if Assigned(FControl) then begin
      SavePosition;
      Control.SetFocus;
    end;
    else Key := wKey;
  end;
end;

procedure TsohoCustomQuickSearch.EraseLastSymbol;
begin
  Locked := True;
  Text := Copy(Text, 1, Length(Text) - 1);
  SelStart := Length(Text);
  SelLength := 0;
  Locked := False;
end;

procedure TsohoCustomQuickSearch.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  SelStart := 0;
  SelLength := Length(Text);
end;

procedure TsohoCustomQuickSearch.InternalSearch(IndexNumb: LongInt);
begin
  with FDataSet do begin
    DisableControls;
    First;
    MoveBy(IndexNumb);
    EnableControls;
  end;
end;

procedure TsohoCustomQuickSearch.Search(Direct : ShortInt);
var PrevIndexNumb : LongInt;
    ws : string;
begin
  if not FActive then Exit;
  PrevIndexNumb := FIndexNumb;
  ws := AnsiUpperCase(Text);
  repeat
    inc(FIndexNumb, Direct);
  until (FIndexNumb < 0) or (FIndexNumb >= FIndexSize) or
        (Pos(ws, FIndex^[FIndexNumb]^)<>0) or (ws = EmptyStr);
  SearchSuccess := (FIndexNumb >= 0) and (FIndexNumb < FIndexSize);
  if not SearchSuccess then begin
    FIndexNumb := PrevIndexNumb;
    Exit;
  end;
  InternalSearch(FIndexNumb);
end;

procedure TsohoCustomQuickSearch.SetActive(Value : boolean);
begin
  if Value then Activate
  else DeActivate;
end;

procedure TsohoCustomQuickSearch.SetFieldsList(Value: TStrings);
begin
  DeActivate;
  FieldsList.Assign(Value);
end;

procedure TsohoCustomQuickSearch.DoAfterOpen;
begin
  if Assigned(FAfterOpen) then FAfterOpen(DataSet);
  LoadData;
end;

procedure TsohoCustomQuickSearch.DoAfterPost(DataSet:TDataSet);
var i : LongInt;
    wFIndex : PIndex;
    wFIDIndex : PIDIndex;
begin
  if Assigned(FAfterPost) then FAfterPost(DataSet);
  i := FindID(DataSet);
  if i >= 0 then begin {Edit}
    DisposeStr(FIndex^[i]);
    FIndex^[i] := NewStr(GetIndex);
  end else begin {Append}
    Inc(FIndexSize);
    GetMem(wFIDIndex, FIndexSize * SizeOf(LongInt));
    GetMem(wFIndex, FIndexSize * SizeOf(Pointer));
    move(FIndex^, wFIndex^, (FIndexSize-1) * SizeOf(Pointer));
    move(FIDIndex^, wFIDIndex^, (FIndexSize - 1) * SizeOf(LongInt));
    wFIDIndex^[FIndexSize - 1] := DataSet.FieldByName(KeyField).AsInteger;
    wFIndex^[FIndexSize - 1] := NewStr(GetIndex);
    FreeMem(FIndex, (FIndexSize - 1) * SizeOf(Pointer));
    FreeMem(FIDIndex, (FIndexSize - 1) * SizeOf(LongInt));
    FIndex := wFIndex;
    FIDIndex := wFIDIndex;
  end;
end;

procedure TsohoCustomQuickSearch.DoAfterClose;
begin
  if Assigned(FAfterClose) then FAfterClose(DataSet);
  FreeData;
end;

procedure TsohoCustomQuickSearch.DoBeforeDelete(DataSet:TDataSet);
var i: LongInt;
begin
  i := FindID(DataSet);
  DisposeStr(FIndex^[i]);
  FIndex^[i] := NewStr(EmptyStr);
  if Assigned(FBeforeDelete) then FBeforeDelete(DataSet);
end;

procedure TsohoCustomQuickSearch.ActivateEvents;
begin {     }
  with FDataSet do begin
    FAfterOpen := AfterOpen;
    FAfterClose := AfterClose;
    FBeforeDelete := BeforeDelete;
    FAfterPost := AfterPost;
    AfterOpen := DoAfterOpen;
    AfterClose := DoAfterClose;
    BeforeDelete := DoBeforeDelete;
    AfterPost := DoAfterPost;
  end;
end;

procedure TsohoCustomQuickSearch.DeActivateEvents;
begin
  with FDataSet do begin
    AfterOpen := FAfterOpen;
    AfterClose := FAfterClose;
    BeforeDelete := FBeforeDelete;
    AfterPost := FAfterPost;
  end;
end;

procedure TsohoCustomQuickSearch.LoadData;
var Index: LongInt;
begin
  try
    Test;
  except
    raise;
  end;
  if FIndex <> nil then FreeData;
  FIndexSize := FDataSet.RecordCount;
  GetMem(FIDIndex, FIndexSize * SizeOf(LongInt));
  GetMem(FIndex, FIndexSize * SizeOf(Pointer));
  with FDataSet do begin
    {   }
    DisableControls;
    First;
    for Index := 0 to pred(FIndexSize) do begin
      FIndex^[Index] := NewStr(GetIndex);
      FIDIndex^[Index] := FieldByName(KeyField).AsInteger;
      Next;
    end;
    EnableControls;
    FIndexNumb := 0;
  end;
end;

procedure TsohoCustomQuickSearch.FreeData;
var Index: LongInt;
begin
  if FIndex = nil then Exit;
  for Index := 0 to pred(FIndexSize) do
    DisposeStr(FIndex^[Index]);
  FreeMem(FIndex, FIndexSize * SizeOf(Pointer));
  FIndex := nil;
  FreeMem(FIDIndex, FIndexSize * SizeOf(LongInt));
  FIDIndex := nil;
  FIndexSize := 0;
end;

procedure TsohoCustomQuickSearch.Test;
var Index: LongInt;
begin {   }
  if not FDataSet.Active then
    raise EQuickSearch.Create(sohoQSDataSetNotActive);
  if FieldsList.Count = 0 then
    raise EQuickSearch.Create(sohoQSFieldListIsEmpty);
  with FDataSet do begin
    if FindField(KeyField) = nil then
      raise EQuickSearch.Create(Format(sohoQSKeyFieldNotFound, [KeyField]));
    for Index := 0 to pred(FieldsList.Count) do
       if FindField(FieldsList[Index]) = nil then
         raise EQuickSearch.Create(Format(sohoQSFieldNotFound,
           [FieldsList[Index]]));
  end;
end;

function TsohoCustomQuickSearch.FindID(DataSet : TDataSet) : LongInt;
var Index, ID: LongInt;
begin
  result := -1;
  ID := DataSet.FieldByName(KeyField).AsInteger;
  for Index := 0 to pred(FIndexSize) do
    if FIDIndex^[Index] = ID then begin
      result := Index;
      exit;
    end;
end;

function TsohoCustomQuickSearch.GetIndex : string;
var Index: LongInt;
begin {      }
  result := '';
  with FDataSet do
    for Index := 0 to pred(FieldsList.Count) do
      result := result +
        AnsiUpperCase(FieldByName(FieldsList[Index]).AsString) + '|';
end;

end.
