{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoCtmTVCont;

{$I SOHOLIB.INC}

interface

uses Classes, Controls, SoTools, SoFolder, SoCtmRec,
     SoChkBox, SoDBGrid, DBGrids, SoGauges, SoCtmTre, SoUnit,
     Dialogs, Graphics, Menus, SoCtmFld, DB, SoFrmStorage,
     SohoBtns, SysUtils, Windows, Forms, SoCtmDBGridAddon,
     ComCtrls
     {$IFDEF Delphi4_or_5}, ImgList{$ENDIF};

type

  TsohoCustomTVContainer = class;

  TsohoCustomTreeViewForm = class(TForm)
  private
    { Others }
    FOpenD: TOpenDialog;
    FSaveD: TSaveDialog;
    FStorage: TsohoFormStorage;
    FContainer: TsohoCustomTVContainer;
    ModalMode: boolean;
    CancelM: TMenuItem;
  protected
    { DB Engine depended items }
    FTreeView: TsohoCustomTreeView;
    FRecordForm: TsohoCustomRecordForm;
    FFolder: TsohoCustomFolder;
    FGroupsDS: TDataSet;
    FItemsDS: TDataSet;
    FDBAddon: TsohoCustomDBGridAddon;

    procedure GotoGroupMClick(Sender: TObject);
    procedure FindMClick(Sender: TObject);
    procedure SaveIdsMClick(Sender: TObject);
    procedure LoadIdsMClick(Sender: TObject);
    procedure ClearIdsMClick(Sender: TObject);
    procedure SelectMClick(Sender: TObject);
    procedure CancelMClick(Sender: TObject);
    procedure TreeViewActivated; virtual;

    procedure DBAddonBeforeOperation(Sender: TObject);
    procedure DBAddonAfterOperation(Sender: TObject);
{$IFDEF Delphi3}
    procedure CloseForm(Sender: TObject; var Action: TCloseAction);
{$ENDIF}
  public
    NewItemId,
    NewItemHash,
    NewGrpId: LongInt;

    {$IFDEF Delphi5}
    procedure GroupsCustomDrawItem(
        Sender: TCustomTreeView; Node: TTreeNode; State: TCustomDrawState;
        Stage: TCustomDrawStage; var PaintImages, DefaultDraw: Boolean);
    {$ENDIF}
    {$IFDEF Delphi5}
    procedure DoDestroy; override;
    {$ENDIF}
    {$IFDEF Delphi4}
    procedure BeforeDestruction; override;
    {$ENDIF}
    {$IFDEF Delphi3}	
    destructor Destroy; override;
    {$ENDIF}

    {$IFNDEF Delphi3}
    procedure DoClose(var Action: TCloseAction); override;
    {$ENDIF}

    constructor Create(aOwner: TComponent); override;
    procedure TreeViewNewItem(Sender: TObject;
      GroupId: Longint; var ItemId, ItemHash: Longint; var Allow: Boolean);
    procedure TreeViewItemEdit(Sender: TObject; GroupId,
      ItemId: Longint; var Allow: Boolean);
    procedure RecordCardGetValue(Sender: TObject;
      FieldName: String; var Value: String);

    property TreeView: TsohoCustomTreeView read FTreeView;
    property RecordForm: TsohoCustomRecordForm read FRecordForm;
    property Folder: TsohoCustomFolder read FFolder;
    property GroupsDS: TDataSet read FGroupsDS;
    property ItemsDS: TDataSet read FItemsDS;
  end;

  TsohoCustomTVContainer = class(TComponent)
  {        
    ,    .     
         -    
    . ,      
    .         
      MDIChild.        
      - TsohoCustomTreeView, TsohoCustomRecordForm  TsohoCustomFolder. 
    TsohoCustomTVContainer -      
     .   ,      
        , , TsohoTVContainer }
  private
    FOnProgress   : TsohoProgressEvent; {04/10/98}
    FOnException  : TsohoTreeOnException;
    FGauge        : TsohoGauge; {04/10/98}
    FFolderName   : TFileName;
    FFolderDir    : TDirName;
    FItemId       : LongInt;
    FCloseGroups  : boolean;
    FEditOnDblClick : boolean;

    FFocusField   : string;
    FFixedGrpName : string;
    FFixedGrpId   : LongInt;
    FRootName     : string;
    FIncludeSubTree : boolean;
    FEnabledMenus   : SetOfByte;
    FTreeViewMenu   : TPopupMenu;

    FNameGrID      : string;
    FHashField     : string;
    FOrderField    : string;
    FNameParentID  : string;
    FNameGroup     : string;
    FItemsName     : string;
    FGroupsName    : string;
    FGroupPreffix,
    FItemsPreffix  : string;

    FSaveFile      : TIniFileName;
    FOptions       : TsohoTreeOptions;
    FSaveOptions   : TsohoTreeSaveOptions;
    FGroupsSQLFile : TSQLFileName;
    FItemsSQLFile  : TSQLFileName;
    FGroupsTable   : String;
    FItemsTable    : String;
    FMarkedColor   : TColor;
    FMarkedFont    : TFont;
    FTreeViewForm  : TsohoCustomTreeViewForm;

    FResults       : TsohoDBGridResults;
    FClearResults  : boolean;
    FByFilter      : boolean;

    FOkBtn,
    FCancelBtn      : TsohoBitBtn;
    FSortC          : TsohoCheckBox;

    { Events }
    FOnFilter       : TsohoTreeOnFilter;
    FOnItemDelete   : TsohoTreeOnDeleteItem;
    FOnGrpDelete    : TsohoTreeOnDeleteGroup;

    FOnNewItem      : TsohoTreeOnNewItem;
    FOnOldNewItem   : TsohoTreeOnNewItem;

    FOnMoveItem     : TsohoTreeOnMoveItem;
    FOnNewGrp       : TsohoTreeOnNewGroup;

    FOnItemEdit     : TsohoTreeOnEditItem;
    FOnOldItemEdit  : TsohoTreeOnEditItem;

    FOnGrpEdit      : TsohoTreeOnEditGroup;
    FOnGetItems     : TsohoTreeOnGetItems;

    FOnCustomEditGrp : TsohoTreeCustomGroup;
    FOnCustomNewGrp  : TsohoTreeCustomGroup;

    FOnGetCellsProps: TGetCellPropsEvent;
    FOnDrawCells    : TDrawDataCellEvent;
    FOnFormCreate   : TNotifyEvent;
    FOnFormDestroy  : TNotifyEvent;
    FAfterGrpDelete : TNotifyEvent;
    FOnChange       : TNotifyEvent;
    FOnGroupSelect  : TNotifyEvent;
    FOnGridDblClick    : TNotifyEvent;
    FOnGridKeyDown     : TKeyEvent;
    FOnGroupsKeyDown   : TKeyEvent;
    FOnGroupsMouseDown : TMouseEvent;
    FOnGroupsMouseMove : TMouseMoveEvent;
    FOnGroupsDragOver  : TDragOverEvent;
    FOnGroupsDragDrop  : TDragDropEvent;
    FOnGridDragDrop    : TDragDropEvent;
    FOnGridMouseMove   : TMouseMoveEvent;
    FOnGridDragOver    : TDragOverEvent;
    FOnGridMouseDown   : TMouseEvent;

    FHideFields        : TStringList;
    FComplexFields     : TStringList;
    FComplexLabels     : TStringList;
    FSQLHideFields     : TStringList;

    FBeforeSQLGen,
    FAfterSQLGen       : TsohoOnSQLGenerate;
    FOnAddField        : TsohoOnAddField;
    FOnAddFieldToSQL   : TsohoOnAddFieldToSQL;
    FOnGetValue        : TsohoOnGetValue;
    FOnOldGetValue     : TsohoOnGetValue;
    FOnGetTextForField : TsohoOnGetTextForField;
    FOnGetComplexValue : TsohoOnGetComplexValue;
    FOnCardCreate,
    FOnCardDestroy,
    FOnCardSave,
    FOnCardLoad        : TNotifyEvent;

    FCaption           : TCaption;
    FKeyField          : string;
    FAfterExec         : TNotifyEvent;

    FBeforeGroupsOpen  : TDataSetNotifyEvent;
    FBeforeItemsOpen   : TDataSetNotifyEvent;

    FLastGroupId       : LongInt;
    procedure SetMarkedFont (Value : TFont);
    function  GetResults : TsohoDBGridResults;
    procedure SetHideFields    (Value : TStringList);
    procedure SetSQLHideFields (Value : TStringList);
    procedure SetComplexFields (Value : TStringList);
    procedure SetComplexLabels (Value : TStringList);
    function  GetItemId        : LongInt;
    procedure SetItemId        (Value : LongInt);
    procedure SetEnabledMenus  (Value : string);
    function  GetEnabledMenus  : string;
    function GetGauge: TsohoGauge;
    function GetTreeView: TsohoCustomTreeView;
    procedure SetGauge(const Value: TsohoGauge);
    procedure SortChanged(Sender: TObject);
  protected
    { Protected declarations }
    function CreateTreeViewForm: TsohoCustomTreeViewForm; virtual; abstract;
    procedure SetTreeViewProperties; virtual;
    procedure SetFolderProperties; virtual;
    procedure SetRecordProperties; virtual;
    procedure SetQueryProperties; virtual;
    procedure SetDBGridAddonProperties; virtual;
    procedure SetEvents; virtual;
    procedure GridKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure PrepareButtons (Modal : boolean; WithCancel : boolean);
    procedure RestoreSelection;
    procedure DoGetValue     (Sender : TObject; FieldName : string;
                              var Value : string);
    procedure DoNewItem      (Sender: TObject; GroupId: Longint; var ItemId,
                              ItemHash: Longint; var Allow: Boolean);
    procedure DoEditItem     (Sender: TObject; GroupId, ItemId: Longint;
                              var Allow: Boolean);
    procedure AddMenuItems;
    procedure DoFormCreate (Reopen, Modal, WithCancel : boolean); virtual;
    procedure DoFormDestroy(DoFree : boolean); virtual;

    procedure DoProgress (Stage : TProgressStage; aPercentDone : byte;
              const Msg : string); virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function  DoHandleExceptions (Sender: TObject; E: Exception): boolean;
  public
    constructor Create (AOwner : TComponent); override;
    destructor  Destroy; override;

    function    GetTreeMenuItem (Tag : LongInt) : TMenuItem;
    {        Tag }
    procedure   CloseTreeViewForm (DoFree : boolean);
    {    }
    function    GroupsQuery : TDataSet;
    {   DataSet c  }
    function    ItemsQuery  : TDataSet;
    {   DataSet c  }
    function    ReCreateExecute (Modal : boolean; WithCancel : boolean) : boolean;
    {        Ok, Cancel.    }
    function    Execute (Reopen, Modal : boolean; WithCancel : boolean) : boolean;
    {     Ok, Cancel.    }
    function    MDIExecute : boolean;
    {     MDIChild- }
    property    Results  : TsohoDBGridResults read GetResults;
    {   .  TsohoDBGrid }
    property    ItemId   : LongInt read GetItemId write SetItemId default -1;
    {   .  ,   
              }

    property    FixedGrpName : string  read FFixedGrpName write FFixedGrpName;
    {  TsohoCustomTreeView }
    property    FixedGrpId   : LongInt read FFixedGrpId   write FFixedGrpId;
    {  TsohoCustomTreeView }
    property    TreeViewForm : TsohoCustomTreeViewForm read FTreeViewForm;
    {       }
    property    TreeView     : TsohoCustomTreeView read GetTreeView;
    {   TsohoCustomTreeView    }
  published

    property ByFilter : boolean read FByFilter write FByFilter default true;
    {  TsohoCustomTreeView }
    property EditOnDblClick : boolean read FEditOnDblClick write FEditOnDblClick default true;
    {  TsohoCustomTreeView }

    property FocusField   : string read FFocusField write FFocusField;
    {  TsohoCustomRecordForm }

    property TreeViewMenu : TPopupMenu read FTreeViewMenu write FTreeViewMenu;
    {   PopupMenu,      TreeView }
    property ClearResults : boolean read FClearResults write FClearResults default true;
    {  ClearResults = true,    TreeViewForm  
         Results.     
        }

    property FolderName  : TFileName read FFolderName  write FFolderName;
    {  TsohoCustomFolder }
    property FolderDir   : TDirName  read FFolderDir   write FFolderDir;
    {  TsohoCustomFolder }
    property EnabledMenus : string read GetEnabledMenus write SetEnabledMenus;
    {  TsohoCustomFolder }

    property IncludeSubTree : boolean read FIncludeSubTree write FIncludeSubTree;
    {  TsohoCustomTreeView }
    property RootName      : string    read FRootName write FRootName;
    {  TsohoCustomTreeView }


    property GroupsSQLFile : TSQLFileName read FGroupsSQLFile  write FGroupsSQLFile;
    {   sql-    }
    property GroupsTable   : String    read FGroupsTable write FGroupsTable;
    {    }

    property CloseGroups   : boolean   read FCloseGroups  write FCloseGroups default true;
    {  TsohoCustomTreeView }

    property ItemsTable    : String   read FItemsTable  write FItemsTable;
    {     }
    property ItemsSQLFile : TSQLFileName  read FItemsSQLFile  write FItemsSQLFile;
    {   sql-    }
    property NameGroupID   : string    read FNameGrID    write FNameGrID;
    {  TsohoCustomTreeView }
    property NameParentID: string read FNameParentID write FNameParentID;
    {  TsohoCustomTreeView }
    property NameGroup: string read FNameGroup write FNameGroup;
    {  TsohoCustomTreeView }
    property OrderField    : string    read FOrderField  write FOrderField;
    {  TsohoCustomTreeView }
    property HashField     : string    read FHashField   write FHashField;
    {  TsohoCustomTreeView }
    property Options       : TsohoTreeOptions read FOptions write FOptions;
    {  TsohoCustomTreeView }
    property SaveOptions   : TsohoTreeSaveOptions read FSaveOptions write FSaveOptions default [trSaveSplitter];
    {  TsohoCustomTreeView }
    property ItemsName     : string    read FItemsName   write FItemsName;
    {  TsohoCustomTreeView }
    property GroupsName    : string    read FGroupsName   write FGroupsName;
    {  TsohoCustomTreeView }
    property GroupPreffix : string read FGroupPreffix write FGroupPreffix;
    {  TsohoCustomTreeView }
    property ItemsPreffix : string read FItemsPreffix write FItemsPreffix;
    {  TsohoCustomTreeView }
    property SaveFile      : TIniFileName read FSaveFile write FSaveFile;
    {  TsohoCustomTreeView }
    property MarkedColor   : TColor    read FMarkedColor write FMarkedColor;
    {  TsohoCustomTreeView }
    property MarkedFont    : TFont     read FMarkedFont  write SetMarkedFont;
    {  TsohoCustomTreeView }
    property Gauge         : TsohoGauge read GetGauge write SetGauge;
    {    TsohoGauge,     
       .  ""    
       OnProgress }
    property Caption      : TCaption  read FCaption      write FCaption;
    {  TsohoCustomRecordForm }
    property KeyField     : string    read FKeyField     write FKeyField;
    {  TsohoCustomRecordForm }
    property HideFields    : TStringList read FHideFields    write SetHideFields;
    {  TsohoCustomRecordForm }
    property SQLHideFields : TStringList read FSQLHideFields write SetSQLHideFields;
    {  TsohoCustomRecordForm }
    property ComplexFields : TStringList read FComplexFields write SetComplexFields;
    {  TsohoCustomRecordForm }
    property ComplexLabels : TStringList read FComplexLabels write SetComplexLabels;
    {  TsohoCustomRecordForm }

    property OnGetTextForField : TsohoOnGetTextForField read FOnGetTextForField write FOnGetTextForField;
    {  TsohoCustomRecordForm }
    property OnGetComplexValue : TsohoOnGetComplexValue read FOnGetComplexValue write FOnGetComplexValue;
    {  TsohoCustomRecordForm }
    property BeforeSQL  : TsohoOnSQLGenerate read FBeforeSQLGen write FBeforeSQLGen;
    {  TsohoCustomRecordForm }
    property AfterSQL   : TsohoOnSQLGenerate read FAfterSQLGen  write FAfterSQLGen;
    {  TsohoCustomRecordForm }
    property OnAddField : TsohoOnAddField    read FOnAddField   write FOnAddField;
    {  TsohoCustomRecordForm }
    property OnAddFieldToSQL : TsohoOnAddFieldToSQL read FOnAddFieldToSQL write FOnAddFieldToSQL;
    {  TsohoCustomRecordForm }
    property OnGetValue : TsohoOnGetValue    read FOnGetValue   write FOnGetValue;
    {  TsohoCustomRecordForm }
    property AfterSQLExecute : TNotifyEvent  read FAfterExec    write FAfterExec;
    {  TsohoCustomRecordForm }
    property OnCardCreate   : TNotifyEvent   read FOnCardCreate write FOnCardCreate;
    {  TsohoCustomRecordForm }
    property OnCardDestroy  : TNotifyEvent   read FOnCardDestroy  write FOnCardDestroy;
    {  TsohoCustomRecordForm }
    property OnCardSave     : TNotifyEvent   read FOnCardSave   write FOnCardSave;
    {  TsohoCustomRecordForm }
    property OnCardLoad     : TNotifyEvent   read FOnCardLoad   write FOnCardLoad;
    {  TsohoCustomRecordForm }
    property OnFilter       : TsohoTreeOnFilter      read FOnFilter       write FOnFilter;
  {  TsohoCustomTreeView }
    property OnItemDelete   : TsohoTreeOnDeleteItem  read FOnItemDelete   write FOnItemDelete;
  {  TsohoCustomTreeView }
    property OnGrpDelete    : TsohoTreeOnDeleteGroup read FOnGrpDelete    write FOnGrpDelete;
  {  TsohoCustomTreeView }
    property OnNewItem      : TsohoTreeOnNewItem     read FOnNewItem      write FOnNewItem;
  {  TsohoCustomTreeView }
    property OnNewGrp       : TsohoTreeOnNewGroup    read FOnNewGrp       write FOnNewGrp;
  {  TsohoCustomTreeView }
    property OnItemEdit     : TsohoTreeOnEditItem    read FOnItemEdit     write FOnItemEdit;
  {  TsohoCustomTreeView }
    property OnItemMove     : TsohoTreeOnMoveItem    read FOnMoveItem     write FOnMoveItem;
  {  TsohoCustomTreeView }
    property OnGrpEdit      : TsohoTreeOnEditGroup   read FOnGrpEdit      write FOnGrpEdit;
  {  TsohoCustomTreeView }
    property OnCustomEditGrp: TsohoTreeCustomGroup   read FOnCustomEditGrp write FOnCustomEditGrp;
  {  TsohoCustomTreeView }
    property OnCustomNewGrp : TsohoTreeCustomGroup   read FOnCustomNewGrp write FOnCustomNewGrp;
  {  TsohoCustomTreeView }
    property AfterGrpDelete : TNotifyEvent           read FAfterGrpDelete write FAfterGrpDelete;
  {  TsohoCustomTreeView }
    property OnGridDblClick     : TNotifyEvent    read FOnGridDblClick    write FOnGridDblClick;
  {  TsohoCustomTreeView }
    property OnGridKeyDown      : TKeyEvent       read FOnGridKeyDown     write FOnGridKeyDown;
  {  TsohoCustomTreeView }
    property OnGridDragDrop     : TDragDropEvent  read FOnGridDragDrop    write FOnGridDragDrop;
  {  TsohoCustomTreeView }
    property OnGridMouseMove    : TMouseMoveEvent read FOnGridMouseMove   write FOnGridMouseMove;
  {  TsohoCustomTreeView }
    property OnGridMouseDown    : TMouseEvent     read FOnGridMouseDown   write FOnGridMouseDown;
  {  TsohoCustomTreeView }
    property OnGridDragOver     : TDragOverEvent  read FOnGridDragOver    write FOnGridDragOver;
  {  TsohoCustomTreeView }
    property OnTreeViewKeyDown   : TKeyEvent       read FOnGroupsKeyDown   write FOnGroupsKeyDown;
  {  TsohoCustomTreeView }
    property OnTreeViewMouseDown : TMouseEvent     read FOnGroupsMouseDown write FOnGroupsMouseDown;
  {  TsohoCustomTreeView }
    property OnTreeViewMouseMove : TMouseMoveEvent read FOnGroupsMouseMove write FOnGroupsMouseMove;
  {  TsohoCustomTreeView }
    property OnTreeViewDragOver  : TDragOverEvent  read FOnGroupsDragOver  write FOnGroupsDragOver;
  {  TsohoCustomTreeView }
    property OnTreeViewDragDrop  : TDragDropEvent  read FOnGroupsDragDrop  write FOnGroupsDragDrop;
  {  TsohoCustomTreeView }
    property OnChange           : TNotifyEvent    read FOnChange          write FOnChange;
  {  TsohoCustomTreeView }
    property OnGroupSelect      : TNotifyEvent    read FOnGroupSelect     write FOnGroupSelect;
  {  TsohoCustomTreeView }
    property OnGetItems: TsohoTreeOnGetItems read FOnGetItems write FOnGetItems;
  {  TsohoCustomTreeView }
    property OnGetCellProps: TGetCellPropsEvent read FOnGetCellsProps write FOnGetCellsProps;
    {  TsohoDBGrid }
    property OnDrawDataCell   : TDrawDataCellEvent   read FOnDrawCells write FOnDrawCells;
    {  TsohoDBGrid }

    property BeforeGroupsOpen  : TDataSetNotifyEvent read FBeforeGroupsOpen write FBeforeGroupsOpen;
    {        }
    property BeforeItemsOpen   : TDataSetNotifyEvent read FBeforeItemsOpen  write FBeforeItemsOpen;
    {        }

    property OnFormCreate   : TNotifyEvent read FOnFormCreate  write FOnFormCreate;
    {       }
    property OnFormDestroy  : TNotifyEvent read FOnFormDestroy write FOnFormDestroy;
    {       }
    property OnProgress     : TsohoProgressEvent read FOnProgress write FOnProgress; {04/10/98}
    {        }
    property OnException: TsohoTreeOnException read FOnException write FOnException;
    {      .  ,
            TsohoCustomTreeView, 
         }
  end;

const

    {  TsohoTreeView }
    btnDelGroup  = 0;
    btnNewGroup  = 1;
    btnEditGroup = 2;
    btnDelItem   = 3;
    btnNewItem   = 4;
    btnEditItem  = 5;
    btnRefresh   = 6;
    btnReNumber  = 7;

implementation
uses SoStrUtils, ExtCtrls, SoCtmRgs, IniFiles, SoDBCns,
     SoAppUtils, SoVCLUtils;

{$R *.DFM}

{ TsohoCustomTreeViewForm }

procedure TsohoCustomTreeViewForm.CancelMClick(Sender: TObject);
begin
  if FContainer <> nil then FContainer.ItemId  := -1;
  ModalResult := mrCancel;
end;

procedure TsohoCustomTreeViewForm.ClearIdsMClick(Sender: TObject);
begin
  FTreeView.Results.Clear;
end;

{$IFDEF Delphi3}
procedure TsohoCustomTreeViewForm.CloseForm(Sender: TObject; var Action: TCloseAction);
begin
  if FormStyle = fsMDIChild then Action := caFree;
end;
{$ENDIF}

{$IFDEF Delphi5}
procedure TsohoCustomTreeViewForm.GroupsCustomDrawItem(
  Sender: TCustomTreeView; Node: TTreeNode; State: TCustomDrawState;
  Stage: TCustomDrawStage; var PaintImages, DefaultDraw: Boolean);
var
  NodeRect : TRect;
begin
  if (Stage = cdPostPaint) and (Node.Level > 0) and
     Node.HasChildren then begin
    NodeRect := Node.DisplayRect(True);
    with FTreeView.GroupTreeView.Canvas do begin
      Font.Color := clBlue;
      Font.Style := Font.Style - [fsBold];
      Brush.Color := FTreeView.GroupTreeView.Color;
      TextOut(NodeRect.Right + 3, NodeRect.Top + 1, '(' + IntToStr(Node.Count) + ')');
    end;
  end;
end;
{$ENDIF}

constructor TsohoCustomTreeViewForm.Create(aOwner: TComponent);
begin
  inherited;
  Icon.Handle := LoadIcon(hInstance, 'MAINICON');

  FTreeView := nil;
  FRecordForm := nil;
  FFolder := nil;
  FGroupsDS := nil;
  FItemsDS := nil;
  FDBAddon := nil;

  FOpenD := TOpenDialog.Create(Self);
  with FOpenD do begin
    DefaultExt := 'IDS';
    Filter := sohoIDSFiles;
    Options := [ofHideReadOnly, ofFileMustExist];
    Title := sohoLoadMarkedRecords;
  end;

  FSaveD := TSaveDialog.Create(Self);
  with FSaveD do begin
    DefaultExt := 'IDS';
    Filter := sohoIDSFiles;
    Options := [ofOverwritePrompt, ofHideReadOnly, ofPathMustExist];
    Title := sohoSaveMarkedRecords;
  end;

  FStorage := TsohoFormStorage.Create(Self);

  FContainer := nil;
{$IFDEF Delphi3}
  OnClose := CloseForm;
{$ENDIF}
end;

{$IFDEF Delphi5}
procedure TsohoCustomTreeViewForm.DoDestroy;
{$ENDIF}
{$IFDEF Delphi4}
procedure TsohoCustomTreeViewForm.BeforeDestruction;
{$ENDIF}
{$IFDEF Delphi3}
destructor TsohoCustomTreeViewForm.Destroy;
{$ENDIF}
begin
  FStorage.SaveFormPlacement;
  FTreeView.SaveTreeViewState;
  FDBAddon.SaveStatement(FStorage);
  FDBAddon.Free;
  if FContainer <> nil then FContainer.FTreeViewForm := nil;
  inherited;
end;

procedure TsohoCustomTreeViewForm.FindMClick(Sender: TObject);
begin
  // nothing to do now
end;

procedure TsohoCustomTreeViewForm.GotoGroupMClick(Sender: TObject);
var GrpId: LongInt;
    GrpNode: TTreeNode;
begin
  if FItemsDS.FieldByName(FTreeView.NameGroupId).IsNull then exit;
  GrpId := FItemsDS.FieldByName(TreeView.NameGroupId).AsInteger;
  GrpNode := FTreeView.GetNodeByID(GrpId);
  // FoundItem := FTreeView.GroupTreeView.Items[GrpIndex];
  while (GrpNode.Parent <> nil) and (not GrpNode.Expanded) do
    GrpNode := GrpNode.Parent;
  GrpNode.Expand(false);
  FTreeView.GroupTreeView.Selected := GrpNode;
end;

procedure TsohoCustomTreeViewForm.LoadIdsMClick(Sender: TObject);
begin
  if not FOpenD.Execute then exit;
  try
    FTreeView.DBGrid.Results.LoadFromFile(FOpenD.FileName);
  except
  end;
end;

procedure TsohoCustomTreeViewForm.SaveIdsMClick(Sender: TObject);
begin
  if not FSaveD.Execute then exit;
  try
    FTreeView.DBGrid.Results.SaveToFile(FSaveD.FileName);
  except
  end;
end;

procedure TsohoCustomTreeViewForm.SelectMClick(Sender: TObject);
begin
  if FContainer <> nil then begin
    FContainer.FItemId  := FTreeView.ItemId;
    FContainer.FLastGroupId := FTreeView.GetIdByNode(
      FTreeView.GroupTreeView.Selected);
    FContainer.FResults.Assign(FTreeView.Results);
    if not ModalMode then Close
    else ModalResult := mrOk;
  end;
end;

procedure TsohoCustomTreeViewForm.TreeViewNewItem(Sender: TObject;
  GroupId: Longint; var ItemId, ItemHash: Longint; var Allow: Boolean);
begin
  NewItemId   := ItemId;
  NewItemHash := ItemHash;
  NewGrpId    := GroupId;
  Allow := FRecordForm.Insert;
  FRecordForm.Close;
end;

procedure TsohoCustomTreeViewForm.TreeViewItemEdit(Sender: TObject; GroupId,
  ItemId: Longint; var Allow: Boolean);
begin
  Allow := FRecordForm.Edit;
  FRecordForm.Close;
end;

procedure TsohoCustomTreeViewForm.RecordCardGetValue(Sender: TObject;
  FieldName: String; var Value: String);
begin
  if FieldName = 'MDFDATE' then Value := DateToStr(Date);
  if FieldName = 'MDFTIME' then Value := TimeToStr(Time);
  if FieldName = 'IDMDFDAUTH' then begin
    Value := '0';
    if SingleRegister <> nil then Value := IntToStr(SingleRegister.UserID);
  end;
  if FRecordForm.Mode <> rmInsert then exit;
  if FieldName = AnsiUpperCase(FRecordForm.KeyField) then Value := IntToStr(NewItemId);
  if FieldName = AnsiUpperCase(FTreeView.NameGroupId) then Value := IntToStr(NewGrpId);
  if FieldName = AnsiUpperCase(FTreeView.HashField) then Value := IntToStr(NewItemHash);
  if FieldName = 'PROTECT' then begin
    Value := '0';
    if SingleRegister <> nil then Value := IntToStr(SingleRegister.DataLevel);
  end;
end;

procedure TsohoCustomTreeViewForm.TreeViewActivated;
begin
  with FDBAddon do begin
    SetEvents;
    SetDataSet;
  end;
end;

procedure TsohoCustomTreeViewForm.DBAddonAfterOperation(Sender: TObject);
begin
  FFolder.LoadDBGridColumns;
end;

procedure TsohoCustomTreeViewForm.DBAddonBeforeOperation(Sender: TObject);
begin
  FFolder.SaveDBGridColumns;
end;

{$IFNDEF Delphi3}
procedure TsohoCustomTreeViewForm.DoClose(var Action: TCloseAction);
begin
  inherited;
  if FormStyle = fsMDIChild then Action := caFree;
end;
{$ENDIF}


{ TsohoCustomTVContainer }
function TsohoCustomTVContainer.DoHandleExceptions (Sender: TObject; E: Exception): boolean;
begin
  Result := false;
  if Assigned(FOnException) then Result := FOnException(Sender, E);
end;

procedure TsohoCustomTVContainer.DoProgress (Stage : TProgressStage; aPercentDone : byte;
          const Msg : string);
begin
  if Assigned(FGauge) then
    with FGauge do begin
     case Stage of
       psStarting : begin
         MinValue := 0;
         MaxValue := 100;
         Progress := aPercentDone;
         Caption  := Msg;
       end;
       psRunning  : begin
         Progress := aPercentDone;
         Caption  := Msg;
       end;
       psEnding   : begin
         Progress := 0;
         Caption  := Msg;
       end;
     end;
    end;
  if Assigned(FOnProgress) then FOnProgress(Self, Stage, aPercentDone, Msg);
end;

procedure TsohoCustomTVContainer.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(aComponent, Operation);
  if (aComponent = FGauge) and (Operation = opRemove) then FGauge := nil;
end;

function TsohoCustomTVContainer.GetTreeMenuItem (Tag : LongInt) : TMenuItem;
begin
  Result := nil;
  if FTreeViewForm = nil then exit;
  if TreeView.DBGrid.PopupMenu = nil then exit;
  Result := FindMenuItemByTag(TreeView.DBGrid.PopupMenu, Tag);
end;

function TsohoCustomTVContainer.GroupsQuery: TDataSet;
begin
  Result := nil;
  if FTreeViewForm = nil then exit;
  Result := FTreeViewForm.FGroupsDS;
end;

function TsohoCustomTVContainer.ItemsQuery  : TDataSet;
begin
  Result := nil;
  if FTreeViewForm = nil then exit;
  Result := FTreeViewForm.FItemsDS;
end;

function  TsohoCustomTVContainer.GetTreeView : TsohoCustomTreeView;
begin
  Result := nil;
  if FTreeViewForm = nil then exit;
  Result := FTreeViewForm.FTreeView;
end;

procedure TsohoCustomTVContainer.DoFormCreate (Reopen, Modal, WithCancel : boolean);
begin
  DoProgress (psRunning, 45, gdSetEvents);
  if Reopen then SetEvents;
  DoProgress (psRunning, 60, gdPrepareButtons);
  PrepareButtons(Modal, WithCancel);
  DoProgress (psRunning, 90, gdQueryActivate);
  FSortC.Checked := FTreeViewForm.FStorage.ReadBool('Options', 'Sort', false);
  if Reopen then begin
    TreeView.Active := true;
    FTreeViewForm.TreeViewActivated;
    FTreeViewForm.FDBAddon.LoadStatement(FTreeViewForm.FStorage);
  end;
  DoProgress (psRunning, 99, gdRestoring);
  if Reopen then RestoreSelection;
  if FClearResults then TreeView.DBGrid.Results.Clear;
  FTreeViewForm.ModalMode := Modal;
  if Assigned(FOnFormCreate) then FOnFormCreate(FTreeViewForm);
end;

procedure TsohoCustomTVContainer.DoFormDestroy (DoFree : boolean);
begin
  if FOkBtn <> nil then FOkBtn.Free;
  if FCancelBtn <> nil then FCancelBtn.Free;
  FOkBtn     := nil;
  FCancelBtn := nil;
  if FSortC <> nil then begin
    FTreeViewForm.FStorage.WriteBool('Options', 'Sort', FSortC.Checked);
    FSortC.Free;
  end;
  FSortC     := nil;
  if Assigned(FOnFormDestroy) then FOnFormDestroy(FTreeViewForm);
  if DoFree then FTreeViewForm.Free
  else FTreeViewForm.Hide;
end;

procedure TsohoCustomTVContainer.CloseTreeViewForm(DoFree : boolean);
begin
  if FTreeViewForm = nil then exit;
  DoFormDestroy(DoFree);
end;

procedure TsohoCustomTVContainer.SetMarkedFont (Value : TFont);
begin
  FMarkedFont.Assign(Value);
end;

procedure TsohoCustomTVContainer.DoNewItem      (Sender: TObject; GroupId: Longint; var ItemId,
                              ItemHash: Longint; var Allow: Boolean);
begin
  if Assigned(FOnNewItem) then FOnNewItem(Sender, GroupId, ItemId,
    ItemHash, Allow)
  else FTreeViewForm.TreeViewNewItem(Sender, GroupId, ItemId,
    ItemHash, Allow);
end;

procedure TsohoCustomTVContainer.DoEditItem (Sender: TObject; GroupId, ItemId: Longint; var Allow: Boolean);
begin
  if Assigned(FOnItemEdit) then FOnItemEdit(Sender, GroupId, ItemId, Allow)
  else
    FTreeViewForm.TreeViewItemEdit(Sender, GroupId, ItemId,
       Allow);
end;

procedure TsohoCustomTVContainer.DoGetValue (Sender : TObject; FieldName : string; var Value : string);
begin
  FTreeViewForm.RecordCardGetValue(Sender, FieldName, Value);
  if Assigned(FOnGetValue) then FOnGetValue(Sender, FieldName, Value);
end;

procedure TsohoCustomTVContainer.AddMenuItems;
var InsIndex, Index : LongInt;
    MnuItem : TMenuItem;

  function AddMenuItem(const Caption: string; Click: TNotifyEvent;
    ImageIndex: LongInt): TMenuItem;
  begin
    Result := NewItem(Caption, 0, false, true, Click, 0, EmptyStr);
    {$IFDEF Delphi4_or_5}
    Result.ImageIndex := ImageIndex;
    {$ENDIF}
  end;

begin
  with FTreeViewForm.FTreeView.DBGrid.PopupMenu do begin
     InsIndex := Items.Count - 1;
     {    -       
        }
     while (InsIndex>=0)and (Items[InsIndex].Caption <> '-') do Dec(InsIndex);
     if InsIndex<0 then InsIndex := 0;
     if FTreeViewMenu <> nil then begin
       Items.Insert(InsIndex, NewItem('-', 0, false, true, nil, 0, ''));
       inc(InsIndex);
       for Index := 0 to pred(FTreeViewMenu.Items.Count) do begin
         MnuItem := CopyMenuItem(Self, FTreeViewMenu.Items[Index]);
         Items.Insert(InsIndex, MnuItem);
         inc(InsIndex);
       end;
       Items.Insert(InsIndex, NewItem('-', 0, false, true, nil, 0, ''));
     end;
     Items.Add(AddMenuItem('-', nil, -1));
     Items.Add(AddMenuItem(sohoTVContGotoGroupItem, FTreeViewForm.GotoGroupMClick, 0));
     Items.Add(AddMenuItem(sohoTVContSearchItem, FTreeViewForm.FindMClick, 1));
     Items.Add(AddMenuItem('-', nil, -1));
     Items.Add(AddMenuItem(sohoTVContSaveMarkedItem, FTreeViewForm.SaveIdsMClick, 2));
     Items.Add(AddMenuItem(sohoTVContLoadMarkedItem, FTreeViewForm.LoadIdsMClick, 3));
     Items.Add(AddMenuItem(sohoTVContClearItem, FTreeViewForm.ClearIdsMClick, 4));
     Items.Add(AddMenuItem('-', nil, -1));
     Items.Add(AddMenuItem(sohoTVContSelectItem, FTreeViewForm.SelectMClick, 5));
     FTreeViewForm.CancelM := AddMenuItem(sohoTVContCancelItem, FTreeViewForm.CancelMClick, 6);
     Items.Add(FTreeViewForm.CancelM);
     {$IFDEF Delphi4_or_5}
     FTreeViewForm.FTreeView.DBGrid.PopupMenu.Images := TVImages;
     {$ENDIF}
  end;
end;

procedure TsohoCustomTVContainer.SetDBGridAddonProperties;
begin
  with FTreeViewForm.FDBAddon do begin
    Grid := TreeView.DBGrid;
    Active := false;
    AllowSorting := true;
    AllowFiltering := true;
    TurnOffFilter := false;
    BeforeOperation := FTreeViewForm.DBAddonBeforeOperation;
    AfterOperation := FTreeViewForm.DBAddonAfterOperation;
  end;
end;

procedure TsohoCustomTVContainer.SetTreeViewProperties;
begin
  with FTreeViewForm.FTreeView do begin
     EditOnDblClick := FEditOnDblClick;
     IncludeSubTree := FIncludeSubTree;
     RootName      := FRootName;
     GroupsSQLFile := FGroupsSQLFile;
     GroupsTable   := FGroupsTable;
     ItemsTable    := FItemsTable;
     ItemsSQLFile  := FItemsSQLFile;
     NameGroupID   := FNameGrID;
     OrderField    := FOrderField;
     NameParentID  := FNameParentID;
     NameGroup     := FNameGroup;
     HashField     := FHashField;
     Options       := FOptions;
     SaveOptions   := FSaveOptions;
     ItemsName     := FItemsName;
     GroupsName    := FGroupsName;
     CloseGroups   := FCloseGroups;
     SaveFile      := FSaveFile;
     KeyField	   := FKeyField;
     DBGrid.MarkedColor := FMarkedColor;
     DBGrid.MarkedFont.Assign(FMarkedFont);
     ByFilter      := FByFilter;
     OnFilter      := FOnFilter;
     OnItemDelete  := FOnItemDelete;
     OnGrpDelete   := FOnGrpDelete;
     GroupPreffix  := FGroupPreffix;
     ItemsPreffix  := FItemsPreffix;
     FixedGrpName  := FFixedGrpName;
     FixedGrpId    := FFixedGrpId;
     if FClearResults then FResults.Clear;
     Results.Assign(FResults);
     OnGetItems    := FOnGetItems;
     FOnOldNewItem  := OnNewItem;
     OnNewItem      := DoNewItem;

     FOnOldItemEdit := OnItemEdit;
     OnItemEdit     := DoEditItem;

     OnNewGrp       := FOnNewGrp;

     OnCustomEditGrp := FOnCustomEditGrp;
     OnCustomNewGrp  := FOnCustomNewGrp;

     OnItemMove     := FOnMoveItem;
     OnGrpEdit      := FOnGrpEdit;
     OnGridDblClick     := FOnGridDblClick;
     OnGridKeyDown      := GridKeyDown;
     OnGridDragDrop     := FOnGridDragDrop;
     OnGridMouseMove    := FOnGridMouseMove;
     OnGridMouseDown    := FOnGridMouseDown;
     OnGridDragOver     := FOnGridDragOver;
     OnTreeViewKeyDown   := FOnGroupsKeyDown;
     OnTreeViewMouseDown := FOnGroupsMouseDown;
     OnTreeViewMouseMove := FOnGroupsMouseMove;
     OnTreeViewDragOver  := FOnGroupsDragOver;
     OnTreeViewDragDrop  := FOnGroupsDragDrop;
     OnChange           := FOnChange;
     OnGroupSelect      := FOnGroupSelect;
     OnGetCellProps     := FOnGetCellsProps;
     OnDrawDataCell     := FOnDrawCells;
{$IFDEF Delphi5}
     GroupTreeView.OnAdvancedCustomDrawItem := FTreeViewForm.GroupsCustomDrawItem;
{$ENDIF}
     AddMenuItems;
     OnException := DoHandleExceptions;
  end;
end;

procedure TsohoCustomTVContainer.SetFolderProperties;
begin
  with FTreeViewForm, FFolder do begin
     if FolderDir <> '' then
       FolderDir := FFolderDir;
     FolderName   := FFolderName;
     TableName    := FItemsTable;
     EnabledMenus := SetToStr(FEnabledMenus);
  end;
end;

procedure TsohoCustomTVContainer.SetRecordProperties;
begin
  with FTreeViewForm.FRecordForm do begin
     Caption       := FCaption;
     KeyField      := FKeyField;
     TableName     := FItemsTable;
     FocusField    := FFocusField;
     HideFields.Assign(FHideFields);
     SQLHideFields.Assign(FSQLHideFields);
     ComplexFields.Assign(FComplexFields);
     ComplexLabels.Assign(FComplexLabels);
     OnGetTextForField := FOnGetTextForField;
     OnGetComplexValue := FOnGetComplexValue;
     BeforeSQL         := FBeforeSQLGen;
     AfterSQL          := FAfterSQLGen;
     OnAddField        := FOnAddField;
     OnAddFieldToSQL   := FOnAddFieldToSQL;
     FOnOldGetValue    := OnGetValue;
     OnGetValue        := DoGetValue;
     AfterSQLExecute   := FAfterExec;
     OnCardCreate      := FOnCardCreate;
     OnCardDestroy     := FOnCardDestroy;
     OnCardSave        := FOnCardSave;
     OnCardLoad        := FOnCardLoad;
  end;
end;

procedure TsohoCustomTVContainer.SetQueryProperties;
var FGrpsSQL, FItemsSQL : string;
begin
  with FTreeViewForm do begin
     FGrpsSQL  := FGroupsSQLFile;
     FItemsSQL := FItemsSQLFile;

     if SingleRegister<>nil then begin
       FGrpsSQL  := SingleRegister.PathToSQL + FGroupsSQLFile;
       FItemsSQL := SingleRegister.PathToSQL + FItemsSQLFile;
     end;

     with FGroupsDS do begin
       if not FileExists(FGrpsSQL) then begin
         ErrorMsg(Format(sohoTVContFileNotFound, [FGrpsSQL]));
         Abort;
       end;
       BeforeOpen := FBeforeGroupsOpen;
     end;

     with FItemsDS do begin
       if not FileExists(FItemsSQL) then begin
         ErrorMsg(Format(sohoTVContFileNotFound, [FItemsSQL]));
         Abort;
       end;
       BeforeOpen := FBeforeItemsOpen;
     end;
  end;
end;

procedure TsohoCustomTVContainer.SetEvents;
begin
  if FTreeViewForm = nil then exit;
  with FTreeViewForm do begin
    SetTreeViewProperties;
    SetFolderProperties;
    SetRecordProperties;
    SetQueryProperties;
    SetDBGridAddonProperties;
    Caption := FCaption;
    FStorage.IniSection := FContainer.Name + '.TreeViewForm';
    // FStorage.IniSection := Owner.Name + '.' + ClassName + '.' + Name;
  end;
end;

function  TsohoCustomTVContainer.GetResults : TsohoDBGridResults;
begin
  if FTreeViewForm <> nil then Result := FTreeViewForm.FTreeView.Results
  else Result := FResults;
end;

procedure TsohoCustomTVContainer.SetHideFields    (Value : TStringList);
begin
  FHideFields.Assign(Value);
end;

procedure TsohoCustomTVContainer.SetSQLHideFields (Value : TStringList);
begin
  FSQLHideFields.Assign(Value);
end;

procedure TsohoCustomTVContainer.SetComplexFields (Value : TStringList);
begin
  FComplexFields.Assign(Value);
end;

procedure TsohoCustomTVContainer.SetComplexLabels (Value : TStringList);
begin
  FComplexLabels.Assign(Value);
end;

function  TsohoCustomTVContainer.GetItemId        : LongInt;
begin
  if FTreeViewForm = nil then Result := FItemId
  else Result := FTreeViewForm.FTreeView.ItemId;
end;

procedure TsohoCustomTVContainer.SetItemId        (Value : LongInt);
begin
  FItemId := Value;
  if FTreeViewForm <> nil then FTreeViewForm.FTreeView.ItemId := Value;
end;

procedure TsohoCustomTVContainer.SetEnabledMenus  (Value : string);
begin
  FEnabledMenus := StrToSet(Value);
end;

function  TsohoCustomTVContainer.GetEnabledMenus  : string;
begin
  Result := SetToStr(FEnabledMenus);
end;

constructor TsohoCustomTVContainer.Create (AOwner : TComponent);
var Form : TForm;
begin
  inherited Create(AOwner);
  FOptions     := [trDeleteGroup, trAddGroup, trEditGroup, trDeleteItem,
                   trAddItem, trEditItem, trRefresh, trReNumber,
                   trMoveItem,trMoveGroup];
  FSaveOptions  := [trSaveSplitter, trSaveViewChild];
  FEnabledMenus := StrToSet('0-9,11-12,14-15,17-18,20-21,');
  FNameGrID     := 'IDGrp';
  FOrderField   := 'NumOrder';
  FHashField    := 'Hash';
  FNAmeParentId := 'IdParent';
  FNameGroup := 'Name';
  FItemsName    := sohoTreeDefItemName;
  FGroupsName   := sohoTreeDefGroupName;
  FKeyField     := 'Id';
  FEditOnDblClick := true;
  FCloseGroups  := true;
  FItemId       := -1;
  FFixedGrpId   := -1;
  FFixedGrpName := '';
  FMarkedFont   := TFont.Create;
  FByFilter     := true;
  with FMarkedFont do begin
     Form := GetOwnerForm(Self);
     if Form <> nil then Assign(Form.Font);
     Color := clBlack;
  end;
  FMarkedColor := $00E5E5E5;

  FTreeViewForm := nil;
  FHideFields    := TStringList.Create;
  with FHideFields do begin
     Clear;
     Add('ID');
     Add('IDGRP');
     Add('HASH');
     Add('PROTECT');
     Add('MDFDATE');
     Add('MDFTIME');
     Add('IDMDFAUTH');
  end;
  FSQLHideFields := TStringList.Create;
  FComplexFields := TStringList.Create;
  FComplexLabels := TStringList.Create;
  FOkBtn         := nil;
  FCancelBtn     := nil;
  FSortC         := nil;
  FLastGroupId   := -1;
  FResults       := TsohoDBGridResults.Create;
  FClearResults  := true;
end;

destructor TsohoCustomTVContainer.Destroy;
begin
  FHideFields.Free;
  FSQLHideFields.Free;
  FComplexFields.Free;
  FComplexLabels.Free;
  FMarkedFont.Free;
  FResults.Free;
  inherited Destroy;
end;

procedure TsohoCustomTVContainer.PrepareButtons (Modal : boolean; WithCancel : boolean);
var BtnParent : TPanel;
begin
  if FOkBtn<>nil then FOkBtn.Free;
  if FCancelBtn<>nil then FCancelBtn.Free;
  FOkBtn     := nil;
  FCancelBtn := nil;

  BtnParent := FTreeViewForm.FTreeView.BtnsPanel;
  FSortC  := TsohoCheckBox.Create(FTreeViewForm);
  with FSortC do begin
    Parent  := BtnParent;
    Top     := FTreeViewForm.FTreeView.ChildCheckBox.Top + 15;
    Width   := BtnParent.Width-4;
    Left    := 2;
    Caption := sohoTVContSortBtnCaption;
    Hint    := sohoTVContSortBtnHint;
    OnChange := SortChanged;
  end;
  FOkBtn := TsohoBitBtn.Create(FTreeViewForm);
  with FOkBtn do begin
    Parent := BtnParent;
    Top    := FTreeViewForm.FTreeView.ChildCheckBox.Top + 35;
    Height := 24;
    Width  := BtnParent.Width-4;
    Left   := 2;
    if Modal then begin
      Caption := 'OK';
      Tag     := 1;
    end
    else begin
      Tag     := 0;
      Caption := sohoTVContBtnClose;
    end;
    OnClick := FTreeViewForm.SelectMClick;
  end;
  FTreeViewForm.CancelM.Visible := WithCancel;
  if not WithCancel then exit;
  FCancelBtn := TsohoBitBtn.Create(FTreeViewForm);
  with FCancelBtn do begin
    Parent  := BtnParent;
    Top     := FOkBtn.Top + FOkBtn.Height+1;
    Height  := 24;
    Width   := BtnParent.Width-4;
    Left    := 2;
    OnClick := FTreeViewForm.CancelMClick;
    Caption := sohoExlRepCancelBtnCaption;
  end;
end;

procedure TsohoCustomTVContainer.RestoreSelection;
var GroupNode: TTreeNode;
begin
  FTreeViewForm.FStorage.RestoreFormPlacement;
  FTreeViewForm.FTreeView.LoadTreeViewState;
  if FItemId > 0 then FTreeViewForm.FTreeView.ItemId := FItemId;
  if FLastGroupId > 0 then begin
     GroupNode := FTreeViewForm.FTreeView.GetNodeByID(FLastGroupId);
     if GroupNode <> nil then
       FTreeViewForm.FTreeView.GroupTreeView.Selected := GroupNode;
       FTreeViewForm.FTreeView.RefreshGroup;
  end
  else FTreeViewForm.FTreeView.RefreshGroup;
end;

function  TsohoCustomTVContainer.ReCreateExecute (Modal : boolean; WithCancel : boolean) : boolean;
begin
  PushCursor(crHourGlass);
  DoProgress (psStarting, 0, gdPreparing);
  if FTreeViewForm <> nil then begin
    {    ! }
    FTreeViewForm.Free;
  end;
  try
    DoProgress (psRunning, 33, gdFormCreate);
    FTreeViewForm := CreateTreeViewForm;
    with TreeViewForm do begin
        FContainer := Self;
        Result := not Modal;
        DoFormCreate(true, Modal, WithCancel);
        DoProgress (psRunning, 100, gdFormShow);
        if Modal then begin
          DoProgress (psEnding, 100, '');
          RestoreCursor;
          Result := ShowModal = mrOk
        end
        else begin
          RestoreCursor;
          Show;
          DoProgress (psEnding, 100, '');
        end;
    end;
  finally
    TreeViewForm.Free;
  end;
end;

function  TsohoCustomTVContainer.Execute (Reopen, Modal : boolean; WithCancel : boolean) : boolean;
begin
  Result := false;
  PushCursor(crHourGlass);
  DoProgress (psStarting, 0, gdPreparing);
  if TreeViewForm <> nil then begin
    if not Reopen then begin
      DoFormCreate(false, Modal, WithCancel);
      DoProgress (psRunning, 100, gdFormShow);
      if Modal then begin
        DoProgress (psEnding, 100, '');
        RestoreCursor;
        Result := FTreeViewForm.ShowModal = mrOk;
        DoFormDestroy(false);
      end
      else begin
        RestoreCursor;
        Result := true;
        FTreeViewForm.Show;
        DoProgress (psEnding, 100,'');
      end;
      exit;
    end
    else begin
      FTreeViewForm.Free;
      FTreeViewForm := nil;
    end;
  end;
  try
    DoProgress (psRunning, 33, gdFormCreate);
    FTreeViewForm := CreateTreeViewForm;
    with TreeViewForm do begin
        FContainer := Self;
        Result := not Modal;
        DoFormCreate(true, Modal ,WithCancel);
        DoProgress (psRunning, 100, gdFormShow);
        if Modal then begin
          DoProgress (psEnding, 100,'');
          RestoreCursor;
          Result := ShowModal = mrOk;
          DoFormDestroy(false);
        end
        else begin
          RestoreCursor;
          Show;
          DoProgress (psEnding, 100,'');
        end;
    end;
  except ErrorMsg(Format(sohoTVContFormCreateError, [Name]));
  end;
end;

function  TsohoCustomTVContainer.MDIExecute : boolean;
var Form : TForm;
    BtnParent : TPanel;
begin
  Result := false;
  PushCursor(crHourGlass);
  DoProgress (psStarting, 0, gdPreparing);
  if TreeViewForm<>nil then begin
    Result := true;
    TreeViewForm.BringToFront;
    exit;
  end;
  try
    Form := GetOwnerForm(Self);
    if Form = nil then exit;
    if Form.FormStyle <> fsMDIForm then exit;
    DoProgress (psRunning, 33, gdFormCreate);
    FTreeViewForm := CreateTreeViewForm;
    with TreeViewForm do begin
        FContainer := Self;
        FormStyle := fsMDIChild;
        DoProgress (psRunning, 60, gdSetEvents);
        SetEvents;
        FTreeViewForm.Caption := FCaption;
        FStorage.IniSection := FContainer.Name + '.TreeViewForm';
        DoProgress (psRunning, 99, gdRestoring);
        FTreeView.Active := true;
        RestoreSelection;
        ModalMode   := false;

        BtnParent := FTreeViewForm.FTreeView.BtnsPanel;
        FSortC  := TsohoCheckBox.Create(FTreeViewForm);
        with FSortC do begin
          Parent  := BtnParent;
          Top     := FTreeViewForm.FTreeView.ChildCheckBox.Top + 15;
          Width   := BtnParent.Width-4;
          Left    := 2;
          Caption := sohoTVContSortBtnCaption;
          Hint    := sohoTVContSortBtnHint;
          OnChange := SortChanged;
        end;

        if Assigned(FOnFormCreate) then FOnFormCreate(FTreeViewForm);
        DoProgress (psRunning, 100, gdFormShow);

        with FTreeViewForm.FDBAddon do begin
          SetEvents;
          SetDataSet;
        end;

        RestoreCursor;
        Show;
        DoProgress (psEnding, 100, '');
        Result := true;
    end;
  except ErrorMsg(Format(sohoTVContFormCreateError, [Name]));
  end;
end;

procedure TsohoCustomTVContainer.GridKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var ItemId: LongInt;
begin
  if Assigned(FOnGridKeyDown) then FOnGridKeyDown(Sender, Key, Shift);
  if Assigned(FTreeViewForm) then begin
    with FTreeViewForm, FTreeView.DBGrid do
     if (Key = vk_Space) and not
        (DataSource.DataSet.State in [dsEdit, dsInsert]) then begin
       ItemId := FItemsDS.FieldByName(FRecordForm.KeyField).AsInteger;
       if Results.IndexOf(ItemId)=-1 then Results.Add(ItemId)
       else Results.Remove(ItemId);
     end;
    if (Key = vk_Escape) and (FTreeViewForm.FormStyle <> fsMDIChild) then
      FTreeViewForm.Close;
  end;
end;

function TsohoCustomTVContainer.GetGauge: TsohoGauge;
begin
  Result := FGauge;
end;

procedure TsohoCustomTVContainer.SetGauge(const Value: TsohoGauge);
begin
  FGauge := Value;
end;

procedure TsohoCustomTVContainer.SortChanged(Sender: TObject);
begin
  FTreeViewForm.FDBAddon.Active := FSortC.Checked;
end;

end.
