{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoDBCtrls;

interface

{$I SOHOLIB.INC}

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
     SoCtrls, DBCtrls, DB;

type

  TsohoDBEdit = class(TsohoCustomEdit)
{  TsohoCustomEdit  DBaware . }
  private
    { Private declarations }
    FDataLink: TFieldDataLink;
    FCanvas: TControlCanvas;
    FAlignment: TAlignment;
    FFocused: Boolean;
FAutoSetValueKind: boolean; // ## 30.07.2000
  protected
    { Protected declarations }
    procedure ActiveChange(Sender: TObject);
    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetReadOnly: Boolean;
    // function GetTextMargins: TPoint;
    procedure ResetMaxLength;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetFocused(Value: Boolean);
    procedure SetReadOnly(Value: Boolean);
    procedure UpdateData(Sender: TObject);
    procedure WMCut(var Message: TMessage); message WM_CUT;
    procedure WMPaste(var Message: TMessage); message WM_PASTE;
    procedure WMUndo(var Message: TMessage); message WM_UNDO;
    // procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;

    procedure Change; override;
    // function EditCanModify: Boolean; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure CheckValueKind;
    procedure DoRightClick(Sender: TObject); override;
    // procedure Reset; override;
  public
    { Public declarations }
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Loaded; override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    {$IFDEF Delphi4_or_5}
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    function UseRightToLeftAlignment: Boolean; override;
    {$ENDIF}
    property Field: TField read GetField;
  published
    property EditStyle;
    property Shadow;
    property HighLight;
    property ShowMessage;
    property FromChars;
    property ToChars;
    property RightControl;
    property CenterDialog default true;
    property Multiline;
    property DisplayFormat;
    property ValueKind;
    property BtnGlyph;
    property BtnHint;
    property BtnNumGlyphs;
    property Images;
    property ImageIndex;
property AutoSetValueKind: boolean read FAutoSetValueKind write FAutoSetValueKind default true; // ## 30.07.2000
    property AutoSelect;
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnUndo;
    property OnModify;
    property OnRightClick;
{$IFDEF Delphi4_or_5}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
{$ENDIF}
{$IFDEF Delphi5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF Delphi4_or_5}
    property OnResize;
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
  end;

implementation
{ TsohoDBEdit }

procedure TsohoDBEdit.ActiveChange(Sender: TObject);
begin
  ResetMaxLength;
  CheckValueKind;
end;

procedure TsohoDBEdit.Change;
begin
  FDataLink.Modified;
  inherited Change;
end;

procedure TsohoDBEdit.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

constructor TsohoDBEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  inherited ReadOnly := True;
  ControlStyle := ControlStyle + [csReplicatable];
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
  FDataLink.OnActiveChange := ActiveChange;
FAutoSetValueKind := true;
end;

{$HINTS OFF}
type

  THackFieldDataLink = class(TDataLink)
  private
    FField: TField;
    FFieldName: string;
    FControl: TComponent;
    FEditing: Boolean;
    FModified: Boolean;
  end;
{$HINTS ON}

procedure TsohoDBEdit.DataChange(Sender: TObject);
begin
  if Assigned(FDataLink.Field) then begin
    if FAlignment <> FDataLink.Field.Alignment then begin
      // EditText := '';  {forces update}
      FAlignment := FDataLink.Field.Alignment;
    end;
    // EditMask := FDataLink.Field.EditMask;
    if not (csDesigning in ComponentState) then begin
      if (FDataLink.Field.DataType in [ftString{$IFDEF Delphi4_or_5}, ftWideString{$ENDIF}]) and
         (MaxLength = 0) then MaxLength := FDataLink.Field.Size;
    end;
    if FFocused and FDataLink.CanModify then Text := FDataLink.Field.Text
    else begin
      Text := FDataLink.Field.DisplayText;
      if FDataLink.Editing and THackFieldDataLink(FDataLink).FModified then
        Modified := True;
    end;
  end
  else begin
    FAlignment := taLeftJustify;
    // EditMask := '';
    if csDesigning in ComponentState then Text := Name
    else Text := '';
  end;
end;

destructor TsohoDBEdit.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FCanvas.Free;
  inherited Destroy;
end;

{function TsohoDBEdit.EditCanModify: Boolean;
begin
  Result := FDataLink.Edit;
end;}

procedure TsohoDBEdit.EditingChange(Sender: TObject);
begin
  inherited ReadOnly := not FDataLink.Editing;
end;

{$IFDEF Delphi4_or_5}
function TsohoDBEdit.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;
{$ENDIF}

function TsohoDBEdit.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

function TsohoDBEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TsohoDBEdit.GetField: TField;
begin
  Result := FDataLink.Field;
end;

function TsohoDBEdit.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

{function TsohoDBEdit.GetTextMargins: TPoint;
begin

end;}

procedure TsohoDBEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if (Key = VK_DELETE) or ((Key = VK_INSERT) and (ssShift in Shift)) then
    FDataLink.Edit;
end;

procedure TsohoDBEdit.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  if (Key in [#32..#255]) and (FDataLink.Field <> nil) and
    not FDataLink.Field.IsValidChar(Key) then begin
    MessageBeep(0);
    Key := #0;
  end;
  case Key of
    ^H, ^V, ^X, #32..#255: FDataLink.Edit;
    #27: begin
      FDataLink.Reset;
      SelectAll;
      Key := #0;
    end;
  end;
end;

procedure TsohoDBEdit.Loaded;
begin
  inherited Loaded;
  ResetMaxLength;
  if (csDesigning in ComponentState) then DataChange(Self);
end;

procedure TsohoDBEdit.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TsohoDBEdit.ResetMaxLength;
var F: TField;
begin
  if (MaxLength > 0) and Assigned(DataSource) and
     Assigned(DataSource.DataSet) then begin
    F := DataSource.DataSet.FindField(DataField);
    if Assigned(F) and (F.DataType in [ftString{$IFDEF Delphi4_or_5}, ftWideString{$ENDIF}]) and
       (F.Size = MaxLength) then MaxLength := 0;
  end;
end;

procedure TsohoDBEdit.CheckValueKind;
begin
  if (not Assigned(FDataLink)) or
     (not Assigned(FDataLink.Field)) then exit;
 if not FAutoSetValueKind then exit;
  case FDataLink.Field.DataType of    //
    ftSmallint, ftInteger, ftWord
    {$IFDEF Delphi5},ftLargeInt{$ENDIF}: ValueKind := vkInteger;
    ftFloat, ftCurrency: ValueKind := vkFloat;
    ftTime: ValueKind := vkTime;
    ftDate: ValueKind := vkDate;
    ftDateTime: ValueKind := vkDateTime;
    else ValueKind := vkString;
  end;    // case
end;

procedure TsohoDBEdit.SetDataField(const Value: string);
begin
  if not (csDesigning in ComponentState) then ResetMaxLength;
  FDataLink.FieldName := Value;
  if Assigned(FDataLink.Field) then CheckValueKind;
end;

procedure TsohoDBEdit.SetDataSource(Value: TDataSource);
begin
  if not (FDataLink.DataSourceFixed and
    (csLoading in ComponentState)) then FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

procedure TsohoDBEdit.SetFocused(Value: Boolean);
begin
  if FFocused <> Value then begin
    FFocused := Value;
    // if (FAlignment <> taLeftJustify) and not IsMasked then Invalidate;
    // FDataLink.Reset;
  end;
end;

procedure TsohoDBEdit.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

{$IFDEF Delphi4_or_5}
function TsohoDBEdit.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;
{$ENDIF}

procedure TsohoDBEdit.UpdateData(Sender: TObject);
begin
  // ValidateEdit;
  FDataLink.Field.Text := Text;
end;

{$IFDEF Delphi4_or_5}
function TsohoDBEdit.UseRightToLeftAlignment: Boolean;
begin
  Result := DBUseRightToLeftAlignment(Self, Field);
end;
{$ENDIF}

procedure TsohoDBEdit.WMCut(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TsohoDBEdit.CMEnter(var Message: TCMEnter);
begin
  SetFocused(True);
  inherited;
  if SysLocale.FarEast and FDataLink.CanModify then
    inherited ReadOnly := False;
end;

procedure TsohoDBEdit.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    SelectAll;
    SetFocus;
    raise;
  end;
  SetFocused(False);
  // CheckCursor;
  DoExit;
end;

{procedure TsohoDBEdit.WMPaint(var Message: TWMPaint);
begin

end;}

procedure TsohoDBEdit.WMPaste(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TsohoDBEdit.WMUndo(var Message: TMessage);
begin
  FDataLink.Edit;
  inherited;
end;

procedure TsohoDBEdit.DoRightClick(Sender: TObject);
var OldText, NewText: string;
begin
  OldText := Text;
  inherited;
  NewText := Text;
  if (NewText <> OldText) and (FDataLink.CanModify) then begin
   FDataLink.Edit;
   FDataLink.Field.AsString := NewText;
  end;
end;

end.