{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoDBGuide;

{$I SOHOLIB.INC}
interface
uses Classes, SoCtmDBGuide, SohoRec, SoFolder, SoDBGridAddon, DB;

type

  TsohoBDEGuideForm = class(TsohoCtmDBGuideForm)
  public
    constructor CreateGuide(aOwner: TComponent; DBGuide : TsohoCustomDBGuide); override;
  end;

  TsohoDBGuide = class(TsohoCustomDBGuide)
  {        
    ,    .     
         -    
    . ,      
    .         
      MDIChild.        
      - TsohoDBGrid, TsohoRecordForm  TsohoFolder. 
      TsohoDBGuide -      
     .}
  private
    FDatabaseName: string;
  protected
    function CreateGuideForm: TsohoCtmDBGuideForm; override;
    procedure SetEvents; override;
    procedure SetFolderProperties; override;
    procedure SetRecordProperties; override;
    procedure SetQueryProperties; override;
    procedure SetDBGridAddonProperties; override;
    function DoDeleteRecord : boolean; override;
    procedure AfterEditRecord; override;
  published
    property DatabaseName: String read FDatabaseName write FDatabaseName;
{      }
  end;

implementation
uses SoBDERtn, DBTables, SoCtmRgs, Forms, SoStrUtils, SysUtils;

{ TsohoDBGuideForm }

constructor TsohoBDEGuideForm.CreateGuide(aOwner: TComponent; DBGuide : TsohoCustomDBGuide);
begin
  inherited;
  FRecordForm := TsohoRecordForm.Create(Self);
  FFolder := TsohoFolder.Create(Self);
  FGuideDS := TQuery.Create(Self);
  FDBAddon := TsohoDBGridAddon.Create(Self);
end;

{ TsohoDBGuide }

procedure TsohoDBGuide.AfterEditRecord;
begin
  // nothing to do
end;

function TsohoDBGuide.CreateGuideForm: TsohoCtmDBGuideForm;
begin
  Result := TsohoBDEGuideForm.CreateGuide(Application, Self);
end;

function TsohoDBGuide.DoDeleteRecord: boolean;
var SQLText : string;
begin
  if ExtractFileExt(FolderTable) <> EmptyStr then
    SQLText := 'delete from ' + SingleQuotes(FolderTable)
  else
    SQLText := 'delete from ' + FolderTable;
  SQLText := SQLText +' where ' +
    KeyField + '= ' + GuideForm.GuideDS.FieldByName(KeyField).AsString;
  Result := ExecuteSQLText(SQLText, FDatabaseName, NillParams);
end;

procedure TsohoDBGuide.SetDBGridAddonProperties;
begin
  inherited;
  GuideForm.DBAddon.Grid := GuideForm.GuideGrid; 
end;

procedure TsohoDBGuide.SetEvents;
begin
  if (FDataBaseName = EmptyStr) and (SingleRegister <> nil) then
     FDataBaseName := SingleRegister.PathToData;
  GuideForm.GuideSource.DataSet := GuideForm.GuideDS;
  inherited;
end;

procedure TsohoDBGuide.SetFolderProperties;
begin
  inherited;
  with GuideForm do begin
    if Folder.Grid = GuideGrid then exit;
    TsohoFolder(Folder).DataBaseName := FDataBaseName;
    Folder.Grid := GuideGrid;
    Folder.Activate;
  end
end;

procedure TsohoDBGuide.SetQueryProperties;
var FSQL: string;
begin
  inherited;
  with GuideForm do begin
    if GuideDS.Active then exit;
    FSQL := SQLFile;
    if SingleRegister <> nil then
      FSQL := SingleRegister.PathToSQL + SQLFile;

    with TQuery(GuideDS), SQL do begin
      DataBaseName := FDataBaseName;
      LoadFromFile(FSQL);
    end;
  end;
end;

procedure TsohoDBGuide.SetRecordProperties;
begin
  inherited;
  with GuideForm do begin
    TsohoRecordForm(RecordForm).DataBaseName := FDataBaseName;
    TsohoRecordForm(RecordForm).Folder := Folder;
  end;
end;

end.
 