{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
     TsohoDBNavigator
}
unit SoDBNav;

{$I SOHOLIB.INC}

interface

uses Windows, Messages, Classes, DB, Graphics, ExtCtrls, Buttons, Controls
     {$IFDEF Delphi4_or_5}, Imglist{$ENDIF};

const
  InitRepeatPause = 400;  { pause before repeat timer (ms) }
  RepeatPause     = 100;  { pause before hint window displays (ms)}
  SpaceSize       =  5;   { size of space between special buttons }

type

  TsohoNavButton = class;
  TsohoNavDataLink = class;

  TsohoNavGlyph = (ngEnabled, ngDisabled);
  TsohoNavigateBtn = (snbFirst, snbPrior, snbNext, snbLast,
                  snbInsert, snbDelete, snbEdit, snbPost, snbCancel, snbRefresh);
  TButtonSet = set of TsohoNavigateBtn;
  TsohoNavButtonStyle = set of (nsAllowTimer, nsFocusRect);

  TsohoNavClick = procedure (Sender: TObject; Button: TsohoNavigateBtn) of object;
{   OnClick.  Button ,      }

  TsohoDBNavigator = class (TCustomPanel)
{   ,     TDBNavigator       TImageList
      -  .}
  private
    FDataLink: TsohoNavDataLink;
    FVisibleButtons: TButtonSet;
    FHints: TStrings;
    ButtonWidth: Integer;
    MinBtnSize: TPoint;
    FOnNavClick: TsohoNavClick;
    FBeforeAction: TsohoNavClick;
    FocusedButton: TsohoNavigateBtn;
    FConfirmDelete: Boolean;
    FFlat: Boolean;
    procedure ClickHandler(Sender: TObject);
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure InitButtons;
    procedure InitHints;
    procedure BtnMouseDown (Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SetVisible(Value: TButtonSet);
    procedure NavigatorAdjustSize (var W: Integer; var H: Integer);
    procedure HintsChanged(Sender: TObject);
    procedure SetHints(Value: TStrings);
    procedure SetFlat(Value: Boolean);
    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  protected
    Buttons: array[TsohoNavigateBtn] of TsohoNavButton;
    procedure DataChanged;
    procedure EditingChanged;
    procedure ActiveChanged;
    procedure Loaded; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure BtnClick(Index: TsohoNavigateBtn);
    procedure RefreshButtons;
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property VisibleButtons: TButtonSet read FVisibleButtons write SetVisible
      default [snbFirst, snbPrior, snbNext, snbLast, snbInsert, snbDelete,
        snbEdit, snbPost, snbCancel, snbRefresh];
    property Align;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Flat: Boolean read FFlat write SetFlat default False;
    property Ctl3D;
    property Hints: TStrings read FHints write SetHints;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property ConfirmDelete: Boolean read FConfirmDelete write FConfirmDelete default True;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property BeforeAction: TsohoNavClick read FBeforeAction write FBeforeAction;
    property OnClick: TsohoNavClick read FOnNavClick write FOnNavClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnStartDrag;
    property OnResize;
{$IFDEF Delphi4_or_5}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property ImeMode;
    property ImeName;
{$ENDIF}
{$IFDEF Delphi5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF Delphi4_or_5}
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
  end;

{ TsohoNavButton }

  TsohoNavButton = class(TSpeedButton)
  private
    FIndex: TsohoNavigateBtn;
    FNavStyle: TsohoNavButtonStyle;
    FRepeatTimer: TTimer;
    FIsMouse: boolean;
    FDown: boolean;
    procedure TimerExpired(Sender: TObject);
  protected
    procedure Paint; override;
    procedure CMMouseEnter(var message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var message: TMessage); message CM_MOUSELEAVE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  public
    destructor Destroy; override;
    property NavStyle: TsohoNavButtonStyle read FNavStyle write FNavStyle;
    property Index : TsohoNavigateBtn read FIndex write FIndex;
  end;

{ TsohoNavDataLink }

  TsohoNavDataLink = class(TDataLink)
  private
    FNavigator: TsohoDBNavigator;
  protected
    procedure EditingChanged; override;
    procedure DataSetChanged; override;
    procedure ActiveChanged; override;
  public
    constructor Create(ANav: TsohoDBNavigator);
    destructor Destroy; override;
  end;

var BtnImages: TImageList;

implementation
uses SohoBtns, SoVCLUtils, CommCtrl, SoDBCns;

{$R SODBNAV.R32}

procedure TsohoDBNavigator.RefreshButtons;
begin
  DataChanged;
  EditingChanged;
end;

var
  BtnTypeName: array[TsohoNavigateBtn] of PChar = ('FIRST', 'PRIOR', 'NEXT',
    'LAST', 'INSERT', 'DELETE', 'EDIT', 'POST', 'CANCEL', 'REFRESH');
  BtnHintId: array[TsohoNavigateBtn] of string = (sohoNavFirst, sohoNavPrior,
    sohoNavNext, sohoNavLast, sohoNavInsert, sohoNavDelete, sohoNavEdit,
    sohoNavPost, sohoNavCancel, sohoNavRefresh);

constructor TsohoDBNavigator.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csAcceptsControls, csSetCaption] + [csOpaque];
  if not NewStyleControls then ControlStyle := ControlStyle + [csFramed];
  FDataLink := TsohoNavDataLink.Create(Self);
  FVisibleButtons := [snbFirst, snbPrior, snbNext, snbLast, snbInsert,
    snbDelete, snbEdit, snbPost, snbCancel, snbRefresh];
  FHints := TStringList.Create;
  TStringList(FHints).OnChange := HintsChanged;
  InitButtons;
  BevelOuter := bvNone;
  BevelInner := bvNone;
  Width := 241;
  Height := 25;
  ButtonWidth := 0;
  FocusedButton := snbFirst;
  FConfirmDelete := True;
  FullRepaint := False;
end;

destructor TsohoDBNavigator.Destroy;
begin
  FDataLink.Free;
  FHints.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TsohoDBNavigator.InitButtons;
var
  I: TsohoNavigateBtn;
  Btn: TsohoNavButton;
  X: Integer;
begin
  MinBtnSize := Point(20, 18);
  X := 0;
  for I := Low(Buttons) to High(Buttons) do
  begin
    Btn := TsohoNavButton.Create (Self);
    Btn.Flat := Flat;
    Btn.Index := I;
    Btn.Visible := I in FVisibleButtons;
    Btn.Enabled := True;
    Btn.SetBounds (X, 0, MinBtnSize.X, MinBtnSize.Y);
    // Btn.NumGlyphs := 1;
    Btn.Enabled := False;
    Btn.Enabled := True;
    Btn.OnClick := ClickHandler;
    Btn.OnMouseDown := BtnMouseDown;
    Btn.Parent := Self;
    Buttons[I] := Btn;
    X := X + MinBtnSize.X;
  end;
  InitHints;
  Buttons[snbPrior].NavStyle := Buttons[snbPrior].NavStyle + [nsAllowTimer];
  Buttons[snbNext].NavStyle  := Buttons[snbNext].NavStyle + [nsAllowTimer];
end;

procedure TsohoDBNavigator.InitHints;
var
  I: Integer;
  J: TsohoNavigateBtn;
begin
  for J := Low(Buttons) to High(Buttons) do Buttons[J].Hint := BtnHintId[J];
  J := Low(Buttons);
  for I := 0 to (FHints.Count - 1) do begin
    if FHints.Strings[I] <> '' then Buttons[J].Hint := FHints.Strings[I];
    if J = High(Buttons) then Exit;
    Inc(J);
  end;
end;

procedure TsohoDBNavigator.HintsChanged(Sender: TObject);
begin
  InitHints;
end;

procedure TsohoDBNavigator.SetFlat(Value: Boolean);
var
  I: TsohoNavigateBtn;
begin
  if FFlat <> Value then begin
    FFlat := Value;
    for I := Low(Buttons) to High(Buttons) do Buttons[I].Flat := Value;
  end;
end;

procedure TsohoDBNavigator.SetHints(Value: TStrings);
begin
  FHints.Assign(Value);
end;

procedure TsohoDBNavigator.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

procedure TsohoDBNavigator.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TsohoDBNavigator.SetVisible(Value: TButtonSet);
var
  I: TsohoNavigateBtn;
  W, H: Integer;
begin
  W := Width;
  H := Height;
  FVisibleButtons := Value;
  for I := Low(Buttons) to High(Buttons) do
    Buttons[I].Visible := I in FVisibleButtons;
  NavigatorAdjustSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds (Left, Top, W, H);
  Invalidate;
end;

procedure TsohoDBNavigator.NavigatorAdjustSize (var W: Integer; var H: Integer);
var
  Count: Integer;
  MinW: Integer;
  I: TsohoNavigateBtn;
  Space, Temp, Remain: Integer;
  X: Integer;
begin
  if (csLoading in ComponentState) then Exit;
  if Buttons[snbFirst] = nil then Exit;

  Count := 0;
  for I := Low(Buttons) to High(Buttons) do
    if Buttons[I].Visible then Inc(Count);
  if Count = 0 then Inc(Count);

  MinW := Count * MinBtnSize.X;
  if W < MinW then W := MinW;
  if H < MinBtnSize.Y then H := MinBtnSize.Y;

  ButtonWidth := W div Count;
  Temp := Count * ButtonWidth;
  if Align = alNone then W := Temp;

  X := 0;
  Remain := W - Temp;
  Temp := Count div 2;
  for I := Low(Buttons) to High(Buttons) do begin
    if Buttons[I].Visible then begin
      Space := 0;
      if Remain <> 0 then begin
        Dec(Temp, Remain);
        if Temp < 0 then begin
          Inc(Temp, Count);
          Space := 1;
        end;
      end;
      Buttons[I].SetBounds(X, 0, ButtonWidth + Space, Height);
      Inc(X, ButtonWidth + Space);
    end
    else
      Buttons[I].SetBounds (Width + 1, 0, ButtonWidth, Height);
  end;
end;

procedure TsohoDBNavigator.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  W, H: Integer;
begin
  W := AWidth;
  H := AHeight;
  if not HandleAllocated then NavigatorAdjustSize (W, H);
  inherited SetBounds (ALeft, ATop, W, H);
end;

procedure TsohoDBNavigator.WMSize(var Message: TWMSize);
var
  W, H: Integer;
begin
  inherited;
  { check for minimum size }
  W := Width;
  H := Height;
  NavigatorAdjustSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds(Left, Top, W, H);
  Message.Result := 0;
end;

procedure TsohoDBNavigator.ClickHandler(Sender: TObject);
begin
  BtnClick (TsohoNavButton (Sender).Index);
end;

procedure TsohoDBNavigator.BtnMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  OldFocus: TsohoNavigateBtn;
begin
  OldFocus := FocusedButton;
  FocusedButton := TsohoNavButton (Sender).Index;
  if TabStop and (GetFocus <> Handle) and CanFocus then begin
    SetFocus;
    if (GetFocus <> Handle) then exit;
  end
  else if TabStop and (GetFocus = Handle) and (OldFocus <> FocusedButton) then begin
    Buttons[OldFocus].Invalidate;
    Buttons[FocusedButton].Invalidate;
  end;
end;

procedure TsohoDBNavigator.BtnClick(Index: TsohoNavigateBtn);
begin
  if (DataSource <> nil) and (DataSource.State <> dsInactive) then begin
    if not (csDesigning in ComponentState) and Assigned(FBeforeAction) then
      FBeforeAction(Self, Index);
    with DataSource.DataSet do begin
      case Index of
        snbPrior: Prior;
        snbNext: Next;
        snbFirst: First;
        snbLast: Last;
        snbInsert: Insert;
        snbEdit: Edit;
        snbCancel: Cancel;
        snbPost: Post;
        snbRefresh: Refresh;
        snbDelete:
          if not FConfirmDelete or
            (FConfirmDelete and OkCancelMsg(sohoDBNavDeleteRecord)) then Delete;
      end;
    end;
  end;
  if not (csDesigning in ComponentState) and Assigned(FOnNavClick) then
    FOnNavClick(Self, Index);
end;

procedure TsohoDBNavigator.WMSetFocus(var Message: TWMSetFocus);
begin
  Buttons[FocusedButton].Invalidate;
end;

procedure TsohoDBNavigator.WMKillFocus(var Message: TWMKillFocus);
begin
  Buttons[FocusedButton].Invalidate;
end;

procedure TsohoDBNavigator.KeyDown(var Key: Word; Shift: TShiftState);
var
  NewFocus: TsohoNavigateBtn;
  OldFocus: TsohoNavigateBtn;
begin
  OldFocus := FocusedButton;
  case Key of
    VK_RIGHT:
      begin
        NewFocus := FocusedButton;
        repeat
          if NewFocus < High(Buttons) then
            NewFocus := Succ(NewFocus);
        until (NewFocus = High(Buttons)) or (Buttons[NewFocus].Visible);
        if NewFocus <> FocusedButton then
        begin
          FocusedButton := NewFocus;
          Buttons[OldFocus].Invalidate;
          Buttons[FocusedButton].Invalidate;
        end;
      end;
    VK_LEFT:
      begin
        NewFocus := FocusedButton;
        repeat
          if NewFocus > Low(Buttons) then
            NewFocus := Pred(NewFocus);
        until (NewFocus = Low(Buttons)) or (Buttons[NewFocus].Visible);
        if NewFocus <> FocusedButton then
        begin
          FocusedButton := NewFocus;
          Buttons[OldFocus].Invalidate;
          Buttons[FocusedButton].Invalidate;
        end;
      end;
    VK_SPACE:
      begin
        if Buttons[FocusedButton].Enabled then
          Buttons[FocusedButton].Click;
      end;
  end;
end;

procedure TsohoDBNavigator.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;

procedure TsohoDBNavigator.DataChanged;
var
  UpEnable, DnEnable: Boolean;
begin
  UpEnable := Enabled and FDataLink.Active and not FDataLink.DataSet.BOF;
  DnEnable := Enabled and FDataLink.Active and not FDataLink.DataSet.EOF;
  Buttons[snbFirst].Enabled := UpEnable;
  Buttons[snbPrior].Enabled := UpEnable;
  Buttons[snbNext].Enabled := DnEnable;
  Buttons[snbLast].Enabled := DnEnable;
  Buttons[snbDelete].Enabled := Enabled and FDataLink.Active and
    FDataLink.DataSet.CanModify and
    not (FDataLink.DataSet.BOF and FDataLink.DataSet.EOF);
end;

procedure TsohoDBNavigator.EditingChanged;
var
  CanModify: Boolean;
begin
  CanModify := Enabled and FDataLink.Active and FDataLink.DataSet.CanModify;
  Buttons[snbInsert].Enabled := CanModify;
  Buttons[snbEdit].Enabled := CanModify and not FDataLink.Editing;
  Buttons[snbPost].Enabled := CanModify and FDataLink.Editing;
  Buttons[snbCancel].Enabled := CanModify and FDataLink.Editing;
  Buttons[snbRefresh].Enabled := CanModify;
end;

procedure TsohoDBNavigator.ActiveChanged;
var
  I: TsohoNavigateBtn;
begin
  if not (Enabled and FDataLink.Active) then
    for I := Low(Buttons) to High(Buttons) do
      Buttons[I].Enabled := False
  else
  begin
    DataChanged;
    EditingChanged;
  end;
end;

procedure TsohoDBNavigator.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if not (csLoading in ComponentState) then
    ActiveChanged;
end;

procedure TsohoDBNavigator.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if not (csLoading in ComponentState) then
    ActiveChanged;
  if Value <> nil then Value.FreeNotification(Self);
end;

function TsohoDBNavigator.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TsohoDBNavigator.Loaded;
var
  W, H: Integer;
begin
  inherited Loaded;
  W := Width;
  H := Height;
  NavigatorAdjustSize (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds (Left, Top, W, H);
  InitHints;
  ActiveChanged;
end;

{TsohoNavButton}
procedure TsohoNavButton.CMMouseEnter(var message: TMessage);
begin
  inherited;
  FIsMouse := true;
  Invalidate;
end;

procedure TsohoNavButton.CMMouseLeave(var message: TMessage);
begin
  inherited;
  FIsMouse := false;
  Invalidate;
end;

destructor TsohoNavButton.Destroy;
begin
  if FRepeatTimer <> nil then FRepeatTimer.Free;
  inherited Destroy;
end;

procedure TsohoNavButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  FDown := true;
  inherited MouseDown (Button, Shift, X, Y);
  if nsAllowTimer in FNavStyle then
  begin
    if FRepeatTimer = nil then FRepeatTimer := TTimer.Create(Self);
    FRepeatTimer.OnTimer := TimerExpired;
    FRepeatTimer.Interval := InitRepeatPause;
    FRepeatTimer.Enabled  := True;
  end;
end;

procedure TsohoNavButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
                                  X, Y: Integer);
begin
  FDown := false;
  inherited MouseUp (Button, Shift, X, Y);
  if FRepeatTimer <> nil then begin
    FRepeatTimer.Enabled  := False;
    FRepeatTimer.Free;
    FRepeatTimer := nil;
  end;
end;

procedure TsohoNavButton.TimerExpired(Sender: TObject);
begin
  FRepeatTimer.Interval := RepeatPause;
  if (FState = bsDown) and MouseCapture then
  begin
    try
      Click;
    except
      FRepeatTimer.Enabled := False;
      FRepeatTimer.Free;
      FRepeatTimer := nil;
      raise;
    end;
  end;
end;

procedure TsohoNavButton.Paint;
var
  R, TextBounds: TRect;
  IsFocused: boolean;
  TmpGlyph: TButtonGlyph;
  GlyphPos: TPoint;
  DisOffset: integer;
begin
  // inherited Paint;
  R := Bounds(0, 0, Width, Height);
  DisOffset := 0;
  if not Enabled then Inc(DisOffset);

  TmpGlyph := TButtonGlyph.Create;
  BtnImages.GetBitmap(2*ord(FIndex) + DisOffset, TmpGlyph.Glyph);
  TmpGlyph.Color := TmpGlyph.Glyph.Canvas.Pixels[0, TmpGlyph.Glyph.Height - 1];

  IsFocused := (GetFocus = Parent.Handle) and
     (FIndex = TsohoDBNavigator (Parent).FocusedButton);

  if not Flat then
    NewDrawButtonFace (Canvas, R, Color, FIsMouse and Enabled, FDown, IsFocused, false, false)
  else begin
    if FIsMouse or FDown then
      DrawButtonFace(Canvas, R, Color, 0, bsNew, false, FDown, IsFocused);
  end;
  TmpGlyph.CalcButtonLayout(Canvas, R, Caption, blGlyphLeft, -1,
    0, GlyphPos, TextBounds);
  ImageList_DrawEx(BtnImages.Handle, 2*ord(FIndex) + DisOffset, Canvas.Handle,
    GlyphPos.X, GlyphPos.Y, 0, 0, CLR_NONE, CLR_NONE, ILD_TRANSPARENT);
  TmpGlyph.Free;

  if IsFocused then begin
    R := Bounds(0, 0, Width, Height);
    InflateRect(R, -3, -3);
    if FState = bsDown then
      OffsetRect(R, 1, 1);
    DrawFocusRect(Canvas.Handle, R);
  end;
end;

{ TsohoNavDataLink }

constructor TsohoNavDataLink.Create(ANav: TsohoDBNavigator);
begin
  inherited Create;
  FNavigator := ANav;
end;

destructor TsohoNavDataLink.Destroy;
begin
  FNavigator := nil;
  inherited Destroy;
end;

procedure TsohoNavDataLink.EditingChanged;
begin
  if FNavigator <> nil then FNavigator.EditingChanged;
end;

procedure TsohoNavDataLink.DataSetChanged;
begin
  if FNavigator <> nil then FNavigator.DataChanged;
end;

procedure TsohoNavDataLink.ActiveChanged;
begin
  if FNavigator <> nil then FNavigator.ActiveChanged;
end;

procedure InitImagesForButtons;
begin
  BtnImages := TImageList.Create(nil);
  BtnImages.Width := 14;
  BtnImages.Height := 13;
  BtnImages.ResourceLoad(rtBitmap, 'SOHONAVBTNS', clAqua);
end;

initialization
  InitImagesForButtons;
finalization
  BtnImages.Free;
end.
