{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
          ,
      
}
unit SoDate;

{$I SOHOLIB.INC}
{$DEFINE RUSSIAN_DEF}

interface
uses SysUtils, Forms, Classes;

function GetNullDateStr: string;
{  ,  "" , : '00/00/00' }
function DateToSQLDate(Date: TDateTime): string;
{     mm/dd/yyyy     SQL-.
      ,      
   }
function IsNullDate(const Text: string): Boolean;
{    ?      GetNullDateStr, 
     }
function WStrToDate(const Date: string): TDateTime;
{    .      exception }
function WStrToDateTime(const DateTime: string): TDateTime;
{    /.      exception }
function WDateTimeToStr(Date: TDateTime): string;
{      .       GetNullDateStr }
function WDateToStr(Date: TDateTime): string;
{    .       GetNullDateStr }
function LongDateToStr(Date: TDateTime): string;
{    ,       }
function GetPeriod(var FromDate, ToDate: TDateTime): Boolean;
{        }
function GetMaxDate(var ToDate: TDateTime): Boolean;
{       . ,
         - 
       }
procedure GetDatesOnDay(var FromDate, ToDate: TDateTime);
{        }
procedure GetDatesOnWeek(var FromDate, ToDate: TDateTime);
{         }
procedure GetDatesOnMonth(var FromDate, ToDate: TDateTime);
{         }
procedure GetDatesOnQuarter(var FromDate, ToDate: TDateTime);
{         }
procedure GetDatesOnYear(var FromDate, ToDate: TDateTime);
{         }
function TimeToShortTime (Time : TDateTime) : string;
{         Time }

{$IFDEF RUSSIAN_DEF}
var
  sohoShortDayNames: array[1..7] of string;
{      ,   . , ,   ..  }
  sohoLongDayNames: array[1..7] of string;
{      ,   . , ,   ..  }
{$ENDIF}

implementation
uses SoPrdF, SoStrUtils;

function GetNullDateStr: string;
begin
  Result := '00' + DateSeparator + '00' + DateSeparator + '00';
end;

function DateToSQLDate(Date: TDateTime): string;
var Year, Month, Day: Word;
begin
  if trunc(Date) = 0 then Result := 'null'
  else begin
    DecodeDate(Date, Year, Month, Day);
    Result := SingleQuotes(IntToStr(Month) + '/' + IntToStr(Day) + '/' + IntToStr(Year));
  end;
end;

function IsNullDate(const Text: string): Boolean;
begin
  Result := (GetNoSpacePercent(Text) = 0) or (Text = GetNullDateStr);
end;

type
  TDateOrder = (doMDY, doDMY, doYMD);

function GetDateOrder(const DateFormat: string): TDateOrder;
var
  I: Integer;
begin
  Result := doMDY;
  I := 1;
  while I <= Length(DateFormat) do
  begin
    case Chr(Ord(DateFormat[I]) and $DF) of
      'E': Result := doYMD;
      'Y': Result := doYMD;
      'M': Result := doMDY;
      'D': Result := doDMY;
    else
      Inc(I);
      Continue;
    end;
    Exit;
  end;
  Result := doMDY;
end;

function DecodeStrToDate(const aDate: string): TDateTime;
var Order: TDateOrder;
    S1, S2, S3, sDay, sMonth, sYear, SMonthFormat: string;
    iDay, iMonth, iYear: integer;
    wDay, wMonth, wYear: word;
    I: Integer;
    PartNames: array [1..12] of string;
begin
  Result := 0.0;
  Order := GetDateOrder(ShortDateFormat);
  SetDivisers([DateSeparator]);

  if WordsInString(aDate) <> 3 then Exit;

  S1 := GetSomeWords(aDate, 1, 1);
  S2 := GetSomeWords(aDate, 2, 2);
  S3 := GetSomeWords(aDate, 3, 3);
  case Order of
    doMDY: begin
      sDay := S2;
      sMonth := S1;
      sYear := S3;
    end;
    doDMY: begin
      sDay := S1;
      sMonth := S2;
      sYear := S3;
    end;
    doYMD: begin
      sDay := S3;
      sMonth := S2;
      sYear := S1;
    end;
  end;

  S1 := GetSomeWords(ShortDateFormat, 1, 1);
  S2 := GetSomeWords(ShortDateFormat, 2, 2);
  S3 := GetSomeWords(ShortDateFormat, 3, 3);

  case Order of
    doMDY: SMonthFormat := S1;
    doDMY,
    doYMD: SMonthFormat := S2;
  end;

  wYear := 0;
  if length(sYear) = 2 then begin
    DecodeDate(Date, wYear, wMonth, wDay);
    wYear := trunc(wYear / 100) * 100;
  end;
  iYear := StrToIntDef(sYear, -1) + wYear;
  if iYear = -1 then exit;

  if length(SMonthFormat) <= 2 then
    iMonth := StrToIntDef(sMonth, -1)
  else begin
    if length(SMonthFormat) = 3 then begin
      for I := 1 to 12 do
        PartNames[I] := AnsiUpperCase(ShortMonthNames[I]);
      iMonth := IndexOfStr(AnsiUpperCase(sMonth), ShortMonthNames) + 1
    end
    else begin
      for I := 1 to 12 do
        PartNames[I] := AnsiUpperCase(copy(LongMonthNames[I], 1, 2));
      iMonth := IndexOfStr(AnsiUpperCase(copy(sMonth, 1, 2)), PartNames) + 1;
    end;
    if iMonth = 0 then exit;
  end;

  iDay := StrToIntDef(sDay, -1);
  if iDay = -1 then exit;

  Result := EncodeDate(iYear, iMonth, iDay);
end;

function WStrToDate(const Date: string): TDateTime;
begin
  if not IsNullDate(Date) then
    Result := DecodeStrToDate(Date)
  else
    Result := 0.0;
end;

function WStrToDateTime(const DateTime: string): TDateTime;
begin
  if not IsNullDate(DateTime) then Result := StrToDateTime(DateTime)
  else Result := 0.0;
end;

function WDateToStr(Date: TDateTime): string;
begin
  if trunc(Date) = 0 then Result := GetNullDateStr
  else Result := DateToStr(Date);
end;

function WDateTimeToStr(Date: TDateTime): string;
begin
  if trunc(Date) = 0 then Result := GetNullDateStr
  else Result := DateTimeToStr(Date);
end;

function LongDateToStr(Date: TDateTime): string;
begin
  DateTimeToString(Result, LongTimeFormat, Date);
end;

var Year, Month, Day, CurWD: Word;
    Offset: LongInt;

procedure GetDatesOnDay(var FromDate, ToDate: TDateTime);
begin
  FromDate := Date;
  ToDate := Date + 1;
end;

procedure GetDatesOnWeek(var FromDate, ToDate: TDateTime);
begin
  CurWD := DayOfWeek(Date);
  Dec(CurWD);
  if CurWD = 0 then CurWD := 7;
  Offset := curWD - 1;
  FromDate := Date - Offset;
  ToDate := FromDate + 6;
end;

procedure GetDatesOnMonth(var FromDate, ToDate: TDateTime);
begin
  DecodeDate(Date, Year, Month, Day);
  FromDate := EncodeDate(Year, Month, 1);
  if Month = 12 then begin
    inc(Year);
    Month := 0;
  end;
  ToDate := EncodeDate(Year, Month + 1, 1);
end;

procedure GetDatesOnQuarter(var FromDate, ToDate: TDateTime);
begin
  DecodeDate(Date, Year, Month, Day);
  case Month of
    1, 2, 3: begin
      FromDate := EncodeDate(Year, 1, 1);
      ToDate := EncodeDate(Year, 4, 1);
    end;
    4, 5, 6: begin
      FromDate := EncodeDate(Year, 4, 1);
      ToDate := EncodeDate(Year, 7, 1);
    end;
    7, 8, 9: begin
      FromDate := EncodeDate(Year, 7, 1);
      ToDate := EncodeDate(Year, 10, 1);
    end;
    10, 11, 12: begin
      FromDate := EncodeDate(Year, 10, 1);
      ToDate := EncodeDate(Year + 1, 1, 1);
    end;
  end;
end;

procedure GetDatesOnYear(var FromDate, ToDate: TDateTime);
begin
  DecodeDate(Date, Year, Month, Day);
  FromDate := EncodeDate(Year, 1, 1);
  ToDate := EncodeDate(Year + 1, 1, 1);
end;

function GetPeriod;
begin
  try
    Result := False;
    sohoGetPeriodForm := TsohoGetPeriodForm.Create(Application);
    Result := sohoGetPeriodForm.Execute(FromDate, ToDate);
  finally
    sohoGetPeriodForm.Free;
  end;
end;

function GetMaxDate;
begin
  try
    Result := False;
    sohoGetPeriodForm := TsohoGetPeriodForm.Create(Application);
    Result := sohoGetPeriodForm.MaxDate(ToDate);
  finally
    sohoGetPeriodForm.Free;
  end;
end;

function TimeToShortTime (Time : TDateTime) : string;
var Hour, Min, Sec, MSec : word;
    Tmp : string;
begin
  DecodeTime(Time, Hour, Min, Sec, MSec);
  Tmp := IntToStr(Hour);
  Result := AlignStr(Tmp, taRightJustify, '0', 2);
  Tmp := IntToStr(Min);
  Result := Result + ':'+ AlignStr(Tmp, taRightJustify, '0', 2);
end;

var Index : integer;
    ShTmp, LngTmp : string;

{$IFDEF RUSSIAN_DEF}
initialization
  ShTmp := ShortDayNames[1];
  LngTmp := LongDayNames[1];
  for Index := 7 downto 2 do begin
    sohoShortDayNames[Index-1] := ShortDayNames[Index];
    sohoLongDayNames[Index-1]  := LongDayNames[Index];
  end;
  sohoShortDayNames[7] := ShTmp;
  sohoLongDayNames[7]  := LngTmp;
{$ENDIF}

end.

