{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
         Dialogs
     SohoLib - TsohoBitBtn  TsohoEdit
}
unit SoDialogs;

{$I SOHOLIB.INC}

interface
uses Classes, Forms, Controls, SoMultimedia;

type
  TMsgDlgType = (mtWarning, mtError, mtInformation, mtConfirmation, mtCustom);
  TMsgDlgBtn = (mbYes, mbNo, mbOK, mbCancel, mbAbort, mbRetry, mbIgnore,
    mbAll, mbNoToAll, mbYesToAll, mbHelp);
  TMsgDlgButtons = set of TMsgDlgBtn;

const
  mbYesNoCancel = [mbYes, mbNo, mbCancel];
  mbOKCancel = [mbOK, mbCancel];
  mbAbortRetryIgnore = [mbAbort, mbRetry, mbIgnore];

function CreateMessageDialog(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; const CheckCaption: string): TForm;
function CreateSpecMessageDialog(const Msg: string; DlgType: TMsgDlgType;
  Buttons: array of string; const CheckCaption: string): TForm;

function MessageDlg(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer;
function MessageDlgPos(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; X, Y: Integer): Integer;
function MessageDlgPosHelp(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; X, Y: Integer;
  const HelpFileName: string): Integer;
function MessageCheckDlg(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint;
  const CheckCaption: string; var Checked: boolean): Integer;

function SpecMessageDlg(const Msg: string; DlgType: TMsgDlgType;
  Buttons: array of string): Integer;

procedure ShowMessage(const Msg: string);
procedure ShowMessageFmt(const Msg: string; Params: array of const);
procedure ShowMessagePos(const Msg: string; X, Y: Integer);

{ Input dialog }

function InputBox(const ACaption, APrompt, ADefault: string): string;
function InputQuery(const ACaption, APrompt: string;
  var Value: string): Boolean;

var

  DialogsSound: TsohoSound;

implementation
uses SoCmnCns, Windows, Graphics, ExtCtrls, SoCtrls, StdCtrls, SohoBtns,
     SysUtils, SoUtils, SoChkBox, Math;

function GetAveCharSize(Canvas: TCanvas): TPoint;
var
  I: Integer;
  Buffer: array[0..51] of Char;
begin
  for I := 0 to 25 do Buffer[I] := Chr(I + Ord('A'));
  for I := 0 to 25 do Buffer[I + 26] := Chr(I + Ord('a'));
  GetTextExtentPoint(Canvas.Handle, Buffer, 52, TSize(Result));
  Result.X := Result.X div 52;
end;

type

  TMessageForm = class(TForm)
  private
    procedure HelpButtonClick(Sender: TObject);
  public
    {$IFDEF Delphi4_or_5}
    constructor CreateNew(AOwner: TComponent); reintroduce;
    {$ELSE}
    constructor CreateNew(AOwner: TComponent);
    {$ENDIF}
  end;

constructor TMessageForm.CreateNew(AOwner: TComponent);
var
  NonClientMetrics: TNonClientMetrics;
begin
  inherited CreateNew(AOwner);
  NonClientMetrics.cbSize := sizeof(NonClientMetrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NonClientMetrics, 0) then
    Font.Handle := CreateFontIndirect(NonClientMetrics.lfMessageFont);
end;

procedure TMessageForm.HelpButtonClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

var
  Captions: array[TMsgDlgType] of Pointer = (@SMsgDlgWarning, @SMsgDlgError,
    @SMsgDlgInformation, @SMsgDlgConfirm, nil);
  IconIDs: array[TMsgDlgType] of PChar = (IDI_EXCLAMATION, IDI_HAND,
    IDI_ASTERISK, IDI_QUESTION, nil);
  ButtonNames: array[TMsgDlgBtn] of string = (
    'Yes', 'No', 'OK', 'Cancel', 'Abort', 'Retry', 'Ignore', 'All', 'NoToAll',
    'YesToAll', 'Help');
  ButtonCaptions: array[TMsgDlgBtn] of Pointer = (
    @SMsgDlgYes, @SMsgDlgNo, @SMsgDlgOK, @SMsgDlgCancel, @SMsgDlgAbort,
    @SMsgDlgRetry, @SMsgDlgIgnore, @SMsgDlgAll, @SMsgDlgNoToAll, @SMsgDlgYesToAll,
    @SMsgDlgHelp);
  ModalResults: array[TMsgDlgBtn] of Integer = (
    mrYes, mrNo, mrOk, mrCancel, mrAbort, mrRetry, mrIgnore, mrAll, mrNoToAll,
    mrYesToAll, 0);

var
  ButtonWidths : array[TMsgDlgBtn] of integer;  // initialized to zero
  SpecButtonWidths : array[0..10] of integer;  // initialized to zero

const
  mcHorzMargin = 8;
  mcVertMargin = 8;
  mcHorzSpacing = 10;
  mcVertSpacing = 10;
  mcButtonWidth = 50;
  mcButtonHeight = 14;
  mcButtonSpacing = 4;

function CreateMessageDialog(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; const CheckCaption: string): TForm;
var
  DialogUnits: TPoint;
  HorzMargin, VertMargin, HorzSpacing, VertSpacing, ButtonWidth,
  ButtonHeight, ButtonSpacing, ButtonCount, ButtonGroupWidth,
  IconTextWidth, IconTextHeight, X, ALeft, ACheckHeight,
  Index: Integer;
  B, DefaultButton, CancelButton: TMsgDlgBtn;
  IconID: PChar;
  TextRect: TRect;
  Ch: TsohoCheckBox;
begin
  Result := TMessageForm.CreateNew(Application);
  with Result do begin
    {$IFDEF Delphi4_or_5}
    BiDiMode := Application.BiDiMode;
    {$ENDIF}
    BorderStyle := bsDialog;
    Canvas.Font := Font;
    DialogUnits := GetAveCharSize(Canvas);
    HorzMargin := MulDiv(mcHorzMargin, DialogUnits.X, 4);
    VertMargin := MulDiv(mcVertMargin, DialogUnits.Y, 8);
    HorzSpacing := MulDiv(mcHorzSpacing, DialogUnits.X, 4);
    VertSpacing := MulDiv(mcVertSpacing, DialogUnits.Y, 8);
    ButtonWidth := MulDiv(mcButtonWidth, DialogUnits.X, 4);
    for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do begin
      if B in Buttons then begin
        if ButtonWidths[B] = 0 then begin
          TextRect := Rect(0,0,0,0);
          Windows.DrawText( canvas.handle,
            PChar(LoadResString(ButtonCaptions[B])), -1,
            TextRect, DT_CALCRECT or DT_LEFT or DT_SINGLELINE
            {$IFDEF Delphi4_or_5}
            or DrawTextBiDiModeFlagsReadingOnly
            {$ENDIF}
            );
          with TextRect do ButtonWidths[B] := Right - Left + 8;
        end;
        if ButtonWidths[B] > ButtonWidth then
          ButtonWidth := ButtonWidths[B];
      end;
    end;
    ButtonHeight := MulDiv(mcButtonHeight, DialogUnits.Y, 8);
    ButtonSpacing := MulDiv(mcButtonSpacing, DialogUnits.X, 4);
    SetRect(TextRect, 0, 0, Screen.Width div 2, 0);
    DrawText(Canvas.Handle, PChar(Msg), Length(Msg)+1, TextRect,
      DT_EXPANDTABS or DT_CALCRECT or DT_WORDBREAK
      {$IFDEF Delphi4_or_5}
      or DrawTextBiDiModeFlagsReadingOnly
      {$ENDIF}
      );
    IconID := IconIDs[DlgType];
    IconTextWidth := TextRect.Right;
    IconTextHeight := TextRect.Bottom;
    if IconID <> nil then begin
      Inc(IconTextWidth, 32 + HorzSpacing);
      if IconTextHeight < 32 then IconTextHeight := 32;
    end;
    ButtonCount := 0;
    for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
      if B in Buttons then Inc(ButtonCount);
    ButtonGroupWidth := 0;
    if ButtonCount <> 0 then
      ButtonGroupWidth := ButtonWidth * ButtonCount +
        ButtonSpacing * (ButtonCount - 1);
    ClientWidth := GetMax(IconTextWidth, ButtonGroupWidth) + HorzMargin * 2;
    ACheckHeight := 0;
    if CheckCaption <> '' then begin
      Ch := TsohoCheckBox.Create(Result);
      with Ch do begin
        ACheckHeight := Canvas.TextHeight('W') * 2;
        SetBounds(2 * HorzMargin + 32 + TextRect.Left,
                  TextRect.Bottom + VertSpacing + 5,
                  TextRect.Right,
                  TextRect.Bottom);
        Caption := CheckCaption;
        VertAlign := baTop;
        WordWrap := true;
        Name := 'CHECK';
        Parent := Result;
        Height := ACheckHeight;
      end;
    end;
    ClientHeight := GetMax(ACheckHeight + (TextRect.Bottom - TextRect.Top),
      IconTextHeight) + ButtonHeight + VertSpacing +
      VertMargin * 2;
    Left := (Screen.Width div 2) - (Width div 2);
    Top := (Screen.Height div 2) - (Height div 2);
    if DlgType <> mtCustom then
      Caption := LoadResString(Captions[DlgType]) else
      Caption := Application.Title;
    if IconID <> nil then
      with TImage.Create(Result) do begin
        Name := 'Image';
        Parent := Result;
        Picture.Icon.Handle := LoadIcon(0, IconID);
        SetBounds(HorzMargin, VertMargin, 32, 32);
      end;
    with TLabel.Create(Result) do begin
      Name := 'Message';
      Parent := Result;
      WordWrap := True;
      Caption := Msg;
      BoundsRect := TextRect;
      {$IFDEF Delphi4_or_5}
      BiDiMode := Result.BiDiMode;
      {$ENDIF}
      ALeft := IconTextWidth - TextRect.Right + HorzMargin;
      {$IFDEF Delphi4_or_5}
      if UseRightToLeftAlignment then
        ALeft := Result.ClientWidth - ALeft - Width;
      {$ENDIF}
      SetBounds(ALeft, VertMargin,
        TextRect.Right, TextRect.Bottom);
    end;
    if mbOk in Buttons then DefaultButton := mbOk else
      if mbYes in Buttons then DefaultButton := mbYes else
        DefaultButton := mbRetry;
    if mbCancel in Buttons then CancelButton := mbCancel else
      if mbNo in Buttons then CancelButton := mbNo else
        CancelButton := mbOk;
    X := (ClientWidth - ButtonGroupWidth) div 2;
    Index := 0;
    for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
      if B in Buttons then
        with TsohoBitBtn.Create(Result) do begin
          Name := ButtonNames[B];
          Parent := Result;
          Caption := LoadResString(ButtonCaptions[B]);
          ModalResult := ModalResults[B];
          if B = DefaultButton then Default := True;
          if B = CancelButton then Cancel := True;
          TabOrder := Index;
          inc(Index);
          SetBounds(X,
            GetMax(ACheckHeight + (TextRect.Bottom - TextRect.Top), IconTextHeight)
            + VertMargin + VertSpacing, ButtonWidth, ButtonHeight);
          Inc(X, ButtonWidth + ButtonSpacing);
          if B = mbHelp then
            OnClick := TMessageForm(Result).HelpButtonClick;
          Sound := DialogsSound;
        end;
  end;
end;

function CreateSpecMessageDialog(const Msg: string; DlgType: TMsgDlgType;
  Buttons: array of string; const CheckCaption: string): TForm;
var
  DialogUnits: TPoint;
  HorzMargin, VertMargin, HorzSpacing, VertSpacing, ButtonWidth,
  ButtonHeight, ButtonSpacing, ButtonCount, ButtonGroupWidth,
  IconTextWidth, IconTextHeight, X, ALeft, ACheckHeight,
  Index: Integer;
  B: Integer;
  IconID: PChar;
  TextRect: TRect;
  Ch: TsohoCheckBox;
begin
  Result := TMessageForm.CreateNew(Application);
  with Result do begin
    {$IFDEF Delphi4_or_5}
    BiDiMode := Application.BiDiMode;
    {$ENDIF}
    BorderStyle := bsDialog;
    Canvas.Font := Font;
    DialogUnits := GetAveCharSize(Canvas);
    HorzMargin := MulDiv(mcHorzMargin, DialogUnits.X, 4);
    VertMargin := MulDiv(mcVertMargin, DialogUnits.Y, 8);
    HorzSpacing := MulDiv(mcHorzSpacing, DialogUnits.X, 4);
    VertSpacing := MulDiv(mcVertSpacing, DialogUnits.Y, 8);
    ButtonWidth := MulDiv(mcButtonWidth, DialogUnits.X, 4);
    for B := Low(Buttons) to High(Buttons) do begin
      if SpecButtonWidths[B] = 0 then begin
        TextRect := Rect(0,0,0,0);
        Windows.DrawText(Canvas.Handle, PChar(Buttons[B]), -1,
          TextRect, DT_CALCRECT or DT_LEFT or DT_SINGLELINE
          {$IFDEF Delphi4_or_5}
          or DrawTextBiDiModeFlagsReadingOnly
          {$ENDIF}
          );
        with TextRect do SpecButtonWidths[B] := Right - Left + 8;
      end;
      if SpecButtonWidths[B] > ButtonWidth then
        ButtonWidth := SpecButtonWidths[B];
    end;
    ButtonHeight := MulDiv(mcButtonHeight, DialogUnits.Y, 8);
    ButtonSpacing := MulDiv(mcButtonSpacing, DialogUnits.X, 4);
    SetRect(TextRect, 0, 0, Screen.Width div 2, 0);
    DrawText(Canvas.Handle, PChar(Msg), Length(Msg)+1, TextRect,
      DT_EXPANDTABS or DT_CALCRECT or DT_WORDBREAK
      {$IFDEF Delphi4_or_5}
      or DrawTextBiDiModeFlagsReadingOnly
      {$ENDIF}
      );
    IconID := IconIDs[DlgType];
    IconTextWidth := TextRect.Right;
    IconTextHeight := TextRect.Bottom;
    if IconID <> nil then begin
      Inc(IconTextWidth, 32 + HorzSpacing);
      if IconTextHeight < 32 then IconTextHeight := 32;
    end;
    ButtonCount := High(Buttons) + 1;
    ButtonGroupWidth := 0;
    if ButtonCount <> 0 then
      ButtonGroupWidth := ButtonWidth * ButtonCount +
        ButtonSpacing * (ButtonCount - 1);
    ClientWidth := GetMax(IconTextWidth, ButtonGroupWidth) + HorzMargin * 2;
    ACheckHeight := 0;
    if CheckCaption <> '' then begin
      Ch := TsohoCheckBox.Create(Result);
      with Ch do begin
        ACheckHeight := Canvas.TextHeight('W') * 2;
        SetBounds(2 * HorzMargin + 32 + TextRect.Left,
                  TextRect.Bottom + VertSpacing + 5,
                  TextRect.Right,
                  TextRect.Bottom);
        Caption := CheckCaption;
        VertAlign := baTop;
        WordWrap := true;
        Name := 'CHECK';
        Parent := Result;
        Height := ACheckHeight;
      end;
    end;
    ClientHeight := GetMax(ACheckHeight + (TextRect.Bottom - TextRect.Top),
      IconTextHeight) + ButtonHeight + VertSpacing +
      VertMargin * 2;
    Left := (Screen.Width div 2) - (Width div 2);
    Top := (Screen.Height div 2) - (Height div 2);
    if DlgType <> mtCustom then
      Caption := LoadResString(Captions[DlgType]) else
      Caption := Application.Title;
    if IconID <> nil then
      with TImage.Create(Result) do begin
        Name := 'Image';
        Parent := Result;
        Picture.Icon.Handle := LoadIcon(0, IconID);
        SetBounds(HorzMargin, VertMargin, 32, 32);
      end;
    with TLabel.Create(Result) do begin
      Name := 'Message';
      Parent := Result;
      WordWrap := True;
      Caption := Msg;
      BoundsRect := TextRect;
      {$IFDEF Delphi4_or_5}
      BiDiMode := Result.BiDiMode;
      {$ENDIF}
      ALeft := IconTextWidth - TextRect.Right + HorzMargin;
      {$IFDEF Delphi4_or_5}
      if UseRightToLeftAlignment then
        ALeft := Result.ClientWidth - ALeft - Width;
      {$ENDIF}
      SetBounds(ALeft, VertMargin,
        TextRect.Right, TextRect.Bottom);
    end;
    X := (ClientWidth - ButtonGroupWidth) div 2;
    Index := 0;
    for B := Low(Buttons) to High(Buttons) do
        with TsohoBitBtn.Create(Result) do begin
          Parent := Result;
          Caption := Buttons[B];
          ModalResult := B + 1;
          if B = 1 then Default := True;
          TabOrder := Index;
          inc(Index);
          SetBounds(X,
            GetMax(ACheckHeight + (TextRect.Bottom - TextRect.Top), IconTextHeight)
            + VertMargin + VertSpacing, ButtonWidth, ButtonHeight);
          Inc(X, ButtonWidth + ButtonSpacing);
          Sound := DialogsSound;
        end;
  end;
end;

function MessageDlg(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer;
begin
  Result := MessageDlgPosHelp(Msg, DlgType, Buttons, HelpCtx, -1, -1, '');
end;

function SpecMessageDlg(const Msg: string; DlgType: TMsgDlgType;
  Buttons: array of string): Integer;
begin
  with CreateSpecMessageDialog(Msg, DlgType, Buttons, '') do
    try
      Position := poScreenCenter;
      Result := ShowModal;
    finally
      Free;
    end;
end;

function MessageDlgPos(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; X, Y: Integer): Integer;
begin
  Result := MessageDlgPosHelp(Msg, DlgType, Buttons, HelpCtx, X, Y, '');
end;

function MessageDlgPosHelp(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; X, Y: Integer;
  const HelpFileName: string): Integer;
begin
  with CreateMessageDialog(Msg, DlgType, Buttons, '') do
    try
      HelpContext := HelpCtx;
      HelpFile := HelpFileName;
      if X >= 0 then Left := X;
      if Y >= 0 then Top := Y;
      if (Y < 0) and (X < 0) then Position := poScreenCenter;
      Result := ShowModal;
    finally
      Free;
    end;
end;

function MessageCheckDlg(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint;
  const CheckCaption: string; var Checked: boolean): Integer;
var Ch: TsohoCheckBox;
    Dlg: TForm;
begin
  Dlg := CreateMessageDialog(Msg, DlgType, Buttons, CheckCaption);
  with Dlg do
    try
      Ch := TsohoCheckBox(Dlg.FindComponent('CHECK'));
      if Assigned(Ch) then
        Ch.Checked := Checked;
      HelpContext := HelpCtx;
      Position := poScreenCenter;
      Result := ShowModal;
      if Assigned(Ch) then Checked := Ch.Checked;
    finally
      Free;
    end;
end;

procedure ShowMessage(const Msg: string);
begin
  ShowMessagePos(Msg, -1, -1);
end;

procedure ShowMessageFmt(const Msg: string; Params: array of const);
begin
  ShowMessage(Format(Msg, Params));
end;

procedure ShowMessagePos(const Msg: string; X, Y: Integer);
begin
  MessageDlgPos(Msg, mtCustom, [mbOK], 0, X, Y);
end;

{ Input dialog }

function InputQuery(const ACaption, APrompt: string;
  var Value: string): Boolean;
var
  Form: TForm;
  Prompt: TLabel;
  Edit: TsohoEdit;
  DialogUnits: TPoint;
  ButtonTop, ButtonWidth, ButtonHeight: Integer;
begin
  Result := False;
  Form := TForm.Create(Application);
  with Form do
    try
      Canvas.Font := Font;
      DialogUnits := GetAveCharSize(Canvas);
      BorderStyle := bsDialog;
      Caption := ACaption;
      ClientWidth := MulDiv(180, DialogUnits.X, 4);
      ClientHeight := MulDiv(63, DialogUnits.Y, 8);
      Position := poScreenCenter;
      Prompt := TLabel.Create(Form);
      with Prompt do begin
        Parent := Form;
        AutoSize := True;
        Left := MulDiv(8, DialogUnits.X, 4);
        Top := MulDiv(8, DialogUnits.Y, 8);
        Caption := APrompt;
      end;
      Edit := TsohoEdit.Create(Form);
      with Edit do begin
        Parent := Form;
        Left := Prompt.Left;
        Top := MulDiv(19, DialogUnits.Y, 8);
        Width := MulDiv(164, DialogUnits.X, 4);
        MaxLength := 255;
        Text := Value;
        SelectAll;
      end;
      ButtonTop := MulDiv(41, DialogUnits.Y, 8);
      ButtonWidth := MulDiv(50, DialogUnits.X, 4);
      ButtonHeight := MulDiv(14, DialogUnits.Y, 8);
      with TsohoBitBtn.Create(Form) do begin
        Parent := Form;
        Caption := SMsgDlgOK;
        ModalResult := mrOk;
        Default := True;
        SetBounds(MulDiv(38, DialogUnits.X, 4), ButtonTop, ButtonWidth,
          ButtonHeight);
        Sound := DialogsSound;
      end;
      with TsohoBitBtn.Create(Form) do begin
        Parent := Form;
        Caption := SMsgDlgCancel;
        ModalResult := mrCancel;
        Cancel := True;
        SetBounds(MulDiv(92, DialogUnits.X, 4), ButtonTop, ButtonWidth,
          ButtonHeight);
        Sound := DialogsSound;
      end;
      if ShowModal = mrOk then begin
        Value := Edit.Text;
        Result := True;
      end;
    finally
      Form.Free;
    end;
end;

function InputBox(const ACaption, APrompt, ADefault: string): string;
begin
  Result := ADefault;
  InputQuery(ACaption, APrompt, Result);
end;

initialization
  DialogsSound := nil;
end.