{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
     TsohoEnhTreeView -   TTreeView 
   .     
      (ExtendedTreeView)
}
unit SoEnhTreeView;

interface

{$I SOHOLIB.INC}

uses Windows, Messages, Classes, Graphics, ComCtrls, Controls,
     SoCommCtrl, CommCtrl
     {$IFDEF Delphi4_or_5}, ImgList {$ENDIF};

type

  TTVPaintStyle = (psPrePaint, psPostPaint, psPaint);
  TTVPaintStyles = set of TTVPaintStyle;

  {$IFDEF Delphi4_or_5}
  TTVPrePaintEvent = procedure(Sender: TObject;
    Node: TTreeNode;
    dwItemSpec: DWORD;  // this is control specific, but it's how to specify an item.  valid only with CDDS_ITEM bit set
    uItemState: UINT;
    Canvas: TCanvas;
    Rect: TRect;
    var clrText, clrTextBk: COLORREF) of object;
  {$ELSE}
  TTVPrePaintEvent = procedure(Sender: TObject;
    Node: TTreeNode;
    dwItemSpec: DWORD;  // this is control specific, but it's how to specify an item.  valid only with CDDS_ITEM bit set
    uItemState: UINT;
    Canvas: TCanvas;
    Rect: TRect;
    var clrText, clrTextBk: TColor) of object;
  {$ENDIF}

  TTVPaintEvent = procedure(Sender: TObject;
    Node: TTreeNode;
    Canvas: TCanvas;
    Rect: TRect;
    dwItemSpec: DWORD;  // this is control specific, but it's how to specify an item.  valid only with CDDS_ITEM bit set
    uItemState: UINT) of object;

  TsohoEnhCheckState = (csChecked, csUnchecked, csPartChecked);

  TsohoNodeClickNotify = procedure (Sender : TObject; Node : TTreeNode; Shift: TShiftState) of object;
  TsohoNodeSelectNotify = procedure (Sender : TObject; Node : TTreeNode) of object;
  TsohoNodeCheckNotify = procedure (Sender : TObject; Node : TTreeNode) of object;

  TsohoEnhTreeView = class(TTreeView)
  private
    FSortType: TSortType;
    FPaintStyle: TTVPaintStyles;
    FTrackSelect: Boolean;
    FOnPrePaint: TTVPrePaintEvent;
    FOnPaint: TTVPaintEvent;
    FOnPostPaint: TTVPaintEvent;
    FOnNodeClick : TsohoNodeClickNotify;
    FStates : TImageList;
    FToolTips: Boolean;
    FShowChecks : boolean;
    FAutoChecks : boolean;
    FOnCheckClick : TsohoNodeClickNotify;
    FOnNodeSelect: TsohoNodeSelectNotify;
    FOnNodeCheckClick: TsohoNodeCheckNotify;
  protected
    function GetItemText(aNode: TTreeNode): string;
    procedure SetSortType(Value: TSortType);
    procedure CMEnter(var Message: TMessage); message CM_Enter;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CNNotify(var Msg: TWMNotify); message CN_NOTIFY;
    procedure Loaded;override;
    procedure SetCheckForChilds (aItem : TTreeNode; aChecked : boolean);
    procedure SetCheckForParent (aItem : TTreeNode);
    procedure WMNotify(var Message: TWMNotify); message WM_NOTIFY;
    procedure SetTrackSelect (Value : boolean);
    procedure SetChecks (Value : boolean);
    procedure SetToolTips (Value: boolean);
    function GetToolTips: boolean;
    procedure NodeSelected(Node: TTreeNode); virtual;
    procedure NodeCheckClicked(Node: TTreeNode); virtual;
    procedure SetPaintStyle(Value: TTVPaintStyles);
    {$IFDEF Delphi4_or_5}
    procedure Prepaint(
       Node: TTreeNode;
       dwItemSpec: DWORD;  // this is control specific, but it's how to specify an item.  valid only with CDDS_ITEM bit set
       uItemState: UINT;
       Canvas: TCanvas;
       Rect: TRect;
       var clrText, clrTextBk: COLORREF); virtual;
    {$ELSE}
    procedure Prepaint(
       Node: TTreeNode;
       dwItemSpec: DWORD;  // this is control specific, but it's how to specify an item.  valid only with CDDS_ITEM bit set
       uItemState: UINT;
       Canvas: TCanvas;
       Rect: TRect;
       var clrText, clrTextBk: TColor); virtual;
    {$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy;override;

    procedure ClearItems;
    function AlphaSort: Boolean;
    function CustomSort(SortProc: TTVCompare; Data: Longint): Boolean;
    procedure LoadFromFile(const aFileName: string);
    procedure SaveToFile(const aFileName: string);
    procedure GetItemList(aList: TStrings);
    procedure SetItemList(aList: TStrings);

    function GetNodeState (Node : TTreeNode) : TsohoEnhCheckState;
    procedure SetNodeState (Node : TTreeNode; Value : TsohoEnhCheckState);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure WMRButtonDown(var Message: TWMRButtonDown); message WM_RBUTTONDOWN;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure KeyPress(var Key: Char);override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    function  IsNodeBold(Node: TTreeNode): Boolean;
    procedure MakeNodeBold(Node: TTreeNode; Value: Boolean);
  published
    property AutoChecks : boolean read FAutoChecks write FAutoChecks default true;
    property ShowChecks : boolean read FShowChecks write SetChecks default false;
    property PaintStyle: TTVPaintStyles read FPaintStyle write SetPaintStyle default [];
    property SortType: TSortType read FSortType write SetSortType default stNone;
    property ToolTips: boolean read GetToolTips write SetToolTips;
    property TrackSelect: Boolean read FTrackSelect write SetTrackSelect default True;
    property OnPrePaint: TTVPrePaintEvent read FOnPrePaint write FOnPrePaint;
    property OnPaint: TTVPaintEvent read FOnPaint write FOnPaint;
    property OnPostPaint: TTVPaintEvent read FOnPostPaint write FOnPostPaint;
    property OnNodeClick : TsohoNodeClickNotify read FOnNodeClick write FOnNodeClick;
    property OnCheckNode : TsohoNodeClickNotify read FOnCheckClick write FOnCheckClick;
    property OnNodeSelect: TsohoNodeSelectNotify read FOnNodeSelect write FOnNodeSelect;
    property OnNodeCheckClick: TsohoNodeCheckNotify read FOnNodeCheckClick write FOnNodeCheckClick;
  end;

function InitCommonControlsEx(dwICC: DWORD): Boolean;

implementation
uses SysUtils, SoCmnCns;

{$R SOENHTREEVIEW.R32}

const
  cctrl = 'comctl32.dll';

type
  TInitCommonControlsEx = record
    dwSize: DWORD;
    dwICC: DWORD;
  end;

function InitCommonControlsEx_(var TInitCommonControlsRec: TInitCommonControlsEx): Boolean;
  stdcall external cctrl name 'InitCommonControlsEx';

function InitCommonControlsEx;
var
  InitCommonControlsExRec: TInitCommonControlsEx;
begin
  InitCommonControlsExRec.dwSize := SizeOf(InitCommonControlsExRec);
  InitCommonControlsExRec.dwICC := dwICC;
  Result := InitCommonControlsEx_(InitCommonControlsExRec);
end;

function DefaultTreeViewSort(Node1, Node2: TTreeNode; lParam: Integer): Integer; stdcall;
begin
  Result := lstrcmp(PChar(Node1.Text), PChar(Node2.Text));
end;

const  TVS_NOTOOLTIPS = $0080;

procedure TsohoEnhTreeView.NodeSelected(Node: TTreeNode);
begin
  if Assigned(Node) and Assigned(FOnNodeSelect) then
    FOnNodeSelect(Self, Node);
end;

function TsohoEnhTreeView.GetToolTips: boolean;
begin
  FToolTips := (GetWindowLong(Handle, GWL_STYLE) and TVS_NOTOOLTIPS) <> 0;
  Result := FToolTips;
end;

procedure TsohoEnhTreeView.SetToolTips (Value: boolean);
var State : longint;
begin
  if FToolTips = Value then exit;
  FToolTips := Value;
  State := GetWindowLong(Handle, GWL_STYLE);
  if FToolTips then State := State or TVS_NOTOOLTIPS
  else State := State xor TVS_NOTOOLTIPS;
  SetWindowLong(Handle, GWL_STYLE, State);
end;

procedure TsohoEnhTreeView.WMNotify(var Message: TWMNotify);
begin
 inherited;
 // if FToolTips then inherited;
end;

procedure TsohoEnhTreeView.SetTrackSelect (Value : boolean);
begin
  if FTrackSelect = Value then exit;
  FTrackSelect := Value;
  RecreateWnd;
end;

procedure TsohoEnhTreeView.SetChecks (Value : boolean);
begin
  if FShowChecks = Value then exit;
  FShowChecks := Value;
  if csDesigning in ComponentState then exit;
  if FShowChecks then begin
    StateImages := FStates;
  end
  else StateImages := nil;
end;

procedure TsohoEnhTreeView.KeyUp(var Key: Word; Shift: TShiftState);
begin
  inherited;
  if Key in [vk_Up, vk_Down, vk_Home, vk_End, vk_Next,
    vk_Prior] then NodeSelected(Selected);
  if (Key = vk_F2) and (not ReadOnly) and Assigned(Selected)
    then Selected.EditText;
end;

procedure TsohoEnhTreeView.KeyPress(var Key: Char);
var Item : TTreeNode;
begin
  if FShowChecks and (ReadOnly) and (Key = #32) then begin
    Key := #00;
    Item := Selected;
    if Assigned(FOnCheckClick) then FOnCheckClick(Self, Item, []);
    if FAutoChecks then begin
      SetCheckForChilds(Item, Item.StateIndex <= 1);
      SetCheckForParent(Item);
    end;
    Invalidate;
  end
  else inherited KeyPress(Key);
end;

procedure TsohoEnhTreeView.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var ItemRect : TRect;
    Pnt : TPoint;
begin
  if Button <> mbLeft then inherited MouseUp(Button, Shift, X, Y)
  else begin
    inherited MouseUp(Button, Shift, X, Y);
    if not Assigned(Selected) then exit;
    ItemRect := Selected.DisplayRect(true);
    Pnt.X := X; Pnt.Y := Y;
    if PtInRect(ItemRect, Pnt) then
      if Assigned(FOnNodeClick) then FOnNodeClick(Self, Selected, Shift);
  end;
end;

procedure TsohoEnhTreeView.WMRButtonDown(var Message: TWMRButtonDown);
var Item : TTreeNode;
begin;
  if RightClickSelect then begin
    Item := GetNodeAt(Message.XPos, Message.YPos);
    if (Item <> nil) and (Selected <> Item) then begin
      Selected := Item;
      NodeSelected(Item);
    end;
  end;
  inherited;
end;

procedure TsohoEnhTreeView.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var Item : TTreeNode;
    ItemRect : TRect;
    Offset : integer;
begin
  if Button <> mbLeft then begin
    inherited MouseDown(Button, Shift, X, Y);
  end
  else begin
    Item := GetNodeAt(X, Y);
    if not FShowChecks then begin
      inherited MouseDown(Button, Shift, X, Y);
      NodeSelected(Item);
      exit;
    end;

    if not ((Item = nil) or (Item <> Selected) or
          (Item.StateIndex = -1)) then begin
      ItemRect := Item.DisplayRect(true);
      Offset := 14;
      if (Images <> nil) and (Item.ImageIndex <> -1) then
        Offset := Offset + Images.Width;
      if (X >= ItemRect.Left - Offset) and
         (X <= ItemRect.Left - Offset + 14) then begin
        if Assigned(FOnCheckClick) then FOnCheckClick(Self, Item, []);
        NodeCheckClicked(Item);
        if FAutoChecks then begin
          SetCheckForChilds(Item, Item.StateIndex <= 1);
          SetCheckForParent(Item);
        end;
        Invalidate;
      end
      else inherited MouseDown(Button, Shift, X, Y);
    end
    else inherited MouseDown(Button, Shift, X, Y);
    NodeSelected(Item);
  end;
end;

procedure TsohoEnhTreeView.SetCheckForChilds (aItem : TTreeNode; aChecked : boolean);
var Index : LongInt;
begin
  if aChecked then aItem.StateIndex := 2
  else aItem.StateIndex := 1;
  for Index := 0 to pred(aItem.Count) do
    SetCheckForChilds(aItem.Item[Index], aChecked);
end;

procedure TsohoEnhTreeView.SetCheckForParent (aItem : TTreeNode);
var Index, CheckCount, Almost : LongInt;
    Tmp : TTreeNode;
begin
  Tmp := aItem.Parent;
  if Tmp = nil then exit;
  CheckCount := 0;
  Almost := 0;
  for Index := 0 to pred(Tmp.Count) do begin
    if Tmp.Item[Index].StateIndex = 2 then inc(CheckCount);
    if Tmp.Item[Index].StateIndex = 3 then inc(Almost);
  end;
  if CheckCount = Tmp.Count then Tmp.StateIndex := 2
  else
     if (CheckCount = 0) and (Almost = 0) then Tmp.StateIndex := 1
     else Tmp.StateIndex := 3;
  if Tmp.Parent <> nil then begin
     SetCheckForParent (Tmp);
  end;
end;

procedure TsohoEnhTreeView.Loaded;
begin
  inherited Loaded;
  if csDesigning in ComponentState then exit;
  FStates.ResourceLoad(rtBitmap, 'SOHOCHECKTREEVIEW', clWhite);
end;

function TsohoEnhTreeView.GetNodeState (Node : TTreeNode) : TsohoEnhCheckState;
begin
  if (Node.StateIndex < 2) or (Node.StateIndex > 3) then Result := csUnchecked
  else
    if Node.StateIndex = 2 then Result := csChecked
    else Result := csPartChecked;
end;

procedure TsohoEnhTreeView.SetNodeState (Node : TTreeNode; Value : TsohoEnhCheckState);
begin
  case Value of    //
    csChecked      : Node.StateIndex := 2;
    csUnchecked    : Node.StateIndex := 1;
    csPartChecked  : Node.StateIndex := 3;
  end;    // case
  // if Assigned(FOnCheckClick) then FOnCheckClick
end;

constructor TsohoEnhTreeView.Create;
begin
  inherited Create(AOwner);
  FTrackSelect := True;
  FShowChecks := false;
  FAutoChecks := true;
  FStates := TImageList.Create(Self);
  Cursor := crHandPoint;
  FPaintStyle := [];
  FSortType := stNone;
end;

destructor TsohoEnhTreeView.Destroy;
begin
  FStates.Free;
  inherited Destroy;
end;

procedure TsohoEnhTreeView.CreateParams;
const
  TrackSelectStyles: array[Boolean] of DWORD = (0, TVS_TRACKSELECT);
begin
  if not InitCommonControlsEx(ICC_TREEVIEW_CLASSES) then
    raise Exception.Create(sohoEnhTreeComCtrl32Error);

  inherited CreateParams(Params);
  with Params do
    Style := Style or TrackSelectStyles[FTrackSelect];
end;

procedure TsohoEnhTreeView.CNNotify;
{$IFDEF Delphi3}
  procedure CallPaintEvent(PaintEvent: TTVPaintEvent);
  var
    Canvas: TCanvas;
  begin
    Canvas := TCanvas.Create;
    with TNMTVCustomDraw(Pointer(Msg.NMHdr)^), nmcd do
    try
      Canvas.Handle := hdc;
      Canvas.Brush := Self.Brush;
      Canvas.Font := Font;
      PaintEvent(
        Self,
        TTreeNode(lItemlParam),
        Canvas,
        rc,
        dwItemSpec,
        uItemState);
    finally
      Canvas.Free;
    end;
  end;

var Canvas: TCanvas;
    LogFont: TLogFont;
{$ENDIF}
begin
  inherited;

  {$IFDEF Delphi3} //    3    
  with Msg, TNMTVCustomDraw(Pointer(NMHdr)^), nmcd do
  case Hdr.Code of
    NM_CUSTOMDRAW:            
    begin
      case dwDrawStage of
        (*
           CDDS_PREPAINT is at the beginning of the paint cycle. You
           implement custom draw by returning the proper value. In this
           case, we're requesting item-specific notifications.
        *)
        CDDS_PREPAINT:
          // Request pre-paint notifications for each item.
          if Assigned(FOnPrePaint) or Assigned(FOnPaint) or Assigned(FOnPostPaint) or
            (FPaintStyle <> []) then
            Result := CDRF_NOTIFYITEMDRAW;
        (*
          CDDS_ITEMPREPAINT is sent when the control is about to paint an
          item. We're getting this because we returned CDRF_NOTIFYITEMDRAW
          in response to CDDS_PREPAINT before.
        *)
        CDDS_ITEMPREPAINT:
        begin
          if Assigned(FOnPrePaint) or (psPrepaint in FPaintStyle) then
          begin
            (*
              To change the text and background colors in a list view
              control, set the clrText and clrTextBk members of the
              NMLVCUSTOMDRAW structure to the desired color.

              This differs from most other controls that support
              CustomDraw. To change the text and background colors for
              the others, call SetTextColor and SetBkColor on the provided HDC.
            *)
            Canvas := TCanvas.Create;
            try
              Canvas.Handle := hdc;
              Canvas.Brush := Self.Brush;
              Canvas.Font := Font;
              Prepaint(
                TTreeNode(lItemlParam),
                dwItemSpec,
                uItemState,
                Canvas,
                rc,
                clrText, clrTextBk);
                if GetObject(Canvas.Font.Handle, SizeOf(LogFont), @LogFont) <> 0 then
                begin
                  Canvas.Handle := 0;  // disconnect from hdc
                  // don't delete the stock font
                  SelectObject(hdc, CreateFontIndirect(LogFont));
                end;
            finally
              Canvas.Free;
            end;
            Result := Result or CDRF_NEWFONT;
          end;
          if Assigned(FOnPaint) then // we wouldn't get here if haven't asked this
                                         // we will not ask this until Assigned(FOnPostPaint
                                         // BUT I leave this check. Just in case
          begin
            CallPaintEvent(FOnPaint);
            Result := Result or CDRF_SKIPDEFAULT;
          end;
          if Assigned(FOnPostPaint) or (psPostPaint in FPaintStyle) then
            Result := Result or CDRF_NOTIFYPOSTPAINT;
        end;
        CDDS_ITEMPOSTPAINT:
          if Assigned(FOnPostPaint) or (psPostpaint in FPaintStyle) then // we wouldn't get here if haven't asked this
                                         // we will not ask this until Assigned(FOnPostPaint
                                         // BUT I leave this check. Just in case
            CallPaintEvent(FOnPostPaint);
      end;
    end;
  end;
  {$ENDIF}
end;

function TsohoEnhTreeView.IsNodeBold(Node: TTreeNode): Boolean;
var Item: TTVItem;
begin
  Result := False;
  if not Assigned(Node) then Exit;
  with Item do begin
    mask := TVIF_STATE;
    hItem := Node.ItemId;
    if TreeView_GetItem(Node.Handle, Item) then
      Result := (State and TVIS_BOLD) <> 0;
  end;
end;

procedure TsohoEnhTreeView.MakeNodeBold(Node: TTreeNode; Value: Boolean);
var Item: TTVItem;
    Template: Integer;
begin
  if not Assigned(Node) then Exit;

  if Value then Template := -1
  else Template := 0;

  with Item do begin
    mask := TVIF_STATE;
    hItem := Node.ItemId;
    StateMask := TVIS_BOLD;
    State := StateMask and Template;
  end;
  TreeView_SetItem(Node.Handle, Item);
end;


{$IFDEF Delphi4_or_5}
procedure TsohoEnhTreeView.Prepaint(Node: TTreeNode; dwItemSpec: DWORD;
  uItemState: UINT; Canvas: TCanvas; Rect: TRect; var clrText, clrTextBk: COLORREF);
begin
  if Assigned(FOnPrePaint) then FOnPrePaint(Self, Node, dwItemSpec,
    uItemState, Canvas, Rect, clrText, clrTextBk);
end;
{$ELSE}
procedure TsohoEnhTreeView.Prepaint(Node: TTreeNode; dwItemSpec: DWORD;
  uItemState: UINT; Canvas: TCanvas;  Rect: TRect; var clrText, clrTextBk: TColor);
begin
  if Assigned(FOnPrePaint) then FOnPrePaint(Self, Node, dwItemSpec,
    uItemState, Canvas, Rect, clrText, clrTextBk);
end;
{$ENDIF}

procedure TsohoEnhTreeView.SetPaintStyle(Value: TTVPaintStyles);
begin
  if FPaintStyle = Value then exit;
  FPaintStyle := Value;
  Repaint;
end;

procedure TsohoEnhTreeView.CMEnter(var Message: TMessage);
begin
  inherited;
  if Assigned(Selected) then NodeSelected(Selected);
end;

procedure TsohoEnhTreeView.SetSortType(Value: TSortType);
begin
  if SortType <> Value then begin
     FSortType := Value;
     if ((SortType in [stData, stBoth]) and Assigned(OnCompare)) or
        (SortType in [stText, stBoth]) then AlphaSort;
  end;
end;

procedure TsohoEnhTreeView.LoadFromFile(const aFileName: string);
var aList: TStringList;
begin
  aList := TStringList.Create;
  Items.BeginUpdate;
  try
    aList.LoadFromFile(aFileName);
    SetItemList(AList);
  finally
    Items.EndUpdate;
    AList.Free;
  end;
end;

procedure TsohoEnhTreeView.SaveToFile(const aFileName: string);
var aList: TStringList;
begin
  aList := TStringList.Create;
  try
    GetItemList(aList);
    aList.SaveToFile(aFileName);
  finally
    aList.Free;
  end;
end;

procedure TsohoEnhTreeView.SetItemList(AList: TStrings);
var aLevel, aOldLevel, i, Cnt: LongInt;
    S: string;
    ANewStr: string;
    AParentNode: TTreeNode;
    TmpSort: TSortType;

    function GetBufStart(Buffer: PChar; var aLevel: Integer): PChar;
    begin
      ALevel := 0;
      while Buffer^ in [' ', #9] do begin
        Inc(Buffer);
        Inc(ALevel);
      end;
      Result := Buffer;
    end;
begin

  SendMessage(Handle, TVM_DELETEITEM, 0, Longint(TVI_ROOT));
  AOldLevel := 0;
  AParentNode := nil;

  TmpSort := SortType;
  SortType := stNone;
  try
    for Cnt := 0 to pred(AList.Count) do begin
      S := AList[Cnt];
      if (Length(S) = 1) and (S[1] = Chr($1A)) then Break;
      ANewStr := GetBufStart(PChar(S), ALevel);
      if (ALevel > AOldLevel) or (AParentNode = nil) then begin
         if ALevel - AOldLevel > 1 then
           raise Exception.Create(sohoEnhTreeInvalidLevel);
      end
      else begin
         for i := aOldLevel downto aLevel do begin
           AParentNode := AParentNode.Parent;
           if (AParentNode = nil) and (i - ALevel > 0) then
              raise Exception.Create(sohoEnhTreeInvalidLevel);
         end;
      end;
      aParentNode := Items.AddChild(aParentNode, ANewStr);
      aOldLevel := aLevel;
    end;
  finally
    SortType := TmpSort;
  end;
end;

procedure TsohoEnhTreeView.GetItemList(AList: TStrings);
var Index, Cnt: LongInt;
    ANode: TTreeNode;
begin
  AList.Clear;
  Cnt := Items.Count - 1;
  ANode := Items.GetFirstNode;
  for Index := 0 to Cnt do begin
    AList.Add(GetItemText(ANode));
    ANode := ANode.GetNext;
  end;
end;

function TsohoEnhTreeView.GetItemText(ANode: TTreeNode): string;
begin
 Result := StringOfChar(' ', ANode.Level) + ANode.Text;
end;

function TsohoEnhTreeView.AlphaSort: Boolean;
begin
  if HandleAllocated then Result := CustomSort(nil, 0)
  else Result := False;
end;

procedure TsohoEnhTreeView.ClearItems;
begin
  SendMessage(Handle, TVM_DELETEITEM, 0, Longint(TVI_ROOT));
end;

function TsohoEnhTreeView.CustomSort(SortProc: TTVCompare; Data: Longint): Boolean;
var SortCB: TTVSortCB;
    Node: TTreeNode;
begin
  Result := False;
  if HandleAllocated then begin
     with SortCB do begin
       if not Assigned(SortProc) then lpfnCompare := @DefaultTreeViewSort
       else lpfnCompare := SortProc;
       hParent := TVI_ROOT;
       lParam := Data;
       Result := TreeView_SortChildrenCB(Handle, SortCB, 0);
     end;
     if Items.Count > 0 then begin
       Node := Items.GetFirstNode;
       while Node <> nil do begin
         if Node.HasChildren then Node.CustomSort(SortProc, Data);
         Node := Node.GetNext;
       end;
     end;
  end;
end;

procedure TsohoEnhTreeView.NodeCheckClicked(Node: TTreeNode);
begin
  if Assigned(FOnNodeCheckClick) then FOnNodeCheckClick(Self, Node);
end;

end.
