{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoExpertTree;

{$I SOHOLIB.INC}

interface
uses Windows, Messages, SysUtils, Classes, Graphics, Controls,
     Forms, SohoBtns, ExtCtrls;

const

    DefHorizontalSpace  = 4;
    DefVerticalSpace    = 8;
    DefBoxHeight        = 17;
    DefBoxWidth         = 16;
    DefTopMargin        = 4;
    DefLeftMargin       = 4;

    DefLineColor        = clWhite;
    DefSelectedBoxColor = clLime;
    DefBoxColor         = clGray;
    
    DefHeight           = 140;
    DefWidth            = 133;
    DefFontColor        = clWhite;
    DefColor            = clBlack;

type
   TOnChanging = function(Sender: TObject; var NextPage: LongInt): boolean of object;   TOnChange = procedure(Sender: TObject) of object;
   TsohoExpertTree = class(TGraphicControl)
  private
    FRects : TList;
    FItems : TStrings;
    FItemIndex : Integer;
    FOnChanging : TOnChanging;
    FOnChange   : TOnChange;

    FLineColor        : TColor;
    FSelectedBoxColor : TColor;
    FBoxColor         : TColor;

    FHorizontalSpace  : Integer;
    FVerticalSpace    : Integer;
    FBoxHeight        : Integer;
    FBoxWidth         : Integer;
    FLeftMargin       : Integer;
    FTopMargin        : Integer;

    FNotebook: TNotebook;
    // FOldPagesChange: TNotifyEvent;
    FBtns: array [0..3] of TsohoBitBtn;
    FOnOldClicks: array [0..3] of TNotifyEvent;
  protected
    procedure SetInteger(Index: integer; Value: integer); virtual;
    function GetInteger(Index: integer): integer; virtual;
    procedure SetColor (Index: integer; Value: TColor); virtual;
    function GetColor(Index: integer): TColor; virtual;
    procedure SetItemIndex(Value : Integer);
    procedure SetItems(NewItems : TStrings);
    procedure AllocateRects;
    procedure FillBox( Index : Integer; Live : Boolean );
    procedure Change;
    function Changing(var NextPage: LongInt): boolean;
    procedure SetNotebook(Value: TNotebook); virtual;
    procedure SetButton(Index: integer; Value: TsohoBitBtn); virtual;
    function GetButton(Index: integer): TsohoBitBtn; virtual;
    procedure Notification (aComponent: TComponent; aOperation: TOperation); override;
    procedure CheckButtons; virtual;
    procedure SyncPages;
    procedure ButtonClick(Sender: TObject); virtual;
    procedure MouseMove (Shift: TShiftState; X, Y: Integer); override;
    function  MouseToNewPage(CursorPos: TPoint): LongInt;
  public
    // procedure Click; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;
  published
    property Align;
    property Color default DefColor;
    property Font;

    property Items: TStrings read FItems write SetItems;
    property ItemIndex: Integer read FItemIndex write SetItemIndex;

    property LineColor: TColor index 0 read GetColor
       write SetColor default DefLineColor;
    property SelectedBoxColor: TColor index 1 read GetColor
       write SetColor default DefSelectedBoxColor;
    property BoxColor: TColor index 2 read GetColor
       write SetColor default DefBoxColor;

    property HorizontalSpace: Integer index 0 read GetInteger
        write SetInteger default DefHorizontalSpace;
    property VerticalSpace: Integer index 1 read GetInteger
        write SetInteger default DefVerticalSpace;
    property BoxHeight: Integer index 2 read GetInteger
        write SetInteger default DefBoxHeight;
    property BoxWidth: Integer index 3 read GetInteger
        write SetInteger default DefBoxWidth;
    property LeftMargin: Integer index 4 read GetInteger
        write SetInteger default DefLeftMargin;
    property TopMargin: Integer index 5 read GetInteger
        write SetInteger default DefTopMargin;

    property Notebook: TNotebook read FNotebook write SetNotebook;
    property FirstBtn: TsohoBitBtn index 0 read GetButton write SetButton;
    property PriorBtn: TsohoBitBtn index 1 read GetButton write SetButton;
    property NextBtn: TsohoBitBtn index 2 read GetButton write SetButton;
    property LastBtn: TsohoBitBtn index 3 read GetButton write SetButton;

    property OnChanging: TOnChanging read FOnChanging write FOnChanging;
    property OnChange: TOnChange read FOnChange write FOnChange;
  end;

implementation

constructor TsohoExpertTree.Create(aOwner: TComponent);
var Index: LongInt;
begin
  inherited;
  ControlStyle := ControlStyle + [csDesignInteractive] -
    [csAcceptsControls];
  FItems := TStringList.Create;
  FItems.Add('Expert tree');
  FItems.Add('Step one');
  FItems.Add('Step two');
  FItems.Add('Step three');
  FItems.Add('Finish');
  FRects := TList.Create;
  FItemIndex := 0;
  FHorizontalSpace := DefHorizontalSpace;
  FVerticalSpace := DefVerticalSpace;
  FBoxHeight := DefBoxHeight;
  FBoxWidth := DefBoxWidth;
  FBoxColor := DefBoxColor;
  FSelectedBoxColor := DefSelectedBoxColor;
  FLineColor := DefLineColor;
  Font.Color := DefFontColor;
  Color := DefColor;
  Width := DefWidth;
  Height := DefHeight;
  FTopMargin := DefTopMargin;
  FLeftMargin:= DefLeftMargin;
  FNotebook := nil;
  for Index := 0 to 3 do Fbtns[Index] := nil;
  for Index := 0 to 3 do FOnOldClicks[Index] := nil;
end;

destructor TsohoExpertTree.Destroy;
var Index: LongInt;
begin
  {if Assigned(FNotebook) then
    TStringList(FNotebook.Pages).OnChange := FOldPagesChange;}
  if not (csDesigning in ComponentState) then
    for Index := 0 to 3 do
      if Assigned(Fbtns[Index]) then
        Fbtns[Index].OnClick := FOnOldClicks[Index];
  FItems.Clear;
  AllocateRects;
  FItems.Free;
  FRects.Free;
  inherited Destroy;
end;

procedure TsohoExpertTree.SyncPages;
begin
  if Assigned(FNotebook) and
     (FNotebook.Pages.Count <> FItems.Count) then
     FNotebook.Pages.Assign(FItems);
end;

procedure TsohoExpertTree.SetNotebook(Value: TNotebook);
begin
  if FNotebook = Value then exit;
  FNotebook := Value;
  if (csDesigning in ComponentState) then SyncPages;
  // FNotebook.Pages.Assign(FItems);
  // FOldPagesChange := TStringList(FNotebook.Pages).OnChange;
  // TStringList(FNotebook.Pages).OnChange := SyncPages;
end;

procedure TsohoExpertTree.CheckButtons;
begin
  if Assigned(Fbtns[0]) then
    Fbtns[0].Enabled := (FItemIndex <> -1) and (FItemIndex <> 0);
  if Assigned(Fbtns[1]) then
    Fbtns[1].Enabled := (FItemIndex <> -1) and (FItemIndex <> 0);
  if Assigned(Fbtns[2]) then
    Fbtns[2].Enabled := (FItemIndex <> -1) and (FItemIndex < pred(FItems.Count));
  if Assigned(Fbtns[3]) then
    Fbtns[3].Enabled := (FItemIndex <> -1) and (FItemIndex < pred(FItems.Count));
end;

procedure TsohoExpertTree.ButtonClick(Sender: TObject);
var Index, NewPage: LongInt;
begin
  if not Assigned(Sender) then exit;
  for Index := 0 to 3 do
    if Fbtns[Index] = Sender then begin
      case Index of
        0: NewPage := 0;
        1: NewPage := ItemIndex - 1;
        2: NewPage := ItemIndex + 1;
        else NewPage := FItems.Count - 1;
      end;
      if Changing(NewPage) then begin
        if (NewPage >= 0) and (NewPage < FItems.Count) then
           ItemIndex := NewPage;
      end;
      if Assigned(FOnOldClicks[Index]) then FOnOldClicks[Index](Sender);
    end;
end;

procedure TsohoExpertTree.SetButton(Index: integer; Value: TsohoBitBtn);
begin
  if Fbtns[Index] = Value then exit;
  if not (csDesigning in ComponentState) and
     Assigned(Fbtns[Index]) then Fbtns[Index].OnClick := FOnOldClicks[Index];
  Fbtns[Index] := Value;
  if not (csDesigning in ComponentState) and Assigned(Value) then begin
    FOnOldClicks[Index] := Value.OnClick;
    Value.OnClick := ButtonClick;
  end;
  CheckButtons;
end;

function TsohoExpertTree.GetButton(Index: integer): TsohoBitBtn;
begin
  Result := Fbtns[Index];
end;

procedure TsohoExpertTree.Notification (aComponent: TComponent; aOperation: TOperation);
var Index: LongInt;
begin
  inherited;
  if (aComponent = FNotebook) and (aOperation = opRemove) then
    FNotebook := nil;
  for Index := 0 to 3 do
    if (aComponent = Fbtns[Index]) and (aOperation = opRemove) then
       SetButton(Index, nil);
end;

procedure TsohoExpertTree.SetInteger(Index: integer; Value: integer);
begin
  if GetInteger(Index) = Value then exit;
  case Index of
    0: FHorizontalSpace := Value;
    1: FVerticalSpace := Value;
    2: FBoxHeight := Value;
    3: FBoxWidth := Value;
    4: FLeftMargin := Value;
    else FTopMargin := Value;
  end;
  Invalidate;
end;

function TsohoExpertTree.GetInteger(Index: integer): integer;
begin
  case Index of
    0: Result := FHorizontalSpace;
    1: Result := FVerticalSpace;
    2: Result := FBoxHeight;
    3: Result := FBoxWidth;
    4: Result := FLeftMargin;
    else Result := FTopMargin;
  end;
end;

procedure TsohoExpertTree.SetColor (Index: integer; Value: TColor);
begin
  if GetColor(Index) = Value then exit;
  case Index of
    0: FLineColor := Value;
    1: FSelectedBoxColor := Value;
    else FBoxColor := Value;
  end;
  Invalidate;
end;

function TsohoExpertTree.GetColor(Index: integer): TColor;
begin
  case Index of
    0: Result := FLineColor;
    1: Result := FSelectedBoxColor;
    else Result := FBoxColor;
  end;
end;

procedure TsohoExpertTree.SetItems(NewItems : TStrings);
begin
  FItems.Assign(NewItems);
  Invalidate;
  CheckButtons;
end;

procedure TsohoExpertTree.SetItemIndex(Value : Integer);
begin
  if (Value < 0) or (Value > FItems.Count - 1) or
     (Value = FItemIndex) then exit;
  FillBox(ItemIndex, false);
  FItemIndex := Value;
  FillBox(ItemIndex, true);
  if Assigned(FNotebook) and
     (FNotebook.Pages.Count >= FItems.Count) then
     FNotebook.PageIndex := ItemIndex;
  // Invalidate;
  CheckButtons;
  if not (csLoading in ComponentState) then Change;
end;

procedure TsohoExpertTree.AllocateRects;
var Index: Integer;
    P: Pointer;
begin
  if FRects.Count < FItems.Count then
    for Index := FRects.Count to Pred(FItems.Count) do begin
      GetMem(P, SizeOf(TRect));
      FRects.Add(P);
    end
  else if FRects.Count > FItems.Count then
    for Index := Pred(FRects.Count) downto FItems.Count do begin
      FreeMem(FRects[Index]);
      FRects.Delete(Index);
    end;
end;

procedure TsohoExpertTree.Paint;
var Index: Integer;
    X, Y: Integer;
begin
  AllocateRects;
  Canvas.Brush.Color := Color;
  Canvas.FillRect( Rect( 0 , 0 , Width , Height ) );
  Y := TopMargin + VerticalSpace;
  for Index := 0 to Pred( FItems.Count ) do begin
    if ( Index > 0 ) and ( Index < Pred( FItems.Count ) ) then
      X := LeftMargin + HorizontalSpace + BoxWidth + HorizontalSpace
    else
      X := LeftMargin + HorizontalSpace;
    TRect(FRects[Index]^) := Rect(X, Y,
      X + BoxWidth + HorizontalSpace + Canvas.TextWidth(FItems[Index]),
      Y + BoxHeight);
    FillBox(Index, ItemIndex = Index);
    Inc(Y, BoxHeight + VerticalSpace * 2);
    Canvas.Pen.Color := LineColor;
    if (Index = 0) then begin
      Canvas.MoveTo(X + BoxWidth, Y - VerticalSpace - BoxHeight);
      Canvas.LineTo(X + HorizontalSpace + BoxWidth + BoxWidth div 2,
                    Y - VerticalSpace - BoxHeight);
      Canvas.LineTo(X + HorizontalSpace + BoxWidth + BoxWidth div 2,
                    Y + VerticalSpace);
    end
    else if (Index = pred(FItems.Count) - 1) then begin
      Canvas.MoveTo(X + BoxWidth div 2, Y - VerticalSpace * 2);
      Canvas.LineTo(X + BoxWidth div 2, Y + VerticalSpace);
      Canvas.LineTo(X - BoxWidth - 1, Y + VerticalSpace);
    end
    else if (Index < pred(FItems.Count)) then begin
      Canvas.MoveTo(X + BoxWidth div 2, Y - VerticalSpace * 2);
      Canvas.LineTo(X + BoxWidth div 2, Y);
    end;
  end;
  X := LeftMargin + HorizontalSpace + (BoxWidth div 2);
  Y := TopMargin + VerticalSpace + BoxHeight;
  Canvas.MoveTo(X, Y);
  Inc(Y, VerticalSpace * 4 + (BoxHeight * (FItems.Count - 2)) +
      ((VerticalSpace * 2) * (FItems.Count - 3)));
  Canvas.Pen.Color := LineColor;
  Canvas.LineTo(X, Y);
end;

procedure TsohoExpertTree.Change;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

function TsohoExpertTree.Changing(var NextPage: LongInt): boolean;
begin
  if (NextPage < 0) or (NextPage >= FItems.Count) or
     (FItems.Count = 0) then begin
     Result := false;
     exit;
  end;
  Result := true;
  //   
  if NextPage = 0 then begin
    if not Assigned(Fbtns[0]) then Result := true
    else Result := Fbtns[0].Enabled;
  end;
  if NextPage = ItemIndex - 1 then begin
    if not Assigned(Fbtns[1]) then Result := true
    else Result := Fbtns[1].Enabled;
  end;
  if NextPage = pred(FItems.Count) then begin
    if not Assigned(Fbtns[3]) then Result := true
    else Result := Fbtns[3].Enabled;
  end;
  if NextPage = ItemIndex + 1 then begin
    if not Assigned(Fbtns[2]) then Result := true
    else Result := Fbtns[2].Enabled;
  end;
  //    ,    ,    
  if Assigned(FOnChanging) then
    Result := Result and FOnChanging(Self, NextPage);
end;

function TsohoExpertTree.MouseToNewPage(CursorPos: TPoint): LongInt;
var Index, PageIndex: Integer;
begin
  Result := -1;
  for Index := 0 to pred(FRects.Count) do
    if PtInRect(TRect(FRects[Index]^), CursorPos) and
      (ItemIndex <> Index) then begin
       PageIndex := Index;
       if Changing(PageIndex) then Result := PageIndex;
       exit;
    end;
end;

procedure TsohoExpertTree.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var Index: Integer;
    P: TPoint;
begin
  inherited;
  P := Point(X, Y);
  Index := MouseToNewPage(P);
  if Index <> -1 then ItemIndex := Index;
end;

procedure TsohoExpertTree.MouseMove (Shift: TShiftState; X, Y: Integer);
var Index: Integer;
    NewCursor: TCursor;
begin
  inherited;
  Index := MouseToNewPage(Point(X, Y));
  NewCursor := crDefault;
  if Index <> -1 then NewCursor := crHandPoint;
  if Cursor <> NewCursor then Cursor := NewCursor; 
end;

procedure TsohoExpertTree.FillBox( Index : Integer; Live : Boolean );
var BoxRect: TRect;
    TextRect: TRect;
begin
  if (Index < 0) or (Index >= FItems.Count) then exit;
  Canvas.Brush.Style := bsSolid;
  if Live then Canvas.Brush.Color := SelectedBoxColor
  else Canvas.Brush.Color := BoxColor;
  BoxRect := TRect(FRects[Index]^);
  TextRect := BoxRect;
  BoxRect.Right := BoxRect.Left + BoxWidth;
  TextRect.Left := BoxRect.Right + HorizontalSpace;
  TextRect.Right := ClientRect.Right;
  if (Index = 0) or (Index = pred(FItems.Count)) then
    Inc(TextRect.Left, HorizontalSpace + BoxWidth div 2);
  Canvas.FillRect(BoxRect);
  Canvas.Font := Font;
  if Live then Canvas.Font.Style := Canvas.Font.Style + [fsBold];
  Canvas.Brush.Color := Color;
  Canvas.FillRect(TextRect);
  Canvas.Brush.Style := bsClear;
  Canvas.TextOut(TextRect.Left, TextRect.Top +
     ((BoxHeight - Canvas.TextHeight(FItems[Index])) div 2),
     FItems[Index]);
  TRect(FRects[Index]^) := Rect (BoxRect.Left, BoxRect.Top,
    BoxRect.Left + BoxWidth + HorizontalSpace +
    Canvas.TextWidth(FItems[Index]), BoxRect.Top + BoxHeight);
end;

end.

