{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{           Partial copyright (c) AllexSoft             }
{              Written by Serg Vostrikov,               }
{                Alexander Govyadinov,                  }
{                    Victor Mylcin                      }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoFldFrm;

{$I SOHOLIB.INC}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, StdCtrls, SohoBtns, SoChkBox, SoRadGrp, SoCtrls, ComCtrls, DB,
  SoUnit, SoDBGrid, SohoSpin;

type
  TsohoFolderForm = class(TForm)
    TabSet: TPageControl;
    ColumnPage: TTabSheet;
    TitlePage: TTabSheet;
    FormatL: TsohoLabel;
    WidthL: TsohoLabel;
    FormatE: TsohoEdit;
    WidthE: TsohoSpinEdit;
    FieldCaptionL: TsohoLabel;
    TitleM: TMemo;
    AlignGroup: TsohoRadioGroup;
    HeightL: TsohoLabel;
    TitleHeightE: TsohoSpinEdit;
    ShowTitleC: TsohoCheckBox;
    AlignTitle: TsohoRadioGroup;
    OkB: TsohoBitBtn;
    CancelB: TsohoBitBtn;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FunctionNumber : integer;
  end;

var sohoFolderForm: TsohoFolderForm;

function SetField (Field : TField; EnabledMenus : SetOfByte; Grid : TsohoDBGrid;
         TabIndex : integer; FuncNumber : integer) : boolean;

implementation
uses SoCtmFld, SoStrUtils, DBGrids, SoRTTI, SoVCLUtils, SoCmnCns, SoDBCns;

{$R *.DFM}

procedure PrepareForm (Field : TField; EnabledMenus : SetOfByte; Grid : TsohoDBGrid);
var aIndex : integer;
begin
     with Field, sohoFolderForm do begin
       AlignGroup.ItemIndex := integer(Alignment);
       if IsProperty(Field,'DisplayFormat') then
         FormatE.AsString := GetStringProperty(Grid.SelectedField,'DisplayFormat')
       else FormatE.Enabled := false;
       WidthE.AsInteger := Grid.ColWidthsByFieldIndex[Grid.SelectedIndex];
       SetDivisers(['~']);
       with TitleM, Lines do begin
         Clear;
         for aIndex := 1 to WordsInString(DisplayLabel) do
          Add(GetSomeWords(DisplayLabel,aIndex,aIndex));
       end;
       AlignTitle.ItemIndex   := integer(Grid.TitleAlignment);
       TitleHeightE.AsInteger := Grid.TitleLines;
       ShowTitleC.Checked     := dgTitles in Grid.Options;

       {  ,   ! }
       TitleM.Enabled       := (mDisplaylabel+1 in EnabledMenus);
       WidthE.Enabled       := (mDisplayWidth+1 in EnabledMenus);
       // TsohoBitBtn(WidthE.Button).Enabled  := (mDisplayWidth+1 in EnabledMenus);
       FormatE.Enabled      := FormatE.Enabled and (mDisplayFormat+1 in EnabledMenus);
       AlignGroup.Enabled   := (mFieldAlign+1 in EnabledMenus);
       TitleHeightE.Enabled := (mTitleLines+1 in EnabledMenus);
       // TsohoBitBtn(WidthE.Button).Enabled := (mTitleLines+1 in EnabledMenus);
       AlignTitle.Enabled   := (mTitleAlign+1 in EnabledMenus);
       ShowTitleC.Enabled   := AlignTitle.Enabled and TitleHeightE.Enabled;

       Caption := Caption + ' [' + Field.FieldName + ']';
     end;
end;

procedure ChangeField (Field : TField; EnabledMenus : SetOfByte; Grid : TsohoDBGrid);
var TmpTitle : string;
    aIndex   : integer;
begin
     with Field, sohoFolderForm do begin
       {  ,   ! }
       if TitleM.Enabled then begin
          TmpTitle := '';
          for aIndex := 0 to pred(TitleM.Lines.Count) do
            TmpTitle := TmpTitle + TitleM.Lines.Strings[aIndex]+'~';
          if TmpTitle[length(TmpTitle)]='~' then TmpTitle := Copy(TmpTitle,1,length(TmpTitle)-1);
          DisplayLabel := TmpTitle;
       end;
       if WidthE.Enabled then Grid.ColWidthsByFieldIndex[Grid.SelectedIndex] := WidthE.AsInteger;
       if FormatE.Enabled then
         SetStringProperty(Grid.SelectedField,'DisplayFormat',FormatE.AsString);
       if AlignGroup.Enabled then Alignment := TAlignment(AlignGroup.ItemIndex);
       if TitleHeightE.Enabled and (TitleHeightE.AsInteger>0) then
          Grid.TitleLines := TitleHeightE.AsInteger;
       if AlignTitle.Enabled then
          Grid.TitleAlignment := TAlignment(AlignTitle.ItemIndex);
       if ShowTitleC.Enabled then begin
         if ShowTitleC.Checked then Grid.Options := Grid.Options + [dgTitles]
         else Grid.Options := Grid.Options - [dgTitles];
       end;
     end;
end;

function SetField (Field : TField; EnabledMenus : SetOfByte; Grid : TsohoDBGrid;
         TabIndex : integer; FuncNumber : integer) : boolean;
begin
     sohoFolderForm := TsohoFolderForm.Create(Application);
     try
        PrepareForm(Field, EnabledMenus, Grid);
        with sohoFolderForm do begin
          // SetPageIndex(TabSet,TabIndex);
          FunctionNumber := FuncNumber;
          Result := ShowModal=mrOk;
        end;
        if Result then ChangeField(Field, EnabledMenus, Grid);
     finally
        sohoFolderForm.Free;
     end;
end;

procedure TsohoFolderForm.FormShow(Sender: TObject);
begin
     case FunctionNumber of
       mDisplaylabel   : begin
         TitleM.SetFocus;
         TitleM.SelStart := 0;
       end;
       mDisplayWidth   : WidthE.SetFocus;
       mDisplayFormat  : FormatE.SetFocus;
       mFieldAlign     : AlignGroup.SetFocus;
       mTitleLines     : begin
         SetPageIndex(TabSet, 1);
         TitleHeightE.SetFocus;
       end;
       mTitleAlign     : begin
         SetPageIndex(TabSet, 1);
         AlignTitle.SetFocus;
       end;
     end;
end;

procedure TsohoFolderForm.FormCreate(Sender: TObject);
begin
  Caption := sohoFolderDialogTitle;
  ColumnPage.Caption := sohoFolderDialogColumnPage;
  TitlePage.Caption := sohoFolderDialogTitlePage;
  FormatL.Caption := sohoFolderDialogFormat;
  WidthL.Caption := sohoFolderDialogWidth;
  FieldCaptionL.Caption := sohoFolderDialogFieldCaption;
  AlignGroup.Caption := sohoFolderDialogAlignGroup;
  with AlignGroup, Items do begin
    Clear;
    Add(sohoFolderDialogAlignLeft);
    Add(sohoFolderDialogAlignRight);
    Add(sohoFolderDialogAlignCenter);
  end;

  HeightL.Caption := sohoFolderDialogHeight;
  ShowTitleC.Caption := sohoFolderDialogShowTitle;
  AlignTitle.Caption := sohoFolderDialogAlignTitle;
  with AlignTitle, Items do begin
    Clear;
    Add(sohoFolderDialogAlignLeft);
    Add(sohoFolderDialogAlignRight);
    Add(sohoFolderDialogAlignCenter);
  end;

  OkB.Caption := SOKButton;
  CancelB.Caption := SCancelButton;
end;

end.
