{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoFltForm;

{$I SOHOLIB.INC}

interface

uses SysUtils, Classes, Forms, SoChkBox, SoCtrls, SohoBtns,
     StdCtrls, Controls;

type
  TsohoFilterForm = class(TForm)
    ShowOnlyL: TsohoLabel;
    And1C: TsohoCheckBox;
    Or1C: TsohoCheckBox;
    Cond2C: TsohoComboBox;
    Value1E: TsohoEdit;
    Value2E: TsohoEdit;
    CommentL: TsohoLabel;
    OkB: TsohoBitBtn;
    CancelB: TsohoBitBtn;
    Cond1C: TsohoComboBox;
    CaseC: TsohoCheckBox;
    procedure And1CChange(Sender: TObject);
    procedure Or1CChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure OkBClick(Sender: TObject);
  private
    { Private declarations }
    function Execute (
       var Cond1, Cond2: integer;
       var IsAnd, CaseSens: boolean;
       var Value1, Value2: string
      ): boolean;
  public
    { Public declarations }
  end;

var sohoFilterForm: TsohoFilterForm;

function GetFilterConditions (
   var Cond1, Cond2: integer;
   var IsAnd, CaseSens: boolean;
   var Value1, Value2: string
  ): boolean;

implementation
uses SoDBCns, SoCmnCns, SoVClUtils;

{$R *.DFM}

function GetFilterConditions (
   var Cond1, Cond2: integer;
   var IsAnd, CaseSens: boolean;
   var Value1, Value2: string
  ): boolean;
begin
  try
    sohoFilterForm := TsohoFilterForm.Create(Application);
    Result := sohoFilterForm.Execute(Cond1, Cond2, IsAnd, CaseSens, Value1, Value2);
  finally
    sohoFilterForm.Free;
  end;
end;

procedure TsohoFilterForm.And1CChange(Sender: TObject);
begin
  Or1C.Checked := not And1C.Checked;
end;

procedure TsohoFilterForm.Or1CChange(Sender: TObject);
begin
  And1C.Checked := not Or1C.Checked;
end;

procedure TsohoFilterForm.FormCreate(Sender: TObject);
begin
  Caption := sohoFltCaption;
  OkB.Caption := SOKButton;
  CancelB.Caption := SCancelButton;
  ShowOnlyL.Caption := sohoFltShowOnly;
  And1C.Caption := sohoFltAnd;
  Or1C.Caption := sohoFltOr;
  CommentL.Caption := sohoFltComment;
  CaseC.Caption := sohoFltCaseSensitive;

  with Cond1C, Items do begin
    Add('');
    Add(sohoFltEqual);
    Add(sohoFltNotEqual);
    Add(sohoFltGreater);
    Add(sohoFltNotGreater);
    Add(sohoFltLess);
    Add(sohoFltNotLess);
    Add(sohoFltStartWith);
    Add(sohoFltNotStartWith);
    Add(sohoFltFinishOn);
    Add(sohoFltNotFinishOn);
    Add(sohoFltContain);
    Add(sohoFltNotContain);
  end;
  Cond2C.Items.Assign(Cond1C.Items);
  Cond1C.ItemIndex := 1;
  Cond2C.ItemIndex := 0;
end;

function TsohoFilterForm.Execute(var Cond1, Cond2: integer;
  var IsAnd, CaseSens: boolean; var Value1, Value2: string): boolean;
begin
  Cond1C.ItemIndex := Cond1;
  Cond2C.ItemIndex := Cond2;
  Value1E.AsString := Value1;
  Value2E.AsString := Value2;
  And1C.Checked := IsAnd;
  CaseC.Checked := CaseSens;
  Result := ShowModal = mrOk;
  if not Result then exit;
  Cond1 := Cond1C.ItemIndex;
  Cond2 := Cond2C.ItemIndex;
  Value1 := Value1E.AsString;
  Value2 := Value2E.AsString;
  IsAnd := And1C.Checked;
  CaseSens := CaseC.Checked;
end;

procedure TsohoFilterForm.OkBClick(Sender: TObject);
begin
  if Cond1C.ItemIndex = 0 then begin
    ErrorMsg(sohoFltNoOperation);
    Cond1C.SetFocus;
    exit;
  end;
  if (Cond2C.ItemIndex = 0) and
     (Value2E.AsString <> '') then Cond2C.ItemIndex := Cond1C.ItemIndex;
  ModalResult := mrOk;
end;

end.
