{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoFrmProps;

{$I SOHOLIB.INC}

interface

uses
  SysUtils, Messages, Classes, Graphics, Controls, Forms, StdCtrls,
  Buttons, ExtCtrls, Consts, {$IFDEF Delphi6}DesignEditors, DesignIntf{$ELSE}DsgnIntf{$ENDIF}, SoChkBox, ComCtrls,
  SohoBtns, SoCtrls, SoListView, SoFrmStorage, SoProps;

type

{$IFNDEF Delphi4_or_5}
  IDesigner = TDesigner;
{$ENDIF}

{ TFormPropsDlg }

  TSoFormPropsDlg = class(TForm)
    Bevel1: TBevel;
    ComponentsL: TsohoLabel;
    PropertiesL: TsohoLabel;
    SavedPropertiesL: TsohoLabel;
    UpBtn: TsohoBitBtn;
    DownBtn: TsohoBitBtn;
    StoredList: TsohoListView;
    PropertiesList: TListBox;
    ComponentsList: TListBox;
    FormBox: TGroupBox;
    ActiveCtrlBox: TsohoCheckBox;
    PositionBox: TsohoCheckBox;
    StateBox: TsohoCheckBox;
    AddButton: TsohoBitBtn;
    DeleteButton: TsohoBitBtn;
    ClearButton: TsohoBitBtn;
    OkB: TsohoBitBtn;
    CancelB: TsohoBitBtn;
    procedure AddButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure StoredListClick(Sender: TObject);
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure PropertiesListDblClick(Sender: TObject);
    procedure ComponentsListClick(Sender: TObject);
    procedure PropertiesListClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure StoredListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    FCompOwner: TComponent;
    FDesigner: IDesigner;
    FLastProperty: TStringList;
    function CheckAddItem(const CompName, PropName: string): boolean;
    // procedure AddItem(IdxComp, IdxProp: Integer; AUpdate: Boolean);
    procedure BuildLists(StoredProps: TStrings);
    procedure CheckButtons;
    procedure SetStoredList(AList: TStrings);
  public
    { Public declarations }
  end;

{ TFormStorageEditor }

  TsohoFormStorageEditor = class(TComponentEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

{ TStoredPropsProperty }

  TsohoStoredPropsProperty = class(TClassProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure Edit; override;
  end;

{ Show component editor }
function ShowStorageDesigner(ACompOwner: TComponent; ADesigner: IDesigner;
  AStoredList: TStrings; var Options: TsohoPlacementOptions): Boolean;

implementation
uses Windows, TypInfo, SoDesCns, SoStrUtils, SoVCLUtils, SoCmnCns;

{$R *.DFM}

{ TsohoFormStorageEditor }

procedure TsohoFormStorageEditor.ExecuteVerb(Index: Integer);
var
  Storage: TsohoFormStorage;
  Opt: TsohoPlacementOptions;
begin
  Storage := Component as TsohoFormStorage;
  if Index = 0 then begin
    Opt := Storage.Options;
    if ShowStorageDesigner(TComponent(Storage.Owner), Designer,
      Storage.StoredProps, Opt) then
    begin
      Storage.Options := Opt;
      Storage.SetNotification;
      Designer.Modified;
    end;
  end;
end;

function TsohoFormStorageEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := sohoFormStorage;
    else Result := '';
  end;
end;

function TsohoFormStorageEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

{ TsohoStoredPropsProperty }

function TsohoStoredPropsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog] - [paSubProperties];
end;

function TsohoStoredPropsProperty.GetValue: string;
begin
  if TStrings(GetOrdValue).Count > 0 then Result := inherited GetValue
  else Result := '(None)';
end;

procedure TsohoStoredPropsProperty.Edit;
var
  Storage: TsohoFormStorage;
  Opt: TsohoPlacementOptions;
begin
  Storage := GetComponent(0) as TsohoFormStorage;
  Opt := Storage.Options;
  if ShowStorageDesigner(Storage.Owner as TComponent, Designer,
    Storage.StoredProps, Opt) then
  begin
    Storage.Options := Opt;
    Storage.SetNotification;
    Designer.Modified;
  end;
end;

{ Show component editor }

procedure ListViewToStrings (FromLV: TListView; ToLB: TStrings);
var Index: LongInt;
    Key, Tmp: string;
begin
  ToLB.Clear;
  for Index := 0 to pred(FromLV.Items.Count) do begin
    Key := FromLV.Items[Index].Caption;
    Tmp := ChangeChars(Key, sNameDelim, '.');
    if AnsiUpperCase(Tmp) <> AnsiUpperCase(FromLV.Items[Index].SubItems[0]) then
       ToLB.AddObject(FromLV.Items[Index].SubItems[0] + sKeyDelim + Key,
         TObject(FromLV.Items[Index].Data))
    else
       ToLB.AddObject(FromLV.Items[Index].SubItems[0],
         TObject(FromLV.Items[Index].Data));
  end;
end;

function ShowStorageDesigner(ACompOwner: TComponent; ADesigner: IDesigner;
  AStoredList: TStrings; var Options: TsohoPlacementOptions): Boolean;
begin
  with TSoFormPropsDlg.Create(Application) do
  try
    FCompOwner := ACompOwner;
    FDesigner := ADesigner;
    Screen.Cursor := crHourGlass;
    try
      UpdateStoredList(ACompOwner, AStoredList, True);
      SetStoredList(AStoredList);
      BuildLists(AStoredList);
      ActiveCtrlBox.Checked := fpActiveControl in Options;
      PositionBox.Checked := fpPosition in Options;
      StateBox.Checked := fpState in Options;
    finally
      Screen.Cursor := crDefault;
    end;
    Result := ShowModal = mrOk;
    if Result then begin
      // AStoredList.Assign(StoredList.Items);
      ListViewToStrings(StoredList, AStoredList);
      Options := [];
      if ActiveCtrlBox.Checked then Include(Options, fpActiveControl);
      if PositionBox.Checked then Include(Options, fpPosition);
      if StateBox.Checked then Include(Options, fpState);
    end;
  finally
    Free;
  end;
end;

{ TSoFormPropsDlg }

function TSoFormPropsDlg.CheckAddItem(const CompName, PropName: string): boolean;
var Tmp: string;
    Index: LongInt;    
begin
  Result := true;
  Tmp := AnsiUpperCase(CompName + '.' + PropName);
  for Index := 0 to pred(StoredList.Items.Count) do
    if AnsiUpperCase(StoredList.Items[Index].SubItems[0]) =
       Tmp then begin
       Result := false;
       exit;
  end;
end;

procedure TSoFormPropsDlg.BuildLists(StoredProps: TStrings);
var Index: LongInt;
begin
  ComponentsList.Clear;
  if FCompOwner is TCustomForm then begin
    for Index := 0 to pred(FCompOwner.ComponentCount) do
      if FCompOwner.Components[Index].Name <> '' then
       ComponentsList.Items.AddObject(FCompOwner.Components[Index].Name,
         FCompOwner.Components[Index]);
  end;
  CheckButtons;
end;

procedure TSoFormPropsDlg.SetStoredList(AList: TStrings);
var Item: TListItem;
    Index: LongInt;
    CompName, PropName, KeyName: string;
begin
  StoredList.Items.Clear;
  if not Assigned(aList) then exit;
  for Index := 0 to pred(aList.Count) do begin
    Item := StoredList.Items.Add;
    ParseStoredItem(aList[Index], CompName, PropName, KeyName);
    if KeyName = '' then KeyName := CompName + sNameDelim + PropName;
    Item.Caption := KeyName;
    Item.SubItems.Add(CompName + '.' + PropName);
    Item.Data := aList.Objects[Index];
  end;
end;

procedure TSoFormPropsDlg.CheckButtons;
var Enable: boolean;
begin
  AddButton.Enabled := (ComponentsList.ItemIndex >= 0) and
    (PropertiesList.ItemIndex >= 0);
  Enable := (StoredList.Items.Count > 0) and
            Assigned(StoredList.Selected);
  DeleteButton.Enabled := Enable;
  ClearButton.Enabled := Enable;
  UpBtn.Enabled := Enable and Assigned(StoredList.Selected);
  DownBtn.Enabled := Enable and Assigned(StoredList.Selected);
end;

procedure TSoFormPropsDlg.AddButtonClick(Sender: TObject);
var Item: TListItem;
    Index: LongInt;
begin
  if PropertiesList.ItemIndex = -1 then exit;
  FLastProperty.Clear;
  for Index := 0 to pred(PropertiesList.Items.Count) do begin
    if not PropertiesList.Selected[Index] then Continue;
    FLastProperty.Add(PropertiesList.Items[Index]);
    if not CheckAddItem(
      ComponentsList.Items[ComponentsList.ItemIndex],
      PropertiesList.Items[Index]) then Continue;
    Item := StoredList.Items.Add;
    Item.Caption := ComponentsList.Items[ComponentsList.ItemIndex] + sNameDelim +
      PropertiesList.Items[Index];
    Item.SubItems.Add(ComponentsList.Items[ComponentsList.ItemIndex] + '.' +
      PropertiesList.Items[Index]);
    Item.Data := ComponentsList.Items.Objects[ComponentsList.ItemIndex];
  end;
  CheckButtons;
end;

procedure TSoFormPropsDlg.ClearButtonClick(Sender: TObject);
begin
  if StoredList.Items.Count > 0 then begin
    SetStoredList(nil);
    if FDesigner <> nil then FDesigner.Modified;
  end;
end;

procedure TSoFormPropsDlg.StoredListClick(Sender: TObject);
begin
  CheckButtons;
end;

procedure TSoFormPropsDlg.UpBtnClick(Sender: TObject);
var WasIndex: LongInt;
    P1, P2: string;
    Item: TListItem;
begin
  if Assigned(StoredList.Selected) and
     (StoredList.Selected.Index > 0) then begin
     P1 := StoredList.Selected.Caption;
     P2 := StoredList.Selected.SubItems[0];
     WasIndex := StoredList.Selected.Index;
     StoredList.Items.Delete(WasIndex);
     Item := StoredList.Items.Insert(WasIndex - 1);
     Item.Caption := P1;
     Item.SubItems.Add(P2);
     StoredList.Selected := Item;
  end;
  if FDesigner <> nil then FDesigner.Modified;
  CheckButtons;
end;

procedure TSoFormPropsDlg.DownBtnClick(Sender: TObject);
var WasIndex: LongInt;
    P1, P2: string;
    Item: TListItem;
begin
  if Assigned(StoredList.Selected) and
     (StoredList.Selected.Index < pred(StoredList.Items.Count)) then begin
     P1 := StoredList.Selected.Caption;
     P2 := StoredList.Selected.SubItems[0];
     WasIndex := StoredList.Selected.Index;
     StoredList.Items.Delete(WasIndex);
     Item := StoredList.Items.Insert(WasIndex + 1);
     Item.Caption := P1;
     Item.SubItems.Add(P2);
     StoredList.Selected := Item;
  end;
  if FDesigner <> nil then FDesigner.Modified;
  CheckButtons;
end;

procedure TSoFormPropsDlg.PropertiesListDblClick(Sender: TObject);
begin
  if AddButton.Enabled then AddButtonClick(nil);
end;

procedure TSoFormPropsDlg.ComponentsListClick(Sender: TObject);
var Props: TsohoPropInfoList;
    Index, FoundIndex: LongInt;
begin
  PropertiesList.Clear;
  if ComponentsList.ItemIndex < 0 then exit;
  Props := TsohoPropInfoList.Create(
    ComponentsList.Items.Objects[ComponentsList.ItemIndex], tkProperties);
  for Index := 0 to pred(Props.Count) do
    PropertiesList.Items.Add(Props.Items[Index].Name);
  Props.Free;
  for Index:= 0 to pred(FLastProperty.Count) do begin
    FoundIndex := PropertiesList.Items.IndexOf(FLastProperty[Index]);
    if FoundIndex <> -1 then
      PropertiesList.Selected[FoundIndex] := true;
  end;
end;

procedure TSoFormPropsDlg.PropertiesListClick(Sender: TObject);
begin
  CheckButtons;
end;

procedure TSoFormPropsDlg.DeleteButtonClick(Sender: TObject);
begin
  if not Assigned(StoredList.Selected) then exit;
  StoredList.Items.Delete(StoredList.Selected.Index);
end;

procedure TSoFormPropsDlg.StoredListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Assigned(StoredList.Selected) and (Key = vk_F12) then InfoMsg('   ');
    // StoredList.Selected.EditCaption;
end;

procedure TSoFormPropsDlg.FormCreate(Sender: TObject);
begin
  FLastProperty := TStringList.Create;

  Caption := sohoFormStorage;

  ComponentsL.Caption := sohoFormStorageComponents;
  PropertiesL.Caption := sohoFormStorageProperties;
  SavedPropertiesL.Caption := sohoFormStorageSavedProperties;
  FormBox.Caption := sohoFormStorageFormBox;
  ActiveCtrlBox.Caption := sohoFormStorageActiveCtrlBox;
  PositionBox.Caption := sohoFormStoragePositionBox;
  StateBox.Caption := sohoFormStorageStateBox;
  AddButton.Caption := sohoFormStorageAddButton;
  DeleteButton.Caption := sohoFormStorageDeleteButton;
  ClearButton.Caption := sohoFormStorageClearButton;

  OkB.Caption := SOKButton;
  CancelB.Caption := SCancelButton;
end;

procedure TSoFormPropsDlg.FormDestroy(Sender: TObject);
begin
  FLastProperty.Free;
end;

procedure TSoFormPropsDlg.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = vk_F2) and (ActiveControl = StoredList)
    and (StoredList.Selected <> nil) then
      StoredList.Selected.EditCaption
end;

end.