{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Adapted by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoFrmStorage;

{$I SOHOLIB.INC}

interface

uses Windows, Controls, Messages, Classes, Forms, IniFiles, Graphics;

type
  TsohoPlacementOption = (fpState, fpPosition, fpActiveControl);
  TsohoPlacementOptions = set of TsohoPlacementOption;
  TsohoPlacementOperation = (poSave, poRestore);

  TsohoFormStorage = class(TComponent)
  private
    FActive: Boolean;
    FIniFileName: PString;
    FIniSection: PString;
    FIni, FIniFile: TIniFile;
    FOptions: TsohoPlacementOptions;
    FSaved: Boolean;
    FRestored: Boolean;
    FDestroying: Boolean;
    FDefMaximize: Boolean;
    FSaveFormShow: TNotifyEvent;
    FSaveFormDestroy: TNotifyEvent;
    FSaveFormCloseQuery: TCloseQueryEvent;
    FOnSavePlacement: TNotifyEvent;
    FOnRestorePlacement: TNotifyEvent;

    FStoredProps: TStrings;
    procedure SetStoredProps(Value: TStrings);

    function GetIniSection: string;
    procedure SetIniSection(const Value: string);
    function GetIniFileName: string;
    procedure SetIniFileName(const Value: string);
    function GetIniFile: TObject;
    procedure UpdatePlacement;
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormDestroy(Sender: TObject);
  protected

    procedure SaveProperties; virtual;
    procedure RestoreProperties; virtual;
    procedure WriteState(Writer: TWriter); override;

    procedure Loaded; override;
    procedure Save; dynamic;
    procedure Restore; dynamic;
    procedure SavePlacement; virtual;
    procedure RestorePlacement; virtual;
    function DoReadString(const Section, Ident, Default: string): string; virtual;
    procedure DoWriteString(const Section, Ident, Value: string); virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    procedure IniNeeded;
    procedure IniFree;

    procedure SetNotification;
    procedure SetEvents;
    procedure RestoreEvents;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SaveFormPlacement;
    procedure RestoreFormPlacement;
    procedure EraseSections;

    procedure WriteString(const IniSection, Ident, Value: string);
    {   ini-  }
    procedure WriteInteger(const IniSection, Ident: string; Value: Longint);
    {   ini-   }
    procedure WriteBool(const IniSection, Ident: string; Value: boolean);
    {   ini-   }
    procedure WriteColor(const IniSection, Ident: string; Value: TColor);
    {   ini-  }
    procedure WriteFont(const IniSection, Ident: string; Value: tFont);
    {   ini-  }
    function ReadString(const IniSection, Ident, DefValue: string): string;
    {   ini-  }
    procedure ReadFont(const IniSection, Ident: string; var Value: TFont; DefValue: tFont);
    {   ini-  }
    function ReadInteger(const IniSection, Ident: string; DefValue: Longint): Longint;
    {   ini-   }
    function ReadBool(const IniSection, Ident: string; DefValue: boolean): boolean;
    {   ini-   }
    function ReadColor(const IniSection, Ident: string; DefValue: TColor): TColor;
    {   ini-  }

    procedure WriteStrings(const IniSection: string; const Idents: array of string;
      Values: array of string);
    {   ini-  }
    procedure WriteIntegers(const IniSection: string; const Idents: array of string;
      Values: array of Longint);
    {   ini-   }
    procedure WriteBools(const IniSection: string; const Idents: array of string;
      Values: array of boolean);
    {   ini-   }
    procedure WriteColors(const IniSection: string; const Idents: array of string;
      Values: array of TColor);
    {   ini-  }
    procedure WriteFonts(const IniSection: string; const Idents: array of string;
      Values: array of TFont);
    {   ini-  }
    procedure ReadStrings(const IniSection: string; const Idents: array of string;
      var Values: array of string; const DefValues: array of string);
    {   ini-  }
    procedure ReadFonts(const IniSection: string; const Idents: array of string;
      var Values: array of TFont; DefValues: array of TFont);
    {   ini-  }
    procedure ReadIntegers(const IniSection: string; const Idents: array of string;
      var Values: array of LongInt; DefValues: array of Longint);
    {   ini-   }
    procedure ReadBools(const IniSection: string; const Idents: array of string;
      var Values: array of boolean; DefValues: array of boolean);
    {   ini-   }
    procedure ReadColors(const IniSection: string; const Idents: array of string;
      var Values: array of TColor; DefValues: array of TColor);
    {   ini-  }

    procedure ReadSection(const IniSection: string; Strings: TStrings);
    {   ini-     }
    procedure ReadSectionValues(const IniSection: string; Strings: TStrings);
    {   ini-      }
    procedure ReadKeyValues(const IniSection: string; Strings: TStrings);
    {   ini-   }
    procedure DeleteIdent(const IniSection, Ident: string);
    {   ini-  }
    procedure RenameSection(const OldSection, NewSection: string);
    {   ini-  }
    procedure RenameIdent(const IniSection, OldIdent, NewIdent: string);
    {   ini-  }
    procedure EraseSection(const IniSection: string);
    {   ini-  }
    procedure Clear;
    {  ,    Section }

    property IniFileObject: TObject read GetIniFile;
    property IniFile: TIniFile read FIniFile;
  published
    property Active: Boolean read FActive write FActive default True;
    property IniFileName: string read GetIniFileName write SetIniFileName;
    property IniSection: string read GetIniSection write SetIniSection;
    property Options: TsohoPlacementOptions read FOptions write FOptions default [fpState, fpPosition];
    property StoredProps: TStrings read FStoredProps write SetStoredProps;
    property OnSavePlacement: TNotifyEvent read FOnSavePlacement
      write FOnSavePlacement;
    property OnRestorePlacement: TNotifyEvent read FOnRestorePlacement
      write FOnRestorePlacement;
  end;

implementation
uses SysUtils, SoStrUtils, SoVCLUtils, SoUtils, SoAppUtils, SoProps;

const
{ The following string should not be localized }
  siActiveCtrl = 'ActiveControl';
  siVisible    = 'Visible';
  siFlags      = 'Flags';
  siShowCmd    = 'ShowCmd';
  siNormPos    = 'NormPos';
  siMinMaxPos  = 'MinMaxPos';
  siPixels     = 'PixelsPerInch';
  siMDIChild   = 'MDI Children';
  siListCount  = 'Count';
  siItem       = 'Item%d';

function FindPart(const HelpWilds, InputStr: string): Integer;
var
  I, J: Integer;
  Diff: Integer;
begin
  I := Pos('?', HelpWilds);
  if I = 0 then begin
    { if no '?' in HelpWilds }
    Result := Pos(HelpWilds, InputStr);
    Exit;
  end;
  { '?' in HelpWilds }
  Diff := Length(InputStr) - Length(HelpWilds);
  if Diff < 0 then begin
    Result := 0;
    Exit;
  end;
  { now move HelpWilds over InputStr }
  for I := 0 to Diff do begin
    for J := 1 to Length(HelpWilds) do begin
      if (InputStr[I + J] = HelpWilds[J]) or
        (HelpWilds[J] = '?') then
      begin
        if J = Length(HelpWilds) then begin
          Result := I + 1;
          Exit;
        end;
      end
      else Break;
    end;
  end;
  Result := 0;
end;

function IsWild(InputStr, Wilds: string; IgnoreCase: Boolean): Boolean;

 function SearchNext(var Wilds: string): Integer;
 { looking for next *, returns position and string until position }
 begin
   Result := Pos('*', Wilds);
   if Result > 0 then Wilds := Copy(Wilds, 1, Result - 1);
 end;

var
  CWild, CInputWord: Integer; { counter for positions }
  I, LenHelpWilds: Integer;
  MaxInputWord, MaxWilds: Integer; { Length of InputStr and Wilds }
  HelpWilds: string;
begin
  if Wilds = InputStr then begin
    Result := True;
    Exit;
  end;
  repeat { delete '**', because '**' = '*' }
    I := Pos('**', Wilds);
    if I > 0 then
      Wilds := Copy(Wilds, 1, I - 1) + '*' + Copy(Wilds, I + 2, MaxInt);
  until I = 0;
  if Wilds = '*' then begin { for fast end, if Wilds only '*' }
    Result := True;
    Exit;
  end;
  MaxInputWord := Length(InputStr);
  MaxWilds := Length(Wilds);
  if IgnoreCase then begin { upcase all letters }
    InputStr := AnsiUpperCase(InputStr);
    Wilds := AnsiUpperCase(Wilds);
  end;
  if (MaxWilds = 0) or (MaxInputWord = 0) then begin
    Result := False;
    Exit;
  end;
  CInputWord := 1;
  CWild := 1;
  Result := True;
  repeat
    if InputStr[CInputWord] = Wilds[CWild] then begin { equal letters }
      { goto next letter }
      Inc(CWild);
      Inc(CInputWord);
      Continue;
    end;
    if Wilds[CWild] = '?' then begin { equal to '?' }
      { goto next letter }
      Inc(CWild);
      Inc(CInputWord);
      Continue;
    end;
    if Wilds[CWild] = '*' then begin { handling of '*' }
      HelpWilds := Copy(Wilds, CWild + 1, MaxWilds);
      I := SearchNext(HelpWilds);
      LenHelpWilds := Length(HelpWilds);
      if I = 0 then begin
        { no '*' in the rest, compare the ends }
        if HelpWilds = '' then Exit; { '*' is the last letter }
        { check the rest for equal Length and no '?' }
        for I := 0 to LenHelpWilds - 1 do begin
          if (HelpWilds[LenHelpWilds - I] <> InputStr[MaxInputWord - I]) and
            (HelpWilds[LenHelpWilds - I]<> '?') then
          begin
            Result := False;
            Exit;
          end;
        end;
        Exit;
      end;
      { handle all to the next '*' }
      Inc(CWild, 1 + LenHelpWilds);
      I := FindPart(HelpWilds, Copy(InputStr, CInputWord, MaxInt));
      if I= 0 then begin
        Result := False;
        Exit;
      end;
      CInputWord := I + LenHelpWilds;
      Continue;
    end;
    Result := False;
    Exit;
  until (CInputWord > MaxInputWord) or (CWild > MaxWilds);
  { no completed evaluation }
  if CInputWord <= MaxInputWord then Result := False;
  if (CWild <= MaxWilds) and (Wilds[MaxWilds] <> '*') then Result := False;
end;

function CrtResString: string;
begin
  Result := Format('(%dx%d)', [GetSystemMetrics(SM_CXSCREEN),
    GetSystemMetrics(SM_CYSCREEN)]);
end;

procedure WritePosStr(IniFile: TIniFile; const Section, Ident, Value: string);
begin
  IniFile.WriteString(Section, Ident + CrtResString, Value);
  IniFile.WriteString(Section, Ident, Value);
end;

procedure InternalWriteFormPlacement(Form: TForm; IniFile: TIniFile;
  const Section: string);
var
  Placement: TWindowPlacement;
begin
  // if not DirectoryExists(ExtractFilePath(IniFileName)) then exit;
  Placement.Length := SizeOf(TWindowPlacement);
  GetWindowPlacement(Form.Handle, @Placement);
  with Placement, TForm(Form) do begin
    if (Form = Application.MainForm) and IsIconic(Application.Handle) then
      ShowCmd := SW_SHOWMINIMIZED;
    if (FormStyle = fsMDIChild) and (WindowState = wsMinimized) then
      Flags := Flags or WPF_SETMINPOSITION;
    IniFile.WriteInteger(Section, siFlags, Flags);
    IniFile.WriteInteger(Section, siShowCmd, ShowCmd);
    IniFile.WriteInteger(Section, siPixels, Screen.PixelsPerInch);
    WritePosStr(IniFile, Section, siMinMaxPos, Format('%d,%d,%d,%d',
      [ptMinPosition.X, ptMinPosition.Y, ptMaxPosition.X, ptMaxPosition.Y]));
    WritePosStr(IniFile, Section, siNormPos, Format('%d,%d,%d,%d',
      [rcNormalPosition.Left, rcNormalPosition.Top, rcNormalPosition.Right,
      rcNormalPosition.Bottom]));
  end;
end;

procedure SaveFormPlacement(Form: TForm; const IniFileName: string);
var IniFile: TIniFile;
begin
  if not DirectoryExists(ExtractFilePath(IniFileName)) then exit;
  IniFile := TIniFile.Create(IniFileName);
  try
    InternalWriteFormPlacement(Form, IniFile, Form.ClassName);
  finally
    IniFile.Free;
  end;
end;

{$HINTS OFF}
type

  THackForm = class(TScrollingWinControl)
  private
    FActiveControl: TWinControl;
    FFocusedControl: TWinControl;
    FBorderIcons: TBorderIcons;
    FBorderStyle: TFormBorderStyle;
{$IFDEF Delphi4_or_5}
    FSizeChanging: Boolean;
{$ENDIF}
    FWindowState: TWindowState; { !! }
  end;

  THackComponent = class(TComponent);
{$HINTS ON}

function ReadPosStr(IniFile: TIniFile; const Section, Ident: string): string;
begin
  Result := IniFile.ReadString(Section, Ident + CrtResString, '');
  if Result = '' then Result := IniFile.ReadString(Section, Ident, '');
end;

procedure InternalReadFormPlacement(Form: TForm; IniFile: TIniFile;
  const Section: string; LoadState, LoadPosition: Boolean);
const
  Delims = [',',' '];
var
  PosStr: string;
  Placement: TWindowPlacement;
  WinState: TWindowState;
  DataFound: Boolean;
begin
  if not (LoadState or LoadPosition) then Exit;
  Placement.Length := SizeOf(TWindowPlacement);
  GetWindowPlacement(Form.Handle, @Placement);
  with Placement, TForm(Form) do begin
    if not IsWindowVisible(Form.Handle) then
      ShowCmd := SW_HIDE;
    if LoadPosition then begin
      DataFound := False;
      Flags := IniFile.ReadInteger(Section, siFlags, Flags);
      PosStr := ReadPosStr(IniFile, Section, siMinMaxPos);
      SetDivisers(Delims);
      if PosStr <> '' then begin
        DataFound := True;
        ptMinPosition.X := StrToIntDef(GetSomeWords(PosStr, 1, 1), 0);
        ptMinPosition.Y := StrToIntDef(GetSomeWords(PosStr, 2, 2), 0);
        ptMaxPosition.X := StrToIntDef(GetSomeWords(PosStr, 3, 3), 0);
        ptMaxPosition.Y := StrToIntDef(GetSomeWords(PosStr, 4, 4), 0);
      end;
      PosStr := ReadPosStr(IniFile, Section, siNormPos);
      if PosStr <> '' then begin
        DataFound := True;
        rcNormalPosition.Left := StrToIntDef(GetSomeWords(PosStr, 1, 1), Left);
        rcNormalPosition.Top := StrToIntDef(GetSomeWords(PosStr, 2, 2), Top);
        rcNormalPosition.Right := StrToIntDef(GetSomeWords(PosStr, 3, 3), Left + Width);
        rcNormalPosition.Bottom := StrToIntDef(GetSomeWords(PosStr, 4, 4), Top + Height);
      end;
      if Screen.PixelsPerInch <> IniFile.ReadInteger(Section, siPixels,
        Screen.PixelsPerInch) then DataFound := False;
      if DataFound then begin
        if not (BorderStyle in [bsSizeable {$IFDEF WIN32}, bsSizeToolWin {$ENDIF}]) then
          rcNormalPosition := Rect(rcNormalPosition.Left, rcNormalPosition.Top,
            rcNormalPosition.Left + Width, rcNormalPosition.Top + Height);
        if rcNormalPosition.Right > rcNormalPosition.Left then begin
          if (Position in [poScreenCenter {$IFDEF Delphi4_or_5}, poDesktopCenter {$ENDIF}]) and
            not (csDesigning in ComponentState) then
          begin
            THackComponent(Form).SetDesigning(True);
            try
              Position := poDesigned;
            finally
              THackComponent(Form).SetDesigning(False);
            end;
          end;
          SetWindowPlacement(Handle, @Placement);
        end;
      end;
    end;
    if LoadState then begin
      WinState := wsNormal;
      { default maximize MDI main form }
      if ((Application.MainForm = Form) {$IFDEF Delphi4_or_5} or
        (Application.MainForm = nil) {$ENDIF}) and ((FormStyle = fsMDIForm) or
        ((FormStyle = fsNormal) and (Position = poDefault))) then
        WinState := wsMaximized;
      ShowCmd := IniFile.ReadInteger(Section, siShowCmd, SW_HIDE);
      case ShowCmd of
        SW_SHOWNORMAL, SW_RESTORE, SW_SHOW:
          WinState := wsNormal;
        SW_MINIMIZE, SW_SHOWMINIMIZED, SW_SHOWMINNOACTIVE:
          WinState := wsMinimized;
        SW_MAXIMIZE: WinState := wsMaximized;
      end;
{$IFDEF WIN32}
      if (WinState = wsMinimized) and ((Form = Application.MainForm)
        {$IFDEF Delphi4_or_5} or (Application.MainForm = nil) {$ENDIF}) then
      begin
        THackForm(Form).FWindowState := wsNormal;
        PostMessage(Application.Handle, WM_SYSCOMMAND, SC_MINIMIZE, 0);
        Exit;
      end;
{$ENDIF}
      if FormStyle in [fsMDIChild, fsMDIForm] then
        THackForm(Form).FWindowState := WinState
      else WindowState := WinState;
    end;
    Update;
  end;
end;

procedure RestoreFormPlacement(Form: TForm; const IniFileName: string);
var IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(IniFileName);
  try
    InternalReadFormPlacement(Form, IniFile, Form.ClassName, True, True);
  finally
    IniFile.Free;
  end;
end;

{ TsohoFormStorage }
constructor TsohoFormStorage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIniFileName := NullStr;
  FIniSection := NullStr;
  FActive := True;
  if AOwner is TForm then FOptions := [fpState, fpPosition]
  else FOptions := [];
  FStoredProps := TStringList.Create;
end;

destructor TsohoFormStorage.Destroy;
begin
  if not (csDesigning in ComponentState) then RestoreEvents;
  DisposeStr(FIniFileName);
  DisposeStr(FIniSection);
  FStoredProps.Free;
  FStoredProps := nil;
  inherited Destroy;
end;

procedure TsohoFormStorage.Loaded;
var
  Loading: Boolean;
begin
  Loading := csLoading in ComponentState;
  inherited Loaded;
  if not (csDesigning in ComponentState) then
    if Loading then SetEvents;
  UpdateStoredList(Owner, FStoredProps, True);
end;

procedure TsohoFormStorage.SetEvents;
var Form: TForm;
begin
  Form := GetOwnerForm(Self);
  if not Assigned(Form) then exit;
  with Form do begin
    FSaveFormShow := OnShow;
    OnShow := FormShow;
    FSaveFormCloseQuery := OnCloseQuery;
    OnCloseQuery := FormCloseQuery;
    FSaveFormDestroy := OnDestroy;
    OnDestroy := FormDestroy;
    FDefMaximize := (biMaximize in BorderIcons);
  end;
end;

procedure TsohoFormStorage.RestoreEvents;
var Form: TForm;
begin
  Form := GetOwnerForm(Self);
  if not Assigned(Form) then exit;
  with Form do begin
    OnShow := FSaveFormShow;
    OnCloseQuery := FSaveFormCloseQuery;
    OnDestroy := FSaveFormDestroy;
  end;
end;

procedure TsohoFormStorage.FormShow(Sender: TObject);
begin
  if Active then
  try
    RestoreFormPlacement;
  except
    Application.HandleException(Self);
  end;
  if Assigned(FSaveFormShow) then FSaveFormShow(Sender);
end;

procedure TsohoFormStorage.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var Form: TForm;
begin
  if Assigned(FSaveFormCloseQuery) then
    FSaveFormCloseQuery(Sender, CanClose);
  Form := GetOwnerForm(Self);
  if not Assigned(Form) then exit;
  if CanClose and Active and (Form.Handle <> 0) then
  try
    SaveFormPlacement;
  except
    Application.HandleException(Self);
  end;
end;

procedure TsohoFormStorage.FormDestroy(Sender: TObject);
begin
  if Active and not FSaved then begin
    FDestroying := True;
    try
      SaveFormPlacement;
    except
      Application.HandleException(Self);
    end;
    FDestroying := False;
  end;
  if Assigned(FSaveFormDestroy) then FSaveFormDestroy(Sender);
end;

procedure TsohoFormStorage.UpdatePlacement;
const
{$IFDEF WIN32}
  Metrics: array[bsSingle..bsSizeToolWin] of Word =
    (SM_CXBORDER, SM_CXFRAME, SM_CXDLGFRAME, SM_CXBORDER, SM_CXFRAME);
{$ELSE}
  Metrics: array[bsSingle..bsDialog] of Word =
    (SM_CXBORDER, SM_CXFRAME, SM_CXDLGFRAME);
{$ENDIF}
var
  Placement: TWindowPlacement;
  Form: TForm;
begin
  Form := GetOwnerForm(Self);
  if not Assigned(Form) then exit;
  if Form.HandleAllocated and
     not (csLoading in ComponentState) then
      Placement.Length := SizeOf(TWindowPlacement);
  GetWindowPlacement(Form.Handle, @Placement);
  if not IsWindowVisible(Form.Handle) then
    Placement.ShowCmd := SW_HIDE;
  if Form.BorderStyle <> bsNone then begin
    Placement.ptMaxPosition.X := -GetSystemMetrics(Metrics[Form.BorderStyle]);
    Placement.ptMaxPosition.Y := -GetSystemMetrics(Metrics[Form.BorderStyle] + 1);
  end
  else Placement.ptMaxPosition := Point(0, 0);
  SetWindowPlacement(Form.Handle, @Placement);
end;

function TsohoFormStorage.GetIniFile: TObject;
begin
  Result := FIniFile;
end;

function TsohoFormStorage.GetIniFileName: string;
begin
  Result := FIniFileName^;
  if (Result = '') and not (csDesigning in ComponentState) then begin
    Result := GetRunDir+DefaultName+'.ini';
  end;
end;

procedure TsohoFormStorage.SetIniFileName(const Value: string);
begin
  AssignStr(FIniFileName, Value);
end;

function TsohoFormStorage.GetIniSection: string;
begin
  Result := FIniSection^;
  if (Result = '') and not (csDesigning in ComponentState) then
    Result := Owner.ClassName;
end;

procedure TsohoFormStorage.SetIniSection(const Value: string);
begin
  AssignStr(FIniSection, Value);
end;

procedure TsohoFormStorage.Save;
begin
  if Assigned(FOnSavePlacement) then FOnSavePlacement(Self);
end;

procedure TsohoFormStorage.Restore;
begin
  if Assigned(FOnRestorePlacement) then FOnRestorePlacement(Self);
end;

procedure WriteFormPlacement(Form: TForm; IniFile: TIniFile;
  const Section: string);
begin
  InternalWriteFormPlacement(Form, IniFile, Section);
end;

procedure TsohoFormStorage.SavePlacement;
var Form: TForm;
begin
  Form := GetOwnerForm(Self);
  if not Assigned(Form) then exit;
  if not DirectoryExists(ExtractFilePath(IniFileName)) then exit;
  WriteFormPlacement(Form, FIniFile, IniSection);
  if (fpActiveControl in Options) and (Form.ActiveControl <> nil) then
    FIniFile.WriteString(IniSection, siActiveCtrl, Form.ActiveControl.Name);
  FIniFile.WriteBool(IniSection, siVisible, FDestroying);
  SaveProperties;
end;

procedure ReadFormPlacement(Form: TForm; IniFile: TIniFile;
  const Section: string; LoadState, LoadPosition: Boolean);
begin
  InternalReadFormPlacement(Form, IniFile, Section, LoadState, LoadPosition);
end;

procedure TsohoFormStorage.RestorePlacement;
var Form: TForm;
begin
  Form := GetOwnerForm(Self);
  if not Assigned(Form) then exit;
  ReadFormPlacement(Form, FIniFile, IniSection, fpState in Options,
    fpPosition in Options);
  FRestored := True;
  RestoreProperties;
end;

procedure TsohoFormStorage.IniNeeded;
begin
  if IniFileObject = nil then begin
    FIniFile := TIniFile.Create(IniFileName);
  end;
end;

procedure TsohoFormStorage.IniFree;
begin
  if IniFileObject <> nil then begin
    IniFileObject.Free;
    FIniFile := nil;
  end;
end;

procedure TsohoFormStorage.DeleteIdent(const IniSection, Ident: string);
var S1: TStringList;
  index, I      : Integer;
  aIdent, aValue: string;
begin
  S1 := TStringList.Create;
  FIni := TIniFile.Create(IniFileName);
  FIni.ReadSection(IniSection, S1);
  index := S1.IndexOf(Ident);
  if index > -1 then begin
    S1.Clear;
    FIni.ReadSectionValues(IniSection, S1);
    FIni.EraseSection(IniSection);
    for I := 0 to S1.Count - 1 do
      if I <> index then begin
        aIdent := Copy(S1[I], 1, Pos('=', S1[I]) - 1);
        aValue := S1.Values[aIdent];
        FIni.WriteString(IniSection, aIdent, aValue);
      end;
  end;
  FIni.Free;
end;

procedure TsohoFormStorage.RenameSection(const OldSection, NewSection: string);
var S1: TStringList;
  aIdent, aValue: string;
  I             : Integer;
begin
  S1 := TStringList.Create;
  FIni := TIniFile.Create(IniFileName);
  FIni.ReadSectionValues(OldSection, S1);
  FIni.EraseSection(OldSection);
  for I := 0 to S1.Count - 1 do begin
    aIdent := Copy(S1[I], 1, Pos('=', S1[I]) - 1);
    aValue := S1.Values[aIdent];
    FIni.WriteString(NewSection, aIdent, aValue);
  end;
  FIni.Free;
end;

procedure TsohoFormStorage.RenameIdent(const IniSection, OldIdent, NewIdent: string);
var S1: TStringList;
  index, I      : Integer;
  aIdent, aValue: string;
begin
  S1 := TStringList.Create;
  FIni := TIniFile.Create(IniFileName);
  FIni.ReadSection(IniSection, S1);
  index := S1.IndexOf(OldIdent);
  if index > -1 then begin
    S1.Clear;
    FIni.ReadSectionValues(IniSection, S1);
    FIni.EraseSection(IniSection);
    for I := 0 to S1.Count - 1 do begin
      if I <> index then begin
        aIdent := Copy(S1[I], 1, Pos('=', S1[I]) - 1);
        aValue := S1.Values[aIdent];
      end
      else begin
        aIdent := NewIdent;
        aValue := S1.Values[Copy(S1[I], 1, Pos('=', S1[I]) - 1)];
      end;
      FIni.WriteString(IniSection, aIdent, aValue);
    end;
  end;
  FIni.Free;
end;

procedure TsohoFormStorage.ReadSection(const IniSection: string; Strings: TStrings);
begin
  FIni := TIniFile.Create(IniFileName);
  FIni.ReadSection(IniSection, Strings);
  FIni.Free;
end;

procedure TsohoFormStorage.ReadSectionValues(const IniSection: string; Strings: TStrings);
begin
  FIni := TIniFile.Create(IniFileName);
  FIni.ReadSectionValues(IniSection, Strings);
  FIni.Free;
end;

procedure TsohoFormStorage.ReadKeyValues(const IniSection: string; Strings: TStrings);
var Tmp : TStringList;
    Index : LongInt;
begin
  Tmp := TStringList.Create;
  FIni := TIniFile.Create(IniFileName);
  Strings.Clear;
  FIni.ReadSection(IniSection, Tmp);
  for Index := 0 to pred(Tmp.Count) do
    Strings.Add(FIni.ReadString(IniSection, Tmp[Index], ''));
  FIni.Free;
  Tmp.Free;
end;

procedure TsohoFormStorage.WriteString(const IniSection, Ident, Value: string);
begin
  FIni := TIniFile.Create(IniFileName);
  FIni.WriteString(IniSection, Ident, Value);
  FIni.Free;
end;

procedure TsohoFormStorage.WriteInteger(const IniSection, Ident: string; Value: Longint);
begin
  FIni := TIniFile.Create(IniFileName);
  FIni.WriteInteger(IniSection, Ident, Value);
  FIni.Free;
end;

procedure TsohoFormStorage.WriteBool(const IniSection, Ident: string; Value: boolean);
begin
  FIni := TIniFile.Create(IniFileName);
  FIni.WriteBool(IniSection, Ident, Value);
  FIni.Free;
end;

procedure TsohoFormStorage.WriteColor(const IniSection, Ident: string; Value: TColor);
begin
  FIni := TIniFile.Create(IniFileName);
  FIni.WriteString(IniSection, Ident, ColorToString(Value));
  FIni.Free;
end;

procedure TsohoFormStorage.WriteFont(const IniSection, Ident: string; Value: tFont);
begin
  FIni := TIniFile.Create(IniFileName);
  FIni.WriteString(IniSection, Ident, FontToStr(Value));
  FIni.Free;
end;

function TsohoFormStorage.ReadString(const IniSection, Ident, DefValue: string): string;
begin
  FIni := TIniFile.Create(IniFileName);
  Result := FIni.ReadString(IniSection, Ident, DefValue);
  FIni.Free;
end;

procedure TsohoFormStorage.ReadFont(const IniSection, Ident: string; var Value: TFont; DefValue: tFont);
begin
  FIni := TIniFile.Create(IniFileName);
  StrToFont(FIni.ReadString(IniSection, Ident, FontToStr(DefValue)), Value);
  FIni.Free;
end;

function TsohoFormStorage.ReadInteger(const IniSection, Ident: string; DefValue: Longint): Longint;
begin
  FIni := TIniFile.Create(IniFileName);
  Result := FIni.ReadInteger(IniSection, Ident, DefValue);
  FIni.Free;
end;

function TsohoFormStorage.ReadBool(const IniSection, Ident: string; DefValue: boolean): boolean;
begin
  FIni := TIniFile.Create(IniFileName);
  Result := StrToBool(FIni.ReadString(IniSection, Ident, BoolToStr(DefValue)));
  FIni.Free;
end;

procedure TsohoFormStorage.ReadBools(const IniSection: string; const Idents: array of string;
              var Values: array of boolean; DefValues: array of boolean);
var Index, HighBounds : longint;
begin
  FIni := TIniFile.Create(IniFileName);
  HighBounds := GetMin(High(Idents), GetMin(High(Values), High(DefValues)));
  for Index := Low(Idents) to HighBounds do
    Values[Index] := StrToBool(FIni.ReadString(IniSection, Idents[Index],
      BoolToStr(DefValues[Index])));
  FIni.Free;
end;

function TsohoFormStorage.ReadColor(const IniSection, Ident: string; DefValue: TColor): TColor;
begin
  FIni := TIniFile.Create(IniFileName);
  Result := StringToColor(FIni.ReadString(IniSection, Ident, ColorToString(DefValue)));
  FIni.Free;
end;

procedure TsohoFormStorage.EraseSection(const IniSection: string);
begin
  FIni := TIniFile.Create(IniFileName);
  FIni.EraseSection(IniSection);
  FIni.Free;
end;

procedure TsohoFormStorage.Clear;
begin
  FIni := TIniFile.Create(IniFileName);
  FIni.EraseSection(IniSection);
  FIni.Free;
end;

function TsohoFormStorage.DoReadString(const Section, Ident,
  Default: string): string;
begin
  if IniFile <> nil then
    Result := IniFile.ReadString(Section, Ident, Default)
  else begin
    IniNeeded;
    try
      Result := IniFile.ReadString(Section, Ident, Default);
    finally
      IniFree;
    end;
  end;
end;

procedure TsohoFormStorage.DoWriteString(const Section, Ident, Value: string);
begin
  if IniFile <> nil then
    IniFile.WriteString(Section, Ident, Value)
  else begin
    IniNeeded;
    try
      IniFile.WriteString(Section, Ident, Value);
    finally
      IniFree;
    end;
  end;
end;

procedure TsohoFormStorage.EraseSections;
var
  Lines: TStrings;
  I: Integer;
begin
  if IniFileObject = nil then begin
    IniNeeded;
    try
      Lines := TStringList.Create;
      try
        IniFile. ReadSections(Lines);
        for I := 0 to Lines.Count - 1 do begin
          if (Lines[I] = IniSection) or
            (IsWild(Lines[I], IniSection + '.*', False) or
            IsWild(Lines[I], IniSection + '\*', False)) then
            IniFile.EraseSection(Lines[I]);
        end;
      finally
        Lines.Free;
      end;
    finally
      IniFree;
    end;
  end;
end;

procedure TsohoFormStorage.SaveFormPlacement;
begin
  if not DirectoryExists(ExtractFilePath(IniFileName)) then exit;
  if FRestored or not Active then begin
    IniNeeded;
    try
      SavePlacement;
      Save;
      FSaved := True;
    finally
      IniFree;
    end;
  end;
end;

procedure TsohoFormStorage.RestoreFormPlacement;
var cActive: TComponent;
    Form: TForm;
begin
  FSaved := False;
  IniNeeded;
  try
    RestorePlacement;
    FRestored := True;
    Restore;
    Form := GetOwnerForm(Self);
    if (fpActiveControl in Options) and Assigned(Form) then begin
      cActive := Form.FindComponent(IniFile.ReadString(IniSection,
         siActiveCtrl, ''));
      if (cActive <> nil) and (cActive is TWinControl) and
        TWinControl(cActive).CanFocus then
          Form.ActiveControl := TWinControl(cActive);
    end;
  finally
    IniFree;
  end;
  UpdatePlacement;
end;

procedure TsohoFormStorage.SetNotification;
var
  I: Integer;
  Component: TComponent;
begin
  for I := FStoredProps.Count - 1 downto 0 do begin
    Component := TComponent(FStoredProps.Objects[I]);
    if Component <> nil then Component.FreeNotification(Self);
  end;
end;

procedure TsohoFormStorage.SetStoredProps(Value: TStrings);
begin
  FStoredProps.Assign(Value);
  SetNotification;
end;

procedure TsohoFormStorage.WriteState(Writer: TWriter);
begin
  UpdateStoredList(Owner, FStoredProps, True);
  inherited WriteState(Writer);
end;

procedure TsohoFormStorage.Notification(AComponent: TComponent; Operation: TOperation);
var
  I: Integer;
  Component: TComponent;
begin
  inherited Notification(AComponent, Operation);
  if not (csDestroying in ComponentState) and (Operation = opRemove) and
    (FStoredProps <> nil) then
    for I := FStoredProps.Count - 1 downto 0 do begin
      Component := TComponent(FStoredProps.Objects[I]);
      if Component = AComponent then FStoredProps.Delete(I);
    end;
end;

procedure TsohoFormStorage.SaveProperties;
begin
  with TsohoPropsStorage.Create do
  try
    Section := IniSection;
    OnWriteString := DoWriteString;
    OnEraseSection := FIniFile.EraseSection;
    StoreObjectsProps(Owner, FStoredProps);
  finally
    Free;
  end;
end;

procedure TsohoFormStorage.RestoreProperties;
begin
  with TsohoPropsStorage.Create do
  try
    Section := IniSection;
    OnReadString := DoReadString;
    try
      LoadObjectsProps(Owner, FStoredProps);
    except
      { ignore any exceptions }
    end;
  finally
    Free;
  end;
end;

procedure TsohoFormStorage.ReadColors(const IniSection: string;
  const Idents: array of string; var Values: array of TColor;
  DefValues: array of TColor);
var Index, HighBounds : longint;
begin
  FIni := TIniFile.Create(IniFileName);
  HighBounds := GetMin(High(Idents), GetMin(High(Values), High(DefValues)));
  for Index := Low(Idents) to HighBounds do
    Values[Index] := StringToColor(FIni.ReadString(IniSection, Idents[Index],
      ColorToString(DefValues[Index])));
  FIni.Free;
end;

procedure TsohoFormStorage.ReadFonts(const IniSection: string;
  const Idents: array of string; var Values: array of TFont;
  DefValues: array of TFont);
var Index, HighBounds : longint;
begin
  FIni := TIniFile.Create(IniFileName);
  HighBounds := GetMin(High(Idents), GetMin(High(Values), High(DefValues)));
  for Index := Low(Idents) to HighBounds do
    StrToFont( FIni.ReadString(IniSection, Idents[Index], FontToStr(DefValues[Index])),
      Values[Index]);
  FIni.Free;
end;

procedure TsohoFormStorage.ReadIntegers(const IniSection: string;
  const Idents: array of string; var Values: array of LongInt;
  DefValues: array of LongInt);
var Index, HighBounds : longint;
begin
  FIni := TIniFile.Create(IniFileName);
  HighBounds := GetMin(High(Idents), GetMin(High(Values), High(DefValues)));
  for Index := Low(Idents) to HighBounds do
    Values[Index] := FIni.ReadInteger(IniSection, Idents[Index], DefValues[Index]);
  FIni.Free;
end;

procedure TsohoFormStorage.ReadStrings(const IniSection: string;
  const Idents: array of string; var Values: array of string;
  const DefValues: array of string);
var Index, HighBounds : longint;
begin
  FIni := TIniFile.Create(IniFileName);
  HighBounds := GetMin(High(Idents), GetMin(High(Values), High(DefValues)));
  for Index := Low(Idents) to HighBounds do
    Values[Index] := FIni.ReadString(IniSection, Idents[Index], DefValues[Index]);
  FIni.Free;
end;

procedure TsohoFormStorage.WriteBools(const IniSection: string;
  const Idents: array of string; Values: array of boolean);
var Index, HighBounds : longint;
begin
  FIni := TIniFile.Create(IniFileName);
  HighBounds := GetMin(High(Idents), High(Values));
  for Index := Low(Idents) to HighBounds do
    FIni.WriteBool(IniSection, Idents[Index], Values[Index]);
  FIni.Free;
end;

procedure TsohoFormStorage.WriteColors(const IniSection: string;
  const Idents: array of string; Values: array of TColor);
var Index, HighBounds : longint;
begin
  FIni := TIniFile.Create(IniFileName);
  HighBounds := GetMin(High(Idents), High(Values));
  for Index := Low(Idents) to HighBounds do
    FIni.WriteString(IniSection, Idents[Index], ColorToString(Values[Index]));
  FIni.Free;
end;

procedure TsohoFormStorage.WriteFonts(const IniSection: string;
  const Idents: array of string; Values: array of TFont);
var Index, HighBounds : longint;
begin
  FIni := TIniFile.Create(IniFileName);
  HighBounds := GetMin(High(Idents), High(Values));
  for Index := Low(Idents) to HighBounds do
    FIni.WriteString(IniSection, Idents[Index], FontToStr(Values[Index]));
  FIni.Free;
end;

procedure TsohoFormStorage.WriteIntegers(const IniSection: string;
  const Idents: array of string; Values: array of Longint);
var Index, HighBounds : longint;
begin
  FIni := TIniFile.Create(IniFileName);
  HighBounds := GetMin(High(Idents), High(Values));
  for Index := Low(Idents) to HighBounds do
    FIni.WriteInteger(IniSection, Idents[Index], Values[Index]);
  FIni.Free;
end;

procedure TsohoFormStorage.WriteStrings(const IniSection: string;
  const Idents: array of string; Values: array of string);
var Index, HighBounds : longint;
begin
  FIni := TIniFile.Create(IniFileName);
  HighBounds := GetMin(High(Idents), High(Values));
  for Index := Low(Idents) to HighBounds do
    FIni.WriteString(IniSection, Idents[Index], Values[Index]);
  FIni.Free;
end;

end.