{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
      TsohoMDINotifier, 
        
   MDI-.
}
unit SoMDINotifier;

interface

{$I SOHOLIB.INC}

uses Windows, Messages, SysUtils, Classes, Graphics,
     Controls, Forms, SoTools
     {$IFDEF Delphi4_or_5}, SoMenuBar{$ENDIF};

const

  WM_MDINOTIFY = $3F;
  {    .   ,  
     ,      ,
     -   child- }


type

  TsohoMDIChildState = record
{      }
    Child: TForm;
    State: TWindowState;
  end;

  TsohoMDINotifier = class(TComponent)
  {   , 
           fsMDIStyle.
      OnMessage   
       .  ,
    TsohoMDINotifier   
    OnStateChanged.     ,
       MDIChild . 
      :   child
      WindowState  child. 
    MenuBar   TsohoMenuBar 
    MDI- -  , TsohoMDINotifier 
     MenuBar,   
     ,  child-  }
  private
    { Private declarations }
    OwnerWndProc: TFarProc;
    MyWndProc: TFarProc;
    FGrabbed: boolean;
    FOnMessage: TGrabbedMessageEvent;
    FOnStateChanged: TNotifyEvent;
    {$IFDEF Delphi4_or_5}
    FMenuBar: TsohoMenuBar;
    {$ENDIF}
    FForm: TForm;
    FLastChildState: TsohoMDIChildState;
  protected
    { Protected declarations }
    procedure FreeHandlerHook; virtual;
    procedure WNDPROC(var Msg: TMessage); virtual;
    procedure WMDestroy(var Msg: TWMDestroy); message WM_DESTROY;
    procedure WMMDINOTIFY(var Message: TMessage); message WM_MDINOTIFY;
    {$IFDEF Delphi4_or_5}
    procedure Notification (aComponent: TComponent; aOperation: TOperation); override;
    {$ENDIF}
    procedure Loaded; override;
    procedure ChildStateChanged; virtual;
  public
    procedure Hook; virtual;
    procedure DefaultHandler(var Msg); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    {$IFDEF Delphi4_or_5}
    property MenuBar: TsohoMenuBar read FMenuBar write FMenuBar;
    {$ENDIF}
    property OnStateChanged: TNotifyEvent read FOnStateChanged write FOnStateChanged;
    {          }
    property OnMessage: TGrabbedMessageEvent read FOnMessage write FOnMessage;
    {       }
  end;

implementation
uses SoVClUtils, SoCmnCns;

{ TsohoMDINotifier }
procedure TsohoMDINotifier.ChildStateChanged;
begin
  if Assigned(FOnStateChanged) then FOnStateChanged(Self);
  {$IFDEF Delphi4_or_5}
  if not Assigned(FMenuBar) or
     not Assigned(FMenuBar.Parent) or
     (csDestroying in ComponentState) then exit;
  FMenuBar.ShowMDIChildIcon(FLastChildState.State = wsMaximized);
  FMenuBar.ShowMDIButtons(FLastChildState.State = wsMaximized);
  {$ENDIF}
end;

procedure TsohoMDINotifier.WMMDINOTIFY(var Message: TMessage);
var StateChanged: boolean;
begin
  inherited;
  with FLastChildState do begin
    StateChanged := ((Child <> FForm.ActiveMDIChild) or
        (Assigned(FForm.ActiveMDIChild) and
        (State <> FForm.ActiveMDIChild.WindowState)));
    Child := FForm.ActiveMDIChild;
    if Assigned(Child) then
      State := FForm.ActiveMDIChild.WindowState
    else
      State := wsNormal;
  end;
  if StateChanged then ChildStateChanged;
end;

procedure TsohoMDINotifier.Hook;
begin
  FForm := GetOwnerForm(Self);
  if not (csDesigning in ComponentState) and
    (FForm.FormStyle = fsMDIForm) then begin
    OwnerWndProc := TFarProc(GetWindowLong(FForm.ClientHandle, GWL_WNDPROC));
    MyWndProc := MakeObjectInstance(WNDPROC);
    SetWindowLong(FForm.ClientHandle, GWL_WNDPROC, Longint(MyWndProc));
    FGrabbed := True;
  end;
end;

procedure TsohoMDINotifier.Loaded;
begin
  inherited;
  Hook;
end;

constructor TsohoMDINotifier.Create(AOwner: TComponent);
begin
  if not (AOwner is TWinControl) then
    raise ENonWindowOwner.Create(sohoOwnerMustBeWindow);
  inherited Create(AOwner);
  {$IFDEF Delphi4_or_5}
  FMenuBar := nil;
  {$ENDIF}
  FGrabbed := False;
  FLastChildState.Child := nil;
  FLastChildState.State := wsNormal;
end;

procedure TsohoMDINotifier.FreeHandlerHook;
begin
  if not FGrabbed then exit;
  SetWindowLong(FForm.ClientHandle, GWL_WNDPROC, Longint(OwnerWndProc));
  FGrabbed := False;
end;

destructor TsohoMDINotifier.Destroy;
begin
  FreeHandlerHook;
  FreeObjectInstance(MyWndProc);
  inherited Destroy;
end;

procedure TsohoMDINotifier.WMDestroy(var Msg: TWMDestroy);
begin
  FreeHandlerHook;
  inherited;
end;

procedure TsohoMDINotifier.WNDPROC(var Msg: TMessage);
begin
  if Assigned(FOnMessage) then FOnMessage(Self, Msg);
  if Msg.Result = 0 then Dispatch(Msg);
end;

procedure TsohoMDINotifier.DefaultHandler(var Msg);
begin
  with TMessage(Msg) do
    Result := CallWindowProc(OwnerWndProc, FForm.ClientHandle,
    Msg, WPARAM, LPARAM);
end;

{$IFDEF Delphi4_or_5}
procedure TsohoMDINotifier.Notification (aComponent: TComponent; aOperation: TOperation);
begin
  inherited;
  if (aComponent = FMenuBar) and (aOperation = opRemove) then
    FMenuBar := nil;
end;
{$ENDIF}

end.
