{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
    TsohoMRUManager.
}
unit SoMRU;

{$I SOHOLIB.INC}

interface
uses Classes, Menus, SoFrmStorage;

type

  TsohoRecentSelected = procedure (Sender: TObject; ItemIndex: LongInt) of object;
  {    OnSelect  TsohoMRUManager. ItemIndex 
      ,    }

  TsohoMRUManager = class(TComponent)
  {        ,
          
    (More Recently Used).    TsohoMRUManager  ,
           
      ,      ,
    ,  ,  reopen  Delphi.
  }
  private
    FItems: TStrings;
    FMaxCount: LongInt;
    FRecentItem: TMenuItem;
    FOnSelect: TsohoRecentSelected;
    FStorage: TsohoFormStorage;
    FSection: string;
    FAutoSave: boolean;
    FCheckFirst: boolean;
    FHooked: boolean;
    FAddToRecentDocs: boolean;
    FOnSavePlacement: TNotifyEvent;
    FOnRestorePlacement: TNotifyEvent;
  protected
    procedure SetCheckFirst(Value: boolean);
    procedure ItemsChanged(Sender: TObject); virtual;
    procedure RefreshMenuItems; virtual;
    procedure ItemClicked(Sender: TObject);
    procedure SetItems(const Value: TStrings);
    procedure SetMaxCount(const Value: LongInt);
    procedure SetRecentItem(const Value: TMenuItem);
    procedure RemoveSubItems; virtual;
    procedure DoSavePlacement(Sender: TObject);
    procedure DoRestorePlacement(Sender: TObject);
    procedure SetAutoSave(Value: boolean);
    procedure SetStorage(Value: TsohoFormStorage);

    procedure SetEvents;
    procedure RestoreEvents;
  public
    procedure Notification(aComponent: TComponent; aOperation: TOperation); override;
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;

    procedure AddItem(const Value: string);
    {     }
    procedure RemoveItem(const Value: string);
    {        }
    procedure DeleteItem(Index: LongInt);
    {        }
    procedure SaveItems; virtual;
    {     ini-.     
        Storage }
    procedure LoadItems; virtual;
    {     ini-.     
        Storage }
  published
    property Items: TStrings read FItems write SetItems;
    {   }
    property MaxCount: LongInt read FMaxCount write SetMaxCount default 10;
    {  -   .     
       -    ,    
        }
    property AddToRecentDocs: boolean read FAddToRecentDocs write FAddToRecentDocs default false;
    {  ,      MRU-  
       (     -       ) }
    property RecentMenuItem: TMenuItem read FRecentItem write SetRecentItem;
    {    ,       
       Items }
    property CheckFirst: boolean read FCheckFirst write SetCheckFirst default false;
    {  ,    ""    
       RecentMenuItem.    - "" , 
          }
    property Storage: TsohoFormStorage read FStorage write SetStorage;
    {   TsohoFormStorage.     
        ini- }
    property IniSection: string read FSection write FSection;
    {    ini-,      }
    property AutoSave: boolean read FAutoSave write FAutoSave default true;
    {       ini-.
             
         TsohoFormStorage }
    property OnSelect: TsohoRecentSelected read FOnSelect write FOnSelect;
    {  ,       
       RecentMenuItem }
  end;

implementation
uses SoStrUtils, SoAppUtils;

{ TsohoMRUManager }

procedure TsohoMRUManager.AddItem(const Value: string);
var Index: LongInt;
begin
  Index := FItems.IndexOf(Value);
  FItems.BeginUpdate;
  if Index = -1 then FItems.Insert(0, Value)
  else FItems.Move(Index, 0);

  if FItems.Count > FMaxCount then
    while FItems.Count > FMaxCount do FItems.Delete(FItems.Count - 1);

  if FAddToRecentDocs then
    SoAppUtils.AddToRecentDocs(Value);
  FItems.EndUpdate;
end;

constructor TsohoMRUManager.Create(aOwner: TComponent);
begin
  inherited;
  FAddToRecentDocs := false;
  FItems := TStringList.Create;
  TStringList(FItems).OnChange := ItemsChanged;
  FMaxCount := 10;
  FStorage := nil;
  FSection := EmptyStr;
  FAutoSave := true;
  FOnSavePlacement := nil;
  FOnRestorePlacement := nil;
  FHooked := false;
  FCheckFirst := false;
end;

procedure TsohoMRUManager.DeleteItem(Index: Integer);
begin
  FItems.Delete(Index);
end;

destructor TsohoMRUManager.Destroy;
begin
  RestoreEvents;
  RemoveSubItems;
  FRecentItem := nil;
  TStringList(FItems).OnChange := nil;
  FItems.Free;
  inherited;
end;

procedure TsohoMRUManager.DoRestorePlacement(Sender: TObject);
begin
  if Assigned(FOnRestorePlacement) then FOnRestorePlacement(Sender);
  if FAutoSave then LoadItems;
end;

procedure TsohoMRUManager.DoSavePlacement(Sender: TObject);
begin
  if FAutoSave then SaveItems;
  if Assigned(FOnSavePlacement) then FOnSavePlacement(Sender);
end;

procedure TsohoMRUManager.ItemClicked(Sender: TObject);
begin
  if Assigned(FOnSelect) then FOnSelect(Self, (Sender as TMenuItem).Tag);
end;

procedure TsohoMRUManager.ItemsChanged(Sender: TObject);
begin
  RefreshMenuItems;
end;

procedure TsohoMRUManager.Loaded;
begin
  inherited;
  if not (csDesigning in ComponentState) and (FSection = EmptyStr) then
    FSection := Owner.ClassName + '.MRU Manager.' + Name;
  if FAutoSave then SetEvents;
end;

procedure TsohoMRUManager.LoadItems;
begin
  if not Assigned(FStorage) then exit;
  FStorage.IniNeeded;
  FStorage.IniFile.ReadSection(FSection, FItems);
  FStorage.IniFree;
end;

procedure TsohoMRUManager.Notification(aComponent: TComponent;
  aOperation: TOperation);
begin
  inherited;
  if (aComponent = FRecentItem) and (aOperation = opRemove) then begin
    RemoveSubItems;
    FRecentItem := nil;
  end;
  if (aComponent = FStorage) and (aOperation = opRemove) then begin
    RestoreEvents;
    FStorage := nil;
  end;  
end;

procedure TsohoMRUManager.RefreshMenuItems;
var Index: LongInt;
    Item: TMenuItem;
    Caption: string;
    Delimiter: string;
begin
  if not Assigned(FRecentItem) then exit;
  RemoveSubItems;
  Delimiter := Space + Space;
  for Index := 0 to pred(FItems.Count) do begin
    if Index < 10 then Caption := '&' + Char(Ord('0') + Index) + Delimiter
    else
      if Index <= (Ord('Z') + 10) then Caption := '&' +
        Char(Index + Ord('A') - 10) + Delimiter
      else Caption := Space + Delimiter;
    Caption := Caption + FItems[Index];
    Item := NewItem(Caption, 0, Index = 0, true, ItemClicked,
       0, '');
    Item.Tag := Index;
    FRecentItem.Add(Item);
  end;
  if FRecentItem.Count > 0 then
    FRecentItem.Items[0].Checked := FCheckFirst;
  FRecentItem.Enabled := FRecentItem.Count > 0;
end;

procedure TsohoMRUManager.RemoveItem(const Value: string);
var Index: LongInt;
begin
  Index := FItems.IndexOf(Value);
  if Index <> -1 then DeleteItem(Index);
end;

procedure TsohoMRUManager.RemoveSubItems;
var Index: LongInt;
begin
  if not Assigned(FRecentItem) then exit;
  for Index := pred(FRecentItem.Count) downto 0 do
    FRecentItem.Delete(Index);
end;

procedure TsohoMRUManager.SetEvents;
begin
  if csDesigning in ComponentState then exit;
  if Assigned(FStorage) then begin
    FOnSavePlacement := FStorage.OnSavePlacement;
    FOnRestorePlacement := FStorage.OnRestorePlacement;
    FStorage.OnSavePlacement := DoSavePlacement;
    FStorage.OnRestorePlacement := DoRestorePlacement;
    FHooked := true;
  end;
end;

procedure TsohoMRUManager.RestoreEvents;
begin
  if csDesigning in ComponentState then exit;
  if FHooked and Assigned(FStorage) then begin
    FStorage.OnSavePlacement := FOnSavePlacement;
    FStorage.OnRestorePlacement := FOnRestorePlacement;
  end;
  FOnSavePlacement := nil;
  FOnRestorePlacement := nil;
  FHooked := false;
end;

procedure TsohoMRUManager.SaveItems;
var Index: LongInt;
begin
  if not Assigned(FStorage) then exit;
  FStorage.IniNeeded;
  FStorage.IniFile.EraseSection(FSection);
  for Index := 0 to pred(FItems.Count) do
    FStorage.IniFile.WriteString(FSection, FItems[Index], 'Stored');
  FStorage.IniFree;
end;

procedure TsohoMRUManager.SetAutoSave(Value: boolean);
begin
  FAutoSave := Value;
  if csReading in ComponentState then exit;
  if FHooked then RestoreEvents;
  if FAutoSave and Assigned(FStorage) then SetEvents;
end;

procedure TsohoMRUManager.SetCheckFirst(Value: boolean);
begin
  if FCheckFirst = Value then exit;
  FCheckFirst := Value;
  if Assigned(FRecentItem) and (FRecentItem.Count > 0) then
    FRecentItem.Items[0].Checked := FCheckFirst;
end;

procedure TsohoMRUManager.SetItems(const Value: TStrings);
begin
  FItems.Assign(Value);
end;

procedure TsohoMRUManager.SetMaxCount(const Value: LongInt);
var Index: LongInt;
begin
  if (FMaxCount = Value) or (Value < 1) then exit;
  FMaxCount := Value;
  //   
  if Assigned(FRecentItem) then
    for Index := pred(FRecentItem.Count) downto FMaxCount do
      FRecentItem.Delete(Index);
  if FItems.Count > FMaxCount then begin
    TStringList(FItems).OnChange := nil;
    for Index := pred(FItems.Count) downto FMaxCount do
       FItems.Delete(Index);
    TStringList(FItems).OnChange := ItemsChanged;
  end;
end;

procedure TsohoMRUManager.SetRecentItem(const Value: TMenuItem);
begin
  if FRecentItem = Value then exit;
  if Assigned(FRecentItem) then RemoveSubItems;
  FRecentItem := Value;
  RefreshMenuItems;
end;

procedure TsohoMRUManager.SetStorage(Value: TsohoFormStorage);
begin
  if Assigned(FStorage) then RestoreEvents;
  FStorage := Value;
  if not (csLoading in ComponentState) and FAutoSave then SetEvents;
end;

end.
