{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                  (on ElTree basis)                    }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoMemTree;

interface
uses Classes, SoClass;

type

  TsohoMTreeItem = ^RsohoMTreeItem;

  TNodes = class(TsohoList)
  private
    function GetNode(Index: Integer): TsohoMTreeItem;
  public
    property Nodes[Index: Integer]: TsohoMTreeItem read GetNode;
  end;

  RsohoMTreeItem = record
    Parent: TsohoMTreeItem;
    Data: pointer;
    List: TNodes;
  end;

  TIterProc = procedure(Item: TsohoMTreeItem; Index: integer; var ContinueIterate: boolean;
    IterateData: pointer);

  TsohoMTStreamEvent = procedure (Sender: TObject; Item: TsohoMTreeItem;
    Stream: TStream) of object;

  TsohoMemoryTree = class
  private
    FRoot: TsohoMTreeItem;
    FCount: integer;
    FOnItemDelete: TOnPntItemNotify;
    FOnStreamSave,
    FOnStreamLoad: TsohoMTStreamEvent;
    function GetItem(Index: LongInt): TsohoMTreeItem;
  protected
    procedure ItemDeleteEvent(Item: TsohoMTreeItem); virtual;
    procedure DoItemStreamSave(Item: TsohoMTreeItem; Stream: TStream); virtual;
    procedure DoItemStreamLoad(Item: TsohoMTreeItem; Stream: TStream); virtual;
  public
    constructor Create;
    destructor Destroy; override;
    function AddItem(Parent: TsohoMTreeItem; Value: pointer): TsohoMTreeItem;
    function InsertItem(Parent: TsohoMTreeItem; Index: integer; Value: pointer): TsohoMTreeItem;
    procedure DeleteItem(Item: TsohoMTreeItem);
    procedure MoveTo(Item, NewParent: TsohoMTreeItem);
    procedure Clear;
    function GetIndex(Item: TsohoMTreeItem): Integer;
    function GetAbsIndex(Item: TsohoMTreeItem): Integer;
    procedure Iterate(IterateProc: TIterProc; IterateData: pointer);
    procedure SaveToStream(Stream: TStream); virtual;
    procedure LoadFromStream(Stream: TStream); virtual;
    procedure SaveSubTreeToStream(Item: TsohoMTreeItem; Stream: TStream); virtual;
         // Item will be saved too
    procedure LoadSubTreeFromStream(Item: TsohoMTreeItem; Stream: TStream); virtual;
         // Item here is the "parent" item for all items loaded
    property Count: Integer read FCount;
    property Item[index: integer]: TsohoMTreeItem read GetItem;
    property Root: TsohoMTreeItem read FRoot;
    property OnItemDelete: TOnPntItemNotify read FOnItemDelete write FOnItemDelete;
    property OnStreamSave: TsohoMTStreamEvent read FOnStreamSave write FOnStreamSave;
    property OnStreamLoad: TsohoMTStreamEvent read FOnStreamLoad write FOnStreamLoad;
  end;

implementation

procedure TsohoMemoryTree.Iterate(IterateProc: TIterProc; IterateData: pointer);
var j: integer;
    DoContinue: boolean;

  procedure IntIterate(Item: TsohoMTreeItem);
  var
    Index: integer;
  begin
    inc(j);
    if j >= 0 then IterateProc(Item, j, DoContinue, IterateData);
    if not (DoContinue) then exit;
    for Index := 0 to pred(Item^.List.Count) do begin
      IntIterate(TsohoMTreeItem(Item.List[Index]));
      if not (DoContinue) then exit;
    end;
  end;

begin
  j := -2;
  DoContinue := true;
  IntIterate(FRoot);
end;

type

  PGetItemRec = ^TGetItemRec;
  TGetItemRec = record
    j: LongInt;
    TSI: TsohoMTreeItem;
  end;

function TsohoMemoryTree.GetItem(Index: LongInt): TsohoMTreeItem;
var GetItemRec: TGetItemRec;

  procedure IntGetItem(Item: TsohoMTreeItem; Index: integer; var ContinueIterate: boolean;
    IterateData: pointer);
  begin
    if Index = PGetItemRec(IterateData)^.j then begin
      PGetItemRec(IterateData)^.TSI := Item;
      ContinueIterate := false;
    end;
  end;

begin
  if (index < 0) or (index >= FCount) then begin
    Result := nil;
    exit;
  end;
  GetItemRec.TSI := nil;
  GetItemRec.j := Index;
  Iterate(@IntGetItem, @GetItemRec);
  Result := GetItemRec.TSI;
end;

function TsohoMemoryTree.AddItem(Parent: TsohoMTreeItem; Value: pointer): TsohoMTreeItem; { public }
var TI: TsohoMTreeItem;
begin
  New(TI);
  TI^.List := TNodes.Create;
  TI^.Data := Value;
  if Parent = nil then FRoot^.List.Add(TI)
  else Parent^.List.Add(TI);
  if Parent = nil then TI^.Parent := FRoot
  else TI^.Parent := Parent;
  Result := ti;
  inc(FCount);
end;

function TsohoMemoryTree.InsertItem(Parent: TsohoMTreeItem; Index: integer; Value: pointer): TsohoMTreeItem; { public }
var TI: TsohoMTreeItem;
begin
  New(TI);
  TI^.List := TNodes.Create;
  TI^.Data := Value;
  if Parent = nil then begin
    FRoot^.List.Insert(index, TI);
    TI^.Parent := FRoot;
  end
  else begin
    Parent^.List.Insert(Index, TI);
    TI^.Parent := Parent;
  end;
  Result := ti;
  inc(FCount);
end;

procedure TsohoMemoryTree.DeleteItem(Item: TsohoMTreeItem);
begin
  if (Item = nil) then exit;
  while Item^.List.Count > 0 do DeleteItem(Item^.List[0]);
  Item^.List.Free;
  Item^.Parent^.List.Remove(Item);
  ItemDeleteEvent(Item);
  Dispose(Item);
  dec(FCount);
end;

procedure TsohoMemoryTree.Clear;
begin
  while FRoot^.List.Count > 0 do DeleteItem(FRoot^.List[0]);
end;

procedure TsohoMemoryTree.MoveTo(Item, NewParent: TsohoMTreeItem);
begin
  if Item = nil then exit;
  Item^.Parent.List.Remove(Item);
  if NewParent = nil then NewParent := FRoot;
  NewParent^.List.Add(Item);
  Item^.Parent := NewParent;
end;

function TsohoMemoryTree.GetIndex(Item: TsohoMTreeItem): Integer;
begin
  Result := Item^.Parent^.List.IndexOf(Item);
end; { GetIndex }

function TsohoMemoryTree.GetAbsIndex(Item: TsohoMTreeItem): Integer;
var GetItemRec: TGetItemRec;

  procedure IntGetIndex(Item: TsohoMTreeItem; Index: integer; var ContinueIterate: boolean;
    IterateData: pointer);
  begin
    if PGetItemRec(IterateData)^.TSI = Item then begin
      PGetItemRec(IterateData)^.j := index;
      ContinueIterate := false;
    end;
  end;

begin
  if Item = nil then begin
    Result := -1;
    exit;
  end;
  GetItemRec.j := -1;
  GetItemRec.TSI := Item;
  Iterate(@IntGetIndex, @GetItemRec);
  Result := GetItemRec.j;
end;

procedure TsohoMemoryTree.SaveToStream(Stream: TStream);
begin
  SaveSubTreeToStream(FRoot, Stream);
end;

procedure TsohoMemoryTree.LoadFromStream(Stream: TStream);
begin
  LoadSubTreeFromStream(FRoot, Stream);
end;

procedure TsohoMemoryTree.SaveSubTreeToStream(Item: TsohoMTreeItem; Stream: TStream);

  procedure IntSave(Item: TsohoMTreeItem; Stream: TStream; Tree: TsohoMemoryTree);
  var i, j: integer;
  begin
    i := Item^.List.Count;
    Stream.WriteBuffer(i, sizeof(integer));
    if (Item <> FRoot) then
      DoItemStreamSave(Item, Stream);
    for j := 0 to i - 1 do IntSave(TsohoMTreeItem(Item^.List[j]), Stream, Tree);
  end;

begin
  if Item = nil then Item := FRoot;
  IntSave(Item, Stream, self);
end;

procedure TsohoMemoryTree.LoadSubTreeFromStream(Item: TsohoMTreeItem; Stream: TStream);

  procedure IntLoad(Item: TsohoMTreeItem; Stream: TStream; Tree: TsohoMemoryTree);
  var i, j: integer;
    NewItem: TsohoMTreeItem;
  begin
    Stream.ReadBuffer(i, sizeof(integer));
    if Item <> FRoot then
      DoItemStreamLoad(Item, Stream);
    for j := 0 to i - 1 do begin
      NewItem := Tree.AddItem(Item, nil);
      IntLoad(NewItem, Stream, Tree);
    end;
  end;

begin
  if Item = nil then Item := FRoot;
  IntLoad(Item, Stream, self);
end;

procedure TsohoMemoryTree.ItemDeleteEvent(Item: TsohoMTreeItem);
begin
  if Assigned(FOnItemDelete) then FOnItemDelete(Self, Item);
end;

constructor TsohoMemoryTree.Create;
begin
  inherited;
  New(FRoot);
  FRoot.Parent := nil;
  FRoot.Data := nil;
  FRoot^.List := TNodes.Create;
end;

destructor TsohoMemoryTree.Destroy;
begin
  FRoot^.List.Free;
  Dispose(FRoot);
  inherited;
end;

procedure TsohoMemoryTree.DoItemStreamLoad(Item: TsohoMTreeItem;
  Stream: TStream);
begin
  if Assigned(FOnStreamLoad) then FOnStreamLoad(Self, Item, Stream);
end;

procedure TsohoMemoryTree.DoItemStreamSave(Item: TsohoMTreeItem;
  Stream: TStream);
begin
  if Assigned(FOnStreamSave) then FOnStreamSave(Self, Item, Stream);
end;

{ TNodes }

function TNodes.GetNode(Index: Integer): TsohoMTreeItem;
begin
  Result := TsohoMTreeItem(Items[Index]);
end;

end.
