{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoMenuBar;

interface

{$I SOHOLIB.INC}

uses Windows, Messages, SysUtils, Classes, Graphics,
     Controls, Forms, ToolWin, ComCtrls, Menus, ExtCtrls,
     Buttons;

type

  TsohoMenuBar = class;

  TsohoIconControl = class(TCustomControl)
  private
    FImage: TImage;
    FMenuBar: TsohoMenuBar;
    FSysMenu: TPopupMenu;
  protected
    procedure SysPopup(Sender: TObject);
    procedure ItemClicked (Sender: TObject);
    procedure DoPopup; virtual;
    {procedure ImageMouseMove(Sender: TObject; Shift: TShiftState;
              X, Y: Integer);}
    procedure ImageMouseDown (Sender: TObject; Button: TMouseButton;
              Shift: TShiftState; X, Y: Integer);
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetIcon (Value: TIcon);
  end;

  TsohoMDIButtonControl = class(TCustomControl)
  private
    FMin,
    FRest,
    FClose: TSpeedButton;
    FMenuBar: TsohoMenuBar;
  protected
    procedure ButtonClicked (Sender: TObject);
    procedure RecalcSize;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  end;

  TsohoMenuBar = class(TToolBar)
  private
    FMenu: TMainMenu;
    FIsMain: Boolean;
    FIcon: TsohoIconControl;
    FBtns: TsohoMDIButtonControl;
    FOnFormResize: TNotifyEvent;
    procedure SetMenu(const Value: TMainMenu);
  protected
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    procedure Notification(aComponent: TComponent; aOperation: TOperation); override;
    procedure DoFormResize(Sender: TObject);
    function GetMinWidth: LongInt;

    function HandleAppKeyDown(var Message: TWMKey): boolean;
    procedure CMDialogKey(var Message: TWMKey); message CM_DIALOGKEY;
  public
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Loaded; override;
    procedure CreateWnd; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    {      MDIChilds }
    procedure ShowMDIChildIcon(aShow: boolean);
    procedure ShowMDIButtons(aShow: boolean);
  published
    property AutoSize default false;
    property EdgeBorders default [];
    property Menu: TMainMenu read FMenu write SetMenu;
  end;

implementation
uses SoVCLUtils, SoGraph, CommCtrl;

{$R SOMENUBAR.R32}

{ TsohoIconControl }
procedure TsohoIconControl.DoPopup;
var Pnt: TPoint;
begin
  Pnt := Point(Left, Top + Height);
  Pnt := ClientToScreen(Pnt);
  FSysMenu.Popup(Pnt.X, Pnt.Y);
end;

{
procedure TsohoIconControl.ImageMouseMove(Sender: TObject; Shift: TShiftState;
              X, Y: Integer);
var Index, PressedButton: LongInt;
begin
  if not Assigned(FMenuBar) then exit;
  //  ,       MenuBar
  PressedButton := -1;
  for Index := 0 to pred(FMenuBar.ButtonCount) do
    if FMenuBar.Buttons[Index].Down then begin
      PressedButton := Index;
      Break;
    end;
  if PressedButton = -1 then exit;
  FMenuBar.Buttons[PressedButton].Down := false;
  DoPopup;
end;
}

procedure TsohoIconControl.ImageMouseDown (Sender: TObject; Button: TMouseButton;
          Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then DoPopup;
end;

procedure TsohoIconControl.SetIcon (Value: TIcon);
var Bmp: TBitmap;
begin
  if Value.Handle = 0 then
   Value.Handle := LoadIcon(0, IDI_APPLICATION);

  Bmp := CreateBitmapFromIcon(Value, clBtnFace);
  FImage.Picture.Bitmap.Assign(Bmp);
  if Assigned(FMenuBar) then
    FImage.Top := trunc((FMenuBar.Height - FImage.Height)/2);
  Bmp.Free;
end;

procedure TsohoIconControl.ItemClicked (Sender: TObject);
begin
  SendMessage(GetOwnerForm(Self).ActiveMDIChild.Handle,
    WM_SYSCOMMAND, TMenuItem(Sender).Tag, 0);
end;

procedure TsohoIconControl.SysPopup(Sender: TObject);
var FSysHandle: HMenu;
    Item: TMenuItem;
    Index: LongInt;
    fCaption: array [0..255] of char;
    SCaption : string;
    fEnabled: boolean;
    Tmp: LongInt;
begin
  {$IFDEF Delphi4}
  for Index := 0 to pred(FSysMenu.Items.Count) do FSysMenu.Items.Delete(0);
  {$ELSE}
  FSysMenu.Items.Clear;
  {$ENDIF}
  FSysHandle := GetSystemMenu(GetOwnerForm(Self).ActiveMDIChild.Handle, false);
  if FSysHandle = 0 then exit;
  for Index := 0 to pred(GetMenuItemCount(FSysHandle)) do begin
    GetMenuString(FSysHandle, Index, fCaption, 254, MF_BYPOSITION);
    SCaption := StrPas(fCaption);
    if SCaption = '' then sCaption := '-';
    Tmp := GetMenuState(FSysHandle, Index, MF_BYPOSITION);
    fEnabled := ((Tmp and MF_GRAYED) = 0) and ((Tmp and MF_DISABLED) = 0);
    Item := NewItem(sCaption, 0, false, fEnabled, ItemClicked, 0, '');
    FSysMenu.Items.Add(Item);
    Item.Tag := GetMenuItemID(FSysHandle, Index);
    Item.Default := Tmp = MF_DEFAULT;
  end;
end;

constructor TsohoIconControl.Create(aOwner: TComponent);
begin
  inherited;
  FImage := TImage.Create(Self);
  with FImage do begin
    Parent := Self;
    Stretch := true;
    Left := 0;
    Height := 16;
    Width := 16;
    Center := true;
    FImage.OnMouseDown := ImageMouseDown;
    // FImage.OnMouseMove := ImageMouseMove;
  end;
  FSysMenu := TPopupMenu.Create(Self);
  FSysMenu.OnPopup := SysPopup;
  FSysMenu.AutoPopup := false;
  // FImage.PopupMenu := FSysMenu;
end;

destructor TsohoIconControl.Destroy;
begin
  FSysMenu.Free;
  FImage.Free;
  inherited;
end;

{ TsohoMDIButtonControl }
procedure TsohoMDIButtonControl.ButtonClicked (Sender: TObject);
var Child: TForm;
begin
  Child := GetOwnerForm(Self).ActiveMDIChild;
  if not Assigned(Child) then exit;
  case TSpeedButton(Sender).Tag of
    0: ShowWindow(Child.Handle, SW_MINIMIZE);
    1: ShowWindow(Child.Handle, SW_RESTORE);
    2: Child.Close;
  end;
end;

procedure TsohoMDIButtonControl.RecalcSize;
var Index, fLeft: LongInt;
    Form: TForm;
begin
  if not Assigned(FMenuBar) or
     (FMenuBar.ButtonCount = 0) then exit;
  fLeft := 0;
  for Index := 0 to pred(FMenuBar.ButtonCount) do
    fLeft := FLeft + FMenuBar.Buttons[Index].Width;
  if Assigned(FMenuBar.FIcon.Parent) then
    fLeft := FLeft + FMenuBar.FIcon.Width;
  inc(fLeft);
  // Color := clBlue;
  Height := FMenuBar.Height;
  //   + 5   ,    :)
  Width := FMenuBar.Width - fLeft + 15;
  Left  := fLeft;
  // Align := alRight;
  FClose.Left := Width - 1 - FClose.Width;
  FClose.Top := trunc((Height - FClose.Height)/2);
  FRest.Left := FClose.Left - 2 - FRest.Width;
  FRest.Top := FClose.Top;
  FMin.Left := FRest.Left - 1 - FMin.Width;
  FMin.Top := FClose.Top;

  //   ,      
  Form := GetOwnerForm(Self).ActiveMDIChild;
  if not Assigned(Form) then exit;
  FClose.Enabled := biSystemMenu in Form.BorderIcons;
  FRest.Enabled := FClose.Enabled;
  FMin.Enabled := biMinimize in Form.BorderIcons;
end;

constructor TsohoMDIButtonControl.Create(aOwner: TComponent);
begin
  inherited;
  FMin := TSpeedButton.Create(Self);
  with FMin do begin
    Width := 17;
    Height := 15;
    Glyph.Handle := ResBitmap('MDIMINIMIZEBTN');
    Parent := Self;
    Tag := 0;
    OnClick := ButtonClicked;
  end;
  FRest := TSpeedButton.Create(Self);
  with FRest do begin
    Width := 17;
    Height := 15;
    Glyph.Handle := ResBitmap('MDIRESTOREBTN');
    Parent := Self;
    Tag := 1;
    OnClick := ButtonClicked;
  end;
  FClose := TSpeedButton.Create(Self);
  with FClose do begin
    Width := 17;
    Height := 15;
    Glyph.Handle := ResBitmap('MDICLOSEBTN');
    Parent := Self;
    Tag := 2;
    OnClick := ButtonClicked;
  end;
end;

destructor TsohoMDIButtonControl.Destroy;
begin
  FMin.Free;
  FRest.Free;
  FClose.Free;
  inherited;
end;

{ TsohoMenuBar }
procedure TsohoMenuBar.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

procedure TsohoMenuBar.DoFormResize(Sender: TObject);
begin
  if Assigned(FOnFormResize) then FOnFormResize(Sender);
  Width := Width + 1;
end;

procedure TsohoMenuBar.CreateParams(var Params: TCreateParams);
begin
  inherited;
  {with Params do
    Style := Style or CCS_ADJUSTABLE;}
end;

procedure TsohoMenuBar.ShowMDIButtons(aShow: boolean);
var Form: TForm;
begin
  if not aShow then begin
    FBtns.Parent := nil;
    exit;
  end;
  //    -  ?
  Form := GetOwnerForm(Self);
  if not Assigned(Form) then exit;
  if not Assigned(Form.ActiveMDIChild) or
     (Form.ActiveMDIChild.WindowState <> wsMaximized) then begin
    FBtns.Parent := nil;
    exit;
  end;
  FBtns.Parent := nil;
  FBtns.RecalcSize;
  FBtns.Parent := Self;
end;

procedure TsohoMenuBar.ShowMDIChildIcon(aShow: boolean);
var Form: TForm;
//    IconHandle: HIcon;
begin
  if not aShow then begin
    FIcon.Parent := nil;
    exit;
  end;

  //    -  ?
  Form := GetOwnerForm(Self);
  if not Assigned(Form) then exit;
  if not Assigned(Form.ActiveMDIChild) or
     (Form.ActiveMDIChild.WindowState <> wsMaximized) then begin
    FIcon.Parent := nil;
    exit;
  end;
  FIcon.Parent := Self;
  FIcon.Left := 0;
  FIcon.Width := GetSystemMetrics(SM_CXSMICON);
  FIcon.Height := GetSystemMetrics(SM_CYSMICON);
  FIcon.SetIcon(Form.ActiveMDIChild.Icon);
end;

procedure TsohoMenuBar.Loaded;
var Form: TForm;
begin
  inherited;
  if csDesigning in ComponentState then exit;
  Form := GetOwnerForm(Self);
  if Assigned(Form) then begin
    FOnFormResize := Form.OnResize;
    Form.OnResize := DoFormResize;
  end;
end;

procedure TsohoMenuBar.CreateWnd;
begin
  inherited;
  if Assigned(Parent) then begin
    SetBounds(Left, Top, Width, Height);
    FIcon := TsohoIconControl.Create(Self);
    FIcon.FMenuBar := Self;
    FBtns := TsohoMDIButtonControl.Create(Self);
    FBtns.FMenuBar := Self;
  end;
end;

function TsohoMenuBar.GetMinWidth: LongInt;
var Index: LongInt;
begin
  Result := 0;
  for Index := 0 to pred(ButtonCount) do
    Result := Result + Buttons[Index].Width;
  Result := Result + FIcon.Width + FBtns.Width;
  inc(Result);
end;

procedure TsohoMenuBar.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  if Assigned(Parent) then begin
    if not Floating then aWidth := Parent.ClientWidth - 14
    else aWidth := GetMinWidth;
  end;
  inherited;
  if Assigned(FBtns) and
     Assigned(FBtns.Parent) then begin
     FBtns.Parent := nil;
     FBtns.RecalcSize;
     FBtns.Parent := self;
  end;
end;

destructor TsohoMenuBar.Destroy;
var Form: TForm;
begin
  Form := GetOwnerForm(Self);
  if Assigned(Form) then Form.OnResize := FOnFormResize;
  inherited;
end;

constructor TsohoMenuBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIsMain := false;
  Flat := true;
  ShowCaptions := true;
  EdgeBorders := [];
  {ControlStyle := [csCaptureMouse, csClickEvents, csDoubleClicks,
    csMenuEvents, csSetCaption];}
  Wrapable := false;
  Align := alNone;
  AutoSize := false;
  // List := false;
  EdgeInner := esNone;
  EdgeOuter := esNone;
  FIcon := nil;
end;

{procedure TsohoMenuBar.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;}

procedure TsohoMenuBar.Notification(aComponent: TComponent;
  aOperation: TOperation);
begin
  if (aComponent = FMenu) and (aOperation = opRemove) then SetMenu(nil);
  inherited;
end;

procedure TsohoMenuBar.SetMenu(const Value: TMainMenu);
var Index: Integer;
    Button: TToolButton;
    Form: TForm;
begin
  if FMenu = Value then exit;

  Form := GetOwnerForm(Self);
  if not Assigned(Form) then exit;

  if Assigned(FMenu) then begin
    if FIsMain then Form.Menu := FMenu;
    FIsMain := false;
    for Index := pred(ButtonCount) downto 0 do Buttons[Index].Free;
  end;

  FMenu := Value;
  if not Assigned(FMenu) then exit;

  if Form.Menu = FMenu then begin
    FIsMain := true;
    Form.Menu := nil;
  end;

  for Index := 0 to pred(FMenu.Items.Count) do begin
    Button := TToolButton.Create(Self);
    try
      Button.Grouped := true;
      Button.Parent := Self;
      Button.AutoSize := True;
    except
      Button.Free;
      raise;
    end;
  end;
  { Copy attributes from each menu item }
  for Index := 0 to pred(FMenu.Items.Count) do
    Buttons[Index].MenuItem := FMenu.Items[Index];
end;

procedure TsohoMenuBar.CMDialogKey(var Message: TWMKey);
begin
  if not HandleAppKeyDown(Message) then inherited;
end;

function TsohoMenuBar.HandleAppKeyDown(var Message: TWMKey): boolean;
begin
  Result := false;
  if Assigned(FMenu) and FMenu.IsShortCut(Message) then begin
    Result := true;
    Message.Result := 1;
  end;
end;

end.
