{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoMultimedia;

interface
uses Classes, Windows, MMSystem;

type

  TWaveFileName = type string;
  {       WAV- }

  TsohoSound = class(TComponent)
  {     WAV-  
    Windows.       Win16 API,  
          WAV  .  
       : PlayResourceWave.  , 
     SohoLib   TsohoSound,  ""  
     }
  private
    { Private declarations }
    FEnabled: boolean;
    FAsync: boolean;
    FOnSound: TNotifyEvent;
  public
    function PlaySync(Filename: string): boolean;
    {  ,   "",       }
    function PlayAsync(Filename: string): boolean;
    {  ,      ,       }
    function PlayWave(Filename: string): boolean;
    {   }
    function PlayResourceWave(WaveName: string): boolean;
    {  wav   }
    function PlayResourceWaveInstance(hInstance: HINST; WaveName: string): boolean;
    {  wav     hInstance }
    function PlayWaveFromMemory (Buffer: PChar): boolean;
    {  wav   }

    constructor Create(AOwner: TComponent); override;
  published
    property Enabled: boolean read FEnabled write FEnabled default True;
    { /  }
    property Async: boolean read FAsync write FAsync default False;
    {    (false)   (true) }
    property OnSound: TNotifyEvent read FOnSound write FOnSound;
    {       }
  end;

function SetSoundVolume(Value: word): boolean;
{   }
function SetSoundChannelsVolume(Left, Right: word): boolean;
{         }
function GetSoundVolume: word;
{   }
function GetSoundChannelsVolume(var Left, Right: word): boolean;
{         }
function FindSound(Form: TComponent): TsohoSound;
{      }

var

  {      ,    
    wav-,    TsohoSound.PlayWave    
     .     .  ,
       - "Sounds\".   ,  
        exe- + DefSoundsDir.  
    DefSoundsDir     "\"
  }
  DefSoundsDir: string;

implementation
uses SysUtils, SoAppUtils, SoVCLUtils, SoCmnCns;

function SetSoundVolume(Value: word): boolean;
var Volume: DWord;
begin
  Volume := Value + Value shl 16;
  Result := WaveOutSetVolume(0, Volume) = MMSYSERR_NOERROR;
end;

function SetSoundChannelsVolume(Left, Right: word): boolean;
var Volume: DWord;
begin
  Volume := Left + Right shl 16;
  Result := WaveOutSetVolume(0, Volume) = MMSYSERR_NOERROR;
end;

function GetSoundVolume: word;
var Volume: DWord;
    Left, Right: LongInt;
begin
  if WaveOutGetVolume(0, @Volume) = MMSYSERR_NOERROR then begin
    Left := LoWord(Volume);
    Right := HiWord(Volume);
    Result := (Left shr 1) + (Right shr 1);
  end
  else Result := 0;
end;

function GetSoundChannelsVolume(var Left, Right: word): boolean;
var Volume: DWord;
begin
  Left := 0;
  Right := 0;
  Result := WaveOutGetVolume(0, @Volume) = MMSYSERR_NOERROR;
  if Result then begin
    Left := LoWord(Volume);
    Right := HiWord(Volume);
  end;
end;

function FindSound(Form: TComponent): TsohoSound;
var index: Integer;
begin
  Result := nil;
  if Form = nil then exit;
  for index := 0 to Form.ComponentCount - 1 do
    if Form.Components[index] is TsohoSound then begin
      Result := Form.Components[index] as TsohoSound;
      exit;
    end;
end;

{ TsohoSound }
function TsohoSound.PlaySync(Filename: string): boolean;
begin
  Result := False;
  if not FEnabled then exit;
  Result := sndPlaySound(PChar(Filename), 0);
  if Assigned(FOnSound) then FOnSound(Self);
end;

function TsohoSound.PlayAsync(Filename: string): boolean;
begin
  Result := False;
  if not FEnabled then exit;
  Result := sndPlaySound(PChar(Filename), snd_Async);
  if Assigned(FOnSound) then FOnSound(Self);
end;

function TsohoSound.PlayWaveFromMemory (Buffer: PChar): boolean;
const Flags: array[boolean] of Word = (0, snd_Async);
begin
  Result := sndPlaySound(Buffer, snd_Memory or Flags[FAsync]);
end;

function TsohoSound.PlayWave(Filename: string): boolean;
const Flags: array[boolean] of Word = (0, snd_Async);
begin
  Result := False;
  if not FEnabled then exit;
  if ExtractFilePath(FileName) = '' then begin
    if Pos(':', DefSoundsDir) <> 0 then //    
      FileName := DefSoundsDir + FileName
    else
      FileName := GetRunDir + DefSoundsDir+FileName;
  end;
  if ExtractFileExt(FileName) = '' then FileName := FileName + '.wav';
  if not FileExists(FileName) then exit;
  PlayWave := sndPlaySound(PChar(Filename), Flags[FAsync]);
  if Assigned(FOnSound) then FOnSound(Self);
end;

function TsohoSound.PlayResourceWaveInstance(hInstance: HINST; WaveName: string): boolean;
const Flags: array[boolean] of Word = (0, snd_Async);
var WaveHandle: THandle;
    WavePointer: Pointer;
begin
  Result := False;
  if not FEnabled then exit;
  try
    WaveHandle := FindResource(hInstance, PChar(WaveName), RT_RCDATA);
    if WaveHandle <> 0 then begin
      WaveHandle := LoadResource(hInstance, WaveHandle);
      if WaveHandle <> 0 then begin;
        WavePointer := LockResource(WaveHandle);
        Result := sndPlaySound(WavePointer, snd_Memory or Flags[FAsync]);
        UnlockResource(WaveHandle);
        FreeResource(WaveHandle);
        if Assigned(FOnSound) then FOnSound(Self);
      end;
    end;
  except ErrorMsg(name + ': '+ soToolsResourceWavError);
  end;
end;

function TsohoSound.PlayResourceWave(WaveName: string): boolean;
begin
  Result := PlayResourceWaveInstance(hInstance, WaveName);
end;

constructor TsohoSound.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEnabled := True;
end;

initialization
  DefSoundsDir := 'Sounds\';
end.
