{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoPrcUtils;

interface
uses Classes, Windows, TLHelp32, SysUtils;

type

  PThreadInfo = ^TThreadInfo;
  TThreadInfo = record
    ftCreationTime : TFileTime;
    dwUnknown1 : dword;
    dwStartAddress : dword;
    dwOwningPID : dword;
    dwThreadID : dword;
    dwCurrentPriority : dword;
    dwBasePriority : dword;
    dwContextSwitches : dword;
    dwThreadState : dword;
    dwUnknown2 : dword;
    dwUnknown3 : dword;
    dwUnknown4 : dword;
    dwUnknown5 : dword;
    dwUnknown6 : dword;
    dwUnknown : dword;
  end;

  PProcessInfo = ^TProcessInfo;
  TProcessInfo = record
    dwOffset            : dword; // an ofset to the next Process structure
    dwThreadCount       : dword;
    dwUnkown1           : array[0..5] of dword;
    ftCreationTime      : TFileTime;
    dwUnkown2           : dword;
    dwUnkown3           : dword;
    dwUnkown4           : dword;
    dwUnkown5           : dword;
    dwUnkown6           : dword;
    pszProcessName      : PWideChar;
    dwBasePriority      : dword;
    dwProcessID         : dword;
    dwParentProcessID   : dword;
    dwHandleCount       : dword;
    dwUnkown7           : dword;
    dwUnkown8           : dword;
    dwVirtualBytesPeak  : dword;
    dwVirtualBytes      : dword;
    dwPageFaults        : dword;
    dwWorkingSetPeak    : dword;
    dwWorkingSet        : dword;
    dwUnkown9           : dword;
    dwPagedPool         : dword; // kbytes
    dwUnkown10          : dword;
    dwNonPagedPool      : dword; // kbytes
    dwPageFileBytesPeak : dword;
    dwPageFileBytes     : dword;
    dwPrivateBytes      : dword;
    dwUnkown11          : dword;
    dwUnkown12          : dword;
    dwUnkown13          : dword;
    dwUnkown14          : dword;
    ThreadInfo          : PThreadInfo; // Thread list
  end;

  TProcess = class
  public
    ThreadCount         : dword;
    CreationTime        : TFileTime;
    ProcessName         : string;
    BasePriority        : dword;
    ProcessID           : dword;
    ParentProcessID     : dword;
    HandleCount         : dword;
    VirtualBytesPeak    : dword;
    VirtualBytes        : dword;
    PageFaults          : dword;
    WorkingSetPeak      : dword;
    WorkingSet          : dword;
    PagedPool           : dword;
    NonPagedPool        : dword;
    PageFileBytesPeak   : dword;
    PageFileBytes       : dword;
    PrivateBytes        : dword;
  end;

  TProcessList = class(TList)
  private
    function GetProcess(Index: LongInt): TProcess;
    procedure RefreshNTProcesses;
    procedure RefreshWin32Processes;
  public
    procedure Refresh;
    procedure Clear; override;
    destructor Destroy; override;
    property Process[Index: LongInt]: TProcess read GetProcess;
  end;

  TNTQueryFunction = function (dt: dword;  buf: pointer;
                      bufsize: dword; retlen: pointer): dword; stdcall;

implementation
uses SoCmnCns;

var
  LibHandle: THandle;
  NTQuery: TNTQueryFunction;
  IsNT: boolean;

procedure TProcessList.Clear;
var Index: LongInt;
begin
  if Count > 0 then
    for Index := 0 to pred(Count) do TProcess(Items[Index]).Free;
  inherited;
end;

destructor TProcessList.Destroy;
begin
  Clear;
  inherited Destroy;
end;

function TProcessList.GetProcess(Index: Integer): TProcess;
begin
  Result := TProcess(Items[Index]);
end;

type

  PProcessesBuffer = ^TProcessesBuffer;
  TProcessesBuffer = array[0..0] of byte;

procedure TProcessList.RefreshNTProcesses;
var Res, rl, cp: dword;
    PInfo: PProcessInfo;
    Buf: PProcessesBuffer;
    Pcs: TProcess;
begin
  GetMem(Buf, $10000);
  rl := 0;
  Res := NtQuery(5, buf, $10000, @rl);
  if Res = 0 then begin
   cp := 0;
   repeat
    pinfo := PProcessInfo(@buf[cp]);
    cp := cp + pinfo.dwOffset;
    pcs := TProcess.Create;
    with pinfo^ do begin
      pcs.ThreadCount := dwThreadCount;
      pcs.CreationTime := ftCreationTime;
      if pszProcessName <> nil then
        pcs.ProcessName := WideCharToString(pszProcessName)
      else
        pcs.ProcessName := sohoProcSystemIdle;
      pcs.BasePriority := dwBasePriority;
      pcs.ProcessID := dwProcessID;
      pcs.ParentProcessID := dwParentProcessID;
      pcs.HandleCount := dwHandleCount;
      pcs.VirtualBytesPeak := dwVirtualBytesPeak;
      pcs.VirtualBytes := dwVirtualBytes;
      pcs.PageFaults := dwPageFaults;
      pcs.WorkingSetPeak := dwWorkingSetPeak;
      pcs.WorkingSet := dwWorkingSet;
      pcs.PagedPool := dwPagedPool;
      pcs.NonPagedPool := dwNonPagedPool;
      pcs.PageFileBytesPeak := dwPageFileBytesPeak;
      pcs.PageFileBytes := dwPageFileBytes;
      pcs.PrivateBytes := dwPrivateBytes;
      Add(pcs);
    end;
   until pinfo.dwOffset = 0;
  end;
  FreeMem(buf);
end;

procedure TProcessList.RefreshWin32Processes;
var SnapHandle: THandle;
    Process: tagPROCESSENTRY32;
    Prs: TProcess;

    procedure SetProcessInfo;
    begin
      Prs := TProcess.Create;
      with Prs do begin
        ThreadCount := Process.cntThreads;
        HandleCount := Process.cntUsage;
        ProcessID := Process.th32ProcessID;
        ParentProcessID := Process.th32ParentProcessID;
        BasePriority := Process.pcPriClassBase;
        ProcessName := StrPas(Process.szExeFile);
      end;
      Add(Prs);
    end;

begin
  SnapHandle := CreateToolhelp32Snapshot(TH32CS_SNAPALL, 0);
  Process.dwSize := SizeOf(PPROCESSENTRY32W);
  if Process32First(SnapHandle, Process) then begin
    SetProcessInfo;
    while Process32Next(SnapHandle, Process) do SetProcessInfo;
  end;
  CloseHandle(SnapHandle);
end;

procedure TProcessList.Refresh;
begin
  Clear;
  if (@NtQuery = nil) and IsNT then exit;

  if IsNT then RefreshNTProcesses
  else RefreshWin32Processes;
end;

procedure LoadQueryFunction;
var Info : TOSVERSIONINFO;
begin
  LibHandle := 0;
  NTQuery := nil;

  Info.dwOSVersionInfoSize := SizeOf(TOSVERSIONINFO);
  GetVersionEx(Info);
  IsNT := Info.dwPlatformId = VER_PLATFORM_WIN32_NT;

  if IsNT then begin
    LibHandle := LoadLibrary('ntdll.dll');
    if LibHandle <> 0 then
      NTQuery := GetProcAddress(LibHandle, 'NtQuerySystemInformation');
  end;
end;

initialization
  LoadQueryFunction;
finalization;
  if LibHandle <> 0 then FreeLibrary(LibHandle);
end.
