{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoRTTI;

{$I SOHOLIB.INC}

interface
uses Classes, Forms, TypInfo;

procedure RunTimeRegisterComponents(Owner: TComponent);
{     Owner,      
       CreateComponentByClassName }
function CreateComponentByClassName(const ClassName: string;
  Owner: TComponent): TComponent;
{ C     }
function ComponentByName(var Form; AName: string): TComponent;
{        }

function IsProperty(Obj: TPersistent; PropertyName: string): Boolean;
{    PropertyName   Obj? }
function SetFloatProperty(Obj: TPersistent; PropertyName: string; Value: Double): Boolean;
{     PropertyName     Obj }
function SetIntegerProperty(Obj: TPersistent; PropertyName: string; Value: Longint): Boolean;
{     PropertyName     Obj }
function SetStringProperty(Obj: TPersistent; PropertyName: string; Value: string): Boolean;
{     PropertyName     Obj }
function SetClassProperty(Obj: TPersistent; PropertyName: string; Value: Pointer): Boolean;
{     PropertyName     Obj }
function GetFloatProperty(Obj: TPersistent; PropertyName: string): Double;
{    PropertyName     Obj }
function GetIntegerProperty(Obj: TPersistent; PropertyName: string): Longint;
{    PropertyName     Obj }
function GetStringProperty(Obj: TPersistent; PropertyName: string): string;
{    PropertyName     Obj }
function GetClassProperty(Obj: TPersistent; PropertyName: string): Pointer;
{    PropertyName     Obj }
function GetUniqueName (aComponent, aOwner: TComponent): string;
{       }

function StrToIdent(Obj: TPersistent; PropertyName: string; Ident: string): Cardinal;
{       PropertyName  
  .
example:

  StrToIdent(Self, 'Align', 'alLeft') = alLeft
}
function IdentToStr(Obj: TPersistent; PropertyName: string; Value: Cardinal): string;
{         PropertyName
   .
example:

  IdentToStr(Self, 'Align', alLeft) = 'alLeft'
}
function PropType(Obj: TPersistent; PropertyName: string): PTypeInfo;

implementation
uses SysUtils, SoCmnCns;

procedure RunTimeRegisterComponents(Owner: TComponent);
var index: Integer;
begin
  for index := 0 to Owner.ComponentCount - 1 do
    RegisterClass(TPersistentClass(Owner.Components[index].ClassType));
end;

function CreateComponentByClassName(const ClassName: string;
    Owner: TComponent): TComponent;
begin
  Result := TComponentClass(FindClass(ClassName)).Create(Owner);
end;

function ComponentByName(var Form; AName: string): TComponent;
var I: Longint;
begin
  with TForm(Form) do
    for I := 0 to ComponentCount - 1 do
      if Components[I].name = AName then begin
        Result := Components[I];
        exit;
      end;
  Result := nil;
end;

function IsPropertyInfo(Obj: TPersistent; PropertyName: string;
    var PropInfo: PPropInfo): Boolean;
var TypeInfo: PTypeInfo;
begin
  Result := False;
  TypeInfo := Obj.ClassInfo;
  if TypeInfo = nil then exit;
  PropInfo := GetPropInfo(TypeInfo, PropertyName);
  if PropInfo = nil then exit;
  Result := True;
end;

type

  EIdentConvertError = class(Exception);

function EnumValue(EnumType: PTypeInfo; const EnumName: string): Integer;
begin
  Result := GetEnumValue(EnumType, EnumName);
  if Result = -1 then EIdentConvertError.Create(sohoUtilsGetEnumError);
end;

function StrToIdent(Obj: TPersistent; PropertyName: string; Ident: string): Cardinal;
var PropInfo: PPropInfo;
    PropType: PTypeInfo;
begin
  if not IsPropertyInfo(Obj, PropertyName, PropInfo) then
    EIdentConvertError.Create(Format(sohoUtilsPropertyDoesnotExists, [PropertyName]));
  {$IFNDEF WIN32}
  PropType := PPropInfo(PropInfo)^.PropType;
  {$ELSE}
  PropType := PPropInfo(PropInfo)^.PropType^;
  {$ENDIF}
  if PropType^.Kind <> tkEnumeration then
    EIdentConvertError.Create(Format(sohoUtilsPropertyIsNotCardinal, [PropertyName]));
  Result := EnumValue(PropType, Ident);
end;

function IdentToStr(Obj: TPersistent; PropertyName: string; Value: Cardinal): string;
var PropInfo: PPropInfo;
  PropType: PTypeInfo;
begin
  Result := '';
  if not IsPropertyInfo(Obj, PropertyName, PropInfo) then
    EIdentConvertError.Create(Format(sohoUtilsPropertyDoesnotExists, [PropertyName]));
  PropType := PPropInfo(PropInfo)^.PropType^;
  if PropType^.Kind <> tkEnumeration then
    EIdentConvertError.Create(Format(sohoUtilsPropertyIsNotCardinal, [PropertyName]));
  Result := GetEnumName(PropType, Value);
end;

function IsProperty(Obj: TPersistent; PropertyName: string): Boolean;
var PropInfo: PPropInfo;
begin
  Result := IsPropertyInfo(Obj, PropertyName, PropInfo);
end;

function SetFloatProperty(Obj: TPersistent; PropertyName: string; Value: Double): Boolean;
var PropInfo: PPropInfo;
begin
  Result := IsPropertyInfo(Obj, PropertyName, PropInfo);
  if not Result then exit;
  SetFloatProp(Obj, PropInfo, Value);
end;

function SetIntegerProperty(Obj: TPersistent; PropertyName: string; Value: Longint): Boolean;
var PropInfo: PPropInfo;
begin
  Result := IsPropertyInfo(Obj, PropertyName, PropInfo);
  if not Result then exit;
  SetOrdProp(Obj, PropInfo, Value);
end;

function SetStringProperty(Obj: TPersistent; PropertyName: string; Value: string): Boolean;
var PropInfo: PPropInfo;
begin
  Result := IsPropertyInfo(Obj, PropertyName, PropInfo);
  if not Result then exit;
  SetStrProp(Obj, PropInfo, Value);
end;

function SetClassProperty(Obj: TPersistent; PropertyName: string; Value: Pointer): Boolean;
var PropInfo: PPropInfo;
begin
  Result := IsPropertyInfo(Obj, PropertyName, PropInfo);
  if not Result then exit;
  SetOrdProp(Obj, PropInfo, Longint(Value));
end;

function GetFloatProperty(Obj: TPersistent; PropertyName: string): Double;
var PropInfo: PPropInfo;
begin
  Result := 0;
  if not IsPropertyInfo(Obj, PropertyName, PropInfo) then exit;
  Result := GetFloatProp(Obj, PropInfo);
end;

function GetIntegerProperty(Obj: TPersistent; PropertyName: string): Longint;
var PropInfo: PPropInfo;
begin
  Result := 0;
  if not IsPropertyInfo(Obj, PropertyName, PropInfo) then exit;
  Result := GetOrdProp(Obj, PropInfo);
end;

function GetStringProperty(Obj: TPersistent; PropertyName: string): string;
var PropInfo: PPropInfo;
begin
  Result := '';
  if not IsPropertyInfo(Obj, PropertyName, PropInfo) then exit;
  Result := GetStrProp(Obj, PropInfo);
end;

function GetClassProperty(Obj: TPersistent; PropertyName: string): Pointer;
var PropInfo: PPropInfo;
begin
  Result := nil;
  if not IsPropertyInfo(Obj, PropertyName, PropInfo) then exit;
  Result := Pointer(GetOrdProp(Obj, PropInfo));
end;

function PropType(Obj: TPersistent; PropertyName: string): PTypeInfo;
var PropInfo: PPropInfo;
begin
  if not IsPropertyInfo(Obj, PropertyName, PropInfo) then
    EIdentConvertError.Create(Format(sohoUtilsPropertyDoesnotExists, [PropertyName]));
  Result := PPropInfo(PropInfo)^.PropType^;
end;

function GetUniqueName (aComponent, aOwner: TComponent): string;
var Index: Integer;
    Temp: string;
    Comp: TComponent;
begin
  Temp := aComponent.ClassName;
  if Temp[1] = 'T' then System.Delete(Temp, 1, 1);
  Index := 1;
  repeat
    Result := Temp + IntToStr(Index);
    Comp := aOwner.FindComponent(Result);
    Inc(Index);
  until (Comp = nil) or (Comp = aComponent);
end;

end.
