{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
     TsohoRadioGroup
}
unit SoRadGrp;

{$I SOHOLIB.INC}

interface

uses Windows, Messages, Classes, Controls, Forms, StdCtrls;

type

  TsohoCustomRadioGroup = class(TCustomGroupBox)
  private
    FButtons: TList;
    FItems: TStrings;
    FItemIndex: Integer;
    FOldItemIndex: Integer;
    FColumns: Integer;
    FReading: Boolean;
    FUpdating: Boolean;
    FOnChange: TNotifyEvent;
    FOnUndo: TNotifyEvent;
    procedure ArrangeButtons;
    procedure ButtonClick(Sender: TObject);
    procedure ItemsChange(Sender: TObject);
    procedure SetButtonCount(Value: Integer);
    procedure SetColumns(Value: Integer);
    procedure SetItemIndex(Value: Integer);
    procedure SetItems(Value: TStrings);
    procedure UpdateButtons;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure CMEnter(var message: TCMEnter); message CM_ENTER;
  protected
    procedure ReadState(Reader: TReader); override;
    function CanModify: Boolean; virtual;
    procedure Change; virtual;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    property Columns: Integer read FColumns write SetColumns default 1;
    property ItemIndex: Integer read FItemIndex write SetItemIndex default -1;
    property Items: TStrings read FItems write SetItems;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnUndo: TNotifyEvent read FOnUndo write FOnUndo;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Undo;
  end;

  TsohoRadioGroup = class(TsohoCustomRadioGroup)
  published
    property Align;
    property Caption;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ItemIndex;
    property Items;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnChange;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnStartDrag;
    property OnUndo;
  end;

implementation
uses SoChkBox, SoUtils;

type

  TsohoGroupButton = class(TsohoCheckBox)
  private
    FUpdating: boolean;
  protected
    procedure ChangeScale(M, D: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure Changed; override;
  public
    procedure Undo; override;
    constructor InternalCreate(RadioGroup: TsohoCustomRadioGroup);
    destructor Destroy; override;
  end;

{ TsohoGroupButton}
procedure TsohoGroupButton.Changed;
begin
  inherited;
  if FUpdating then exit;
  if Checked then
    TsohoCustomRadioGroup(Parent).ItemIndex :=
      TsohoCustomRadioGroup(Parent).FButtons.IndexOf(Self);
end;

procedure TsohoGroupButton.Undo;
begin
  // inherited;
  // TsohoCustomRadioGroup(Owner).Undo;
end;

constructor TsohoGroupButton.InternalCreate(RadioGroup: TsohoCustomRadioGroup);
begin
  inherited Create(RadioGroup);
  Style := btRadioButton;
  RadioGroup.FButtons.Add(Self);
  Visible := False;
  Enabled := RadioGroup.Enabled;
  ParentShowHint := False;
  OnClick := RadioGroup.ButtonClick;
  Parent := RadioGroup;
end;

destructor TsohoGroupButton.Destroy;
begin
  TsohoCustomRadioGroup(Owner).FButtons.Remove(Self);
  inherited Destroy;
end;

procedure TsohoGroupButton.ChangeScale(M, D: Integer);
begin
end;

procedure TsohoGroupButton.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  TsohoCustomRadioGroup(Parent).KeyPress(Key);
  if (Key = #8) or (Key = ' ') then
    if not TsohoCustomRadioGroup(Parent).CanModify then Key := #0;
end;

procedure TsohoGroupButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if ((Key = ord('z')) or (Key = ord('Z'))) and (ssCtrl in Shift) then
    TsohoCustomRadioGroup(Parent).Undo;
  if (Key = VK_BACK) and (ssAlt in Shift) then
    TsohoCustomRadioGroup(Parent).Undo;
  inherited KeyDown(Key, Shift);
  TsohoCustomRadioGroup(Parent).KeyDown(Key, Shift);
end;

{ TsohoCustomRadioGroup }
procedure TsohoCustomRadioGroup.CMEnter(var message: TCMEnter);
begin
  inherited;
  FOldItemIndex := FItemIndex; //    ,
  //   Undo   ?
end;

procedure TsohoCustomRadioGroup.Change;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

constructor TsohoCustomRadioGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csSetCaption, csDoubleClicks];
  FButtons := TList.Create;
  FItems := TStringList.Create;
  TStringList(FItems).OnChange := ItemsChange;
  FItemIndex := -1;
  FColumns := 1;
end;

destructor TsohoCustomRadioGroup.Destroy;
begin
  SetButtonCount(0);
  TStringList(FItems).OnChange := nil;
  FItems.Free;
  FButtons.Free;
  inherited Destroy;
end;

procedure TsohoCustomRadioGroup.ArrangeButtons;
var
  ButtonsPerCol, ButtonWidth, ButtonHeight, TopMargin, I: Integer;
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;
  DeferHandle: THandle;
begin
  if (FButtons.Count <> 0) and not FReading then
  begin
    DC := GetDC(0);
    SaveFont := SelectObject(DC, Font.Handle);
    GetTextMetrics(DC, Metrics);
    SelectObject(DC, SaveFont);
    ReleaseDC(0, DC);
    ButtonsPerCol := (FButtons.Count + FColumns - 1) div FColumns;
    ButtonWidth := (Width - 10) div FColumns;
    I := Height - Metrics.tmHeight - 5;
    ButtonHeight := I div ButtonsPerCol;
    TopMargin := Metrics.tmHeight + 1 + (I mod ButtonsPerCol) div 2;
    DeferHandle := BeginDeferWindowPos(FButtons.Count);
    for I := 0 to FButtons.Count - 1 do
      with TsohoGroupButton(FButtons[I]) do
      begin
        DeferHandle := DeferWindowPos(DeferHandle, Handle, 0,
          (I div ButtonsPerCol) * ButtonWidth + 8,
          (I mod ButtonsPerCol) * ButtonHeight + TopMargin,
          ButtonWidth, ButtonHeight,
          SWP_NOZORDER or SWP_NOACTIVATE);
        Visible := True;
      end;
    EndDeferWindowPos(DeferHandle);
  end;
end;

procedure TsohoCustomRadioGroup.ButtonClick(Sender: TObject);
begin
  if not FUpdating then
  begin
    FItemIndex := FButtons.IndexOf(Sender);
    Changed;
    Click;
  end;
end;

procedure TsohoCustomRadioGroup.ItemsChange(Sender: TObject);
begin
  if not FReading then
  begin
    if FItemIndex >= FItems.Count then FItemIndex := FItems.Count - 1;
    UpdateButtons;
  end;
end;

procedure TsohoCustomRadioGroup.ReadState(Reader: TReader);
begin
  FReading := True;
  inherited ReadState(Reader);
  FReading := False;
  UpdateButtons;
end;

procedure TsohoCustomRadioGroup.SetButtonCount(Value: Integer);
begin
  while FButtons.Count < Value do TsohoGroupButton.InternalCreate(Self);
  while FButtons.Count > Value do TsohoGroupButton(FButtons.Last).Free;
end;

procedure TsohoCustomRadioGroup.SetColumns(Value: Integer);
begin
  if Value < 1 then Value := 1;
  if Value > 16 then Value := 16;
  if FColumns <> Value then
  begin
    FColumns := Value;
    ArrangeButtons;
    Invalidate;
  end;
end;

procedure TsohoCustomRadioGroup.SetItemIndex(Value: Integer);
begin
  if FReading then FItemIndex := Value else
  begin
    if Value < -1 then Value := -1;
    if Value >= FButtons.Count then Value := FButtons.Count - 1;
    if FItemIndex <> Value then
    begin
      FOldItemIndex := FItemIndex;
      if FItemIndex >= 0 then begin
        TsohoGroupButton(FButtons[FItemIndex]).FUpdating := true;
        TsohoGroupButton(FButtons[FItemIndex]).Checked := False;
        TsohoGroupButton(FButtons[FItemIndex]).FUpdating := false;
      end;
      FItemIndex := Value;
      if FItemIndex >= 0 then begin
        TsohoGroupButton(FButtons[FItemIndex]).FUpdating := true;
        TsohoGroupButton(FButtons[FItemIndex]).Checked := True;
        // TsohoGroupButton(FButtons[FItemIndex]).SetFocus;
        TsohoGroupButton(FButtons[FItemIndex]).FUpdating := false;
      end;
      Change;
    end;
  end;
end;

procedure TsohoCustomRadioGroup.SetItems(Value: TStrings);
begin
  FItems.Assign(Value);
end;

procedure TsohoCustomRadioGroup.UpdateButtons;
var
  I: Integer;
begin
  SetButtonCount(FItems.Count);
  for I := 0 to FButtons.Count - 1 do
    TsohoGroupButton(FButtons[I]).Caption := FItems[I];
  if FItemIndex >= 0 then
  begin
    FUpdating := True;
    TsohoGroupButton(FButtons[FItemIndex]).Checked := True;
    FUpdating := False;
  end;
  ArrangeButtons;
  Invalidate;
end;

procedure TsohoCustomRadioGroup.CMEnabledChanged(var Message: TMessage);
var
  I: Integer;
begin
  inherited;
  for I := 0 to pred(FButtons.Count) do
    TsohoGroupButton(FButtons[I]).Enabled := Enabled;
end;

procedure TsohoCustomRadioGroup.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ArrangeButtons;
end;

procedure TsohoCustomRadioGroup.WMSize(var Message: TWMSize);
begin
  inherited;
  ArrangeButtons;
end;

function TsohoCustomRadioGroup.CanModify: Boolean;
begin
  Result := True;
end;

procedure TsohoCustomRadioGroup.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

procedure TsohoCustomRadioGroup.Undo;
var Tmp1, Tmp2: LongInt;
begin
  Tmp1 := FItemIndex;
  Tmp2 := FOldItemIndex;
  SwapLongInt(Tmp1, Tmp2);
  ItemIndex := Tmp1;
  if Assigned(FOnUndo) then FOnUndo(Self);
end;

end.
