{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{           Partial copyright (c) AllexSoft             }
{           Written by Alexander Govyadinov,            }
{                        Axe,                           }
{                   Serg Vostrikov                      }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
      ,  
       
      .
}
unit SoScaleForm;

{$I SOHOLIB.INC}

interface
uses Forms, Classes;

type

  TsohoScalePosition = class
{   }
    spTop, spLeft, spWidth, spHeight: LongInt;
    spFontSize : LongInt;
    constructor FillFromComponent(Obj: TComponent);
  end;

  TsohoScaleOption = (soProportionalSize, soProportionalFontSize);
  { ,    :
    soProportionalSize -      
    soProportionalFontSize -      
  }

  TsohoScaleOptions = set of TsohoScaleOption;

  TsohoScaleForm = class(TComponent)
  private
    fOnResize: TNotifyEvent;
    VertRatio,HorRatio: Double;
    FScaleOptions: TsohoScaleOptions;
  protected
    SavedHeight : LongInt;
    SavedWidth  : LongInt;
    Form        : TForm;
    procedure DoFirstResize(Sender: TObject);
    procedure DoResize(Sender: TObject);
    procedure Loaded; override;
    procedure ClearPositions; virtual;
  public
    Positions   : TStringList;
    constructor Create(aOwner: TComponent); override;
    // constructor RunTimeCreate(aOwner: TComponent); virtual; 
    destructor Destroy; override;
    procedure SaveComponentsPositions;
    procedure ScaleForm;
    procedure SetEvents; virtual;
    procedure RestoreEvents; virtual;
  published
    property Options: TsohoScaleOptions read FScaleOptions
      write FScaleOptions default [soProportionalSize, soProportionalFontSize];
  end;

implementation
uses Graphics, SoRTTI, SoVCLUtils, SoAppUtils, COntrols;

{ TsohoScalePosition }
constructor TsohoScalePosition.FillFromComponent(Obj:TComponent);
var Font: TFont;
begin
  inherited Create;
  spLeft := GetIntegerProperty(Obj,'Left');
  spTop := GetIntegerProperty(Obj,'Top');
  spWidth := GetIntegerProperty(Obj,'Width');
  spHeight := GetIntegerProperty(Obj,'Height');
  Font := GetClassProperty(Obj, 'Font');
  if Assigned(Font) then spFontSize := Font.Size;
end;

{ TsohoScaleForm }
procedure TsohoScaleForm.ClearPositions;
var Index: LongInt;
begin
  for Index := 0 to pred(Positions.Count) do
    TsohoScalePosition(Positions.Objects[Index]).Free;
  Positions.Clear;
end;

procedure TsohoScaleForm.SetEvents;
begin
  Form := GetOwnerForm(Self);
  if Assigned(Form) then begin
    fOnResize := Form.OnResize;
    Form.OnResize := DoFirstResize; //     
  end;
end;

procedure TsohoScaleForm.RestoreEvents;
begin
  if Assigned(Form) then Form.OnResize := fOnResize;
  ClearPositions;
end;

procedure TsohoScaleForm.Loaded;
begin
  inherited;
  if not (csDesigning in ComponentState) then SetEvents;
end;

constructor TsohoScaleForm.Create;
begin
  inherited Create(aOwner);
  Positions := TStringList.Create; //  
  FScaleOptions := [soProportionalSize, soProportionalFontSize];
end;

destructor TsohoScaleForm.Destroy;
begin
  RestoreEvents;
  ClearPositions;
  Positions.Free;
  inherited;
end;

procedure TsohoScaleForm.DoFirstResize(Sender:TObject);
begin
  if Assigned(fOnResize) then fOnResize(Sender); // 
  SaveComponentsPositions; //,    ...
  Form.OnResize := DoResize; //  
end;

procedure TsohoScaleForm.DoResize(Sender:TObject);
begin
  if Assigned(fOnResize) then fOnResize(Sender); //   
  ScaleForm; //  - 
  //   -  
end;

procedure TsohoScaleForm.SaveComponentsPositions;
var Index: LongInt;
    Obj: TComponent;
begin
  if not Assigned(Form) then exit;
  ClearPositions; //, ,   ,    
  for Index := 0 to pred(Form.ComponentCount) do begin
    Obj := Form.Components[Index];
    if IsProperty(Obj, 'Top') then //,  Top,    ...
       Positions.Objects[Positions.Add(Obj.Name)] := TsohoScalePosition.FillFromComponent(Obj);
       //   -     
  end;
  SavedHeight := Form.ClientHeight;
  SavedWidth := Form.ClientWidth;
end;

procedure TsohoScaleForm.ScaleForm;
var Index: LongInt;
    Obj: TComponent;
    Min: Double;
    Font: TFont;
    Align: TAlign;

    procedure SetPositions(Pos: TsohoScalePosition);
    begin
      with Pos do begin
        SetIntegerProperty(Obj, 'Left', Round(spLeft * HorRatio));
        SetIntegerProperty(Obj, 'Top', Round(spTop * VertRatio));
        SetIntegerProperty(Obj, 'Width', Round(spWidth * HorRatio));
        SetIntegerProperty(Obj, 'Height', Round(spHeight * VertRatio));
      end;
    end;

begin
  if not Assigned(Form) then exit;
  VertRatio := Form.ClientHeight / SavedHeight; //    
  HorRatio := Form.ClientWidth / SavedWidth; //...
  if VertRatio < HorRatio then Min := VertRatio
  else Min := HorRatio;
  // Min := GetMin(VertRatio, HorRatio);
  if soProportionalSize in FScaleOptions then begin
    VertRatio := Min;
    HorRatio := Min;
  end;

  for Index := 0 to pred(Positions.Count) do
    with TsohoScalePosition(Positions.Objects[Index]) do begin
      Obj := ComponentByName(Form,Positions[Index]);
      if IsProperty(Obj, 'Align') then begin
        //    
        Align := TAlign(GetIntegerProperty(Obj, 'Align'));
        if Align in [alLeft, alRight] then
          SetIntegerProperty(Obj, 'Width', Round(spWidth * HorRatio));
        if Align in [alTop, alBottom] then
          SetIntegerProperty(Obj, 'Height', Round(spHeight * VertRatio));
        if Align = alNone then
          SetPositions(TsohoScalePosition(Positions.Objects[Index]));
      end
      else SetPositions(TsohoScalePosition(Positions.Objects[Index]));

      if (soProportionalFontSize in FScaleOptions) and
        IsProperty(Obj, 'Font') then begin
        Font := GetClassProperty(Obj, 'Font');
        Font.Size := Round(spFontSize * Min);
        SetClassProperty(Obj, 'Font', Font);
      end;
    end;
end;

end.
