{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoSpecEdit;

{$I SOHOLIB.INC}

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
     SoCtrls, Dialogs, SoBrsFld;

type

  TsohoDialogKind = (sdOpen, sdSave, sdOpenPicture, sdSavePicture);
  {    TsohoFilenameEdit }

  TsohoFilenameEdit = class(TsohoCustomEdit)
  {     }
  private
    { Private declarations }
    FKind: TsohoDialogKind;
    FHistoryList: TStrings;
    FOptions: TOpenOptions;
    FFilter: string;
    FFilterIndex: Integer;
    FInitialDir: string;
    FTitle: string;
    FDefaultExt: string;
    // FFileName: TFileName;
    FFileEditStyle: TFileEditStyle;
  protected
    { Protected declarations }
    procedure SetHistoryList(Value: TStrings);
    procedure DoRightClick(Sender: TObject); override;
    procedure SetKind (Value: TsohoDialogKind); virtual;
    procedure SetFileName (Value: TFileName);
    function GetFileName: TFileName;
    procedure CreateRightControl; override;
  public
    { Public declarations }
    procedure Loaded; override;
    procedure CreateWnd; override;
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    property Button;
    property FileEditStyle: TFileEditStyle read FFileEditStyle write FFileEditStyle;
    property HistoryList: TStrings read FHistoryList write SetHistoryList;
  published
    {$IFDEF Delphi4_or_5}
    property Action;
    property ParentBiDiMode;
    property Anchors;
    property BiDiMode;
    property DragKind;
    property Constraints;
    {$ENDIF}
    property BtnHint;
    {    }
    property MayBeEmpty default True;
    {     .  , 
      MayBeEmpty = false,     
             
        }
    property CharCase;
    property Color;
    property Font;
    property HideSelection;
    property OEMConvert;
    property AutoSelect;
    property AutoSize;
    property Visible;
    property Ctl3D default True;
    property DragCursor;
    property DragMode;
    property Enabled;
    property HelpContext;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D default True;
    property ParentFont;
    property ParentShowHint;
    property ReadOnly;
    property ShowHint;
    property TabOrder;

    property Kind: TsohoDialogKind read FKind write SetKind default sdOpen;
    property DirectEnter default true;
    property DefaultExt: string read FDefaultExt write FDefaultExt;
    property FileName: TFileName read GetFileName write SetFileName;
    property Filter: string read FFilter write FFilter;
    property FilterIndex: Integer read FFilterIndex write FFilterIndex default 1;
    property InitialDir: string read FInitialDir write FInitialDir;
    {$IFDEF Delphi3}  
    property Options: TOpenOptions read FOptions write FOptions
       default [ofHideReadOnly, ofFileMustExist];
    {$ELSE}
    property Options: TOpenOptions read FOptions write FOptions
       default [ofHideReadOnly, ofFileMustExist, ofEnableSizing];
    {$ENDIF}
    property Title: string read FTitle write FTitle;
        property LightLabel;
    property OnChange;
    property OnModify;
    property OnRightClick;
    property CanExit;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnUndo;
  end;

  TsohoDirectoryEdit = class(TsohoCustomEdit)
  {    .  
         Windows }
  private
    { Private declarations }
    FTitle: string;
    // FDirectory: string;
    FFlags: TsohoBrowseViewFlags;
    FPathInStatus: boolean;
    FRootFolder: TsohoBrowseFolders;
  protected
    { Protected declarations }
    procedure DoRightClick(Sender: TObject); override;
    procedure SetDirectory (Value: string);
    function GetDirectory: string;
    procedure CreateRightControl; override;
  public
    { Public declarations }
    procedure Loaded; override;
    procedure CreateWnd; override;
    constructor Create(aOwner: TComponent); override;
  published
    { Published declarations }
    property BtnHint;
    property MayBeEmpty default True;
    property CharCase;
    property Color;
    property Font;
    property HideSelection;
    property OEMConvert;
    property AutoSelect;
    property AutoSize;
    property Visible;
    property Ctl3D default True;
    property DragCursor;
    property DragMode;
    property Enabled;
    property HelpContext;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D default True;
    property ParentFont;
    property ParentShowHint;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property DirectEnter default true;
    property Title: string read FTitle write FTitle;
    property LightLabel;
    property Directory: string read GetDirectory write SetDirectory;
    property ViewFlags: TsohoBrowseViewFlags read FFlags write FFlags default [bvFileSysDirsOnly, bvStatusText];
    property PathInStatus: boolean read FPathInStatus write FPathInStatus default true;
    property RootFolder: TsohoBrowseFolders read FRootFolder write FRootFolder default bfDesktopExpanded;
    property OnChange;
    property OnModify;
    property OnRightClick;
    property CanExit;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnUndo;
  end;

implementation
uses SoUtils, SoCmnCns, ExtDlgs, SoAppUtils
    {$IFDEF Delphi4_or_5}, ImgList{$ENDIF};

var Pictures: TImageList;

{$R SOSPECEDIT.R32}

{ TsohoFilenameEdit }
procedure TsohoFilenameEdit.CreateRightControl;
begin
  inherited;
  Images := Pictures;
  if ord(Kind) > 1 then ImageIndex := ord(Kind) - 2
  else ImageIndex := ord(Kind);
end;

procedure TsohoFilenameEdit.SetKind (Value: TsohoDialogKind);
begin
  if FKind = Value then exit;
  FKind := Value;
  if ord(FKind) > 1 then ImageIndex := ord(FKind) - 2
  else ImageIndex := ord(FKind);
end;

procedure TsohoFilenameEdit.SetFileName (Value: TFileName);
begin
  Text := Value;
end;

procedure TsohoFilenameEdit.DoRightClick(Sender: TObject);
var Dialog: TOpenDialog;
    FOldNotify: TNotifyEvent;

   procedure SetDialogProperties (Dialog : TOpenDialog);
   begin
     with Dialog do begin
        DefaultExt := FDefaultExt;
        FileName := Text;
        Filter := FFilter;
        FilterIndex := FFilterIndex;
        InitialDir := FInitialDir;
        Options := FOptions;
        Title := FTitle;
     end;
   end;

   procedure GetDialogProperties (Dialog : TOpenDialog);
   begin
     with Dialog do begin
        FDefaultExt := DefaultExt;
        Text := FileName;
        FFilter := Filter;
        FFilterIndex := FilterIndex;
        FOptions := Options;
        FTitle := Title;
        Self.FileName := FileName;
        FInitialDir := ExtractFilePath(FileName);
     end;
   end;

begin
  case FKind of
    sdOpen        : Dialog := TOpenDialog.Create(Self);
    sdSave        : Dialog := TSaveDialog.Create(Self);
    sdOpenPicture : Dialog := TOpenPictureDialog.Create(Self);
    else Dialog := TSavePictureDialog.Create(Self);
  end;
  SetDialogProperties(Dialog);
  if Dialog.Execute then GetDialogProperties(Dialog);
  FOldNotify := OnRightClick;
  if Assigned(FOldNotify) then FOldNotify(Self);
end;

procedure TsohoFilenameEdit.SetHistoryList(Value: TStrings);
begin
  FHistoryList.Assign(Value);
end;

procedure TsohoFilenameEdit.Loaded;
begin
  inherited Loaded;
  RightControl := Assigned(Parent);
  BtnNumGlyphs := 1;
end;

procedure TsohoFilenameEdit.CreateWnd;
begin
  inherited;
  RightControl := Assigned(Parent);
  if csDesigning in ComponentState then Text := '';
end;

constructor TsohoFilenameEdit.Create(aOwner: TComponent);
begin
  inherited;
  FKind := sdOpen;
  ValueKind := vkString;
  FHistoryList := TStringList.Create;
  {$IFDEF Delphi3}
  FOptions := [ofHideReadOnly, ofFileMustExist];
  {$ELSE}
  FOptions := [ofHideReadOnly, ofFileMustExist, ofEnableSizing];
  {$ENDIF}
  FFilter := sohoSpecEditFilter;
  FFilterIndex := 0;
  FInitialDir := '';
  FTitle := sohoSpecEditCaption;
  FDefaultExt := '';
  FFileEditStyle := fsEdit;
  RightControl := true;
end;

destructor TsohoFilenameEdit.Destroy;
begin
  FHistoryList.Free;
  inherited;
end;

function TsohoFilenameEdit.GetFileName: TFileName;
begin
  Result := Text;
end;

{ TsohoDirectoryEdit }
procedure TsohoDirectoryEdit.CreateRightControl;
begin
  inherited;
  Images := Pictures;
  ImageIndex := 0;
end;

procedure TsohoDirectoryEdit.SetDirectory (Value: string);
begin
  Text := Value;
end;

procedure TsohoDirectoryEdit.DoRightClick(Sender: TObject);
var FOldNotify: TNotifyEvent;
begin
  with TsohoBrowseFolder.Create(Self) do begin
    ViewFlags := FFlags;
    RootFolder := FRootFolder;
    Directory := Text;
    if Directory = '' then Directory := GetRunDir;
    PathInStatus := FPathInStatus;
    Title := FTitle;
    if Execute then Self.Directory := Directory;
    Free;
  end;
  FOldNotify := OnRightClick;
  if Assigned(FOldNotify) then FOldNotify(Self);
end;

procedure TsohoDirectoryEdit.Loaded;
begin
  inherited Loaded;
  RightControl := Assigned(Parent);
  BtnNumGlyphs := 1;
end;

procedure TsohoDirectoryEdit.CreateWnd;
begin
  inherited;
  RightControl := Assigned(Parent);
  if csDesigning in ComponentState then Text := '';
end;

constructor TsohoDirectoryEdit.Create(aOwner: TComponent);
begin
  inherited;
  FTitle := sohoSpecBrowseCaption;
  FRootFolder := bfDesktopExpanded;
  FFlags := [bvFileSysDirsOnly, bvStatusText];
  FPathInStatus := true;
  Text := '';
  RightControl := true;
end;

procedure InitImagesForButtons;
begin
  Pictures := TImageList.Create(nil);
  Pictures.Width := 13;
  Pictures.Height := 13;
  Pictures.ResourceLoad(rtBitmap, 'SOHOSPECBTNS', clAqua);
end;

function TsohoDirectoryEdit.GetDirectory: string;
begin
  Result := Text;
end;

initialization
  InitImagesForButtons;
finalization
  Pictures.Free;
end.