{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
    TsohoSplitter
}
unit SoSplit;

{$I SOHOLIB.INC}

interface
uses Classes, Controls {$IFDEF Delphi4_or_5}, ImgList{$ENDIF},
     Windows, Graphics, Messages, Extctrls;

const
  PackLeverMinSize = 5;
  PackLeverMaxSize = 24;

type

  {$IFDEF Delphi3}
  TCanResizeEvent = procedure(Sender: TObject; var NewSize: Integer;
    var Accept: Boolean) of object;
  {        }

  TResizeStyle = (rsNone, rsLine, rsUpdate, rsPattern);
  {   splitter'    }
  {$ENDIF}

  TsohoPackMode = (pmClick, pmDblClick);
  {  ,      , 
    ""  ""   }

  TsohoSplitter = class(TGraphicControl)
  {      TSplitter.
           , 
      ""   "" 
          }
  private
    FActiveControl: TWinControl;
    FFlat: boolean;
    FBevel: Boolean;
    FBrush: TBrush;
    FControl: TControl;
    FDownPos: TPoint;
    FLineDC: HDC;
    FLineVisible: Boolean;
    FMinSize: NaturalNumber;
    FMaxSize: Integer;
    FNewSize: Integer;
    FOldKeyDown: TKeyEvent;
    FOldSize: Integer;
    FPrevBrush: HBrush;
    FResizeStyle: TResizeStyle;
    FSplit: Integer;
    FUnPackWidth: LongInt;
    FPacked: boolean;
    FShowPackLever: boolean;
    FPackLeverColor: TColor;
    FBeforePackActive: boolean;
    FPackMode: TsohoPackMode;
    FPackWidth: LongInt;
    FAfterPacked: boolean;
    FOnLever: boolean;
    FMouseUnder: boolean;
    FOnCanResize: TCanResizeEvent;
    FOnMoved: TNotifyEvent;
    FOnPaint: TNotifyEvent;
    procedure AllocateLineDC;
    procedure CalcSplitSize(X, Y: Integer; var NewSize, Split: Integer);
    procedure DrawLine;
    function FindControl: TControl;
    // procedure FocusKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ReleaseLineDC;
    procedure SetBevel(Value: Boolean);
    procedure UpdateControlSize;
    procedure UpdateSize(X, Y: Integer);

    procedure SetPacked(Value: boolean);
    procedure SetPackLeverColor(Value: TColor);
    procedure SetPackWidth(Value: LongInt);
    procedure SetShowPackLever(Value: boolean);
    procedure SetFlat(Value: boolean);
  protected
    procedure CMMouseEnter (var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave (var Message: TMessage); message CM_MOUSELEAVE;
    {$IFDEF Delphi3}
    function CanResize(var NewSize: Integer): Boolean; virtual;
    {$ELSE}
    function CanResize(var NewSize: Integer): Boolean; reintroduce; virtual;
    {$ENDIF}
    function DoCanResize(var NewSize: Integer): Boolean; virtual;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Paint; override;
    {$IFDEF Delphi3}
    procedure SetAlign(Value: TAlign);
    function GetAlign: TAlign;
    {$ELSE}
    procedure RequestAlign; override;
    {$ENDIF}
    procedure StopSizing; dynamic;

    function GetLeverRect: TRect;
  public
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DblClick; override;
    procedure Click; override;
    property Canvas;
  published
    {$IFDEF Delphi3}
    property Align: TAlign read GetAlign write SetAlign default alLeft;
    {$ELSE}
    property Align default alLeft;
    {$ENDIF}
    property Flat: boolean read FFlat write SetFlat default true;
    { ""  .   , splitter  
           ,    
        }
    property Bevel: Boolean read FBevel write SetBevel default False;
    {   splitter' }
    property Color;
    {  splitter' }
    {$IFDEF Delphi4_or_5}
    property Constraints;
    {$ENDIF}
    property MinSize: NaturalNumber read FMinSize write FMinSize default 30;
    {          
       }
    property ParentColor;
    property ResizeStyle: TResizeStyle read FResizeStyle write FResizeStyle
      default rsPattern;
    {        }
    property Visible;
    property IsPacked: boolean read FPacked write SetPacked default false;
    {  true,        
       splitter ( "") }
    property PackLeverColor: TColor read FPackLeverColor write SetPackLeverColor default clBtnShadow;
    {   "" }
    property PackMode: TsohoPackMode read FPackMode write FPackMode default pmClick;
    {       "" -  ,
         }
    property PackWidth: LongInt read FPackWidth write SetPackWidth default 1;
    {        "" }
    property ShowPackLever: boolean read FShowPackLever write SetShowPackLever default true;
    {   splitter'  ""? }
    property UnPackWidth: LongInt read FUnPackWidth write FUnPackWidth;
    {        "",  ,
              "" }
    property Width;
    property Height;

    property OnCanResize: TCanResizeEvent read FOnCanResize write FOnCanResize;
    {         }
    property OnMoved: TNotifyEvent read FOnMoved write FOnMoved;
    {         }
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
    {      splitter' }
  end;

implementation
uses SoVCLUtils, Forms;

{$R SOSPLIT.R32}

type
  TWinControlAccess = class(TWinControl);

var
  Arrows: TImageList;

{ TsohoSplitter }

constructor TsohoSplitter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align := alLeft;
  Width := 7;
  Height := 50;
  Cursor := crHSplit;
  FMinSize := 30;
  FResizeStyle := rsPattern;
  FOldSize := -1;
  FUnPackWidth := 100;
  FPacked := false;
  FShowPackLever := true;
  FPackLeverColor := clBtnShadow;
  FBeforePackActive := false;
  FPackMode := pmClick;
  FPackWidth := 1;
  FAfterPacked := false;
  FFlat := true;
end;

destructor TsohoSplitter.Destroy;
begin
  FBrush.Free;
  inherited Destroy;
end;

procedure TsohoSplitter.AllocateLineDC;
begin
  FLineDC := GetDCEx(Parent.Handle, 0, DCX_CACHE or DCX_CLIPSIBLINGS
    or DCX_LOCKWINDOWUPDATE);
  {$IFNDEF Delphi3}
  if ResizeStyle = rsPattern then begin
    if FBrush = nil then begin
      FBrush := TBrush.Create;
      FBrush.Bitmap := AllocPatternBitmap(clBlack, clWhite);
    end;
    FPrevBrush := SelectObject(FLineDC, FBrush.Handle);
  end;
  {$ENDIF}
end;

procedure TsohoSplitter.DrawLine;
var
  P: TPoint;
begin
  FLineVisible := not FLineVisible;
  P := Point(Left, Top);
  if Align in [alLeft, alRight] then P.X := Left + FSplit
  else P.Y := Top + FSplit;
  if Align in [alTop, alBottom] then
    with P do PatBlt(FLineDC, X, Y, Width, 2, PATINVERT)
  else
    with P do PatBlt(FLineDC, X, Y, 2, Height, PATINVERT);
end;

procedure TsohoSplitter.ReleaseLineDC;
begin
  if FPrevBrush <> 0 then
    SelectObject(FLineDC, FPrevBrush);
  ReleaseDC(Parent.Handle, FLineDC);
  if FBrush <> nil then begin
    FBrush.Free;
    FBrush := nil;
  end;
end;

function TsohoSplitter.FindControl: TControl;
var
  P: TPoint;
  I: Integer;
  R: TRect;
begin
  Result := nil;
  P := Point(Left, Top);
  case Align of
    alLeft: Dec(P.X);
    alRight: Inc(P.X, Width);
    alTop: Dec(P.Y);
    alBottom: Inc(P.Y, Height);
  else
    Exit;
  end;
  for I := 0 to pred(Parent.ControlCount) do begin
    Result := Parent.Controls[I];
    if Result.Visible and Result.Enabled then begin
      R := Result.BoundsRect;
      if (R.Right - R.Left) = 0 then
        if Align in [alTop, alLeft] then Dec(R.Left)
        else Inc(R.Right);
      if (R.Bottom - R.Top) = 0 then
        if Align in [alTop, alLeft] then Dec(R.Top)
        else Inc(R.Bottom);
      if PtInRect(R, P) then Exit;
    end;
  end;
  Result := nil;
end;

{$IFDEF Delphi4_or_5}
procedure TsohoSplitter.RequestAlign;
begin
  inherited RequestAlign;
  if (Cursor <> crVSplit) and (Cursor <> crHSplit) then Exit;
  if Align in [alBottom, alTop] then Cursor := crVSplit
  else Cursor := crHSplit;
end;
{$ELSE}
procedure TsohoSplitter.SetAlign(Value: TAlign);
begin
  inherited Align := Value;
  if Align in [alBottom, alTop] then Cursor := crVSplit
  else Cursor := crHSplit;
end;

function TsohoSplitter.GetAlign: TAlign;
begin
  Result := inherited Align;
end;
{$ENDIF}


procedure TsohoSplitter.SetFlat(Value: boolean);
begin
  if FFlat = Value then exit;
  FFlat := Value;
  Invalidate;
end;

procedure TsohoSplitter.Paint;
const
  XorColor = $00FFD8CE;
var
  FrameBrush: HBRUSH;
  R, BmpR: TRect;
  fBmp: TBitmap;
  ImageIndex: LongInt;
begin
  R := ClientRect;
  Canvas.Brush.Color := Color;
  Canvas.FillRect(ClientRect);
  if Bevel then begin
    if (FFlat and FMouseUnder) or
       (not Flat) then begin
      if Align in [alLeft, alRight] then InflateRect(R, -1, 2)
      else InflateRect(R, 2, -1);
      OffsetRect(R, 1, 1);
      FrameBrush := CreateSolidBrush(ColorToRGB(clBtnHighlight));
      FrameRect(Canvas.Handle, R, FrameBrush);
      DeleteObject(FrameBrush);
      OffsetRect(R, -2, -2);
      FrameBrush := CreateSolidBrush(ColorToRGB(clBtnShadow));
      FrameRect(Canvas.Handle, R, FrameBrush);
      DeleteObject(FrameBrush);
    end;
  end
  else begin
    if (FFlat and FMouseUnder) then
      Frame3D(Canvas, R, clBtnShadow, clBtnHighlight, 1);
  end;
  if csDesigning in ComponentState then
    { Draw outline }
    with Canvas do begin
      Pen.Style := psDot;
      Pen.Mode := pmXor;
      Pen.Color := XorColor;
      Brush.Style := bsClear;
      Rectangle(0, 0, ClientWidth, ClientHeight);
    end;
  //    ,  
  if FShowPackLever then
    with Canvas do begin
      R := GetLeverRect;
      Brush.Style := bsSolid;
      if FOnLever then Brush.Color := FPackLeverColor
      else Brush.Color := Self.Color;
      FillRect(R);
      //  ,    
      ImageIndex := 0;
      if (
          ((Align = alLeft) and IsPacked) or
          ((Align = alRight) and (not IsPacked))
         ) then ImageIndex := 1;
      if (
          ((Align = alTop) and IsPacked) or
          ((Align = alBottom) and (not IsPacked))
         ) then ImageIndex := 3;
      if (
          ((Align = alTop) and (not IsPacked)) or
          ((Align = alBottom) and IsPacked)
         ) then ImageIndex := 2;
      fBmp := TBitmap.Create;
      fBmp.Height := PackLeverMinSize;
      fBmp.Width := PackLeverMinSize;
      Arrows.GetBitmap(ImageIndex, fBmp);
      if Align in [alLeft, alRight, alNone, alClient] then begin
        BmpR := Rect(R.Left, R.Top + 4, R.Right,
                     R.Top + 4 + PackLeverMinSize);
        BrushCopy(BmpR, fBmp, Rect(0, 0, PackLeverMinSize,
           PackLeverMinSize), clWhite);
        OffsetRect(BmpR, 0, 11);
        BrushCopy(BmpR, fBmp, Rect(0, 0, PackLeverMinSize,
           PackLeverMinSize), clWhite);
      end
      else begin
        BmpR := Rect(R.Left + 4, R.Top,
                     R.Left + 4 + PackLeverMinSize, R.Bottom);
        BrushCopy(BmpR, fBmp, Rect(0, 0, PackLeverMinSize,
           PackLeverMinSize), clWhite);
        OffsetRect(BmpR, 11, 0);
        BrushCopy(BmpR, fBmp, Rect(0, 0, PackLeverMinSize,
           PackLeverMinSize), clWhite);
      end;
      fBmp.Free;
      Pen.Color := clBtnHighlight;
      MoveTo(R.Left, R.Bottom);
      LineTo(R.Left, R.Top);
      LineTo(R.Right, R.Top);
      Pen.Color := clBtnShadow;
      LineTo(R.Right, R.Bottom);
      LineTo(R.Left, R.Bottom);
    end;
  if Assigned(FOnPaint) then FOnPaint(Self);
end;

function TsohoSplitter.DoCanResize(var NewSize: Integer): Boolean;
begin
  Result := CanResize(NewSize);
end;

function TsohoSplitter.CanResize(var NewSize: Integer): Boolean;
begin
  Result := not FPacked;
  if Assigned(FOnCanResize) then FOnCanResize(Self, NewSize, Result);
end;

procedure TsohoSplitter.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  I: Integer;
begin
  FBeforePackActive := PtInRect(GetLeverRect, Point(X,Y));
  inherited MouseDown(Button, Shift, X, Y);
  if Button = mbLeft then begin
    FControl := FindControl;
    FDownPos := Point(X, Y);
    if Assigned(FControl) then begin
      if Align in [alLeft, alRight] then begin
        FMaxSize := Parent.ClientWidth - FMinSize;
        for I := 0 to pred(Parent.ControlCount) do
          with Parent.Controls[I] do
            if Align in [alLeft, alRight] then Dec(FMaxSize, Width);
        Inc(FMaxSize, FControl.Width);
      end
      else begin
        FMaxSize := Parent.ClientHeight - FMinSize;
        for I := 0 to pred(Parent.ControlCount) do
          with Parent.Controls[I] do
            if Align in [alTop, alBottom] then Dec(FMaxSize, Height);
        Inc(FMaxSize, FControl.Height);
      end;
      UpdateSize(X, Y);
      AllocateLineDC;
      with ValidParentForm(Self) do
        if ActiveControl <> nil then begin
          FActiveControl := ActiveControl;
          {FOldKeyDown := TWinControlAccess(FActiveControl).OnKeyDown;
          TWinControlAccess(FActiveControl).OnKeyDown := FocusKeyDown;}
        end;
      if (ResizeStyle in [rsLine, rsPattern]) and
         (not FPacked) then DrawLine;
    end;
  end;
end;

procedure TsohoSplitter.UpdateControlSize;
begin
  if FNewSize <> FOldSize then begin
    case Align of
      alLeft: FControl.Width := FNewSize;
      alTop: FControl.Height := FNewSize;
      alRight:
        begin
          Parent.DisableAlign;
          try
            FControl.Left := FControl.Left + (FControl.Width - FNewSize);
            FControl.Width := FNewSize;
          finally
            Parent.EnableAlign;
          end;
        end;
      alBottom:
        begin
          Parent.DisableAlign;
          try
            FControl.Top := FControl.Top + (FControl.Height - FNewSize);
            FControl.Height := FNewSize;
          finally
            Parent.EnableAlign;
          end;
        end;
    end;
    Update;
    if Assigned(FOnMoved) then FOnMoved(Self);
    FOldSize := FNewSize;
  end;
end;

procedure TsohoSplitter.CalcSplitSize(X, Y: Integer; var NewSize, Split: Integer);
var
  S: Integer;
begin
  if Align in [alLeft, alRight] then Split := X - FDownPos.X
  else Split := Y - FDownPos.Y;
  S := 0;
  case Align of
    alLeft: S := FControl.Width + Split;
    alRight: S := FControl.Width - Split;
    alTop: S := FControl.Height + Split;
    alBottom: S := FControl.Height - Split;
  end;
  NewSize := S;
  if S < FMinSize then NewSize := FMinSize
  else if S > FMaxSize then NewSize := FMaxSize;
  if S <> NewSize then begin
    if Align in [alRight, alBottom] then
      S := S - NewSize else
      S := NewSize - S;
    Inc(Split, S);
  end;
end;

procedure TsohoSplitter.UpdateSize(X, Y: Integer);
begin
  CalcSplitSize(X, Y, FNewSize, FSplit);
end;

procedure TsohoSplitter.CMMouseEnter (var Message: TMessage);
begin
  inherited;
  FMouseUnder := true;
  if FFlat then Invalidate;
end;

procedure TsohoSplitter.CMMouseLeave (var Message: TMessage);
var NeedRepaint: boolean;
begin
  inherited;
  NeedRepaint := FFlat or FOnLever;
  FMouseUnder := false;
  FOnLever := false;
  if NeedRepaint then Invalidate;
end;

procedure TsohoSplitter.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  NewSize, Split: Integer;
begin
  inherited;
  if PtInRect(GetLeverRect, Point(X,Y)) then begin
    if not FOnLever then begin
      FOnLever := true;
      Invalidate;
    end;
  end
  else
    if FOnLever then begin
      FOnLever := false;
      Invalidate;
    end;
  if (ssLeft in Shift) and Assigned(FControl) then
  begin
    CalcSplitSize(X, Y, NewSize, Split);
    if DoCanResize(NewSize) then begin
      if ResizeStyle in [rsLine, rsPattern] then DrawLine;
      FNewSize := NewSize;
      FSplit := Split;
      if ResizeStyle = rsUpdate then UpdateControlSize;
      if ResizeStyle in [rsLine, rsPattern] then DrawLine;
    end;
  end;
end;

procedure TsohoSplitter.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;
  if Assigned(FControl) and
     (not FAfterPacked) and
     (not FPacked) then begin
    if ResizeStyle in [rsLine, rsPattern] then DrawLine;
    UpdateControlSize;
    StopSizing;
  end;
  FAfterPacked := false;
end;

{procedure TsohoSplitter.FocusKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then StopSizing
  else if Assigned(FOldKeyDown) then
    FOldKeyDown(Sender, Key, Shift);
end;}

procedure TsohoSplitter.SetBevel(Value: Boolean);
begin
  FBevel := Value;
  Repaint;
end;

procedure TsohoSplitter.StopSizing;
begin
  if Assigned(FControl) then begin
    if FLineVisible then DrawLine;
    FControl := nil;
    ReleaseLineDC;
    if Assigned(FActiveControl) then begin
      TWinControlAccess(FActiveControl).OnKeyDown := FOldKeyDown;
      FActiveControl := nil;
    end;
  end;
  if Assigned(FOnMoved) then FOnMoved(Self);
end;

procedure TsohoSplitter.Click;
begin
  inherited;
  if (FPackMode = pmClick) and
     (FBeforePackActive) then begin
     FAfterPacked := true;
     IsPacked := not IsPacked;
  end;
end;

procedure TsohoSplitter.DblClick;
begin
  inherited;
  if (FPackMode = pmDblClick) and
     (FBeforePackActive) then begin
     FAfterPacked := true;
     IsPacked := not IsPacked;
  end;
end;

function TsohoSplitter.GetLeverRect: TRect;
begin
  if not FShowPackLever then begin
    Result.Left := -1;
    Result.Top := -1;
    Result.Right := -1;
    Result.Bottom := -1;
    exit;
  end;
  Result := ClientRect;
  with Result do begin
    if Align in [alTop, alBottom] then begin
      Left := Left + (Width - PackLeverMaxSize) div 2;
      Right := Left + PackLeverMaxSize;
      Top := Top + 1;
      Bottom := Bottom - 1;
    end
    else begin
      //  alNone,  ,    alLeft
      Top := Top + (Height - PackLeverMaxSize) div 2;
      Bottom := Top + PackLeverMaxSize;
      Left := Left + 1;
      Right := Right - 1;
    end
  end;
end;

procedure TsohoSplitter.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  if FShowPackLever then begin
    if Align in [alLeft, alRight] then begin
      if aWidth < PackLeverMinSize then aWidth := PackLeverMinSize + 3;
    end;
    if Align in [alTop, alBottom] then begin
      if aHeight < PackLeverMinSize then aHeight := PackLeverMinSize + 2;
    end;
  end;
  inherited;
end;

procedure TsohoSplitter.SetPacked(Value: boolean);
var aCntrl: TControl;
begin
  if FPacked = Value then exit;
  aCntrl := FindControl;
  if not Assigned(aCntrl) then exit;
  FPacked := Value;
  if FPacked then begin
    if Align in [alTop, alBottom] then begin
      FUnPackWidth := aCntrl.Height;
      aCntrl.Height := FPackWidth;
    end
    else begin
      FUnPackWidth := aCntrl.Width;
      aCntrl.Width := FPackWidth;
    end;
  end
  else begin
    if Align in [alTop, alBottom] then aCntrl.Height := FUnPackWidth
    else aCntrl.Width := FUnPackWidth;
  end;
  // FAfterPacked := true;
  Invalidate;
end;

procedure TsohoSplitter.SetPackLeverColor(Value: TColor);
begin
  if FPackLeverColor = Value then exit;
  FPackLeverColor := Value;
  Invalidate;
end;

procedure TsohoSplitter.SetPackWidth(Value: Integer);
var aCntrl: TControl;
begin
  if (FPackWidth = Value) or (PackWidth < 1) then exit;
  FPackWidth := Value;
  if FPacked then begin
    aCntrl := FindControl;
    if Assigned(aCntrl) then begin
      if Align in [alTop, alBottom] then aCntrl.Width := FPackWidth
      else aCntrl.Height := FPackWidth;
    end;
  end;
end;

procedure TsohoSplitter.SetShowPackLever(Value: boolean);
begin
  if FShowPackLever = Value then exit;
  FShowPackLever := Value;
  Invalidate;
end;

initialization
  Arrows := TImageList.Create(nil);
  Arrows.Height := 5;
  Arrows.Width := 5;
  Arrows.ResourceLoad(rtBitmap, 'SOHOPACKLEVEL', clAqua);
finalization
  Arrows.Free;
end.