{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoStatusBar;

interface

{$I SOHOLIB.INC}

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
     ComCtrls;

type

  TsohoStatusBar = class(TStatusBar)
  {      TStatusBar   
      ,     
  }
  private
    { Private declarations }
    FControlSaves: TList;
    FPanelControls: TList;
    FTmp: TStringList;
    FHooked: boolean;
    {$IFDEF Delphi3}
    FAutoHint: Boolean;
    {$ENDIF}
    FOnOldHint: TNotifyEvent;
  protected
    { Protected declarations }
    procedure DrawPanel(Panel: TStatusPanel; const Rect: TRect); override;
    function GetControl (Index: LongInt) : TControl;
    procedure ClearControlSaves;
    procedure AppHint (Sender: TObject); virtual;

    procedure WriteControls (Writer: TWriter);
    procedure ReadControls (Reader: TReader);
    procedure DefineProperties(Filer: TFiler); override;

    procedure DestroyControl(aControl: TControl);
  public
    procedure Notification(aComponent: TComponent; aOperation: TOperation);override;
    procedure Loaded; override;
    constructor Create(aOwner: TComponent);override;
    destructor Destroy; override;

    function GetControlPanelIndex(aControl: TControl): LongInt;
    {        }
    procedure SetControlPanelIndex(aControl: TControl; aPageIndex: LongInt);
    {       }
    procedure AddPanelControl(aControl: TControl; aPanelIndex: LongInt);
    {      }
    procedure RemovePanelControl(aControl: TControl);
    {   (   )  . !  
      ,     . 
      Parent     StatusBar.Parent}
    function PanelControlsCount: LongInt;
    {   -   StatusBar }
    procedure DeletePanelControl(Index: LongInt);
    {   (  )  . !  
      ,     . 
      Parent     StatusBar.Parent}
    procedure ClearStatusPanel(PanelIndex: LongInt);
    {     . !   
          .    
      Parent   StatusBar.Parent }
    function GetControlOnPanel(PanelIndex, ControlIndex: LongInt): TControl;
    {     ControlIndex    StatusBar }
    function GetControlsCountOnPanel(PanelIndex: LongInt): LongInt;
    {  -     StatusBar }
    function IsControlOnPanel(aControl: TControl): boolean;
    {   aControl  StatusBar? }
    property PanelControls[Index: LongInt]: TControl read GetControl;
    {      }
  published
    { Published declarations }
    {$IFDEF Delphi3}
    property AutoHint: boolean read FAutoHint write FAutoHint default true;
    {$ELSE}
    property AutoHint default true;
    {$ENDIF}
  end;

implementation
uses SoVCLUtils;

type

  PControlSave = ^TControlSave;
  TControlSave = record
    Visible: boolean;
    Parent: TWinControl;
    BoundsRect: TRect;
    PanelIndex: LongInt;
  end;

function TsohoStatusBar.IsControlOnPanel(aControl: TControl): boolean;
begin
  Result := FPanelControls.IndexOf(aControl) <> -1;
end;

procedure TsohoStatusBar.DefineProperties(Filer: TFiler);
begin
  inherited;
  Filer.DefineProperty('ZPanelControls', ReadControls, WriteControls,
    true);
end;

procedure TsohoStatusBar.WriteControls(Writer: TWriter);
var Index: LongInt;
begin
  Writer.WriteListBegin;
  for Index := 0 to pred(PanelControlsCount) do
    Writer.WriteString(PanelControls[Index].Name + ',' +
       IntToStr(GetControlPanelIndex(PanelControls[Index])));
  Writer.WriteListEnd;
end;

procedure TsohoStatusBar.ReadControls(Reader: TReader);
var Tmp, ControlName: string;
    PanelIndex, Index, CommaPos: LongInt;
    aControl : TControl;
    Form: TForm;
begin
  Form := GetOwnerForm(Self);
  FTmp.Clear;
  Reader.ReadListBegin;
  try
    for Index := 0 to pred(FPanelControls.Count) do
      RemoveControl(TControl(FPanelControls[0]));
    while not Reader.EndOfList do begin
      Tmp := Reader.ReadString;
      FTmp.Add(Tmp);
      CommaPos := Pos(',', Tmp);
      if CommaPos <> 0 then begin
        ControlName := copy(Tmp, 1, CommaPos - 1);
        PanelIndex := StrToIntDef(copy(Tmp, CommaPos + 1, length(Tmp)), 0);
      end
      else begin
        ControlName := Tmp;
        PanelIndex := 0;
      end;
      if Assigned(Form) then aControl := TControl(Form.FindComponent(ControlName))
      else aControl := nil;
      if Assigned(aControl) then AddPanelControl(aControl, PanelIndex);
    end;
  except
    // May be I'll want to do something here
    raise;
  end;
  Reader.ReadListEnd;
end;

procedure TsohoStatusBar.AppHint (Sender: TObject);
begin
  if AutoHint then begin
    if SimplePanel or (Panels.Count = 0) then SimpleText := Application.Hint
    else Panels[0].Text := Application.Hint;
  end;
  if Assigned(FOnOldHint) then FOnOldHint(Sender);
end;

procedure TsohoStatusBar.Loaded;
var Tmp, ControlName: string;
    PanelIndex, Index, CommaPos: LongInt;
    aControl : TControl;
    Form: TForm;
begin
  inherited;
  FHooked := false;
  if AutoHint and (not (csDesigning in ComponentState)) then begin
    FOnOldHint := Application.OnHint;
    Application.OnHint := AppHint;
    FHooked := true;
  end;
  Form := GetOwnerForm(Self);
  if FTmp.Count > 0 then
   for Index := 0 to pred(FTmp.Count) do begin
      Tmp := FTmp[Index];
      CommaPos := Pos(',', Tmp);
      if CommaPos <> 0 then begin
        ControlName := copy(Tmp, 1, CommaPos - 1);
        PanelIndex := StrToIntDef(copy(Tmp, CommaPos + 1, length(Tmp)), 0);
      end
      else begin
        ControlName := Tmp;
        PanelIndex := 0;
      end;
      if Assigned(Form) then aControl := TControl(Form.FindComponent(ControlName))
      else aControl := nil;
      if Assigned(aControl) then AddPanelControl(aControl, PanelIndex);
   end;
  FTmp.Clear; 
end;

function TsohoStatusBar.GetControlPanelIndex(aControl: TControl): LongInt;
var Index: LongInt;
begin
  Result := -1;
  Index := FPanelControls.IndexOf(aControl);
  if Index = -1 then exit;
  Result := PControlSave(FControlSaves[Index])^.PanelIndex;
end;

procedure TsohoStatusBar.SetControlPanelIndex(aControl: TControl; aPageIndex: LongInt);
var Index: LongInt;
begin
  Index := FPanelControls.IndexOf(aControl);
  if Index = -1 then exit;
  PControlSave(FControlSaves[Index])^.PanelIndex := aPageIndex;
end;

function TsohoStatusBar.GetControl (Index: LongInt) : TControl;
begin
  Result := TControl(FPanelControls[Index]);
end;

procedure TsohoStatusBar.DrawPanel(Panel: TStatusPanel; const Rect: TRect);
var Index: LongInt;
    aControl: TControl;
    aControlWidth: LongInt;
    aControlCount: LongInt;
begin
  aControlCount := GetControlsCountOnPanel(Panel.Index);
  if aControlCount = 0 then exit;
  if (Panel.Index = Panels.Count-1) and SizeGrip then
    aControlWidth := trunc((Rect.Right - Rect.Left - 13) / aControlCount)
  else
    aControlWidth := trunc((Rect.Right - Rect.Left) / aControlCount);
  for Index := 0 to pred(aControlCount) do begin
    aControl := GetControlOnPanel(Panel.Index, Index);
    if Assigned(aControl) then begin
      aControl.Top := Rect.Top;
      aControl.Height := Rect.Bottom - Rect.Top;
      aControl.Left := Rect.Left + Index * aControlWidth;
      aControl.Width := aControlWidth;
   end;
  end;
end;

procedure TsohoStatusBar.Notification(aComponent: TComponent; aOperation: TOperation);
begin
  inherited;
  if (aOperation = opRemove) and (FPanelControls.IndexOf(aComponent) <> -1) then
    DestroyControl(TControl(aComponent));
end;

function TsohoStatusBar.GetControlOnPanel(PanelIndex, ControlIndex: LongInt): TControl;
var Index, FoundIndex: LongInt;
    LastFound: TControl;
begin
  Result := nil;
  if ControlIndex > GetControlsCountOnPanel(PanelIndex) - 1  then exit;
  FoundIndex := -1;
  LastFound := nil;
  for Index := 0 to pred(FPanelControls.Count) do begin
    if PControlSave(FControlSaves[Index])^.PanelIndex = PanelIndex then begin
      inc(FoundIndex);
      LastFound := TControl(FPanelControls[Index]);
    end;
    if FoundIndex = ControlIndex then begin
      Result := LastFound;
      exit;
    end;
  end;
  if FoundIndex = ControlIndex then Result := LastFound;
end;

function TsohoStatusBar.GetControlsCountOnPanel(PanelIndex: LongInt): LongInt;
var Index: LongInt;
begin
  Result := 0;
  for Index := 0 to pred(FPanelControls.Count) do begin
    if PControlSave(FControlSaves[Index])^.PanelIndex = PanelIndex then
      inc(Result);
  end;
end;

constructor TsohoStatusBar.Create(aOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle + [csAcceptsControls]; 
  FPanelControls := TList.Create;
  FControlSaves := TList.Create;
  FTmp := TStringList.Create;
  FHooked := false;
  {$IFDEF Delphi3}
  FAutoHint := true;
  {$ELSE}
  AutoHint := true;
  {$ENDIF}
end;

procedure TsohoStatusBar.ClearControlSaves;
var Index: LongInt;
begin
  for Index := 0 to pred(FControlSaves.Count) do
    Dispose(PControlSave(FControlSaves[Index]));
  FControlSaves.Clear;
end;

destructor TsohoStatusBar.Destroy;
var Index: LongInt;
begin
  if FHooked and (not (csDesigning in ComponentState)) then
    Application.OnHint := FOnOldHint;
  FTmp.Free;
  for Index := 0 to pred(PanelControlsCount) do DeletePanelControl(0);
  FPanelControls.Free;
  ClearControlSaves;
  FControlSaves.Free;
  inherited;
end;

procedure TsohoStatusBar.AddPanelControl(aControl: TControl; aPanelIndex: LongInt);
var Tmp: PControlSave;
begin
  if not (csReading in ComponentState) then begin
    if (Panels.Count - 1 < aPanelIndex) or
       (Self = aControl) then exit;
  end;
  if FPanelControls.IndexOf(aControl) <> -1 then exit;
  New(Tmp);
  with Tmp^ do begin
    Visible := aControl.Visible;
    Parent := aControl.Parent;
    BoundsRect := aControl.BoundsRect;
    PanelIndex := aPanelIndex;
  end;
  if aPanelIndex <= Panels.Count - 1 then
    Panels[aPanelIndex].Style := psOwnerDraw;
  aControl.Parent := Self;
  aControl.Visible := true;
  FControlSaves.Add(Tmp);
  FPanelControls.Add(aControl);
end;

procedure TsohoStatusBar.RemovePanelControl(aControl: TControl);
var FoundIndex: LongInt;
begin
  FoundIndex := FPanelControls.IndexOf(aControl);
  if FoundIndex <> -1 then DeletePanelControl(FoundIndex);
end;

function TsohoStatusBar.PanelControlsCount: LongInt;
begin
  Result := FPanelControls.Count;
end;

procedure TsohoStatusBar.DestroyControl(aControl: TControl);
var FoundIndex: LongInt;
begin
  FoundIndex := FPanelControls.IndexOf(aControl);
  if FoundIndex <> -1 then begin
    FPanelControls.Delete(FoundIndex);
    Dispose(PControlSave(FControlSaves[FoundIndex]));
    FControlSaves.Delete(FoundIndex);
  end;
end;

procedure TsohoStatusBar.DeletePanelControl(Index: LongInt);
var aControl: TControl;
begin
  aControl := TControl(FPanelControls[Index]);
  FPanelControls.Delete(Index);
  aControl.Parent := PControlSave(FControlSaves[Index])^.Parent;
  aControl.BoundsRect := PControlSave(FControlSaves[Index])^.BoundsRect;
  aControl.Visible := PControlSave(FControlSaves[Index])^.Visible;
  Dispose(PControlSave(FControlSaves[Index]));
  FControlSaves.Delete(Index);
end;

procedure TsohoStatusBar.ClearStatusPanel(PanelIndex: LongInt);
var aControl: TControl;
begin
  while GetControlsCountOnPanel(PanelIndex) > 0 do begin
    aControl := GetControlOnPanel(PanelIndex, 0);
    RemoveControl(aControl);
  end;
end;

end.
