{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoStsCtrlEditor;

interface

{$I SOHOLIB.INC}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, SohoBtns, SoCtrls, SoStatusBar, SoClass;

type

  TsohoStatusControls = class
  public
    FPanelControls: TList;
    procedure AddPanelControl(aControl: TControl; aPanelIndex: LongInt);
    procedure RemovePanelControl(aControl: TControl);
    function GetControlPanelIndex(aControl: TControl): LongInt;
    procedure SetControlPanelIndex(aControl: TControl; aPageIndex: LongInt);
    function IndexOfControl(aControl: TControl): LongInt;
    constructor Create;
    destructor Destroy;override;
  end;

  TsohoStatusControlsEditorForm = class(TForm)
    StatusBarL: TsohoLabel;
    OnFormL: TsohoLabel;
    OnStatusL: TsohoLabel;
    FormControlsL: TListBox;
    StatusControlL: TListBox;
    PageIndexL: TsohoLabel;
    PanelIndexC: TsohoComboBox;
    OkB: TsohoBitBtn;
    CancelB: TsohoBitBtn;
    procedure StatusControlLClick(Sender: TObject);
    procedure PanelIndexCChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormControlsLDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure StatusControlLDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure FormControlsLDragDrop(Sender, Source: TObject; X,
      Y: Integer);
    procedure StatusControlLDragDrop(Sender, Source: TObject; X,
      Y: Integer);
  private
    { Private declarations }
    FUpdate: boolean;
    Status: TsohoStatusBar;
    Tmp: TsohoStatusControls;
    function Execute(Value: TsohoStatusBar) : boolean;
    procedure CopyToStatusControls;
    procedure CopyFromStatusControls;
  public
    { Public declarations }
  end;

var
  sohoStatusControlsEditorForm: TsohoStatusControlsEditorForm;

function EditStatusControls(Value: TsohoStatusBar) : boolean;

implementation
uses SoVCLUtils, SoCmnCns, SoDesCns;

{$R *.DFM}

{ TsohoControlRecord }

type
   PControlSave = ^TControlSave;
   TControlSave = record
     Control: TControl;
     PanelIndex: LongInt;
   end;

constructor TsohoStatusControls.Create;
begin
  // inherited;
  FPanelControls:= TList.Create;
end;

destructor TsohoStatusControls.Destroy;
begin
  FPanelControls.Free;
  inherited Destroy;
end;

procedure TsohoStatusControls.AddPanelControl(aControl: TControl; aPanelIndex: LongInt);
var Tmp: PControlSave;
begin
  if IndexOfControl(aControl) <> -1 then exit;
  New(Tmp);
  with Tmp^ do begin
    Control := aControl;
    PanelIndex := aPanelIndex;
  end;
  FPanelControls.Add(Tmp);
end;

procedure TsohoStatusControls.RemovePanelControl(aControl: TControl);
var FoundIndex: LongInt;
begin
  FoundIndex := IndexOfControl(aControl);
  if FoundIndex <> -1 then begin
    Dispose(FPanelControls[FoundIndex]);
    FPanelControls.Delete(FoundIndex);
  end;
end;

function TsohoStatusControls.IndexOfControl(aControl: TControl): LongInt;
var Index: LongInt;
begin
  for Index := 0 to pred(FPanelControls.Count) do
    if PControlSave(FPanelControls[Index])^.Control = aControl then begin
      Result := Index;
      exit;
    end;
  Result := -1;
end;

function TsohoStatusControls.GetControlPanelIndex(aControl: TControl): LongInt;
var Index: LongInt;
begin
  Result := -1;
  Index := IndexOfControl(aControl);
  if Index = -1 then exit;
  Result := PControlSave(FPanelControls[Index])^.PanelIndex;
end;

procedure TsohoStatusControls.SetControlPanelIndex(aControl: TControl; aPageIndex: LongInt);
var Index: LongInt;
begin
  Index := IndexOfControl(aControl);
  if Index = -1 then exit;
  PControlSave(FPanelControls[Index])^.PanelIndex := aPageIndex;
end;

function EditStatusControls(Value: TsohoStatusBar) : boolean;
begin
  try
    sohoStatusControlsEditorForm := TsohoStatusControlsEditorForm.Create(Application);
    Result := sohoStatusControlsEditorForm.Execute(Value);
  finally
    sohoStatusControlsEditorForm.Free;
  end;
end;

procedure TsohoStatusControlsEditorForm.CopyToStatusControls;
var Index: LongInt;
begin
  for Index := 0 to pred(Status.PanelControlsCount) do
    Tmp.AddPanelControl(
      Status.PanelControls[Index],
      Status.GetControlPanelIndex(Status.PanelControls[Index]));
end;

procedure TsohoStatusControlsEditorForm.CopyFromStatusControls;
var Index: LongInt;
begin
  for Index := 0 to pred(Status.PanelControlsCount) do
    Status.RemovePanelControl(Status.PanelControls[0]);
  for Index := 0 to pred(Tmp.FPanelControls.Count) do
    Status.AddPanelControl(
      PControlSave(Tmp.FPanelControls[Index])^.Control,
      PControlSave(Tmp.FPanelControls[Index])^.PanelIndex);
end;

function TsohoStatusControlsEditorForm.Execute(Value: TsohoStatusBar) : boolean;
var Form: TForm;
    Index: LongInt;
begin
  Result := false;
  if not Assigned(Value) then exit;
  Form := GetOwnerForm(Value);
  if not Assigned(Form) then exit;

  Status := Value;
  CopyToStatusControls;
  StatusBarL.Caption := StatusBarL.Caption + ' ' + Value.Name;
  for Index := 0 to pred(Form.ComponentCount) do
    if (Form.Components[Index] is TControl) and
       (Form.Components[Index].Name <> '') and
       (Form.Components[Index] <> Value) and
       not (Status.IsControlOnPanel(TControl(Form.Components[Index]))) then
      FormControlsL.Items.AddObject(Form.Components[Index].Name,
        Form.Components[Index]);
  for Index := 0 to pred(Value.PanelControlsCount) do
      StatusControlL.Items.AddObject(Value.PanelControls[Index].Name,
        Value.PanelControls[Index]);
  for Index := 0 to pred(Value.Panels.Count) do
    PanelIndexC.Items.Add(IntToStr(Index));
  Result := ShowModal = mrOk;
  if Result then CopyFromStatusControls;
end;

procedure TsohoStatusControlsEditorForm.StatusControlLClick(
  Sender: TObject);
var PanelIndex: LongInt;
begin
  with StatusControlL do begin
    PanelIndex := Tmp.GetControlPanelIndex(TControl(Items.Objects[ItemIndex]));
    FUpdate := true;
    PanelIndexC.ItemIndex := PanelIndex;
    FUpdate := false;
  end;
end;

procedure TsohoStatusControlsEditorForm.PanelIndexCChange(Sender: TObject);
begin
  if FUpdate then exit;
  if (PanelIndexC.ItemIndex = -1) or
     (StatusControlL.ItemIndex = -1) then exit;
  Tmp.SetControlPanelIndex(
     TControl(StatusControlL.Items.Objects[StatusControlL.ItemIndex]),
     PanelIndexC.ItemIndex);
end;

procedure TsohoStatusControlsEditorForm.FormCreate(Sender: TObject);
begin
  Tmp := TsohoStatusControls.Create;

  Caption := sohoStatusEditorTitle;
  OnFormL.Caption := sohoStatusEditorOnForm;
  OnStatusL.Caption := sohoStatusEditorOnStatus;
  PageIndexL.Caption := sohoStatusEditorPageIndex;

  OkB.Caption := SOKButton;
  CancelB.Caption := SCancelButton;
end;

procedure TsohoStatusControlsEditorForm.FormDestroy(Sender: TObject);
begin
  Tmp.Free;
end;

procedure TsohoStatusControlsEditorForm.FormControlsLDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := Source = StatusControlL;
end;

procedure TsohoStatusControlsEditorForm.StatusControlLDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := Source = FormControlsL;
end;

procedure TsohoStatusControlsEditorForm.FormControlsLDragDrop(Sender,
  Source: TObject; X, Y: Integer);
var aControl: TControl;
begin
  with StatusControlL do begin
    if ItemIndex = -1 then exit;
    aControl := TControl(Items.Objects[ItemIndex]);
    FormControlsL.Items.AddObject(aControl.Name, aControl);
    Tmp.RemovePanelControl(aControl);
    Items.Delete(ItemIndex);
  end;
end;

procedure TsohoStatusControlsEditorForm.StatusControlLDragDrop(Sender,
  Source: TObject; X, Y: Integer);
var aControl: TControl;
begin
  with FormControlsL do begin
    if ItemIndex = -1 then exit;
    aControl := TControl(Items.Objects[ItemIndex]);
    Tmp.AddPanelControl(aControl, 0);
    StatusControlL.Items.AddObject(Items[ItemIndex], Items.Objects[ItemIndex]);
    Items.Delete(ItemIndex);
  end;
end;

end.
