{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoTVCont;

{$I SOHOLIB.INC}

interface
uses Classes, SoCtmTVCont, SohoRec, SoFolder, SohoTree,
     SoDBGridAddon;

type

  TsohoBDETreeViewForm = class(TsohoCustomTreeViewForm)
  public
    constructor Create(aOwner: TComponent); override;
  end;

  TsohoTVContainer = class(TsohoCustomTVContainer)
  private
    FDatabaseName: string;
  protected
    function CreateTreeViewForm: TsohoCustomTreeViewForm; override;
    procedure SetTreeViewProperties; override;
    procedure SetFolderProperties; override;
    procedure SetRecordProperties; override;
    procedure SetQueryProperties; override;
    procedure SetEvents; override;
  published
    property DatabaseName: string read FDatabaseName write FDatabaseName;
    {   :       ,  BDE-alias }
  end;

implementation
uses SoCtmRgs, Forms, Controls, DBTables, SoStrUtils;

{ TsohoIBTreeViewForm }

constructor TsohoBDETreeViewForm.Create(aOwner: TComponent);
begin
  inherited;
  FTreeView := TsohoTreeView.Create(Self);
  FRecordForm := TsohoRecordForm.Create(Self);
  FFolder := TsohoFolder.Create(Self);
  FGroupsDS := TQuery.Create(Self);
  FItemsDS := TQuery.Create(Self);
  FDBAddon := TsohoDBGridAddon.Create(Self);
end;

{ TsohoTVContainer }

function TsohoTVContainer.CreateTreeViewForm: TsohoCustomTreeViewForm;
begin
  Result := TsohoBDETreeViewForm.Create(Application);
  Result.TreeView.Parent := Result;
  Result.TreeView.Align := alClient;
end;

procedure TsohoTVContainer.SetEvents;
begin
  if TreeViewForm = nil then exit;
  if (FDataBaseName = EmptyStr) and (SingleRegister <> nil) then
     FDataBaseName := SingleRegister.PathToData;
  inherited;
end;

procedure TsohoTVContainer.SetFolderProperties;
begin
  inherited;
  with TreeViewForm do begin
    TsohoFolder(Folder).DataBaseName := FDatabaseName;
    Folder.Auto := false;
  end
end;

procedure TsohoTVContainer.SetQueryProperties;
var FGrpsSQL, FItemsSQL : string;
begin
  inherited;
  with TreeViewForm do begin
    FGrpsSQL  := GroupsSQLFile;
    FItemsSQL := ItemsSQLFile;

    if SingleRegister <> nil then begin
      FGrpsSQL  := SingleRegister.PathToSQL + GroupsSQLFile;
      FItemsSQL := SingleRegister.PathToSQL + ItemsSQLFile;
    end;

    with TQuery(GroupsDS), SQL do begin
      DataBaseName := FDataBaseName;
      LoadFromFile(FGrpsSQL);
    end;
    with TQuery(ItemsDS), SQL do begin
      DataBaseName := FDataBaseName;
      LoadFromFile(FItemsSQL);
    end;
  end;
end;

procedure TsohoTVContainer.SetRecordProperties;
begin
  inherited;
  with TreeViewForm do begin
    TsohoRecordForm(RecordForm).DataBaseName := FDatabaseName;
    TsohoRecordForm(RecordForm).Folder := Folder;
  end;
end;

procedure TsohoTVContainer.SetTreeViewProperties;
begin
  inherited;
  with TreeViewForm, TsohoTreeView(TreeView) do begin
    DataBaseName := FDataBaseName;
    ItemsDataSet := ItemsDS;
    GroupsDataSet := GroupsDS;
  end;
  with TreeViewForm do
    TsohoTreeView(TreeView).Folder := Folder;
end;

end.
