{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{           Partial copyright (c) AllexSoft             }
{           Written by Alexander Govyadinov,            }
{                   Serg Vostrikov                      }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SoUnit;

{$I SOHOLIB.INC}

interface

uses SysUtils, Windows, Messages, Classes, Graphics, Controls,
     Menus, StdCtrls, Outline;

type

   TSQLFileName = type string;
   {  ,  ,       
     SQL- }

   SetOfByte = set of Byte;
   {  .    ApplyActionRights }

   TGetNewIdProc = function  (const Preffix : string): LongInt;
   TSetNewIdProc = procedure (const Preffix : string; Value : LongInt);

function GetNewID(const Preffix: string): Longint;
{     ,     }
procedure SetNewID(const Preffix: string; Value: Longint);
{     ,     }
function OverrideNewIdRoutine (NewGetId : TGetNewIdProc; NewSetId : TSetNewIdProc) : boolean;
{      
       }
function SohoNames(const FieldName: string): Boolean;
{ ,      SOHO
    TsohoFolder }
procedure ApplyActionRights(Component: TComponent;
  Rights: SetOfByte; HideSecret: boolean);
{       -    
      }
function SetToStr(S: SetOfByte): string;
{      }
function StrToSet(S: string): SetOfByte;
{    ,   0, 5, 8-15  }
procedure ApplyOutLineDescription(Outline: TOutLine; const IniFileName: string);
{     OutLine }
procedure SaveOutLineDescription(Outline: TOutLine; const IniFileName: string);
{     OutLine }

implementation
uses SoUtils, SoDialogs, SoDBCns, SoAppUtils, SoStrUtils, SoVCLUtils,
     IniFiles;

var
   FGetId : TGetNewIdProc;
   FSetId : TSetNewIdProc;

{  ,  Tag1  Tag2   (lParam)}
function EncodeData(C: Word; Tag1, Tag2: Byte): Longint;
   {Data Category Number}
begin
   Result := Longint(Tag1) shl 24 + Longint(Tag2) shl 16 + Word(C);
end;

{      }
procedure DecodeData(LPARAM: Longint; var C: Word; var Tag1, Tag2: Byte);
   {lParam Data Category Number}
begin
   Word(C) := LOWORD(LPARAM);
   Tag1 := Hi(HiWord(LPARAM));
   Tag2 := Lo(HiWord(LPARAM));
end;

{      }
procedure AppendCharToString(C: Char; N: Byte; var S: string);
begin
   if N = 0 then Exit;
   while Length(S) < N do S := S + Space;
   S[N] := C;
end;

function SohoNames(const FieldName: string): Boolean;
var TmpFieldName : string;
begin
   TmpFieldName := AnsiUpperCase(FieldName);
   Result := (TmpFieldName = 'ID')
      or (TmpFieldName = 'MDFDATE')
      or (TmpFieldName = 'MDFTIME')
      or (TmpFieldName = 'IDMDFAUTH')
      or (TmpFieldName = 'PROTECT') {Allex 10/04/1998}
      or (Copy(TmpFieldName, 1, 5) = 'IDGRP')
      or (Copy(TmpFieldName, 1, 4) = 'HASH')
      or (Copy(TmpFieldName, 1, 5) = 'ORDER'); {STOP}
end;

procedure ApplyActionRights(Component: TComponent; Rights: SetOfByte;
  HideSecret: boolean);
var I: Longint;
begin
   {
     -      ,
        - ,
     
      "-"
   }
   {  if not assigned(Component) then Exit;}
   with TComponent(Component) do
      for I := 0 to ComponentCount - 1 do begin
         if (Components[I] is TControl) then
            with (Components[I] as TControl) do begin
              if HideSecret then
                Visible := LOWORD(Tag) in Rights
              else
                Enabled := LOWORD(Tag) in Rights
            end;
         if (Components[I] is TMenuItem) then
            with (Components[I] as TMenuItem) do begin
              if HideSecret then
                Visible := LOWORD(Tag) in Rights
              else
                Enabled := LOWORD(Tag) in Rights
            end;
         ApplyActionRights(Components[I], Rights, HideSecret);
      end
end;

function StrToSet(S: string): SetOfByte;
const Digital = ['0'..'9', '-'];
var
   Word             : string; 
   LeftStr, RightStr: string; 
   Left, Right      : Longint;
   I                : Longint;
   Error            : Integer;
begin
   Result := [0];
   {Skip invalid chars}
   I := 1; while (I < Length(S)) and not (S[I] in Digital) do Inc(I);
   Delete(S, 1, I - 1);
   repeat
      {Find WORD}
      I := 1; while (I <= Length(S)) and (S[I] in Digital) do Inc(I);
      Word := Copy(S, 1, I - 1);
      Delete(S, 1, I - 1);
      {Skip invalid chars}
      I := 1; while (I <= Length(S)) and not (S[I] in Digital) do Inc(I);
      Delete(S, 1, I - 1);
      while Pos('--', Word) <> 0 do Delete(Word, Pos('--', Word), 1);
      I := Pos('-', Word); if I = 0 then Word := Word + '-';
      I := Pos('-', Word);
      LeftStr := Copy(Word, 1, I - 1); Val(LeftStr, Left, Error);
      RightStr := Copy(Word, I + 1, 255); Val(RightStr, Right, Error);
      if LeftStr = '' then Left := Right;
      if RightStr = '' then Right := Left;
      if Left > Right
         then Result := Result + [Right..Left]
      else Result := Result + [Left..Right];
   until S = '';
end;

function SetToStr(S: SetOfByte): string;
var Left, Right: Integer;
begin
   Result := ''; if S = [] then Exit;
   Left := 0;
   Right := 0;
   while Right < 256 do
   begin
      {Define start of next section}
      while (Left < 256)
         and not (Left in S)
         do Inc(Left);
      if Left > 255 then Break;
      {Define end of next section}
      Right := Left;
      while (Right < 256)
         and (Right in S)
         do Inc(Right);
      Dec(Right);
      {Append Left&Right to Result}
      Result := Result + IntToStr(Left);
      if Right <> Left then Result := Result + '-' + IntToStr(Right);
      Result := Result + ',';
      Left := Right + 1;
   end;
   if Length(Result) > 0 then DecLength(Result, 1);
end;

function StandartGetNewID (const Preffix : string): LongInt;
var IniFile: TIniFile;
begin
   IniFile := TIniFile.Create('SOHO.INI');
   Result := IniFile.ReadInteger('Common', Preffix + 'ID', 0);
   if Result = 0 then
   begin
      Result := IniFile.ReadInteger('Common', 'Computer', 0);
      if Result = 0 then
         Result := StrToInt(InputBox(sohoUnitComputerNumber, sohoUnitOnlyProvider, '0'));
      IniFile.WriteInteger('Common', 'Computer', Result);
      Result := Result * 100000; {CostYE000 QuantWH000}
   end;
   Inc(Result);
   IniFile.WriteInteger('Common', Preffix + 'ID', Result);
   IniFile.Free;
end;

procedure StandartSetNewID (const Preffix : string; Value : LongInt);
var IniFile: TIniFile;
begin
   IniFile := TIniFile.Create('SOHO.INI');
   IniFile.WriteInteger('Common', Preffix + 'ID', Value);
   IniFile.Free;
end;

function GetNewID;
begin
  if @FGetId = nil then Result := StandartGetNewId(Preffix)
  else Result := FGetId(Preffix);
end;

procedure SetNewID;
begin
  if @FSetId = nil then StandartSetNewId(Preffix, Value)
  else FSetId(Preffix, Value);
end;

function OverrideNewIdRoutine (NewGetId : TGetNewIdProc; NewSetId : TSetNewIdProc) : boolean;
begin
  try
   FGetId := NewGetId;
   FSetId := NewSetId;
   Result := True;
  except
    Result := False;
  end;
end;

procedure SaveOutLineDescription(Outline: TOutLine; const IniFileName: string);
var
   Ini: TIniFile;
   I  : Longint;
begin
   Ini := TIniFile.Create(IniFileName);
   with Ini, Outline do
      for I := 1 to ItemCount do
         WriteBool(Outline.Owner.ClassName + Outline.name + 'OutLine', IntToStr(I), Items[I].Expanded);
   Ini.WriteInteger(Outline.Owner.ClassName + Outline.name + 'OutLine',
      'Selected', Outline.SelectedItem);
   Ini.Free;
end;

procedure ApplyOutLineDescription(Outline: TOutLine; const IniFileName: string);
var
   Ini: TIniFile;
   I  : Longint;
begin
   Ini := TIniFile.Create(IniFileName);
   with Ini, Outline do begin
      for I := 1 to ItemCount do
        if Items[I].Parent.Expanded then
         Items[I].Expanded := ReadBool(Outline.Owner.ClassName + Outline.name + 'OutLine', IntToStr(I), True);
   end;
   I := Ini.ReadInteger(Outline.Owner.ClassName + Outline.name + 'OutLine',
      'Selected', 1);
   if Outline.ItemCount <= I then Outline.SelectedItem := I;
   Ini.Free;
end;

initialization
   FGetId := nil;
   FSetId := nil;
end.

