{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SohoApp;

{$I SOHOLIB.INC}

interface

uses SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms,
     SoTools{$IFDEF Delphi4_or_5}, ActnList {$ENDIF};

const

  DefHintColor = clInfoBk;
  DefHintPause = 500;
  DefHintShortPause = DefHintPause div 10;
  DefHintHidePause = DefHintPause * 5;

{ TsohoApplication }

type

  TsohoHintScrollKind = (skHorizontal, skVertical);
  {    :  ,  }

  TEndSessionNotify = procedure (Sender: TObject; LogOff: boolean;
                      var Allow: boolean) of object;
  {    OnQueryEndSession }

  TsohoApplication = class(TComponent)
  {    ""  TApplication, 
         TApplication 
    design-time.  , TsohoApplication   
      ,   , SearchPrev  FontsDir.
  }
  private
    { Private declarations }
    FHintColor: TColor;
    FHintPause: Integer;
    FScrollHints: boolean;
    FShowHint: Boolean;
    FCanvas: TCanvas;
    FUpdateFormatSettings: Boolean;
    FHintShortPause: Integer;
    FHintHidePause: Integer;
    FShowMainForm: Boolean;
{$IFDEF Delphi3}
    FUpdateMetricSettings: Boolean;
{$ENDIF}
{$IFDEF Delphi4_or_5}
    FHintShortCuts: Boolean;
    FBiDiMode: TBiDiMode;
    FMouseDragImmediate: Boolean;
    FMouseDragThreshold: Integer;
    FOnActionExecute: TActionEvent;
    FOnActionUpdate: TActionEvent;
    FOnShortCut: TShortCutEvent;
{$ENDIF}
    FOnPaintIcon: TNotifyEvent;
    FOnActivate: TNotifyEvent;
    FOnDeactivate: TNotifyEvent;
    FOnException: TExceptionEvent;
    FOnIdle: TIdleEvent;
    FOnHelp: THelpEvent;
    FOnHint: TNotifyEvent;
    FOnMessage: TMessageEvent;
    FOnMinimize: TNotifyEvent;
    FOnRestore: TNotifyEvent;
    FOnShowHint: TShowHintEvent;
    FOnSettingsChanged: TNotifyEvent;
    FOnActiveControlChange: TNotifyEvent;
    FOnActiveFormChange: TNotifyEvent;

    //     
    FOnOldActivate: TNotifyEvent;
    FOnOldDeactivate: TNotifyEvent;
    FOnOldException: TExceptionEvent;
    FOnOldIdle: TIdleEvent;
    FOnOldHelp: THelpEvent;
    FOnOldHint: TNotifyEvent;
    FOnOldMessage: TMessageEvent;
    FOnOldMinimize: TNotifyEvent;
    FOnOldRestore: TNotifyEvent;
    FOnOldShowHint: TShowHintEvent;
    FOnOldActiveControlChange: TNotifyEvent;
    FOnOldActiveFormChange: TNotifyEvent;
{$IFDEF Delphi4_or_5}
    FOnOldActionExecute: TActionEvent;
    FOnOldActionUpdate: TActionEvent;
    FOnOldShortCut: TShortCutEvent;
{$ENDIF}
    FOnQueryEndSession: TEndSessionNotify;

    FScrollSteps : LongInt;
    FScrollKind  : TsohoHintScrollKind;
    FHintAlign   : TAlignment;
    FOnMainCreate: TNotifyEvent;
    WasActived: boolean;
    FSearchPrev: boolean;
    FFonts: TStringList;
    FFontsDir: TTTFDirName;
    FApplicationID: string;
    FParamsList: TStringList;
    FSwitchList: TStringList;
    FSwitchParamsList: TStringList;
    FSwitchChars: string;
    FSwitchMatchCase: boolean;
    FBlockCtrlAltDel: boolean;

    procedure SetBlockCtrlAltDel (Value: boolean);
    function GetParamsList: TStrings;
    function GetSwitchList: TStrings;
    function GetSwitchParamsList: TStrings;
    function GetFonts: TStrings;
    function GetSwitchParam (const SwitchName: string): string;

    procedure SetFonts(Value: TStrings);
    procedure SetFontsDir(Value: TTTFDirName);
    procedure SetAppId(Value: string);

    procedure UpdateAppProps;
    function GetCanvas: TCanvas;
    function GetHintColor: TColor;
    function GetHintPause: Integer;
    function GetShowHint: Boolean;
    procedure SetHintColor(Value: TColor);
    procedure SetHintPause(Value: Integer);
    procedure SetShowHint(Value: Boolean);
    function GetUpdateFormatSettings: Boolean;
    procedure SetUpdateFormatSettings(Value: Boolean);
    function GetHintShortPause: Integer;
    function GetHintHidePause: Integer;
    function GetShowMainForm: Boolean;
    procedure SetHintShortPause(Value: Integer);
    procedure SetHintHidePause(Value: Integer);
    procedure SetShowMainForm(Value: Boolean);
{$IFDEF Delphi3}
    function GetUpdateMetricSettings: Boolean;
    procedure SetUpdateMetricSettings(Value: Boolean);
{$ENDIF}
{$IFDEF Delphi4_or_5}
    function GetHintShortCuts: Boolean;
    function GetBiDiMode: TBiDiMode;
    procedure SetHintShortCuts(Value: Boolean);
    procedure SetBiDiMode(Value: TBiDiMode);
    function GetMouseDragImmediate: Boolean;
    function GetMouseDragThreshold: Integer;
    procedure SetMouseDragImmediate(Value: Boolean);
    procedure SetMouseDragThreshold(Value: Integer);
{$ENDIF}
  protected
    function GetWinDir: string;
    procedure MainFormCreate(Sender: TObject);

    procedure Loaded; override;
    procedure PaintIcon; virtual;
    procedure SettingsChanged; dynamic;
    function MessageHook(var Msg: TMessage): Boolean; virtual;

    //   
    procedure DoActivate(Sender: TObject);
    procedure DoDeactivate(Sender: TObject);
    procedure DoException(Sender: TObject; E: Exception);
    procedure DoIdle(Sender: TObject; var Done: Boolean);
    function DoHelp(Command: Word; Data: Longint;
      var CallHelp: Boolean): Boolean;
    procedure DoHint(Sender: TObject);
    procedure DoMessage(var Msg: TMsg; var Handled: Boolean);
    procedure DoMinimize(Sender: TObject);
    procedure DoRestore(Sender: TObject);
    procedure DoShowHint(var HintStr: string; var CanShow: Boolean;
      var HintInfo: THintInfo);
    procedure DoActiveControlChange(Sender: TObject);
    procedure DoActiveFormChange(Sender: TObject);
{$IFDEF Delphi4_or_5}
    procedure DoActionExecute(Action: TBasicAction; var Handled: Boolean);
    procedure DoActionUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure DoShortCut(var Msg: TWMKey; var Handled: Boolean);
{$ENDIF}
    procedure AddEvents; virtual;
    procedure RemoveEvents; virtual;

    procedure GetCommandParams;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property IconCanvas: TCanvas read GetCanvas; { for painting the icon }

    property ParamsList: TStrings read GetParamsList;
    property SwitchList: TStrings read GetSwitchList;
    property SwitchParamsList: TStrings read GetSwitchParamsList;
    property SwitchParam [const SwitchName: string]: string read GetSwitchParam;

  published
    property BlockCtrlAltDel: boolean read FBlockCtrlAltDel write SetBlockCtrlAltDel default false;
    {   Ctrl-Alt-Del }
    property SwitchChars: string read FSwitchChars write FSwitchChars;
    {  ,     ""  
        .   : "-", "/"}
    property SwitchMatchCase: boolean read FSwitchMatchCase write FSwitchMatchCase default false;
    {      ""   }

    {  ""  }
    property ScrollHints: boolean read FScrollHints write FScrollHints default true;
    property HintSteps : LongInt read FScrollSteps write FScrollSteps default 100;
    property HintKind  : TsohoHintScrollKind read FScrollKind write FScrollKind default skHorizontal;
    property HintAlign   : TAlignment read FHintAlign write FHintAlign default taLeftJustify;
    property SearchPrev: boolean read FSearchPrev write FSearchPrev;
    {      }
    property Fonts: TStrings read GetFonts write SetFonts;
    {        }
    property FontsDir: TTTFDirName read FFontsDir write SetFontsDir;
    {   ,       }
    property ApplicationId: string read FApplicationId write SetAppId;
    {    ,     
           SearchPrev}
    property HintColor: TColor read GetHintColor write SetHintColor default DefHintColor;
    property HintPause: Integer read GetHintPause write SetHintPause default DefHintPause;
    property ShowHint: Boolean read GetShowHint write SetShowHint default True;
    property UpdateFormatSettings: Boolean read GetUpdateFormatSettings
      write SetUpdateFormatSettings default True;
    property HintShortPause: Integer read GetHintShortPause write SetHintShortPause
      default DefHintShortPause;
    property HintHidePause: Integer read GetHintHidePause write SetHintHidePause
      default DefHintHidePause;
    property ShowMainForm: Boolean read GetShowMainForm write SetShowMainForm
      default True;
{$IFDEF Delphi3}
    property UpdateMetricSettings: Boolean read GetUpdateMetricSettings
      write SetUpdateMetricSettings default True;
{$ENDIF}
{$IFDEF Delphi4_or_5}
    property HintShortCuts: Boolean read GetHintShortCuts write SetHintShortCuts
      default True;
    property BiDiMode: TBiDiMode read GetBiDiMode write SetBiDiMode
      default bdLeftToRight;
    property MouseDragImmediate: Boolean read GetMouseDragImmediate
      write SetMouseDragImmediate default True;
    property MouseDragThreshold: Integer read GetMouseDragThreshold
      write SetMouseDragThreshold default 5;
    property OnActionExecute: TActionEvent read FOnActionExecute write FOnActionExecute;
    property OnActionUpdate: TActionEvent read FOnActionUpdate write FOnActionUpdate;
    property OnShortCut: TShortCutEvent read FOnShortCut write FOnShortCut;
{$ENDIF}
    property OnActivate: TNotifyEvent read FOnActivate write FOnActivate;
    property OnDeactivate: TNotifyEvent read FOnDeactivate write FOnDeactivate;
    property OnException: TExceptionEvent read FOnException write FOnException;
    property OnIdle: TIdleEvent read FOnIdle write FOnIdle;
    property OnHelp: THelpEvent read FOnHelp write FOnHelp;
    property OnHint: TNotifyEvent read FOnHint write FOnHint;
    property OnMessage: TMessageEvent read FOnMessage write FOnMessage;
    property OnMinimize: TNotifyEvent read FOnMinimize write FOnMinimize;
    property OnPaintIcon: TNotifyEvent read FOnPaintIcon write FOnPaintIcon;
    property OnRestore: TNotifyEvent read FOnRestore write FOnRestore;
    property OnShowHint: TShowHintEvent read FOnShowHint write FOnShowHint;
    property OnSettingsChanged: TNotifyEvent read FOnSettingsChanged write FOnSettingsChanged;
    property OnActiveControlChange: TNotifyEvent read FOnActiveControlChange write FOnActiveControlChange;
    property OnActiveFormChange: TNotifyEvent read FOnActiveFormChange write FOnActiveFormChange;
    property OnQueryEndSession: TEndSessionNotify read FOnQueryEndSession write FOnQueryEndSession;
  end;

implementation
uses SoStrUtils, SoSingle, SoHints, SoVCLUtils, SoUtils, SoAppUtils;

procedure TsohoApplication.SetAppId(Value: string);
begin
  if Value = EmptyStr then exit;
  FApplicationID := Value;
end;

procedure TsohoApplication.SetFonts;
begin
  FFonts.Assign(Value);
end;

procedure TsohoApplication.MainFormCreate;
var Index : LongInt;
    FileName : string;
    pFont : array [0..255] of char;
begin
  if not WasActived then
    { ,       }
    if (Owner as TForm) <> nil then begin
      if (CmdShow = SW_SHOWMINNOACTIVE) or
        (CmdShow = SW_SHOWMINIMIZED) then (Owner as TForm).WindowState := wsMinimized
      else
        if CmdShow = SW_SHOWMAXIMIZED then (Owner as TForm).WindowState := wsMaximized;
      WasActived := True;
    end;
  if Assigned(FOnMainCreate) then FOnMainCreate(Owner as TForm);
  for Index := 0 to pred(Fonts.Count) do begin
    FileName := FontsDir + Fonts[Index];
    if FileExists(FileName) then begin
      StrPCopy(pFont, FileName);
      AddFontResource(pFont);
    end;
  end;
end;

procedure TsohoApplication.SetFontsDir;
begin
  {    '\'}
  FFontsDir := Value;
  if (Length(FFontsDir) <> 0) and (FFontsDir[Length(FFontsDir)] <> '\')
    then FFontsDir := FFontsDir + '\';
end;

function TsohoApplication.GetWinDir: string;
var Buf: PChar;
begin
  GetMem(Buf, 500);
  GetWindowsDirectory(Buf, 500);
  GetWinDir := StrPas(Buf);
  FreeMem(Buf, 500);
end;

procedure TsohoApplication.AddEvents;
begin
  if not (csDesigning in ComponentState) then begin
    FOnOldActivate := Application.OnActivate;
    FOnOldDeactivate := Application.OnDeactivate;
    FOnOldException := Application.OnException;
    FOnOldIdle := Application.OnIdle;
    FOnOldHelp := Application.OnHelp;
    FOnOldHint := Application.OnHint;
    FOnOldMessage := Application.OnMessage;
    FOnOldMinimize := Application.OnMinimize;
    FOnOldRestore := Application.OnRestore;
    FOnOldShowHint := Application.OnShowHint;
{$IFDEF Delphi4_or_5}
    FOnOldActionExecute := Application.OnActionExecute;
    FOnOldActionUpdate := Application.OnActionUpdate;
    FOnOldShortCut := Application.OnShortCut;

    Application.OnActionExecute := DoActionExecute;
    Application.OnActionUpdate := DoActionUpdate;
    Application.OnShortCut := DoShortCut;
{$ENDIF}
    if Assigned(FOnActivate) then
      Application.OnActivate := DoActivate;
    if Assigned(FOnDeActivate) then
      Application.OnDeactivate := DoDeactivate;
    if Assigned(FOnException) then
      Application.OnException := DoException;
    if Assigned(FOnIdle) then
      Application.OnIdle := DoIdle;
    if Assigned(FOnHelp) then
      Application.OnHelp := DoHelp;
      // Application.OnHint := DoHint;
    if Assigned(FOnMessage) then
      Application.OnMessage := DoMessage;
    if Assigned(FOnMinimize) then
      Application.OnMinimize := DoMinimize;
    if Assigned(FOnRestore) then
      Application.OnRestore := DoRestore;
    if Assigned(FOnShowHint) then
      Application.OnShowHint := DoShowHint;
    if Screen <> nil then begin
      FOnOldActiveControlChange := Screen.OnActiveControlChange;
      FOnOldActiveFormChange := Screen.OnActiveFormChange;
      Screen.OnActiveControlChange := DoActiveControlChange;
      Screen.OnActiveFormChange := DoActiveFormChange;
    end;
  end;
end;

procedure TsohoApplication.RemoveEvents;
begin
{$IFDEF Delphi4_or_5}
  Application.OnActionExecute := FOnOldActionExecute;
  Application.OnActionUpdate := FOnOldActionUpdate;
  Application.OnShortCut := FOnOldShortCut;
{$ENDIF}
  Application.OnActivate := FOnOldActivate;
  Application.OnDeactivate := FOnOldDeactivate;
  Application.OnException := FOnOldException;
  Application.OnIdle := FOnOldIdle;
  Application.OnHelp := FOnOldHelp;
  Application.OnHint := FOnOldHint;
  Application.OnMessage := FOnOldMessage;
  Application.OnMinimize := FOnOldMinimize;
  Application.OnRestore := FOnOldRestore;
  Application.OnShowHint := FOnOldShowHint;
  if Screen <> nil then begin
    Screen.OnActiveControlChange := FOnOldActiveControlChange;
    Screen.OnActiveFormChange := FOnOldActiveFormChange;
  end;
end;

procedure TsohoApplication.DoActivate(Sender: TObject);
begin
  if Assigned(FOnOldActivate) then FOnOldActivate(Sender);
  if Assigned(FOnActivate) then FOnActivate(Sender);
end;

procedure TsohoApplication.DoDeactivate(Sender: TObject);
begin
  if Assigned(FOnOldDeactivate) then FOnOldDeactivate(Sender);
  if Assigned(FOnDeactivate) then FOnDeactivate(Sender);
end;

procedure TsohoApplication.DoException(Sender: TObject; E: Exception);
begin
  if Assigned(FOnOldException) then FOnOldException(Sender, E);
  if Assigned(FOnException) then FOnException(Sender, E);
end;

procedure TsohoApplication.DoIdle(Sender: TObject; var Done: Boolean);
begin
  if Assigned(FOnOldIdle) then FOnOldIdle(Sender, Done);
  if Assigned(FOnIdle) then FOnIdle(Sender, Done);
end;

function TsohoApplication.DoHelp(Command: Word; Data: Longint;
  var CallHelp: Boolean): Boolean;
begin
  Result := true;
  if Assigned(FOnOldHelp) then Result := FOnOldHelp(Command, Data, CallHelp);
  if Assigned(FOnHelp) then Result := FOnHelp(Command, Data, CallHelp);
end;

procedure TsohoApplication.DoHint(Sender: TObject);
begin
  if Assigned(FOnOldHint) then FOnOldHint(Sender);
  if Assigned(FOnHint) then FOnHint(Sender);
end;

procedure TsohoApplication.DoMessage(var Msg: TMsg; var Handled: Boolean);
begin
  if Assigned(FOnOldMessage) then FOnOldMessage(Msg, Handled);
  if (not Handled) and Assigned(FOnMessage) then FOnMessage(Msg, Handled);
end;

procedure TsohoApplication.DoMinimize(Sender: TObject);
begin
  if Assigned(FOnOldMinimize) then FOnOldMinimize(Sender);
  if Assigned(FOnMinimize) then FOnMinimize(Sender);
end;

procedure TsohoApplication.DoRestore(Sender: TObject);
begin
  if Assigned(FOnOldRestore) then FOnOldRestore(Sender);
  if Assigned(FOnRestore) then FOnRestore(Sender);
end;

procedure TsohoApplication.DoShowHint(var HintStr: string; var CanShow: Boolean;
  var HintInfo: THintInfo);
begin
  if Assigned(FOnOldShowHint) then FOnOldShowHint(HintStr, CanShow, HintInfo);
  if Assigned(FOnShowHint) then FOnShowHint(HintStr, CanShow, HintInfo);
end;

procedure TsohoApplication.DoActiveControlChange(Sender: TObject);
begin
  if Assigned(FOnOldActiveControlChange) then FOnOldActiveControlChange(Sender);
  if Assigned(FOnActiveControlChange) then FOnActiveControlChange(Sender);
end;

procedure TsohoApplication.DoActiveFormChange(Sender: TObject);
begin
  if Assigned(FOnOldActiveFormChange) then FOnOldActiveFormChange(Sender);
  if Assigned(FOnActiveFormChange) then FOnActiveFormChange(Sender);
end;

{$IFDEF Delphi4_or_5}

procedure TsohoApplication.DoActionExecute(Action: TBasicAction;
  var Handled: Boolean);
begin
  if Assigned(FOnOldActionExecute) then FOnOldActionExecute(Action, Handled);
  if (not Handled) and Assigned(FOnActionExecute) then FOnActionExecute(Action, Handled);
end;

procedure TsohoApplication.DoActionUpdate(Action: TBasicAction;
  var Handled: Boolean);
begin
  if Assigned(FOnOldActionUpdate) then FOnOldActionUpdate(Action, Handled);
  if (not Handled) and Assigned(FOnActionUpdate) then FOnActionUpdate(Action, Handled);
end;

procedure TsohoApplication.DoShortCut(var Msg: TWMKey; var Handled: Boolean);
begin
  if Assigned(FOnOldShortCut) then FOnOldShortCut(Msg, Handled);
  if Assigned(FOnShortCut) then FOnShortCut(Msg, Handled);
end;
{$ENDIF Delphi4_or_5}

constructor TsohoApplication.Create(aOwner : TComponent);
var Error: boolean;
begin
  inherited Create(AOwner);
  FSearchPrev := True;
  FFonts := TStringList.Create;
  FFontsDir := EmptyStr;
  WasActived := False;
  FApplicationID := '{' + IntToHex(StrToInt(IntToStr(trunc(Date))             ), 6) +
                          IntToHex(WStrToInt(DeleteChars(TimeToStr(Time),[':']), Error), 6) +
                    '}';
  FScrollSteps := 100;
  FScrollKind  := skHorizontal;
  FHintAlign   := taLeftJustify;

  FHintColor := DefHintColor;
  FHintPause := DefHintPause;
  FShowHint := True;
{$IFDEF Delphi3}
  FUpdateMetricSettings := True;
{$ENDIF}
  FHintShortPause := DefHintShortPause;
  FHintHidePause := DefHintHidePause;
  FShowMainForm := True;
{$IFDEF Delphi4_or_5}
  FHintShortCuts := True;
  FBiDiMode := bdLeftToRight;
  FMouseDragImmediate := True;
  FMouseDragThreshold := 5;
{$ENDIF}
  FUpdateFormatSettings := True;
  if not (csDesigning in ComponentState) then
    Application.HookMainWindow(MessageHook);
  FParamsList := TStringList.Create;
  FSwitchList := TStringList.Create;
  FSwitchParamsList := TStringList.Create;
  FSwitchChars := '-/';
  FSwitchMatchCase := false;
  GetCommandParams;
  FBlockCtrlAltDel := false;
  FScrollHints := true;
end;

destructor TsohoApplication.Destroy;
var Index: Integer;
    FileName: string;
    pFont : array [0..255] of char;
begin
  { Remove fonts resources }
  if not (csDesigning in ComponentState) then begin
    for Index := 0 to pred(Fonts.Count) do begin
      FileName := FontsDir + Fonts[Index];
      if FileExists(FileName) then begin
        StrPCopy(pFont, FileName);
        RemoveFontResource(pFont);
      end;
    end;
  end;
  FFonts.Free;
  FParamsList.Free;
  FSwitchList.Free;
  FSwitchParamsList.Free;
  if not (csDesigning in ComponentState) then
    Application.UnhookMainWindow(MessageHook);
  if Self <> nil then RemoveEvents;
  FCanvas.Free;
  if FBlockCtrlAltDel then BlockCtrlAltDel := false;
  inherited Destroy;
end;

procedure TsohoApplication.Loaded;
var OwnerForm: TForm;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then begin
    ScrollSteps := FScrollSteps;
    HorizontalScroll := FScrollKind = skHorizontal;
    SoHints.HintAlign := FHintAlign;
    if FScrollHints then SetSohoHint;
    {   ,   CmdShow }
    OwnerForm := GetOwnerForm(Self);
    FOnMainCreate := OwnerForm.OnCreate;
    OwnerForm.OnCreate := MainFormCreate;
    if FSearchPrev then
      if AlreadyRunning(ApplicationId, True, false) then Application.Terminate;
  end;
  AddEvents;
  UpdateAppProps;
end;

function TsohoApplication.GetCanvas: TCanvas;
begin
  if FCanvas = nil then FCanvas := TCanvas.Create;
  Result := FCanvas;
end;

procedure TsohoApplication.PaintIcon;
var
  PS: TPaintStruct;
begin
  BeginPaint(Application.Handle, PS);
  try
    if FCanvas <> nil then FCanvas.Free;
    FCanvas := TCanvas.Create;
    try
      IconCanvas.Handle := PS.hDC;
      IconCanvas.Brush.Color := clBackground;
      if PS.fErase then IconCanvas.FillRect(PS.rcPaint);
      if Assigned(FOnPaintIcon) then FOnPaintIcon(Self);
    finally
      FCanvas.Free;
      FCanvas := nil;
    end;
  finally
    EndPaint(Application.Handle, PS);
  end;
end;

procedure TsohoApplication.SettingsChanged;
begin
  if Assigned(FOnSettingsChanged) then FOnSettingsChanged(Self);
end;

function TsohoApplication.MessageHook(var Msg: TMessage): Boolean;
begin
  Result := False;
  case Msg.Msg of
    WM_WININICHANGE:
      begin
  {$IFNDEF Delphi3}
        if Application.ShowHint then begin
          Application.ShowHint := False;
          Application.ShowHint := True;
        end;
  {$ENDIF}
        try
          SettingsChanged;
        except
          Application.HandleException(Self);
        end;
      end;
    WM_PAINT:
      if Assigned(FOnPaintIcon) and IsIconic(Application.Handle) then
      begin
        PaintIcon;
        Result := True;
      end;
    WM_QUERYENDSESSION: if Assigned(FOnQueryEndSession) then begin
      Result := true;
      FOnQueryEndSession(Self, Msg.LParam = LongInt(ENDSESSION_LOGOFF),
        Result);
    end
  end;
end;

function TsohoApplication.GetHintColor: TColor;
begin
  if (csDesigning in ComponentState) then Result := FHintColor
  else Result := Application.HintColor;
end;

function TsohoApplication.GetHintPause: Integer;
begin
  if (csDesigning in ComponentState) then Result := FHintPause
  else Result := Application.HintPause;
end;

function TsohoApplication.GetShowHint: Boolean;
begin
  if (csDesigning in ComponentState) then Result := FShowHint
  else Result := Application.ShowHint;
end;

procedure TsohoApplication.SetHintColor(Value: TColor);
begin
  FHintColor := Value;
  if not (csDesigning in ComponentState) then Application.HintColor := Value;
end;

procedure TsohoApplication.SetHintPause(Value: Integer);
begin
  FHintPause := Value;
  if not (csDesigning in ComponentState) then Application.HintPause := Value;
end;

procedure TsohoApplication.SetShowHint(Value: Boolean);
begin
  FShowHint := Value;
  if not (csDesigning in ComponentState) then Application.ShowHint := Value;
end;

function TsohoApplication.GetUpdateFormatSettings: Boolean;
begin
  if (csDesigning in ComponentState) then Result := FUpdateFormatSettings
  else Result := Application.UpdateFormatSettings;
end;

procedure TsohoApplication.SetUpdateFormatSettings(Value: Boolean);
begin
  FUpdateFormatSettings := Value;
  if not (csDesigning in ComponentState) then
    Application.UpdateFormatSettings := Value;
end;

function TsohoApplication.GetHintShortPause: Integer;
begin
  if (csDesigning in ComponentState) then Result := FHintShortPause
  else Result := Application.HintShortPause;
end;

function TsohoApplication.GetHintHidePause: Integer;
begin
  if (csDesigning in ComponentState) then Result := FHintHidePause
  else Result := Application.HintHidePause;
end;

function TsohoApplication.GetShowMainForm: Boolean;
begin
  if (csDesigning in ComponentState) then Result := FShowMainForm
  else Result := Application.ShowMainForm;
end;

procedure TsohoApplication.SetHintShortPause(Value: Integer);
begin
  FHintShortPause := Value;
  if not (csDesigning in ComponentState) then Application.HintShortPause := Value;
end;

procedure TsohoApplication.SetHintHidePause(Value: Integer);
begin
  FHintHidePause := Value;
  if not (csDesigning in ComponentState) then Application.HintHidePause := Value;
end;

procedure TsohoApplication.SetShowMainForm(Value: Boolean);
begin
  FShowMainForm := Value;
  if not (csDesigning in ComponentState) then Application.ShowMainForm := Value;
end;

{$IFDEF Delphi3}
function TsohoApplication.GetUpdateMetricSettings: Boolean;
begin
  if (csDesigning in ComponentState) then Result := FUpdateMetricSettings
  else Result := Application.UpdateMetricSettings;
end;

procedure TsohoApplication.SetUpdateMetricSettings(Value: Boolean);
begin
  FUpdateMetricSettings := Value;
  if not (csDesigning in ComponentState) then
    Application.UpdateMetricSettings := Value;
end;

{$ENDIF Delphi3}

{$IFDEF Delphi4_or_5}

function TsohoApplication.GetHintShortCuts: Boolean;
begin
  if (csDesigning in ComponentState) then Result := FHintShortCuts
  else Result := Application.HintShortCuts;
end;

function TsohoApplication.GetBiDiMode: TBiDiMode;
begin
  if (csDesigning in ComponentState) then Result := FBiDiMode
  else Result := Application.BiDiMode;
end;

function TsohoApplication.GetMouseDragImmediate: Boolean;
begin
  if (csDesigning in ComponentState) or (Mouse = nil) then
    Result := FMouseDragImmediate
  else Result := Mouse.DragImmediate;
end;

function TsohoApplication.GetMouseDragThreshold: Integer;
begin
  if (csDesigning in ComponentState) or (Mouse = nil) then
    Result := FMouseDragThreshold
  else Result := Mouse.DragThreshold;
end;

procedure TsohoApplication.SetMouseDragImmediate(Value: Boolean);
begin
  FMouseDragImmediate := Value;
  if not (csDesigning in ComponentState) and (Mouse <> nil) then
    Mouse.DragImmediate := Value;
end;

procedure TsohoApplication.SetMouseDragThreshold(Value: Integer);
begin
  FMouseDragThreshold := Value;
  if not (csDesigning in ComponentState) and (Mouse <> nil) then
    Mouse.DragThreshold := Value;
end;

procedure TsohoApplication.SetHintShortCuts(Value: Boolean);
begin
  FHintShortCuts := Value;
  if not (csDesigning in ComponentState) then
    Application.HintShortCuts := Value;
end;

procedure TsohoApplication.SetBiDiMode(Value: TBiDiMode);
begin
  FBiDiMode := Value;
  if not (csDesigning in ComponentState) then
    Application.BiDiMode := Value;
end;

{$ENDIF Delphi4_or_5}

procedure TsohoApplication.UpdateAppProps;
begin
  if not (csDesigning in ComponentState) then begin
    with Application do begin
      HintColor := FHintColor;
      HintPause := FHintPause;
      ShowHint := FShowHint;
      HintShortPause := FHintShortPause;
      HintHidePause := FHintHidePause;
      ShowMainForm := FShowMainForm;
      UpdateFormatSettings := FUpdateFormatSettings;
{$IFDEF Delphi3}
      UpdateMetricSettings := FUpdateMetricSettings;
{$ENDIF}
{$IFDEF Delphi4_or_5}
      HintShortCuts := FHintShortCuts;
      BiDiMode := FBiDiMode;
      with Mouse do begin
        DragImmediate := FMouseDragImmediate;
        DragThreshold := FMouseDragThreshold;
      end;
{$ENDIF}
    end;
  end;
end;

function TsohoApplication.GetParamsList: TStrings;
begin
  Result := TStrings(FParamsList);
end;

function TsohoApplication.GetSwitchList: TStrings;
begin
  Result := TStrings(FSwitchList);
end;

function TsohoApplication.GetSwitchParamsList: TStrings;
begin
  Result := TStrings(FSwitchParamsList);
end;

function TsohoApplication.GetFonts: TStrings;
begin
  Result := TStrings(FFonts);
end;

function TsohoApplication.GetSwitchParam (const SwitchName: string): string;
var Index: LongInt;
begin
  Result := EmptyStr;
  Index := FSwitchList.IndexOf(SwitchName);
  if Index = -1 then exit;
  Result := FSwitchParamsList[Index];
end;

procedure TsohoApplication.GetCommandParams;
var TmpParam, SwitchName, SwitchParam: string;
    FirstChar: char;
    Index: LongInt;
begin
  FParamsList.Clear;
  FSwitchList.Clear;
  FSwitchParamsList.Clear;
  if FSwitchMatchCase then FSwitchChars := AnsiUpperCase(FSwitchChars);
  for Index := 1 to ParamCount do begin
    TmpParam := ParamStr(Index);
    FirstChar := TmpParam[1];
    if FSwitchMatchCase then FirstChar := UpCase(FirstChar);
    if Pos(FirstChar, FSwitchChars) <> 0 then begin
      SwitchName  := copy(TmpParam, 2, 1);
      SwitchParam := copy(TmpParam, 4, length(TmpParam));
      if FSwitchList.IndexOf(SwitchName) = -1 then begin
         FSwitchList.Add(SwitchName);
         FSwitchParamsList.Add(SwitchParam);
      end;
    end
    else FParamsList.Add(TmpParam);
  end;
end;

procedure TsohoApplication.SetBlockCtrlAltDel (Value: boolean);
begin
  if FBlockCtrlAltDel = Value then exit;
  FBlockCtrlAltDel := Value;
  SystemParametersInfo(SPI_SCREENSAVERRUNNING, integer(FBlockCtrlAltDel),
    nil, 0);
end;

end.
