{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
    TsohoCustomButton, TsohoTransButton 
 TsohoBitBtn, TsohoSpeedButton
}

unit SohoBtns;

{$I SOHOLIB.INC}

interface

uses Windows, Messages, Classes, Controls, Forms, Graphics,
     StdCtrls, ExtCtrls, Buttons, SoMultimedia, Menus, SoCtrls 
     {$IFDEF Delphi4_or_5}, ImgList, ActnList {$ENDIF};

type

  TsohoButtonKind = (sbOk, sbCancel, sbYes, sbNo, sbClose, sbCustom);

  TsohoButtonPaintEvent = procedure (Sender: TObject; Canvas: TCanvas; Rect: TRect;
    IsDown, IsDefault: boolean) of object;
{    OnPaint  OnAfterPaint }

  TsohoCustomButton = class(TButton)
  private
    { Private Declarations }
    FDrawFocus: boolean;
    FImageList: TImageList;
    FImageIndex: LongInt;
    FInRect: boolean;
    FFlat: boolean;
    FColor: TColor;
    FGlyph: TBitmap;
    FLayout: TButtonLayout;
    FSound: TsohoSound;
    FSoundName: TWaveFileName;
    FIsFocused: boolean;
    FCanvas: TCanvas;
    FBottomColor: TColor;
    FHighlightColor: TColor;
    FBevWidth: Integer;

    FOnPaint,
    FOnAfterPaint: TsohoButtonPaintEvent;
    procedure SetFlat(Value: boolean);
    procedure SetHighlightColor(Value: TColor);
    procedure SetColor(Value: TColor);
    procedure SetBottomColor(Value: TColor);
    procedure SetBevWidth(Value: Integer);
    procedure SetLayout(Value: TButtonLayout);
    procedure SetGlyph(Value: TBitmap);
    procedure SetSoundName(Value: TWaveFileName);
    procedure GlyphChanged(Sender: TObject);
    procedure SetImageList(Value: TImageList);
    procedure SetImageIndex(Value: LongInt);
    procedure SetDrawFocus(Value: boolean);
    function  GetFocused: boolean;
  protected
    { Protected Declarations }
    {$IFDEF Delphi4_or_5}
    procedure ActionChange(Sender: TObject; CheckDefaults: Boolean); override;
    {$ENDIF}
    function MouseUnderButton: boolean;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure CMMouseEnter(var message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var message: TMessage); message CM_MOUSELEAVE;
    procedure CMEnter(var Message: TMessage); message Cm_Enter;
    procedure CMExit(var Message: TMessage); message Cm_Exit;
    procedure CNMeasureItem(var message: TWMMeasureItem); message CN_MEASUREITEM;
    procedure CNDrawItem(var message: TWMDrawItem); message CN_DRAWITEM;
    procedure CMFontChanged(var message: TMessage); message CM_FONTCHANGED;
    procedure CMParentColorChanged(var message: TMessage); message CM_PARENTCOLORCHANGED;
    procedure CMEnabledChanged(var message: TMessage); message CM_ENABLEDCHANGED;
    procedure DrawItem(const DrawItemStruct: TDrawItemStruct); virtual;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure SetButtonStyle(ADefault: boolean); override;
    function GetPalette: HPALETTE; override;
    procedure DoButtonPaint (Canvas: TCanvas; Rect: TRect; IsDown, IsDefault: boolean); virtual;
    procedure DoButtonAfterPaint (Canvas: TCanvas; Rect: TRect; IsDown, IsDefault: boolean); virtual;
    property Images: TImageList read FImageList write SetImageList;
    property ImageIndex: LongInt read FImageIndex write SetImageIndex;
    property BevelWidth: Integer read FBevWidth write SetBevWidth default 1;
    property Canvas: TCanvas read FCanvas;
    property DrawFocus: boolean read FDrawFocus write SetDrawFocus;
    property Color: TColor read FColor write SetColor;
    property HighlightColor: TColor read FHighlightColor write SetHighlightColor default clWhite;
    property BottomColor: TColor read FBottomColor write SetBottomColor default clGray;
    property Sound: TsohoSound read FSound write FSound;
    property ClickSound: TWaveFileName read FSoundName write SetSoundName;
    property Layout: TButtonLayout read FLayout write SetLayout default blGlyphLeft;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property Flat: boolean read FFlat write SetFlat;
    property OnPaint: TsohoButtonPaintEvent read FOnPaint write FOnPaint;
    property OnAfterPaint: TsohoButtonPaintEvent read FOnAfterPaint write FOnAfterPaint;
  public
    { Public Declarations }
    procedure DrawButton(ButtonState: TButtonState; Rect: TRect;
      IsDown, IsDefault: boolean); virtual;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    property IsFocused: boolean read GetFocused write FIsFocused;
  end;

  TsohoTransButton = class(TsohoCustomButton)
  private
    { Private Declarations }
    FKind: TsohoButtonKind;
    FBack: TBitmap;
  protected
    procedure SetKind(Value: TsohoButtonKind);
  public
    { Public Declarations }
    procedure WMKillFocus(var message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMSetFocus(var message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMEraseBkgnd(var message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure DrawButton(ButtonState: TButtonState; Rect: TRect;
      IsDown, IsDefault: boolean); override;
    constructor Create(AOwner: TComponent); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    destructor Destroy; override;
    procedure Click; override;
  published
    { Published Declarations }
    property Align;
    property DrawFocus default true;
    property Images;
    property ImageIndex default -1;
    property Kind: TsohoButtonKind read FKind write SetKind default sbCustom;
    property BevelWidth;
    property HighlightColor;
    property BottomColor;
    property Sound;
    property ClickSound;
    property Cursor default crHandPoint;
    property Layout;
    property Glyph;
    property Flat;
    property OnPaint;
    property OnAfterPaint;
{$IFDEF Delphi4_or_5}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
{$ENDIF}
{$IFDEF Delphi5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF Delphi4_or_5}
    property OnResize;
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
  end;

  TsohoButtonType = (btRectangle, btEllipse);
  TsohoDropDownMenuPos = (dmpBottom, dmpRight);

  TsohoBitBtn = class(TsohoCustomButton)
  private
    FNewGlyph: Pointer;
    FStyle: TButtonStyle;
    FKind: TBitBtnKind;
    fType: TsohoButtonType;
    FBackColor: TColor;
    FLayout: TButtonLayout;
    FSpacing: Integer;
    FMargin: Integer;
    FModifiedGlyph: boolean;
    FMarkDropDown: Boolean;
    FDropDownMenu: TPopupMenu;
    FDragging: boolean;
    FMenuPosition: TsohoDropDownMenuPos;
    FMenuTracking: boolean;
    FDown: Boolean;
    procedure SetGlyph(Value: TBitmap);
    function GetGlyph: TBitmap;
    function GetNumGlyphs: TNumGlyphs;
    procedure SetNumGlyphs(Value: TNumGlyphs);
    procedure GlyphChanged(Sender: TObject);
    function IsCustom: boolean;
    function IsCustomCaption: boolean;
    procedure SetStyle(Value: TButtonStyle);
    procedure SetKind(Value: TBitBtnKind);
    procedure SetType(Value: TsohoButtonType);
    procedure SetBackColor(Value: TColor);
    function GetKind: TBitBtnKind;
    procedure SetLayout(Value: TButtonLayout);
    procedure SetSpacing(Value: Integer);
    procedure SetMargin(Value: Integer);
    procedure SetDropDownMenu(Value: TPopupMenu);
    procedure SetMarkDropDown(Value: Boolean);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DrawPopupMark(Canvas: TCanvas; X, Y: Integer);
    function GetDropDownMenuPos: TPoint;
    function CheckMenuDropDown(const Pos: TSmallPoint;
      Manual: Boolean): Boolean;
    procedure CMMouseEnter(var message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var message: TMessage); message CM_MOUSELEAVE;
    procedure WMLButtonDown(var message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonDblClk(var message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure CMParentColorChanged(var message: TMessage); message CM_PARENTCOLORCHANGED;
    function GetPalette: HPALETTE; override;
    procedure SetButtonStyle(ADefault: boolean); override;
    procedure DoMouseUp(Button: TMouseButton; Shift: TShiftState;
       X, Y: Integer);
  public
    function CheckBtnMenuDropDown: Boolean;
    procedure DrawButton(ButtonState: TButtonState; Rect: TRect;
      IsDown, IsDefault: boolean); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
  published
    property Align;
    property DrawFocus default true;
    property Down: Boolean read FDown write FDown;
    property DropDownMenu: TPopupMenu read FDropDownMenu write SetDropDownMenu;
    property MenuPosition: TsohoDropDownMenuPos read FMenuPosition write FMenuPosition
      default dmpBottom;
    property MarkDropDown: Boolean read FMarkDropDown write SetMarkDropDown default True;
    property Images;
    property ImageIndex default -1;
    property Flat default False;
    property Color;
    property ParentColor;
    property Cancel stored IsCustom;
    property Caption stored IsCustomCaption;
    property Default stored IsCustom;
    property Glyph: TBitmap read GetGlyph write SetGlyph stored IsCustom;
    property Kind: TBitBtnKind read GetKind write SetKind default bkCustom;
    property Layout: TButtonLayout read FLayout write SetLayout default blGlyphLeft;
    property Margin: Integer read FMargin write SetMargin default - 1;
    property ModalResult stored IsCustom;
    property NumGlyphs: TNumGlyphs read GetNumGlyphs write SetNumGlyphs stored IsCustom default 1;
    property ParentShowHint;
    property Style: TButtonStyle read FStyle write SetStyle default bsAutoDetect;
    property Spacing: Integer read FSpacing write SetSpacing default 4;
    property BtnType: TsohoButtonType read fType write SetType default btRectangle;
    property BackColor: TColor read FBackColor write SetBackColor;
    property Cursor default crHandPoint;
    property Sound;
    property ClickSound;
    property OnPaint;
    property OnAfterPaint;
{$IFDEF Delphi4_or_5}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
{$ENDIF}
{$IFDEF Delphi5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF Delphi4_or_5}
    property OnResize;
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
  end;

  { TBitPool }
const
  BitsPerInt = SizeOf(Integer) * 8;

type
  TBitEnum = 0..BitsPerInt - 1;
  TBitSet = set of TBitEnum;
  TBitPool = class
  private
    FSize: Integer;
    FBits: Pointer;
    procedure SetSize(Value: Integer);
    procedure SetBit(index: Integer; Value: boolean);
    function GetBit(index: Integer): boolean;
  public
    destructor Destroy; override;
    function OpenBit: Integer;
    property Bits[index: Integer]: boolean read GetBit write SetBit; default;
    property Size: Integer read FSize write SetSize;
  end;

  TGlyphList = class(TImageList)
  private
    used: TBitPool;
    fCount: Integer;
    function AllocateIndex: Integer;
  public
    {$IFNDEF Win32}
    constructor Create(AWidth, AHeight: Integer);
    {$ELSE}
    constructor CreateSize(AWidth, AHeight: Integer);
    {$ENDIF}
    destructor Destroy; override;
    function Add(IMAGE, Mask: TBitmap): Integer;
    function AddMasked(IMAGE: TBitmap; MaskColor: TColor): Integer;
    procedure Delete(index: Integer);
    property Count: Integer read fCount;
  end;

  TGlyphCache = class
  private
    GlyphLists: TList;
  public
    constructor Create;
    destructor Destroy; override;
    function GetList(AWidth, AHeight: Integer): TGlyphList;
    procedure ReturnList(List: TGlyphList);
    function Empty: boolean;
  end;

  TButtonGlyph = class
  private
    FColor: TColor;
    FOriginal: TBitmap;
    FGlyphList: TGlyphList;
    FIndexs: array[TButtonState] of Integer;
    FTransparentColor: TColor;
    FNumGlyphs: TNumGlyphs;
    FOnChange: TNotifyEvent;
  public
    procedure GlyphChanged(Sender: TObject);
    procedure SetGlyph(Value: TBitmap);
    procedure SetNumGlyphs(Value: TNumGlyphs);
    procedure Invalidate;
    function CreateButtonGlyph(State: TButtonState): Integer;
    procedure DrawButtonGlyph(Canvas: TCanvas; X, Y: Integer;
      State: TButtonState; Transparent: boolean);
    procedure DrawButtonText(Canvas: TCanvas; const Caption: string;
      TextBounds: TRect; State: TButtonState);
    procedure CalcButtonLayout(Canvas: TCanvas; const Client: TRect;
      const Caption: string; Layout: TButtonLayout; Margin, Spacing: Integer;
      var GlyphPos: TPoint; var TextBounds: TRect);
    constructor Create;
    destructor Destroy; override;
    { return the text rectangle }
    function Draw(Canvas: TCanvas; const Client: TRect;
      const Caption: string; Layout: TButtonLayout; Margin, Spacing: Integer;
      State: TButtonState; Transparent: boolean): TRect;
    property Glyph: TBitmap read FOriginal write SetGlyph;
    property NumGlyphs: TNumGlyphs read FNumGlyphs write SetNumGlyphs;
    property Color: TColor read FColor write FColor;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TsohoSpeedButton = class(TGraphicControl)
  private
    FGroupIndex: Integer;
    FGlyph: Pointer;
    FDown: Boolean;
    FDragging: Boolean;
    FAllowAllUp: Boolean;
    FLayout: TButtonLayout;
    FSpacing: Integer;
    FTransparent: Boolean;
    FMargin: Integer;
    FFlat: Boolean;
    FMenuTracking:boolean;
    FMouseInControl: Boolean;
    FDropDownMenu: TPopupMenu;
    FMenuPosition: TsohoDropDownMenuPos;
    FAlignment:TAlignment;
    FImageList: TImageList;
    FImageIndex: LongInt;
    procedure SetAlignment(Value: TAlignment);
    procedure GlyphChanged(Sender: TObject);
    procedure UpdateExclusive;
    function GetGlyph: TBitmap;
    procedure SetGlyph(Value: TBitmap);
    function GetNumGlyphs: TNumGlyphs;
    procedure SetNumGlyphs(Value: TNumGlyphs);
    procedure SetDown(Value: Boolean);
    procedure SetFlat(Value: Boolean);
    procedure SetAllowAllUp(Value: Boolean);
    procedure SetGroupIndex(Value: Integer);
    procedure SetLayout(Value: TButtonLayout);
    procedure SetSpacing(Value: Integer);
    procedure SetTransparent(Value: Boolean);
    procedure SetMargin(Value: Integer);
    procedure SetDropDownMenu(Value: TPopupMenu);
    procedure SetImageIndex(Value: LongInt);
    function CheckMenuDropDown(const Pos: TSmallPoint;
      Manual: Boolean): Boolean;
    procedure UpdateTracking;
    procedure WMLButtonDblClk(var Message: TWMLButtonDown); message WM_LBUTTONDBLCLK;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMButtonPressed(var Message: TMessage); message CM_BUTTONPRESSED;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMSysColorChange(var Message: TMessage); message CM_SYSCOLORCHANGE;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure SetImageList(const Value: TImageList);
  protected
    FState: TButtonState;
    {$IFDEF Delphi4_or_5}
    procedure ActionChange(Sender: TObject; CheckDefaults: Boolean); override;
    {$ENDIF}
    function GetPalette: HPALETTE; override;
    procedure Loaded; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure DoMouseUp(Button: TMouseButton; Shift: TShiftState;
       X, Y: Integer);
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Paint; override;
    property MouseInControl: Boolean read FMouseInControl;
    function GetDropDownMenuPos: TPoint;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment default taCenter;
    property AllowAllUp: Boolean read FAllowAllUp write SetAllowAllUp default False;
    property Images: TImageList read FImageList write SetImageList;
    property ImageIndex: LongInt read FImageIndex write SetImageIndex default -1;
    property GroupIndex: Integer read FGroupIndex write SetGroupIndex default 0;
    property Down: Boolean read FDown write SetDown default False;
    property Caption;
    property DropDownMenu: TPopupMenu read FDropDownMenu write SetDropDownMenu;
    property MenuPosition: TsohoDropDownMenuPos read FMenuPosition write FMenuPosition
      default dmpBottom;
    property Enabled;
    property Flat: Boolean read FFlat write SetFlat default False;
    property Font;
    property Glyph: TBitmap read GetGlyph write SetGlyph;
    property Layout: TButtonLayout read FLayout write SetLayout default blGlyphLeft;
    property Margin: Integer read FMargin write SetMargin default -1;
    property NumGlyphs: TNumGlyphs read GetNumGlyphs write SetNumGlyphs default 1;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Spacing: Integer read FSpacing write SetSpacing default 4;
    property Transparent: Boolean read FTransparent write SetTransparent default True;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
{$IFDEF Delphi4_or_5}
    property Anchors;
    property Action;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
{$ENDIF}
{$IFDEF Delphi5}
    property OnContextPopup;
{$ENDIF}
{$IFDEF Delphi4_or_5}
    property OnResize;
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
  end;

type
  PBitArray = ^TBitArray;
  TBitArray = array[0..4096] of TBitSet;

function NewDrawButtonFace (Canvas: TCanvas; const CLIENT: TRect; FillColor: TColor;
    IsMouseOver, IsDown, IsFocused, IsDefault, DrawFocus: boolean): TRect;

function DrawButtonFace(Canvas: TCanvas; const CLIENT: TRect; FillColor: TColor;
  BevelWidth: Integer; Style: TButtonStyle; IsRounded, IsDown,
  IsFocused: boolean): TRect;

procedure DrawGlyphWithText(const Glyph: TBitmap; Layout: TButtonLayout;
  Text: string; Canvas: TCanvas; Rect: TRect; var TxtRect: TRect);

procedure DrawUsualGlyphWithText(const Glyph: TBitmap; Layout: TButtonLayout;
  Text: string; Canvas: TCanvas; Rect: TRect; var TxtRect: TRect;
  Enabled: boolean);

procedure CreateBrushPattern;

var
  Pattern    : TBitmap;

implementation
uses SysUtils, SoGraph, SoVCLUtils, SoAppUtils, CommCtrl, SoUtils,
     SoCmnCns;

{$R SOHOBTNS.R32}

const
  { TsohoBitBtn data }
  BitBtnResNames: array[TBitBtnKind] of PChar = (
    nil, 'sbBOK', 'sbBCANCEL', 'sbBHELP', 'sbBYES', 'sbBNO', 'sbBCLOSE',
    'sbBABORT', 'sbBRETRY', 'sbBIGNORE', 'sbBALL');
  BitBtnCaptions: array[TBitBtnKind] of string = (
    '', SOKButton, SCancelButton, SHelpButton, SYesButton, SNoButton,
    SCloseButton, SAbortButton, SRetryButton, SIgnoreButton,
    SAllButton);
  BitBtnModalResults: array[TBitBtnKind] of TModalResult = (
    0, mrOk, mrCancel, 0, mrYes, mrNo, 0, mrAbort, mrRetry, mrIgnore,
    mrAll);

  sbResNames: array[TsohoButtonKind] of string = ('sbOK',
    'sbCANCEL', 'sbYES', 'sbNO', 'sbCLOSE', '');
  WFBResNames: array[TsohoButtonKind] of string = ('WFBOK',
    'WFBCANCEL', 'WFBYES', 'WFBNO', 'WFBCLOSE', '');
  sbResCaptions: array[TsohoButtonKind] of string = (wtb_CaptionOk,
    wtb_CaptionCancel, wtb_CaptionYes, wtb_CaptionNo,
    wtb_CaptionClose, '');
  sbModalResults: array[TsohoButtonKind] of TModalResult = (mrOk,
    mrCancel, mrYes, mrNo, mrNone, mrNone);


var
  BitBtnGlyphs: array[TBitBtnKind] of TBitmap;

{ DrawButtonFace - returns the remaining usable area inside the Client rect.}
function SohoDrawButtonFace(Canvas: TCanvas; const CLIENT: TRect; FillColor: TColor;
    BevelWidth: Integer; Style: TButtonStyle; IsRounded, IsDown,
    IsFocused: boolean; Transparent : boolean): TRect;
var
  NewStyle: boolean;
  R       : TRect;
begin
  NewStyle := ((Style = bsAutoDetect) and NewStyleControls) or (Style = bsNew);

  R := CLIENT;
  with Canvas do begin
    if NewStyle then begin
      if not Transparent then begin
        BRUSH.Color := FillColor;
        BRUSH.Style := bsSolid;
        FillRect(R);
      end;

      if IsDown then begin
        Frame3D(Canvas, R, clWindowFrame, clBtnHighlight, 1);
        Frame3D(Canvas, R, clBtnShadow, clBtnFace, 1);
      end
      else begin
        Frame3D(Canvas, R, clBtnHighlight, clWindowFrame, 1);
        Frame3D(Canvas, R, clBtnFace, clBtnShadow, 1);
      end;

      if IsFocused then begin
        InflateRect(R, - 1, - 1);
        BRUSH.Style := bsClear;
        Rectangle(R.Left, R.Top, R.Right, R.Bottom);
      end;
      
    end
    else begin
      Pen.Color := clWindowFrame;
      if not Transparent then begin
        BRUSH.Color := FillColor;
        BRUSH.Style := bsSolid;
      end
      else BRUSH.Style := bsClear;

      Rectangle(R.Left, R.Top, R.Right, R.Bottom);

      { round the corners - only applies to Win 3.1 style buttons }
      if IsRounded then begin
        Pixels[R.Left, R.Top] := clBtnFace;
        Pixels[R.Left, R.Bottom - 1] := clBtnFace;
        Pixels[R.Right - 1, R.Top] := clBtnFace;
        Pixels[R.Right - 1, R.Bottom - 1] := clBtnFace;
      end;

      if IsFocused then begin
        InflateRect(R, - 1, - 1);
        BRUSH.Style := bsClear;
        Rectangle(R.Left, R.Top, R.Right, R.Bottom);
      end;

      InflateRect(R, - 1, - 1);
      if not IsDown then Frame3D(Canvas, R, clBtnHighlight, clBtnShadow, BevelWidth)
      else begin
        Pen.Color := clBtnShadow;
        Polyline([Point(R.Left, R.Bottom - 1), Point(R.Left, R.Top),
          Point(R.Right, R.Top)]);
      end;
    end;
  end;

  Result := CLIENT;
  InflateRect(Result, - BevelWidth, - BevelWidth);
  if IsDown then OffsetRect(Result, 1, 1);
end;

function DrawButtonFace(Canvas: TCanvas; const CLIENT: TRect; FillColor: TColor;
    BevelWidth: Integer; Style: TButtonStyle; IsRounded, IsDown,
    IsFocused: boolean): TRect;
begin
  Result := SohoDrawButtonFace(Canvas, CLIENT, FillColor, BevelWidth,
            Style, IsRounded, IsDown, IsFocused, false);
end;

function NewDrawButtonFace (Canvas: TCanvas; const CLIENT: TRect; FillColor: TColor;
    IsMouseOver, IsDown, IsFocused, IsDefault, DrawFocus: boolean): TRect;
var R: TRect;
begin
    R := Client;

    Canvas.Brush.Color := FillColor;

    if ((IsFocused) or (IsDefault)) and DrawFocus then begin

      Canvas.Pen.Color := clWindowText;
      Canvas.Pen.Width := 1;
      Canvas.Brush.Style := bsSolid;
      Canvas.Rectangle(R.Left, R.Top, R.Right, R.Bottom);

      InflateRect(R, -1, -1);

    end;

    Canvas.FillRect(R);

    if IsMouseOver then begin

      Canvas.Pen.Color := clWindowText;
      Canvas.MoveTo(R.Right-1, R.Top);
      Canvas.LineTo(R.Right-1, R.Bottom-1);
      Canvas.LineTo(R.Left-1, R.Bottom-1);

      Canvas.Pen.Color := clBtnHighlight;
      Canvas.MoveTo(R.Left, R.Bottom-2);
      Canvas.LineTo(R.Left, R.Top);
      Canvas.LineTo(R.Right-1, R.Top);

      Canvas.Pen.Color := clBtnShadow;
      Canvas.MoveTo(R.Right-2, R.Top+1);
      Canvas.LineTo(R.Right-2, R.Bottom-2);
      Canvas.LineTo(R.Left, R.Bottom-2);

    end else begin

      Canvas.Pen.Color := clBtnHighlight;
      Canvas.Pen.Width := 1;
      Canvas.MoveTo(R.Left, R.Bottom-2);
      Canvas.LineTo(R.Left, R.Top);
      Canvas.LineTo(R.Right-1, R.Top);

      Canvas.Pen.Color := clBtnShadow;
      Canvas.LineTo(R.Right-1, R.Bottom-1);
      Canvas.LineTo(R.Left-1, R.Bottom-1);

    end;

    if IsDown then begin
      Canvas.Brush.Color := FillColor;
      Canvas.FillRect(R);
      Canvas.Pen.Color := clBtnShadow;
      Canvas.Pen.Width := 1;
      Canvas.MoveTo(R.Left, R.Bottom-2);
      Canvas.LineTo(R.Left, R.Top);
      Canvas.LineTo(R.Right-1, R.Top);
      Canvas.Pen.Color := clBtnHighlight;
      Canvas.LineTo(R.Right-1, R.Bottom-1);
      Canvas.LineTo(R.Left-1, R.Bottom-1);
    end;

    if IsFocused and IsDefault and DrawFocus then begin

      InflateRect(R, -3, -3);
      Canvas.Pen.Color := clWindowFrame;
      Canvas.Brush.Color := FillColor;
      DrawFocusRect(Canvas.Handle, R);

    end;

    if IsDown then OffsetRect(R, 1, 1);
    Result := R;
end;

function DrawSpeedButtonFace(Canvas: TCanvas; const Client: TRect;
  BevelWidth: Integer; Style: TButtonStyle; IsRounded, IsDown,
  IsFocused: Boolean): TRect;
var
  NewStyle: Boolean;
  R: TRect;
  DC: THandle;
begin
  NewStyle := ((Style = bsAutoDetect) and NewStyleControls) or (Style = bsNew);
    
  R := Client;
  with Canvas do
  begin
    if NewStyle then
    begin
      Brush.Color := clBtnFace;
      Brush.Style := bsSolid;
      DC := Canvas.Handle;    { Reduce calls to GetHandle }
    
      if IsDown then
      begin    { DrawEdge is faster than Polyline }
        DrawEdge(DC, R, BDR_SUNKENINNER, BF_TOPLEFT);              { black     }
        DrawEdge(DC, R, BDR_SUNKENOUTER, BF_BOTTOMRIGHT);          { btnhilite }
        Dec(R.Bottom);
        Dec(R.Right);
        Inc(R.Top);
        Inc(R.Left);
        DrawEdge(DC, R, BDR_SUNKENOUTER, BF_TOPLEFT or BF_MIDDLE); { btnshadow }
      end
      else
      begin
        DrawEdge(DC, R, BDR_RAISEDOUTER, BF_BOTTOMRIGHT);          { black }
        Dec(R.Bottom);
        Dec(R.Right);
        DrawEdge(DC, R, BDR_RAISEDINNER, BF_TOPLEFT);              { btnhilite }
        Inc(R.Top);
        Inc(R.Left);
        DrawEdge(DC, R, BDR_RAISEDINNER, BF_BOTTOMRIGHT or BF_MIDDLE); { btnshadow }
      end;
    end
    else
    begin
      Pen.Color := clWindowFrame;
      Brush.Color := clBtnFace;
      Brush.Style := bsSolid;
      Rectangle(R.Left, R.Top, R.Right, R.Bottom);
    
      { round the corners - only applies to Win 3.1 style buttons }
      if IsRounded then
      begin
        Pixels[R.Left, R.Top] := clBtnFace;
        Pixels[R.Left, R.Bottom - 1] := clBtnFace;
        Pixels[R.Right - 1, R.Top] := clBtnFace;
        Pixels[R.Right - 1, R.Bottom - 1] := clBtnFace;
      end;
    
      if IsFocused then
      begin
        InflateRect(R, -1, -1);
        Brush.Style := bsClear;
        Rectangle(R.Left, R.Top, R.Right, R.Bottom);
      end;
    
      InflateRect(R, -1, -1);
      if not IsDown then
        Frame3D(Canvas, R, clBtnHighlight, clBtnShadow, BevelWidth)
      else
      begin
        Pen.Color := clBtnShadow;
        PolyLine([Point(R.Left, R.Bottom - 1), Point(R.Left, R.Top),
          Point(R.Right, R.Top)]);
      end;
    end;
  end;

  Result := Rect(Client.Left + 1, Client.Top + 1,
    Client.Right - 2, Client.Bottom - 2);
  if IsDown then OffsetRect(Result, 1, 1);
end;

procedure GetCoordsGlyphAndText(const Glyph: TBitmap; Layout: TButtonLayout;
    Text: string; Canvas: TCanvas; Rect: TRect; var TxtRect: TRect;
    GlyphWidth: Integer; var IMAGE: TRect);
var AllW, AllH: Integer;
begin
  AllW := 0;
  AllH := 0;
  with Canvas do begin
    if Glyph <> nil then begin
      case Layout of
        blGlyphLeft,
          blGlyphRight: begin AllW := GlyphWidth + TextWidth(Text);
            AllH := GetMax(Glyph.Height, TextHeight('W'));
          end;
        blGlyphTop,
          blGlyphBottom: begin AllH := Glyph.Height + TextHeight('W');
            AllW := GetMax(GlyphWidth, TextWidth(Text));
          end;
      end;
      case Layout of
        blGlyphLeft: begin with IMAGE do begin
            Left := HorCenter(Rect, AllW);
            Top := VertCenter(Rect, Glyph.Height);
            Right := Left + GlyphWidth;
            Bottom := Top + Glyph.Height;
          end;
          TxtRect.Left := IMAGE.Right + 1;
          TxtRect.Top := VertCenter(Rect, TextHeight('W'));
          TxtRect.Right := TxtRect.Left + TextWidth(Text);
          TxtRect.Bottom := TxtRect.Top + TextHeight('W');
        end;
        blGlyphRight: begin TxtRect.Left := HorCenter(Rect, AllW);;
          TxtRect.Top := VertCenter(Rect, TextHeight('W'));
          TxtRect.Right := TxtRect.Left + TextWidth(Text);
          TxtRect.Bottom := TxtRect.Top + TextHeight('W');
          with IMAGE do begin
            Left := TxtRect.Right + 1;
            Top := VertCenter(Rect, Glyph.Height);
            Right := Left + GlyphWidth;
            Bottom := Top + Glyph.Height;
          end;
        end;
        blGlyphTop: begin with IMAGE do begin
            Left := HorCenter(Rect, GlyphWidth);
            Top := VertCenter(Rect, AllH);
            Right := Left + GlyphWidth;
            Bottom := Top + Glyph.Height;
          end;
          TxtRect.Left := HorCenter(Rect, TextWidth(Text));
          TxtRect.Top := IMAGE.Bottom + 1;
          TxtRect.Right := TxtRect.Left + TextWidth(Text);
          TxtRect.Bottom := TxtRect.Top + TextHeight('W');
        end;
        blGlyphBottom: begin TxtRect.Left := HorCenter(Rect, TextWidth(Text));
          TxtRect.Top := VertCenter(Rect, AllH);
          TxtRect.Right := TxtRect.Left + TextWidth(Text);
          TxtRect.Bottom := TxtRect.Top + TextHeight('W');
          with IMAGE do begin
            Left := HorCenter(Rect, GlyphWidth);
            Top := TxtRect.Bottom + 1;
            Right := Left + GlyphWidth;
            Bottom := Top + Glyph.Height;
          end;
        end;
      end;
    end
    else begin
      TxtRect.Left := HorCenter(Rect, TextWidth(Text));
      TxtRect.Top := VertCenter(Rect, TextHeight('W'));
      TxtRect.Right := TxtRect.Left + TextWidth(Text);
      TxtRect.Bottom := TxtRect.Top + TextHeight('W');
    end;
  end;
end;

procedure DrawUsualGlyphWithText;
var gW: Integer;
  IMAGE: TRect;
begin
  if Glyph.Width = 2 * Glyph.Height then gW := Glyph.Width div 2
  else gW := Glyph.Width;
  GetCoordsGlyphAndText(Glyph, Layout, Text, Canvas, Rect, TxtRect, gW, IMAGE);
  if Glyph <> nil then begin
    if Enabled or (Glyph.Width <> 2 * Glyph.Height) then
      Canvas.BrushCopy(IMAGE, Glyph, Bounds(0, 0, gW, Glyph.Height),
      Glyph.Canvas.Pixels[0, Glyph.Height - 1])
    else Canvas.BrushCopy(IMAGE, Glyph, Bounds(gW, 0, gW, Glyph.Height),
      Glyph.Canvas.Pixels[0, Glyph.Height - 1]);
  end;
end;

procedure DrawGlyphWithText;
var IMAGE: TRect;
  gW: Integer;
begin
  gW := Glyph.Width div 2;
  GetCoordsGlyphAndText(Glyph, Layout, Text, Canvas, Rect, TxtRect, gW, IMAGE);
  {  }
  with Canvas do begin
    CopyMode := cmSrcAnd;
    CopyRect(IMAGE, Glyph.Canvas, Bounds(gW, 0, gW, Glyph.Height));
    {   }
    CopyMode := cmSrcPaint;
    CopyRect(IMAGE, Glyph.Canvas, Bounds(0, 0, gW, Glyph.Height));
  end;
end;

function GetBitBtnGlyph(Kind: TBitBtnKind): TBitmap;
begin
  if BitBtnGlyphs[Kind] = nil then begin
    BitBtnGlyphs[Kind] := TBitmap.Create;
    BitBtnGlyphs[Kind].Handle := LoadBitmap(HInstance, BitBtnResNames[Kind]);
  end;
  Result := BitBtnGlyphs[Kind];
end;


destructor TBitPool.Destroy;
begin
  SetSize(0);
  inherited Destroy;
end;

procedure TBitPool.SetSize(Value: Integer);
var
  NewMem    : Pointer;
  NewMemSize: Integer;
  OldMemSize: Integer;
  
  function Min(X, Y: Integer): Integer;
  begin
    Result := X;
    if X > Y then Result := Y;
  end;
   begin
  if Value <> Size then begin
    NewMemSize := ((Value + BitsPerInt - 1) div BitsPerInt) * SizeOf(Integer);
    OldMemSize := ((Size + BitsPerInt - 1) div BitsPerInt) * SizeOf(Integer);
    if NewMemSize <> OldMemSize then begin
      if NewMemSize <> 0 then begin
        GetMem(NewMem, NewMemSize);
        FillChar(NewMem^, NewMemSize, 0);
      end
      else NewMem := nil;
      if OldMemSize <> 0 then begin
        if NewMem <> nil then
          Move(FBits^, NewMem^, Min(OldMemSize, NewMemSize));
        FreeMem(FBits, OldMemSize);
      end;
      FBits := NewMem;
    end;
    FSize := Value;
  end;
end;

procedure TBitPool.SetBit(index: Integer; Value: boolean);
begin
  if Value then
    Include(PBitArray(FBits)^[index div BitsPerInt], index mod BitsPerInt)
  else
    Exclude(PBitArray(FBits)^[index div BitsPerInt], index mod BitsPerInt);
end;

function TBitPool.GetBit(index: Integer): boolean;
begin
  Result := index mod BitsPerInt in PBitArray(FBits)^[index div BitsPerInt];
end;

function TBitPool.OpenBit: Integer;
var
  I: Integer; 
  B: TBitSet; 
  J: TBitEnum;
  E: Integer; 
begin
  E := (Size + BitsPerInt - 1) div BitsPerInt - 1;
  for I := 0 to E do
    if PBitArray(FBits)^[I] <> [0..BitsPerInt - 1] then begin
      B := PBitArray(FBits)^[I];
      for J := Low(J) to High(J) do begin
        if not (J in B) then begin
          Result := I * BitsPerInt + J;
          if Result >= Size then Result := -1;
          Exit;
        end;
      end;
    end;
  Result := -1;
end;

{ TGlyphList }

{$IFNDEF Win32}
constructor TGlyphList.Create(AWidth, AHeight: Integer);
begin
  inherited Create(AWidth, AHeight);
  used := TBitPool.Create;
end;
{$ELSE}
constructor TGlyphList.CreateSize(AWidth, AHeight: Integer);
begin
  inherited CreateSize(AWidth, AHeight);
  used := TBitPool.Create;
end;
{$ENDIF}

destructor TGlyphList.Destroy;
begin
  used.Free;
  inherited Destroy;
end;

function TGlyphList.AllocateIndex: Integer;
begin
  Result := used.OpenBit;
  if Result = -1 then begin
    Result := inherited Add(nil, nil);
    used.Size := Result + 1;
  end;
  used[Result] := True;
end;

function TGlyphList.Add(IMAGE, Mask: TBitmap): Integer;
begin
  Result := AllocateIndex;
  Replace(Result, IMAGE, Mask);
  Inc(fCount);
end;

function TGlyphList.AddMasked(IMAGE: TBitmap; MaskColor: TColor): Integer;
begin
  Result := AllocateIndex;
  ReplaceMasked(Result, IMAGE, MaskColor);
  Inc(fCount);
end;

procedure TGlyphList.Delete(index: Integer);
begin
  if used[index] then begin
    Dec(fCount);
    used[index] := False;
  end;
end;

{ TGlyphCache }

constructor TGlyphCache.Create;
begin
  inherited Create;
  GlyphLists := TList.Create;
end;

destructor TGlyphCache.Destroy;
begin
  GlyphLists.Free;
  inherited Destroy;
end;

function TGlyphCache.GetList(AWidth, AHeight: Integer): TGlyphList;
var
  I: Integer;
begin
  for I := GlyphLists.Count - 1 downto 0 do begin
    Result := GlyphLists[I];
    with Result do
      if (AWidth = Width) and (AHeight = Height) then Exit;
  end;
  {$IFNDEF Win32}
  Result := TGlyphList.Create(AWidth, AHeight);
  {$ELSE}
  Result := TGlyphList.CreateSize(AWidth, AHeight);
  {$ENDIF}
  GlyphLists.Add(Result);
end;

procedure TGlyphCache.ReturnList(List: TGlyphList);
begin
  if List = nil then Exit;
  if List.Count = 0 then begin
    GlyphLists.Remove(List);
    List.Free;
  end;
end;

function TGlyphCache.EMPTY: boolean;
begin
  Result := GlyphLists.Count = 0;
end;

var
  GlyphCache: TGlyphCache;

{ TButtonGlyph }
constructor TButtonGlyph.Create;
var
  I: TButtonState;
begin
  inherited Create;
  FColor := clBtnFace;
  FOriginal := TBitmap.Create;
  FOriginal.OnChange := GlyphChanged;
  FTransparentColor := clOlive;
  FNumGlyphs := 1;
  for I := Low(I) to High(I) do
    FIndexs[I] := -1;
  if GlyphCache = nil then GlyphCache := TGlyphCache.Create;
end;

destructor TButtonGlyph.Destroy;
begin
  FOriginal.Free;
  Invalidate;
  if Assigned(GlyphCache) and GlyphCache.EMPTY then begin
    GlyphCache.Free;
    GlyphCache := nil;
  end;
  inherited Destroy;
end;

procedure TButtonGlyph.Invalidate;
var
  I: TButtonState;
begin
  for I := Low(I) to High(I) do begin
    if FIndexs[I] <> -1 then FGlyphList.Delete(FIndexs[I]);
    FIndexs[I] := -1;
  end;
  GlyphCache.ReturnList(FGlyphList);
  FGlyphList := nil;
end;

procedure TButtonGlyph.GlyphChanged(Sender: TObject);
begin
  if Sender = FOriginal then begin
    FTransparentColor := FOriginal.TransparentColor;
    Invalidate;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TButtonGlyph.SetGlyph(Value: TBitmap);
var
  Glyphs: Integer;
begin
  Invalidate;
  FOriginal.Assign(Value);
  if (Value <> nil) and (Value.Height > 0) then begin
    FTransparentColor := Value.TransparentColor;
    if Value.Width mod Value.Height = 0 then begin
      Glyphs := Value.Width div Value.Height;
      if Glyphs > 4 then Glyphs := 1;
      SetNumGlyphs(Glyphs);
    end;
  end;
end;

procedure TButtonGlyph.SetNumGlyphs(Value: TNumGlyphs);
begin
  if (Value <> FNumGlyphs) and (Value > 0) then begin
    Invalidate;
    FNumGlyphs := Value;
  end;
end;

function TButtonGlyph.CreateButtonGlyph(State: TButtonState): Integer;
var
  TmpImage, MonoBmp: TBitmap;     
  iWidth, IHeight  : Integer;     
  IRect, ORect     : TRect;       
  I                : TButtonState;
begin
  if (State = bsDown) and (NumGlyphs < 3) then State := bsUp;
  Result := FIndexs[State];
  if Result <> -1 then Exit;
  iWidth := FOriginal.Width div FNumGlyphs;
  IHeight := FOriginal.Height;
  if (iWidth = 0) or (IHeight = 0) then exit;
  if FGlyphList = nil then begin
    if GlyphCache = nil then GlyphCache := TGlyphCache.Create;
    FGlyphList := GlyphCache.GetList(iWidth, IHeight);
  end;
  TmpImage := TBitmap.Create;
  try
    TmpImage.Width := iWidth;
    TmpImage.Height := IHeight;
    IRect := Rect(0, 0, iWidth, IHeight);
    TmpImage.Canvas.BRUSH.Color := FColor;
    TmpImage.Canvas.FillRect(IRect);
    I := State;
    if Ord(I) >= NumGlyphs then I := bsUp;
    ORect := Rect(Ord(I) * iWidth, 0, (Ord(I) + 1) * iWidth, IHeight);
    case State of
      bsUp, bsDown: begin
        TmpImage.Canvas.BrushCopy(IRect, FOriginal, ORect, FTransparentColor);
        FIndexs[State] := FGlyphList.AddMasked(TmpImage, FColor);
      end;
      bsExclusive: begin
        TmpImage.Canvas.CopyRect(IRect, FOriginal.Canvas, ORect);
        FIndexs[State] := FGlyphList.AddMasked(TmpImage, FTransparentColor);
      end;
      bsDisabled:
        if NumGlyphs > 1 then begin
          TmpImage.Canvas.BrushCopy(IRect, FOriginal, ORect, FTransparentColor);
          FIndexs[State] := FGlyphList.Add(TmpImage, nil);
        end
        else begin
          { Create a disabled version }
          MonoBmp := TBitmap.Create;
          try
            with MonoBmp do begin
              Assign(FOriginal);
              Canvas.BRUSH.Color := clBlack;
              Width := iWidth;
              if Monochrome then begin
                Canvas.Font.Color := clWhite;
                Monochrome := False;
                Canvas.BRUSH.Color := clWhite;
              end;
              Monochrome := True;
            end;
            with TmpImage.Canvas do begin
              BRUSH.Color := FColor;
              FillRect(IRect);
              BRUSH.Color := clBlack;
              Font.Color := clWhite;
              CopyMode := MERGEPAINT;
              Draw(IRect.Left + 1, IRect.Top + 1, MonoBmp);
              CopyMode := SRCAND;
              Draw(IRect.Left, IRect.Top, MonoBmp);
              BRUSH.Color := clBtnShadow;
              Font.Color := clBlack;
              CopyMode := SRCPAINT;
              Draw(IRect.Left, IRect.Top, MonoBmp);
              CopyMode := SRCCOPY;
            end;
            FIndexs[State] := FGlyphList.Add(TmpImage, nil);
          finally
            MonoBmp.Free;
          end;
        end;
    end;
  finally
    TmpImage.Free;
  end;
  Result := FIndexs[State];
  FOriginal.Dormant;
end;

procedure TButtonGlyph.DrawButtonGlyph(Canvas: TCanvas; X, Y: Integer;
    State: TButtonState; Transparent: boolean);
{var
  index: Integer;
begin
  if FOriginal = nil then Exit;
  if (FOriginal.Width = 0) or (FOriginal.Height = 0) then Exit;
  index := CreateButtonGlyph(State);
  FGlyphList.Draw(Canvas, X, Y, index);
end;}
var
  Index: Integer;
begin
  if FOriginal = nil then Exit;
  if (FOriginal.Width = 0) or (FOriginal.Height = 0) then Exit;
  Index := CreateButtonGlyph(State);
  if Transparent or (State = bsExclusive) then
    ImageList_DrawEx(FGlyphList.Handle, Index, Canvas.Handle, X, Y, 0, 0,
      clNone, clNone, ILD_Transparent)
  else
    ImageList_DrawEx(FGlyphList.Handle, Index, Canvas.Handle, X, Y, 0, 0,
      ColorToRGB(FColor), clNone, ILD_Normal);
end;

procedure TButtonGlyph.DrawButtonText(Canvas: TCanvas; const Caption: string;
    TextBounds: TRect; State: TButtonState);
var
  CString: array[0..255] of Char;
begin
  StrPCopy(CString, Caption);
  Canvas.BRUSH.Style := bsClear;
  if State = bsDisabled then begin
    with Canvas do begin
      OffsetRect(TextBounds, 1, 1);
      Font.Color := clWhite;
      DrawText(Handle, CString, Length(Caption), TextBounds, 0);
      OffsetRect(TextBounds, - 1, - 1);
      Font.Color := clDkGray;
      DrawText(Handle, CString, Length(Caption), TextBounds, 0);
    end;
  end
  else
    DrawText(Canvas.Handle, CString, - 1, TextBounds,
      DT_CENTER or DT_VCENTER or DT_SINGLELINE);
end;

procedure TButtonGlyph.CalcButtonLayout(Canvas: TCanvas; const CLIENT: TRect;
    const Caption: string; Layout: TButtonLayout; Margin, SPACING: Integer;
    var GlyphPos: TPoint; var TextBounds: TRect);
var
  TextPos                        : TPoint;
  ClientSize, GlyphSize, TextSize: TPoint;
  TotalSize                      : TPoint;
  CString                        : array[0..255] of Char;
  {SpaceLeft                     : Integer;              }
  
begin
  { calculate the item sizes }
  ClientSize := Point(CLIENT.Right - CLIENT.Left, CLIENT.Bottom -
    CLIENT.Top);
  
  if FOriginal <> nil then
    GlyphSize := Point(FOriginal.Width div FNumGlyphs, FOriginal.Height) else
    GlyphSize := Point(0, 0);
  
  if Length(Caption) > 0 then begin
    TextBounds := Rect(0, 0, CLIENT.Right - CLIENT.Left, 0);
    DrawText(Canvas.Handle, StrPCopy(CString, Caption), - 1, TextBounds,
      DT_CALCRECT);
  end
  else
    TextBounds := Rect(0, 0, 0, 0);
  TextSize := Point(TextBounds.Right - TextBounds.Left, TextBounds.Bottom -
    TextBounds.Top);

  { If the layout has the glyph on the right or the left, then both the
    text and the glyph are centered vertically.  If the glyph is on the top
    or the bottom, then both the text and the glyph are centered horizontally.}
  if Layout in [blGlyphLeft, blGlyphRight] then begin
    GlyphPos.Y := (ClientSize.Y div 2) - (GlyphSize.Y div 2);
    TextPos.Y := (ClientSize.Y div 2) - (TextSize.Y div 2);
  end
  else begin
    GlyphPos.X := (ClientSize.X div 2) - (GlyphSize.X div 2);
    TextPos.X := (ClientSize.X div 2) - (TextSize.X div 2);
  end;
  
  { if there is no text or no bitmap, then Spacing is irrelevant }
  if (TextSize.X = 0) or (GlyphSize.X = 0) then
    SPACING := 0;
  
  { adjust Margin and Spacing }
  if Margin = -1 then begin
    if SPACING = -1 then begin
      TotalSize := Point(GlyphSize.X + TextSize.X, GlyphSize.Y + TextSize.Y);
      if Layout in [blGlyphLeft, blGlyphRight] then
        Margin := (ClientSize.X - TotalSize.X) div 3
      else
        Margin := (ClientSize.Y - TotalSize.Y) div 3;
      SPACING := Margin;
    end
    else begin
      TotalSize := Point(GlyphSize.X + SPACING + TextSize.X, GlyphSize.Y +
        SPACING + TextSize.Y);
      if Layout in [blGlyphLeft, blGlyphRight] then
        Margin := (ClientSize.X div 2) - (TotalSize.X div 2)
      else
        Margin := (ClientSize.Y div 2) - (TotalSize.Y div 2);
    end;
  end
  else begin
    if SPACING = -1 then begin
      TotalSize := Point(ClientSize.X - (Margin + GlyphSize.X), ClientSize.Y -
        (Margin + GlyphSize.Y));
      if Layout in [blGlyphLeft, blGlyphRight] then
        SPACING := (TotalSize.X div 2) - (TextSize.X div 2)
      else
        SPACING := (TotalSize.Y div 2) - (TextSize.Y div 2);
    end;
  end;

  case Layout of
    blGlyphLeft: begin
      GlyphPos.X := Margin;
      TextPos.X := GlyphPos.X + GlyphSize.X + SPACING;
    end;
    blGlyphRight: begin
      GlyphPos.X := ClientSize.X - Margin - GlyphSize.X;
      TextPos.X := GlyphPos.X - SPACING - TextSize.X;
    end;
    blGlyphTop: begin
      GlyphPos.Y := Margin;
      TextPos.Y := GlyphPos.Y + GlyphSize.Y + SPACING;
    end;
    blGlyphBottom: begin
      GlyphPos.Y := ClientSize.Y - Margin - GlyphSize.Y;
      TextPos.Y := GlyphPos.Y - SPACING - TextSize.Y;
    end;
  end;
  
  { fixup the result variables }
  Inc(GlyphPos.X, CLIENT.Left);
  Inc(GlyphPos.Y, CLIENT.Top);
  OffsetRect(TextBounds, TextPos.X + CLIENT.Left, TextPos.Y + CLIENT.Top);
end;

function TButtonGlyph.Draw(Canvas: TCanvas; const CLIENT: TRect;
    const Caption: string; Layout: TButtonLayout; Margin, SPACING: Integer;
    State: TButtonState; Transparent: boolean): TRect;
var
  GlyphPos  : TPoint;
  TextBounds: TRect;
  {CString  : array[0..255] of Char;}
begin
  CalcButtonLayout(Canvas, CLIENT, Caption, Layout, Margin, SPACING,
    GlyphPos, TextBounds);
  DrawButtonGlyph(Canvas, GlyphPos.X, GlyphPos.Y, State, Transparent);
  DrawButtonText(Canvas, Caption, TextBounds, State);
  Result := TextBounds;
end;

procedure CreateBrushPattern;
var
  X, Y: Integer;
begin
  Pattern := TBitmap.Create;
  Pattern.Width := 8;
  Pattern.Height := 8;
  with Pattern.Canvas do begin
    BRUSH.Style := bsSolid;
    BRUSH.Color := clBtnFace;
    FillRect(Rect(0, 0, Pattern.Width, Pattern.Height));
    for Y := 0 to 7 do
      for X := 0 to 7 do
        if (Y mod 2) = (X mod 2) then { toggles between even/odd pixles }
          Pixels[X, Y] := clWhite; { on even/odd rows }
  end;
end;

{ TsohoCustomButton }
{$IFDEF Delphi4_or_5}
procedure TsohoCustomButton.ActionChange(Sender: TObject; CheckDefaults: Boolean);
begin
  inherited;
  if Sender is TCustomAction then
    with TCustomAction(Sender) do
    begin
      if not CheckDefaults or (Self.ImageIndex = -1) then
        Self.ImageIndex := ImageIndex;
    end;
end;
{$ENDIF}

function  TsohoCustomButton.GetFocused: boolean;
var Form: TForm;
begin
  Form := GetOwnerForm(Self);
  Result := FIsFocused;
  if Assigned(Form) then
    Result := Form.ActiveControl = Self;
end;

procedure TsohoCustomButton.SetDrawFocus(Value: boolean);
begin
  if FDrawFocus = Value then exit;
  FDrawFocus := Value;
  Invalidate;
end;

procedure TsohoCustomButton.DoButtonPaint (Canvas: TCanvas; Rect: TRect; IsDown, IsDefault: boolean);
begin
  if Assigned(FOnPaint) then FOnPaint(Self, Canvas, Rect, IsDown, IsDefault);
end;

procedure TsohoCustomButton.DoButtonAfterPaint (Canvas: TCanvas; Rect: TRect; IsDown, IsDefault: boolean);
begin
  if Assigned(FOnAfterPaint) then FOnAfterPaint(Self, Canvas, Rect, IsDown, IsDefault);
end;

procedure TsohoCustomButton.SetImageList(Value: TImageList);
begin
  FImageList := Value;
  Invalidate;
end;

procedure TsohoCustomButton.SetImageIndex(Value: LongInt);
begin
  FImageIndex := Value;
  Invalidate;
end;

procedure TsohoCustomButton.CMParentColorChanged(var message: TMessage);
begin
  if ParentColor then Color := GetParentColor(Self);
end;

function TsohoCustomButton.MouseUnderButton: boolean;
begin
  Result := FInRect;
end;

function TsohoCustomButton.GetPalette: HPALETTE;
begin
  Result := FGlyph.Palette;
end;

procedure TsohoCustomButton.GlyphChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TsohoCustomButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do Style := Style or BS_OWNERDRAW;
end;

procedure TsohoCustomButton.SetColor;
begin
  if FColor = Value then Exit;
  FColor := Value;
  Invalidate;
end;

procedure TsohoCustomButton.SetSoundName;
begin
  FSoundName := Value;
end;

procedure TsohoCustomButton.SetButtonStyle(ADefault: boolean);
begin
  if ADefault <> IsFocused then begin
    IsFocused := ADefault;
    Refresh;
  end;
end;

procedure TsohoCustomButton.CNMeasureItem(var message: TWMMeasureItem);
begin
  with message.MeasureItemStruct^ do begin
    ItemWidth := Width;
    ItemHeight := Height;
  end;
end;

procedure TsohoCustomButton.CNDrawItem(var message: TWMDrawItem);
begin
  if csDestroying in ComponentState then Exit;
  DrawItem(message.DrawItemStruct^);
end;

procedure TsohoCustomButton.Click;
begin
  if FSound <> nil then begin
    if FSoundName <> '' then begin
      if FSoundName[1] = '#' then
        FSound.PlayResourceWave(Copy(FSoundName, 2, Length(FSoundName)))
      else FSound.PlayWave(FSoundName);
    end;
  end;
  inherited Click;
end;

procedure TsohoCustomButton.DrawButton(ButtonState: TButtonState; Rect: TRect;
    IsDown, IsDefault: boolean);
var InnerRect, Txt: TRect;
    TmpBmp: TBitmap;
begin
  Canvas.Font := Self.Font;
  with InnerRect do begin
    Top := Rect.Top + BevelWidth;
    Left := Rect.Left + BevelWidth;
    Right := Rect.Right - BevelWidth;
    Bottom := Rect.Bottom - BevelWidth;
  end;
  if Assigned(FOnPaint) then DoButtonPaint(Canvas, Rect, IsDown, IsDefault)
  else begin
    with Canvas do begin
      InnerRect := SohoDrawButtonFace(Canvas, Rect, FColor, BevelWidth, bsNew, false, IsDown,
       false, false);
      if Assigned(FImageList) then begin
        TmpBmp := TBitmap.Create;
        FImageList.GetBitmap(FImageIndex, TmpBmp);
        DrawGlyphWithText(TmpBmp, Layout, Caption, Canvas, InnerRect, Txt);
        TmpBmp.Free;
      end
      else DrawGlyphWithText(Glyph, Layout, Caption, Canvas, InnerRect, Txt);
      TextInRect(Canvas, Caption, Txt, taLeftJustify);
      BRUSH.Style := bsSolid;
    end;
    DoButtonAfterPaint(Canvas, Rect, IsDown, IsDefault);
    if IsDefault then begin
      Canvas.Pen.Color := HighlightColor;
      InflateRect(Rect, - 2 - BevelWidth, - 2 - BevelWidth);
      if IsDown then OffsetRect(Rect, - 1, - 1);
      if FDrawFocus then DrawFocusRect(Canvas.Handle, Rect);
    end;
  end;
end;

procedure TsohoCustomButton.DrawItem(const DrawItemStruct: TDrawItemStruct);
var State: TButtonState;
  IsDown, IsDefault: boolean;
begin
  FCanvas.Handle := DrawItemStruct.HDC;
  with DrawItemStruct do begin
    IsDown := ItemState and ODS_SELECTED <> 0;
    IsDefault := ItemState and ODS_FOCUS <> 0;
    // IsFocused := ItemState and ODS_FOCUS <> 0;
    if not Enabled then State := bsDisabled
    else if IsDown then State := bsDown
    else State := bsUp;
  end;
  DrawButton(State, ClientRect, IsDown, IsDefault);
  FCanvas.Handle := 0;
end;

procedure TsohoCustomButton.CMFontChanged(var message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TsohoCustomButton.CMEnabledChanged(var message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TsohoCustomButton.SetHighlightColor;
begin
  if FHighlightColor = Value then Exit;
  FHighlightColor := Value;
  Invalidate;
end;

procedure TsohoCustomButton.SetBottomColor;
begin
  if FBottomColor = Value then Exit;
  FBottomColor := Value;
  Invalidate;
end;

procedure TsohoCustomButton.SetBevWidth;
begin
  if FBevWidth = Value then Exit;
  FBevWidth := Value;
  Invalidate;
end;

procedure TsohoCustomButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if (AComponent = FSound) and (Operation = opRemove) then FSound := nil;
  if (AComponent = FImageList) and (Operation = opRemove) then FImageList := nil;
  inherited Notification(AComponent, Operation);
end;

procedure TsohoCustomButton.SetLayout(Value: TButtonLayout);
begin
  if FLayout = Value then Exit;
  FLayout := Value; Invalidate;
end;

procedure TsohoCustomButton.SetGlyph(Value: TBitmap);
begin
  FGlyph.Assign(Value);
  Invalidate;
end;

procedure TsohoCustomButton.CMMouseEnter(var message: TMessage);
begin
  if not Flat then Exit;
  FInRect := True;
  Invalidate;
end;

procedure TsohoCustomButton.CMMouseLeave(var message: TMessage);
begin
  if not Flat then Exit;
  FInRect := False;
  Invalidate;
end;

procedure TsohoCustomButton.CMEnter(var Message: TMessage);
begin
  inherited;
  FIsFocused := true;
end;

procedure TsohoCustomButton.CMExit(var Message: TMessage);
begin
  inherited;
  FIsFocused := false;
end;

procedure TsohoCustomButton.SetFlat(Value: boolean);
begin
  if FFlat = Value then Exit;
  FFlat := Value;
  Invalidate;
end;

constructor TsohoCustomButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDrawFocus := true;;
  FGlyph := TBitmap.Create;
  FGlyph.OnChange := GlyphChanged;
  if Owner is TForm then FSound := FindSound(AOwner as TForm);
  FSoundName := '';
  FLayout := blGlyphLeft;
  FCanvas := TCanvas.Create;
  FHighlightColor := clWhite;
  FBottomColor := clBlack;
  FBevWidth := 1;
  FColor := clBtnFace;
  FFlat := False;
  FInRect := False;
  Cursor := crHandPoint;
  Height := 24;
  Width := 75;
  FImageList := nil;
  FImageIndex := -1;
end;

destructor TsohoCustomButton.Destroy;
begin
  FCanvas.Free;
  FGlyph.Free;
  inherited Destroy;
end;

{ TsohoTransButton }
procedure TsohoTransButton.SetKind(Value: TsohoButtonKind);
begin
  if FKind = Value then Exit;
  FKind := Value;
  case FKind of
    sbOk..sbClose: begin Glyph.Handle := ResBitmap(sbResNames[FKind]);
      default := Value in [sbOk, sbYes];
      CANCEL := Value in [sbCancel, sbNo];
      if ((csLoading in ComponentState) and (Caption = ''))
        or (not (csLoading in ComponentState)) then begin
        default := Value in [sbOk, sbYes];
        CANCEL := Value in [sbCancel, sbNo];
        Caption := sbResCaptions[FKind];
      end;
      ModalResult := sbModalResults[FKind];
    end;
    sbCustom: ModalResult := mrNone;
  end;
  Invalidate;
end;

procedure TsohoTransButton.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var DoRecreate : boolean;
begin
  DoRecreate := (Top <> ATop) or (Left <> ALeft) or (Height <> AHeight) or
    (Width <> AWidth);
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  if DoRecreate then RecreateWnd;
end;

type
  TParentControl = class(TWinControl);

procedure CopyParentImage(Control: TControl; Dest: TCanvas);
var
  I, Count, X, Y, SaveIndex: Integer;
  DC: HDC;
  R, SelfR, CtlR: TRect;
begin
  if (Control = nil) or (Control.Parent = nil) then Exit;
  Count := Control.Parent.ControlCount;
  DC := Dest.Handle;
{$IFDEF WIN32}
  with Control.Parent do ControlState := ControlState + [csPaintCopy];
  try
{$ENDIF}
    with Control do begin
      SelfR := Bounds(Left, Top, Width, Height);
      X := -Left; Y := -Top;
    end;
    { Copy parent control image }
    SaveIndex := SaveDC(DC);
    try
      SetViewportOrgEx(DC, X, Y, nil);
      IntersectClipRect(DC, 0, 0, Control.Parent.ClientWidth,
        Control.Parent.ClientHeight);
      with TParentControl(Control.Parent) do begin
        Perform(WM_ERASEBKGND, DC, 0);
        PaintWindow(DC);
      end;
    finally
      RestoreDC(DC, SaveIndex);
    end;
    { Copy images of graphic controls }
    for I := 0 to Count - 1 do begin
      if Control.Parent.Controls[I] = Control then Break
      else if (Control.Parent.Controls[I] <> nil) and
        (Control.Parent.Controls[I] is TGraphicControl) then
      begin
        with TGraphicControl(Control.Parent.Controls[I]) do begin
          CtlR := Bounds(Left, Top, Width, Height);
          if Bool(IntersectRect(R, SelfR, CtlR)) and Visible then begin
{$IFDEF WIN32}
            ControlState := ControlState + [csPaintCopy];
{$ENDIF}
            SaveIndex := SaveDC(DC);
            try
              SaveIndex := SaveDC(DC);
              SetViewportOrgEx(DC, Left + X, Top + Y, nil);
              IntersectClipRect(DC, 0, 0, Width, Height);
              Perform(WM_PAINT, DC, 0);
            finally
              RestoreDC(DC, SaveIndex);
{$IFDEF WIN32}
              ControlState := ControlState - [csPaintCopy];
{$ENDIF}
            end;
          end;
        end;
      end;
    end;
{$IFDEF WIN32}
  finally
    with Control.Parent do ControlState := ControlState - [csPaintCopy];
  end;
{$ENDIF}
end;

procedure TsohoTransButton.WMEraseBkgnd(var message: TWMEraseBkgnd);
var Rect : TRect;
begin
  if (csDesigning in ComponentState) then inherited
  else begin
    Message.Result := 1;
    Rect := GetClientRect;
    FBack.Free;
    FBack := TBitmap.Create;
    FBack.Height := Rect.Bottom - Rect.Top;
    FBack.Width := Rect.Right - Rect.Left;
    CopyParentImage(Self, FBack.Canvas);
  end;
end;

procedure TsohoTransButton.DrawButton(ButtonState: TButtonState; Rect: TRect;
    IsDown, IsDefault: boolean);
var InnerRect, Txt: TRect;
    TmpBmp : TBitmap;
begin
  if (csDesigning in ComponentState) then
    inherited DrawButton(ButtonState, Rect, IsDown, IsDefault)
  else begin
    Canvas.Font := Self.Font;
    with InnerRect do begin
      Top := Rect.Top + BevelWidth;
      Left := Rect.Left + BevelWidth;
      Right := Rect.Right - BevelWidth;
      Bottom := Rect.Bottom - BevelWidth;
    end;
    if Assigned(FOnPaint) then DoButtonPaint(Canvas, Rect, IsDown, IsDefault)
    else begin
      with Canvas do begin
        BitBlt(Canvas.Handle, 0, 0, Rect.Right - Rect.Left, Rect.Bottom - Rect.Top,
          FBack.Canvas.Handle, 0, 0, SRCCOPY);

        if IsDown then Frame3D(Canvas, InnerRect, BottomColor, HighlightColor, BevelWidth)
        else begin
          if FFlat and (FInRect or (csDesigning in ComponentState)) then
            InnerRect := SohoDrawButtonFace(Canvas, Rect, FColor, BevelWidth, bsNew, false, IsDown,
              Focused, true);
          if not FFlat then
            InnerRect := SohoDrawButtonFace(Canvas, Rect, FColor, BevelWidth, bsNew, false, IsDown,
              Focused, true);
        end;
        if IsDown then OffsetRect(InnerRect, 1, 1);
        BRUSH.Style := bsClear;
        if Assigned(FImageList) then begin
          TmpBmp := TBitmap.Create;
          FImageList.GetBitmap(FImageIndex, TmpBmp);
          DrawGlyphWithText(TmpBmp, Layout, Caption, Canvas, InnerRect, Txt);
          TmpBmp.Free;
        end
        else DrawGlyphWithText(Glyph, Layout, Caption, Canvas, InnerRect, Txt);
        // DrawGlyphWithText(Glyph, Layout, Caption, Canvas, InnerRect, Txt);
        TextInRect(Canvas, Caption, Txt, taLeftJustify);
      end;
      DoButtonAfterPaint(Canvas, Rect, IsDown, IsDefault);
      if IsDefault then begin
        Canvas.Pen.Color := clBlack;
        InflateRect(Rect, - 2 - BevelWidth, - 2 - BevelWidth);
        if FDrawFocus then DrawFocusRect(Canvas.Handle, Rect);
      end;
    end;
  end;
end;

procedure TsohoTransButton.WMKillFocus(var message: TWMKillFocus);
begin
  Repaint;
  inherited;
end;

procedure TsohoTransButton.WMSetFocus(var message: TWMSetFocus);
begin
  Repaint;
  inherited;
end;

constructor TsohoTransButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FKind := sbCustom;
  FBack := TBitmap.Create;
  FInRect := False;
end;

destructor TsohoTransButton.Destroy;
begin
  FBack.Free;
  inherited Destroy;
end;

procedure TsohoTransButton.Click;
begin
  inherited Click;
  Repaint;
end;

procedure TsohoTransButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  if not (csDesigning in ComponentState) then
    Params.ExStyle := Params.ExStyle + WS_EX_TRANSPARENT;
end;

{ TsohoBitBtn }
procedure TsohoBitBtn.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (aComponent = FDropDownMenu) and (Operation = opRemove) then
    FDropDownMenu := nil;
end;

procedure TsohoBitBtn.CMMouseEnter(var message: TMessage);
begin
  FInRect := True;
  Invalidate;
end;

procedure TsohoBitBtn.CMMouseLeave(var message: TMessage);
begin
  FInRect := false;
  Invalidate;
end;

procedure TsohoBitBtn.CMParentColorChanged(var message: TMessage);
begin
  inherited;
  if ParentColor then begin
    if Parent <> nil then FBackColor := GetParentColor(Self)
    else FBackColor := GetControlColor(Owner as TForm);
  end;
end;

procedure TsohoBitBtn.SetType(Value: TsohoButtonType);
begin
  if fType = Value then Exit;
  fType := Value;
  Invalidate;
end;

procedure TsohoBitBtn.SetBackColor(Value: TColor);
begin
  if FBackColor = Value then Exit;
  if not (csLoading in ComponentState) and
    not (csReading in ComponentState) then ParentColor := False;
  FBackColor := Value;
  Repaint;
end;

procedure TsohoBitBtn.WMLButtonDown(var message: TWMLButtonDown);
var X0, Y0, A, B, X, Y: Integer;
begin
  {   ,   -      ? }
  if fType = btRectangle then inherited
  else begin
    A := Width div 2;
    B := Height div 2;
    X0 := Left + A;
    Y0 := Top + B;
    X := Left + message.XPos;
    Y := Top + message.YPos;
    if ((X0 - X) * (X0 - X) / (A * A) +
      (Y0 - Y) * (Y0 - Y) / (B * B)) <= 1 then inherited;
  end;
end;

constructor TsohoBitBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FNewGlyph := TButtonGlyph.Create;
  TButtonGlyph(FNewGlyph).OnChange := GlyphChanged;
  FStyle := bsNew;
  FKind := bkCustom;
  FLayout := blGlyphLeft;
  FSpacing := 4;
  FMargin := -1;
  Cursor := crHandPoint;
  Flat := False;
  Height := 24;
  Width := 75;
  FColor := clBtnFace;
  FBackColor := clBtnFace;
  fType := btRectangle;
  FMarkDropDown := True;
end;

procedure TsohoBitBtn.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  DoMouseUp(Button, Shift, X, Y);
end;

procedure TsohoBitBtn.DoMouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DoClick: Boolean;
begin
  if FDragging and (Button = mbLeft) then begin
    FDragging := False;
    DoClick := (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight);
    {if FGroupIndex = 0 then begin
      FState := rbsUp;
      FMouseInControl := False;
      if DoClick and not (FState in [rbsExclusive, rbsDown]) then
        Repaint
      else Invalidate;
    end
    else }if DoClick then begin
      if Assigned(FDropDownMenu) then begin
        FDown := not FDown;
        Repaint;
      end;
    end
    else begin
      FDown := false;
      Repaint;
    end;
    // if DoClick and not FMenuTracking then Click;
  end;
end;

procedure KillMessage(Wnd: HWnd; Msg: Cardinal);
{ Delete the requested message from the queue, but throw back }
{ any WM_QUIT msgs that PeekMessage may also return.          }
{ Copied from DbGrid.pas                                      }
var
  M: TMsg;
begin
  M.Message := 0;
  if PeekMessage(M, Wnd, Msg, Msg, PM_REMOVE) and (M.Message = WM_QUIT) then
    PostQuitMessage(M.WParam);
end;

procedure TsohoBitBtn.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  P: TPoint;
  Msg: TMsg;
begin
  if FMenuTracking then Exit;
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and Enabled then begin
    if Assigned(FDropDownMenu) then begin
      if not FDown then begin
        FDown := true;
        Repaint;
      end
      else FDown := false;
    end;
    FDragging := True;
    FMenuTracking := True;
    try
      P := GetDropDownMenuPos;
      if CheckMenuDropDown(PointToSmallPoint(P), False) then
        DoMouseUp(Button, Shift, X, Y);
      if PeekMessage(Msg, 0, 0, 0, PM_NOREMOVE) then begin
        if (Msg.Message = WM_LBUTTONDOWN) or (Msg.Message = WM_LBUTTONDBLCLK) then
        begin
          P := ScreenToClient(Msg.Pt);
          if (P.X >= 0) and (P.X < ClientWidth) and (P.Y >= 0)
            and (P.Y <= ClientHeight) then KillMessage(0, Msg.Message);
        end;
      end;
    finally
      FMenuTracking := False;
      FDown := false;
    end;
  end;
end;

procedure TsohoBitBtn.SetDropDownMenu(Value: TPopupMenu);
begin
  FDropDownMenu := Value;
  if Value <> nil then Value.FreeNotification(Self);
  if FMarkDropDown then Invalidate;
end;

procedure TsohoBitBtn.SetMarkDropDown(Value: Boolean);
begin
  if Value <> FMarkDropDown then begin
    FMarkDropDown := Value;
    Invalidate;
  end;
end;

function TsohoBitBtn.GetDropDownMenuPos: TPoint;
begin
  if Assigned(FDropDownMenu) then begin
    if MenuPosition = dmpBottom then begin
      case FDropDownMenu.Alignment of
        paLeft: Result := Point(-1, Height);
        paRight: Result := Point(Width + 1, Height);
        else {paCenter} Result := Point(Width div 2, Height);
      end;
    end
    else { dmpRight } begin
      case FDropDownMenu.Alignment of
        paLeft: Result := Point(Width, -1);
        paRight: Result := Point(-1, -1);
        else {paCenter} Result := Point(Width div 2, Height);
      end;
    end;
  end else Result := Point(0, 0);
end;

function TsohoBitBtn.CheckBtnMenuDropDown: Boolean;
begin
  Result := CheckMenuDropDown(PointToSmallPoint(GetDropDownMenuPos), True);
end;

function TsohoBitBtn.CheckMenuDropDown(const Pos: TSmallPoint;
  Manual: Boolean): Boolean;
var
  Form: TCustomForm;
begin
  Result := False;
  if csDesigning in ComponentState then Exit;
  if Assigned(FDropDownMenu) and (DropDownMenu.AutoPopup or Manual) then
  begin
    Form := GetParentForm(Self);
    if Form <> nil then Form.SendCancelMode(nil);
    DropDownMenu.PopupComponent := Self;
    with ClientToScreen(SmallPointToPoint(Pos)) do DropDownMenu.Popup(X, Y);
    Result := True;
  end;
end;

destructor TsohoBitBtn.Destroy;
begin
  TButtonGlyph(FNewGlyph).Free;
  inherited Destroy;
end;

procedure TsohoBitBtn.SetButtonStyle(ADefault: boolean);
begin
  if ADefault <> IsFocused then begin
    IsFocused := ADefault;
    Refresh;
  end;
end;

procedure TsohoBitBtn.Click;
var Form: TForm;
  CONTROL: TWinControl;
begin
  case FKind of
    bkClose: begin
      Form := GetOwnerForm(Self);
      if Form <> nil then Form.Close
      else inherited Click;
    end;
    bkHelp: begin
      CONTROL := Self;
      while (CONTROL <> nil) and (CONTROL.HelpContext = 0) do
        CONTROL := CONTROL.Parent;
      if CONTROL <> nil then Application.HelpContext(CONTROL.HelpContext)
      else inherited Click;
    end;
  else inherited Click;
  end;
end;

procedure TsohoBitBtn.DrawPopupMark(Canvas: TCanvas; X, Y: Integer);
var aColor: TColor;

  procedure DrawMark;
  var
    I: Integer;
  begin
    with Canvas do begin
      for I := 0 to 6 do begin
        Pixels[X + I, Y - 1] := aColor;
        if (I > 0) and (I < 6) then begin
          Pixels[X + I, Y] := aColor;
          if (I > 1) and (I < 5) then Pixels[X + I, Y + 1] := aColor;
        end;
      end;
      Pixels[X + 3, Y + 2] := aColor;
    end;
  end;

begin
  if not Enabled then begin
    aColor := clBtnHighlight;
    Inc(X, 1); Inc(Y, 1);
    DrawMark;
    Dec(X, 1); Dec(Y, 1);
    aColor := clBtnShadow;
  end
  else AColor := clBtnText;
  DrawMark;
end;

procedure TsohoBitBtn.DrawButton(ButtonState: TButtonState; Rect: TRect;
    IsDown, IsDefault: boolean);
var NewStyle: boolean;
  R, TextBounds: TRect;
  // Bevel        : Integer;

  procedure DrawRectButton;
  var TmpGlyph: TButtonGlyph;
      GlyphPos: TPoint;
  begin
    TmpGlyph := nil;
    if not Assigned(FImageList) then begin
      TButtonGlyph(FNewGlyph).Color := FColor;
      TButtonGlyph(FNewGlyph).FTransparentColor :=
        TButtonGlyph(FNewGlyph).Glyph.Canvas.Pixels[0, TButtonGlyph(FNewGlyph).Glyph.Height - 1];
    end
    else begin
      TmpGlyph := TButtonGlyph.Create;
      FImageList.GetBitmap(FImageIndex, TmpGlyph.Glyph);
      TmpGlyph.Color := TmpGlyph.Glyph.Canvas.Pixels[0, TmpGlyph.Glyph.Height - 1];
    end;
    Canvas.Font := Font;
    if Assigned(FOnPaint) then DoButtonPaint(Canvas, Rect, IsDown or FDown, IsDefault)
    else begin
       if Assigned(TmpGlyph) then begin
         TmpGlyph.CalcButtonLayout(Canvas, R, Caption, FLayout, FMargin,
           FSPACING, GlyphPos, TextBounds);
         if Enabled then
           FImageList.Draw(Canvas, GlyphPos.X, GlyphPos.Y, FImageIndex
{$IFDEF Delphi4_or_5}
                            ,true
{$ENDIF}
)

         else begin
           if NumGlyphs > 1 then
              FImageList.Draw(Canvas, GlyphPos.X, GlyphPos.Y, FImageIndex + 1
{$IFDEF Delphi4_or_5}
		,false
{$ENDIF}
)
           else
              FImageList.Draw(Canvas, GlyphPos.X, GlyphPos.Y, FImageIndex
{$IFDEF Delphi4_or_5}
			,false
{$ENDIF}
);
         end;
         TmpGlyph.DrawButtonText(Canvas, Caption, TextBounds, ButtonState);
       end
       else TextBounds := TButtonGlyph(FNewGlyph).Draw(Canvas, R, Caption, FLayout,
            FMargin, FSpacing, ButtonState, false);
       if FMarkDropDown and Assigned(FDropDownMenu) then begin
         //      
         DrawPopupMark(Canvas, TextBounds.Right + 2,
           trunc((TextBounds.Top + TextBounds.Bottom)/2));
       end;
       DoButtonAfterPaint(Canvas, Rect, IsDown or FDown, IsDefault);
       if IsDefault then begin
         Canvas.BRUSH.Color := FColor;
         if NewStyle then TextBounds := R;
         if NewStyle then begin
           InflateRect(TextBounds, - 2, - 2);
           if IsDown then OffsetRect(TextBounds, - 1, - 1);
         end
         else InflateRect(TextBounds, 1, 1);
       end;
    end;
    if Assigned(FImageList) then TmpGlyph.Free;
  end;

  procedure DrawFocusCircle(ACanvas: TCanvas; var Rect: TRect; BtnColor: TColor);
  begin
    with ACanvas do begin
      Pen.Color := clBlack xor BtnColor;
      Pen.Width := 1;
      Pen.Style := psDot;
      Pen.Mode := pmXor;
      BRUSH.Style := bsClear;
      Ellipse(Rect.Left, Rect.Top, Rect.Right, Rect.Bottom);
    end;
  end;
  
  procedure DrawEllipseButtonText(const Caption: string; TextBounds: TRect);
  var CString: array[0..255] of Char;
  begin
    StrPCopy(CString, Caption);
    Canvas.BRUSH.Style := bsClear;
    if ButtonState = bsDisabled then begin
      with Canvas do begin
        OffsetRect(TextBounds, 1, 1);
        Font.Color := clBtnHighlight;
        DrawText(Handle, CString, Length(Caption), TextBounds,
          DT_CENTER or DT_VCENTER or DT_SINGLELINE);
        OffsetRect(TextBounds, - 1, - 1);
        Font.Color := clBtnShadow;
        DrawText(Handle, CString, Length(Caption), TextBounds,
          DT_CENTER or DT_VCENTER or DT_SINGLELINE);
      end;
    end
    else begin
      Canvas.Font := Font;
      DrawText(Canvas.Handle, CString, - 1, TextBounds,
        DT_CENTER or DT_VCENTER or DT_SINGLELINE);
    end;
  end;

  procedure DrawEllipseButton;
  var X, Y, R1, R2: Integer;
    FaceColor: TColor;
  begin
    if Assigned(FOnPaint) then DoButtonPaint(Canvas, Rect, IsDown or FDown, IsDefault)
    else begin
       TextBounds := R;
       FaceColor := Color;
       with Canvas do begin
         Pen.Style := psSolid;
         Pen.Mode := pmCopy;
         Pen.Width := 1;
         BRUSH.Color := Self.Color;
         BRUSH.Style := bsSolid;
         InflateRect(R, - 1, - 1);
         if IsDown or FDown then begin
           Pen.Color := clBtnShadow;
           Ellipse(R.Left, R.Top, R.Right, R.Bottom);
           FloodFill((R.Left + R.Right) div 2, (R.Top + R.Bottom) div 2, FaceColor,
             fsBorder);
           Pen.Color := clBtnHighlight;
           Ellipse(R.Left + 1, R.Top + 1, R.Right, R.Bottom);
           R1 := trunc((R.Right - R.Left) * sin(45) / 2);
           R2 := trunc((R.Bottom - R.Top) * sin(45) / 2);
           X := R.Left + 1 + (R.Right - R.Left) div 2 - R1;
           Y := R.Top + 1 + (R.Bottom - R.Top) div 2 + R2;
           Pen.Color := clWindowFrame;
           Ellipse(R.Left + 1, R.Top + 1, R.Right - 1, R.Bottom - 1);
           Pen.Color := clBtnShadow;
           Arc(R.Left + 1, R.Top + 1, R.Right - 1, R.Bottom - 1, X, Y, X + 2 * R1, Y - 2 * R2);
         end
         else begin
           Pen.Color := clBtnShadow;
           Ellipse(R.Left, R.Top, R.Right - 1, R.Bottom - 1);
           FloodFill((R.Left + R.Right) div 2, (R.Top + R.Bottom) div 2, FaceColor,
             fsBorder);
           Pen.Color := clWindowFrame;
           Ellipse(R.Left + 2, R.Top + 2, R.Right, R.Bottom);
           R1 := trunc((R.Right - R.Left) * sin(45) / 2);
           R2 := trunc((R.Bottom - R.Top) * sin(45) / 2);
           X := R.Left + 1 + (R.Right - R.Left) div 2 - R1;
           Y := R.Top + 1 + (R.Bottom - R.Top) div 2 + R2;
           Pen.Color := clBtnHighlight;
           Ellipse(R.Left + 1, R.Top + 1, R.Right - 1, R.Bottom - 1);
           Pen.Color := clBtnShadow;
           Arc(R.Left + 1, R.Top + 1, R.Right - 1, R.Bottom - 1, X, Y, X + 2 * R1, Y - 2 * R2);
         end;
       end;
       if IsDown then
         OffsetRect(TextBounds, 1, 1);
       DrawEllipseButtonText(Caption, TextBounds);
       InflateRect(R, - 3, - 3);
       DoButtonAfterPaint(Canvas, Rect, IsDown or FDown, IsDefault);
       if IsDefault then DrawFocusCircle(Canvas, R, Color);
    end;
  end;
begin
  NewStyle := ((Style = bsAutoDetect) and NewStyleControls) or (Style = bsNew);

  Canvas.Font := Self.Font;

  {if NewStyle then Bevel := 1
  else Bevel := 2;}

  R := ClientRect;
  if Flat and not (FInRect or (csDesigning in ComponentState)) then begin
    Canvas.BRUSH.Color := FBackColor;
    Canvas.FillRect(R);
    if fType = btRectangle then DrawRectButton
    else DrawEllipseButton;
  end
  else
    case fType of
      btRectangle: begin
        R := NewDrawButtonFace(Canvas, ClientRect, FColor, FInRect,
           IsDown or FDown,
           IsFocused, IsDefault, FDrawFocus);
        DrawRectButton;
      end;
      btEllipse: begin
        Canvas.BRUSH.Color := FBackColor;
        Canvas.FillRect(R);
        DrawEllipseButton;
      end;
    end;
end;

procedure TsohoBitBtn.WMLButtonDblClk(var message: TWMLButtonDblClk);
begin
  Perform(WM_LBUTTONDOWN, message.KEYS, Longint(message.Pos));
end;

function TsohoBitBtn.GetPalette: HPALETTE;
begin
  Result := Glyph.Palette;
end;

procedure TsohoBitBtn.SetGlyph(Value: TBitmap);
begin
  TButtonGlyph(FNewGlyph).Glyph := Value as TBitmap;
  FModifiedGlyph := True;
  Invalidate;
end;

function TsohoBitBtn.GetGlyph: TBitmap;
begin
  Result := TButtonGlyph(FNewGlyph).Glyph;
end;

procedure TsohoBitBtn.GlyphChanged(Sender: TObject);
begin
  Invalidate;
end;

function TsohoBitBtn.IsCustom: boolean;
begin
  Result := Kind = bkCustom;
end;

procedure TsohoBitBtn.SetStyle(Value: TButtonStyle);
begin
  if Value <> FStyle then begin
    FStyle := Value;
    Invalidate;
  end;
end;

procedure TsohoBitBtn.SetKind(Value: TBitBtnKind);
begin
  if Value <> FKind then begin
    if Value <> bkCustom then begin
      default := Value in [bkOK, bkYes];
      CANCEL := Value in [bkCancel, bkNo];

      if ((csLoading in ComponentState) and (Caption = '')) or
        (not (csLoading in ComponentState)) then begin
          if BitBtnCaptions[Value] <> '' then
              Caption := BitBtnCaptions[Value];
      end;
      
      ModalResult := BitBtnModalResults[Value];
      TButtonGlyph(FNewGlyph).Glyph := GetBitBtnGlyph(Value);
      NumGlyphs := 2;
      FModifiedGlyph := False;
    end;
    FKind := Value;
    Invalidate;
  end;
end;

function TsohoBitBtn.IsCustomCaption: boolean;
begin
  {$IFDEF Delphi3}
   Result := CompareStr(Caption, BitBtnCaptions[FKind]) <> 0;
  {$ELSE}
   Result := AnsiCompareStr(Caption, BitBtnCaptions[FKind]) <> 0;
  {$ENDIF}
end;

function TsohoBitBtn.GetKind: TBitBtnKind;
begin
  if FKind <> bkCustom then
    if ((FKind in [bkOK, bkYes]) and (not default)) or
      ((FKind in [bkCancel, bkNo]) and (not CANCEL)) or
      (ModalResult <> BitBtnModalResults[FKind]) or
      FModifiedGlyph then
      FKind := bkCustom;
  Result := FKind;
end;

procedure TsohoBitBtn.SetLayout(Value: TButtonLayout);
begin
  if FLayout <> Value then begin
    FLayout := Value;
    Invalidate;
  end;
end;

function TsohoBitBtn.GetNumGlyphs: TNumGlyphs;
begin
  Result := TButtonGlyph(FNewGlyph).NumGlyphs;
end;

procedure TsohoBitBtn.SetNumGlyphs(Value: TNumGlyphs);
begin
  if Value < 0 then Value := 1
  else if Value > 4 then Value := 4;
  if Value <> TButtonGlyph(FNewGlyph).NumGlyphs then begin
    TButtonGlyph(FNewGlyph).NumGlyphs := Value;
    Invalidate;
  end;
end;

procedure TsohoBitBtn.SetSpacing(Value: Integer);
begin
  if FSpacing <> Value then begin
    FSpacing := Value;
    Invalidate;
  end;
end;

procedure TsohoBitBtn.SetMargin(Value: Integer);
begin
  if (Value <> FMargin) and (Value >= - 1) then begin
    FMargin := Value;
    Invalidate;
  end;
end;

procedure DestroyLocals;
var I: TBitBtnKind;
begin
  for I := Low(TBitBtnKind) to High(TBitBtnKind) do BitBtnGlyphs[I].Free;
end;

{ TsohoSpeedButton }
constructor TsohoSpeedButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FGlyph := TButtonGlyph.Create;
  TButtonGlyph(FGlyph).OnChange := GlyphChanged;
  SetBounds(0, 0, 23, 22);
  ControlStyle := [csCaptureMouse, csDoubleClicks];
  ParentFont := True;
  Color := clBtnFace;
  FSpacing := 4;
  FMargin := -1;
  FLayout := blGlyphLeft;
  FTransparent := True;
end;

destructor TsohoSpeedButton.Destroy;
begin
  inherited Destroy;
  TButtonGlyph(FGlyph).Free;
end;

procedure TsohoSpeedButton.Paint;
const
  DownStyles: array[Boolean] of Integer = (BDR_RAISEDINNER, BDR_SUNKENOUTER);
  FillStyles: array[Boolean] of Integer = (BF_MIDDLE, 0);
var
  TextBounds, PaintRect: TRect;
  DrawFlags: Integer;
  TmpGlyph: TButtonGlyph;
  GlyphPos: TPoint;
begin
  if not Enabled then begin
    FState := bsDisabled;
    FDragging := False;
  end
  else if FState = bsDisabled then
    if FDown and (GroupIndex <> 0) then FState := bsExclusive
    else  FState := bsUp;
  Canvas.Font := Self.Font;
  PaintRect := Rect(0, 0, Width, Height);
  if not FFlat then begin
    if not Transparent then begin
      DrawFlags := DFCS_BUTTONPUSH or DFCS_ADJUSTRECT;
      if FState in [bsDown, bsExclusive] then
        DrawFlags := DrawFlags or DFCS_PUSHED;
      DrawFrameControl(Canvas.Handle, PaintRect, DFC_BUTTON, DrawFlags);
    end
    else begin
      if FState in [bsDown, bsExclusive] then
        frame3d(canvas, PaintRect ,clBtnShadow,clBtnHighlight, 1)
      else
        frame3d(canvas, PaintRect ,clBtnHighlight,clBtnShadow, 1);
    end;
  end
  else begin
    if (FState in [bsDown, bsExclusive]) or
      (FMouseInControl and (FState <> bsDisabled)) or
      (csDesigning in ComponentState) then
      DrawEdge(Canvas.Handle, PaintRect, DownStyles[FState in [bsDown, bsExclusive]],
        FillStyles[Transparent] or BF_RECT)
    else
    if not Transparent then begin
      Canvas.Brush.Color := Color;
      Canvas.FillRect(PaintRect);
    end;
    InflateRect(PaintRect, -1, -1);
  end;
  if FState in [bsDown, bsExclusive] then begin
{$IFDEF Delphi4_or_5}
    if (FState = bsExclusive) and (not FFlat or not FMouseInControl) then
      Canvas.Brush.Bitmap := AllocPatternBitmap(clBtnFace, clBtnHighlight);
{$ENDIF}
    OffsetRect(PaintRect, 1, 1);
  end;

  TmpGlyph := nil;
  if not Assigned(FImageList) then begin
    TButtonGlyph(FGlyph).Color := Color;
    TButtonGlyph(FGlyph).FTransparentColor :=
      TButtonGlyph(FGlyph).Glyph.Canvas.Pixels[0, TButtonGlyph(FGlyph).Glyph.Height - 1];
  end
  else begin
    TmpGlyph := TButtonGlyph.Create;
    FImageList.GetBitmap(FImageIndex, TmpGlyph.Glyph);
    TmpGlyph.Color := TmpGlyph.Glyph.Canvas.Pixels[0, TmpGlyph.Glyph.Height - 1];
  end;

  if Assigned(TmpGlyph) then begin
    TmpGlyph.CalcButtonLayout(Canvas, PaintRect, Caption, FLayout, FMargin,
      FSpacing, GlyphPos, TextBounds);
    if Enabled then
      FImageList.Draw(Canvas, GlyphPos.X, GlyphPos.Y, FImageIndex
{$IFDEF Delphi4_or_5}
,true
{$ENDIF}
)
    else begin
      if NumGlyphs > 1 then
         FImageList.Draw(Canvas, GlyphPos.X, GlyphPos.Y, FImageIndex + 1
{$IFDEF Delphi4_or_5}
,false
{$ENDIF}
)
      else
         FImageList.Draw(Canvas, GlyphPos.X, GlyphPos.Y, FImageIndex
{$IFDEF Delphi4_or_5}
,false
{$ENDIF}
);
    end;
    TmpGlyph.DrawButtonText(Canvas, Caption, TextBounds, FState);
  end
  else TextBounds := TButtonGlyph(FGlyph).Draw(Canvas, PaintRect, Caption, FLayout,
       FMargin, FSpacing, FState, Transparent);
  if Assigned(TmpGlyph) then TmpGlyph.Free;
end;

procedure TsohoSpeedButton.UpdateTracking;
var P: TPoint;
begin
  if FFlat then begin
    if Enabled then begin
      GetCursorPos(P);
      FMouseInControl := not (FindDragTarget(P, True) = Self);
      if FMouseInControl then Perform(CM_MOUSELEAVE, 0, 0)
      else Perform(CM_MOUSEENTER, 0, 0);
    end;
  end;
end;

procedure TsohoSpeedButton.Loaded;
var
  State: TButtonState;
begin
  inherited Loaded;
  if Enabled then State := bsUp
  else State := bsDisabled;
  TButtonGlyph(FGlyph).CreateButtonGlyph(State);
end;

procedure TsohoSpeedButton.DoMouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DoClick: Boolean;
begin
  if FDragging and (Button = mbLeft) then begin
    FDragging := False;
    DoClick := (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight);
    if DoClick then begin
      if Assigned(FDropDownMenu) then begin
        FDown := not FDown;
        FState := bsUp;
        Repaint;
      end;
    end
    else begin
      FDown := false;
      Repaint;
    end;
  end;
end;

procedure TsohoSpeedButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
   P:TPoint;
   Msg:TMsg;
begin
  if FMenuTracking then Exit;
  inherited MouseDown(Button, Shift, X, Y);
  if (Button = mbLeft) and Enabled then begin
    FDown := not FDown;
    if FDown then FState := bsDown
    else FState := bsUp;
    Repaint;
    FDragging := True;
    FMenuTracking := True;
    try
      P := GetDropDownMenuPos;
      if CheckMenuDropDown(PointToSmallPoint(P), False) then
        MouseUp(Button, Shift, X, Y);
      if PeekMessage(Msg, 0, 0, 0, PM_NOREMOVE) then begin
        if (Msg.Message = WM_LBUTTONDOWN) or (Msg.Message = WM_LBUTTONDBLCLK) then
        begin
          P := ScreenToClient(Msg.Pt);
          if (P.X >= 0) and (P.X < ClientWidth) and (P.Y >= 0)
            and (P.Y <= ClientHeight) then KillMessage(0, Msg.Message);
        end;
      end;
    finally
      FMenuTracking := False;
      FDown := false;
    end;
  end;

end;

procedure TsohoSpeedButton.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  NewState: TButtonState;
begin
  inherited MouseMove(Shift, X, Y);
  if FDragging then begin
    if not FDown then NewState := bsUp
    else NewState := bsExclusive;
    if (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight) then
      if FDown then NewState := bsExclusive else NewState := bsDown;
    if NewState <> FState then begin
      FState := NewState;
      Invalidate;
    end;
  end
end;
    
procedure TsohoSpeedButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DoClick: Boolean;
begin
  inherited MouseUp(Button, Shift, X, Y);
  if FDragging then begin
    FDragging := False;
    DoClick := (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight);
    if FGroupIndex = 0 then begin
      { Redraw face in-case mouse is captured }
      FState := bsUp;
      FMouseInControl := False;
      if DoClick and not (FState in [bsExclusive, bsDown]) then
        Invalidate;
    end
    else
      if DoClick then begin
        SetDown(not FDown);
        if FDown then Repaint;
      end
      else begin
        if FDown then FState := bsExclusive;
        Repaint;
      end;
    if DoClick and not FMenuTracking then Click;
    UpdateTracking;
  end;
end;

function TsohoSpeedButton.GetPalette: HPALETTE;
begin
  Result := Glyph.Palette;
end;

function TsohoSpeedButton.GetGlyph: TBitmap;
begin
  Result := TButtonGlyph(FGlyph).Glyph;
end;
    
procedure TsohoSpeedButton.SetGlyph(Value: TBitmap);
begin
  TButtonGlyph(FGlyph).Glyph := Value;
  Invalidate;
end;
    
function TsohoSpeedButton.GetNumGlyphs: TNumGlyphs;
begin
  Result := TButtonGlyph(FGlyph).NumGlyphs;
end;

procedure TsohoSpeedButton.SetNumGlyphs(Value: TNumGlyphs);
begin
  if Value < 0 then Value := 1
  else if Value > 4 then Value := 4;
  if Value <> TButtonGlyph(FGlyph).NumGlyphs then
  begin
    TButtonGlyph(FGlyph).NumGlyphs := Value;
    Invalidate;
  end;
end;
    
procedure TsohoSpeedButton.GlyphChanged(Sender: TObject);
begin
  Invalidate;
end;
    
procedure TsohoSpeedButton.UpdateExclusive;
var
  Msg: TMessage;
begin
  if (FGroupIndex <> 0) and (Parent <> nil) then
  begin
    Msg.Msg := CM_BUTTONPRESSED;
    Msg.WParam := FGroupIndex;
    Msg.LParam := Longint(Self);
    Msg.Result := 0;
    Parent.Broadcast(Msg);
  end;
end;

procedure TsohoSpeedButton.SetDown(Value: Boolean);
begin
  if FGroupIndex = 0 then Value := False;
  if Value <> FDown then
  begin
    if FDown and (not FAllowAllUp) then Exit;
    FDown := Value;
    if Value then
    begin
      if FState = bsUp then Invalidate;
      FState := bsExclusive
    end
    else
    begin
      FState := bsUp;
      Repaint;
    end;
    if Value then UpdateExclusive;
  end;
end;

procedure TsohoSpeedButton.SetFlat(Value: Boolean);
begin
  if Value <> FFlat then
  begin
    FFlat := Value;
    Invalidate;
  end;
end;
    
procedure TsohoSpeedButton.SetGroupIndex(Value: Integer);
begin
  if FGroupIndex <> Value then
  begin
    FGroupIndex := Value;
    UpdateExclusive;
  end;
end;
    
procedure TsohoSpeedButton.SetLayout(Value: TButtonLayout);
begin
  if FLayout <> Value then begin
    FLayout := Value;
    Invalidate;
  end;
end;
    
procedure TsohoSpeedButton.SetMargin(Value: Integer);
begin
  if (Value <> FMargin) and (Value >= -1) then
  begin
    FMargin := Value;
    Invalidate;
  end;
end;
    
procedure TsohoSpeedButton.SetSpacing(Value: Integer);
begin
  if Value <> FSpacing then
  begin
    FSpacing := Value;
    Invalidate;
  end;
end;

procedure TsohoSpeedButton.SetTransparent(Value: Boolean);
begin
  if Value <> FTransparent then
  begin
    FTransparent := Value;
    if Value then
      ControlStyle := ControlStyle - [csOpaque] else
      ControlStyle := ControlStyle + [csOpaque];
    Invalidate;
  end;
end;

procedure TsohoSpeedButton.SetAllowAllUp(Value: Boolean);
begin
  if FAllowAllUp <> Value then
  begin
    FAllowAllUp := Value;
    UpdateExclusive;
  end;
end;
    
procedure TsohoSpeedButton.WMLButtonDblClk(var Message: TWMLButtonDown);
begin
  if not FMenuTracking then
  begin
    inherited;
    if FDown then DblClick;
  end;
end;
    
procedure TsohoSpeedButton.CMEnabledChanged(var Message: TMessage);
const
  NewState: array[Boolean] of TButtonState = (bsDisabled, bsUp);
begin
  TButtonGlyph(FGlyph).CreateButtonGlyph(NewState[Enabled]);
  UpdateTracking;
  Repaint;
end;
    
procedure TsohoSpeedButton.CMButtonPressed(var Message: TMessage);
var
  Sender: TsohoSpeedButton;
begin
  if Message.WParam = FGroupIndex then
  begin
    Sender := TsohoSpeedButton(Message.LParam);
    if Sender <> Self then
    begin
      if Sender.Down and FDown then
      begin
        FDown := False;
        FState := bsUp;
        Invalidate;
      end;
      FAllowAllUp := Sender.AllowAllUp;
    end;
  end;
end;
    
procedure TsohoSpeedButton.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled and Visible and
      (Parent <> nil) and Parent.Showing then begin
      if GroupIndex = 0 then Click
      else begin
        Down := not Down;
        Click;
      end;
      Result := 1;
     end
     else inherited;
end;

procedure TsohoSpeedButton.CMFontChanged(var Message: TMessage);
begin
  Invalidate;
end;

procedure TsohoSpeedButton.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;
    
procedure TsohoSpeedButton.CMSysColorChange(var Message: TMessage);
begin
  with TButtonGlyph(FGlyph) do begin
    Invalidate;
    CreateButtonGlyph(FState);
  end;
end;

procedure TsohoSpeedButton.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  { Don't draw a border if DragMode <> dmAutomatic since this button is meant to
    be used as a dock client. }
  if FFlat and not FMouseInControl and Enabled and (DragMode <> dmAutomatic)
    and (GetCapture = 0) then
  begin
    FMouseInControl := True;
    Repaint;
  end;
end;

procedure TsohoSpeedButton.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FFlat and FMouseInControl and Enabled and not FDragging then begin
    FMouseInControl := False;
    Invalidate;
  end;
end;

{$IFDEF Delphi4_or_5}
procedure TsohoSpeedButton.ActionChange(Sender: TObject; CheckDefaults: Boolean);
begin
  inherited;
  if Sender is TCustomAction then
    with TCustomAction(Sender) do
    begin
      if not CheckDefaults or (Self.ImageIndex = -1) then
        Self.ImageIndex := ImageIndex;
    end;
end;
{$ENDIF}

function TsohoSpeedButton.GetDropDownMenuPos: TPoint;
begin
  if Assigned(FDropDownMenu) then begin
    if MenuPosition = dmpBottom then begin
      case FDropDownMenu.Alignment of
        paLeft: Result := Point(-1, Height);
        paRight: Result := Point(Width + 1, Height);
        else Result := Point(Width div 2, Height);
      end;
    end
    else
    begin
      case FDropDownMenu.Alignment of
        paLeft: Result := Point(Width, -1);
        paRight: Result := Point(-1, -1);
        else Result := Point(Width div 2, Height);
      end;
    end;
  end else Result := Point(0, 0);
end;

procedure TsohoSpeedButton.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = DropDownMenu) and (Operation = opRemove) then
    DropDownMenu := nil;
  if (AComponent = FImageList) and (Operation = opRemove) then
    FImageList := nil;
end;

procedure TsohoSpeedButton.SetDropDownMenu(Value: TPopupMenu);
begin
  FDropDownMenu := Value;
{$IFDEF WIN32}
  if Value <> nil then Value.FreeNotification(Self);
{$ENDIF}
//  if FMarkDropDown then Invalidate;
end;

function TsohoSpeedButton.CheckMenuDropDown(const Pos: TSmallPoint;
  Manual: Boolean): Boolean;
var
  Form: TCustomForm;
begin
  Result := False;
  if csDesigning in ComponentState then Exit;
  if Assigned(FDropDownMenu) and (DropDownMenu.AutoPopup or Manual) then
  begin
    Form := GetParentForm(Self);
    if Form <> nil then Form.SendCancelMode(nil);
    DropDownMenu.PopupComponent := Self;
    with ClientToScreen(SmallPointToPoint(Pos)) do DropDownMenu.Popup(X, Y);
    Result := True;
  end;
end;

procedure TsohoSpeedButton.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    Invalidate;
  end;
end;

procedure TsohoSpeedButton.SetImageIndex(Value: Integer);
begin
  if FImageIndex = Value then exit;
  FImageIndex := Value;
  Repaint;
end;

procedure TsohoSpeedButton.SetImageList(const Value: TImageList);
begin
  FImageList := Value;
  Repaint;
end;

initialization
  FillChar(BitBtnGlyphs, SizeOf(BitBtnGlyphs), 0);
finalization
  DestroyLocals;
end.

