{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SohoCalc;

{$I SOHOLIB.INC}

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms;

type

   TsohoCalculator = class(TComponent)
   private
     FAutoCalc: boolean;
     FAutoPrecision: boolean;
     FFormat: TFloatFormat;
     FPrecision: LongInt;
     FDigits: LongInt;
     FValue: Double;
     FCaption: TCaption;
     FControl: TWinControl;
     FDisplayFormat: string;
     FColor : TColor;
     FFlat: boolean;
   protected
     procedure Notification (AComponent: TComponent; Operation: TOperation);override;
   public
     constructor Create(aOwner: TComponent); override;
     // destructor Destroy;
     function Execute : boolean;
     property Control: TWinControl read FControl write FControl;
   published
     property AutoCalc: boolean read FAutoCalc write FAutoCalc default true;
     property AutoPrecision: boolean read FAutoPrecision write FAutoPrecision default true;
     property Caption: TCaption read FCaption write FCaption;
     property Value: Double read FValue write FValue;
     property Format: TFloatFormat read FFormat write FFormat;
     property Precision: LongInt read FPrecision write FPrecision;
     property Digits: LongInt read FDigits write FDigits;
     property DisplayFormat: string read FDisplayFormat write FDisplayFormat;
     property Color : TColor read FColor write FColor default clBtnFace;
     property FlatButtons: boolean read FFlat write FFlat default true;
   end;

implementation
uses SoCalcF, SoCmnCns, SoVClUtils;

{ TsohoCalculator }
procedure TsohoCalculator.Notification (AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (AComponent = FControl) and (Operation = opRemove) then
    FControl := nil;
end;

constructor TsohoCalculator.Create(aOwner: TComponent);
begin
  inherited;
  FFlat := true; 
  FColor := clBtnFace;
  FAutoCalc := true;
  FAutoPrecision := true;
  FFormat := ffFixed;
  FPrecision := 6;
  FDigits := 0;
  FValue := 0;
  FControl := nil;
  FCaption := sohoCalcCaption;
  FDisplayFormat := '';
end;

function TsohoCalculator.Execute : boolean;
var Pnt: TPoint;
begin
  try
    sohoCalculatorForm := TsohoCalculatorForm.Create(Application);
    with sohoCalculatorForm, Calculator do begin
      AutoCalc := FAutoCalc;
      AutoPrecision := FAutoPrecision;
      Format := FFormat;
      Precision := FPrecision;
      Digits := FDigits;
      DisplayFormat := FDisplayFormat;
      InputValue := FValue;
      Color := FColor;
      ButtonsFlat(FFlat);
    end;
    if Assigned(Control) then begin
      Pnt := Point(Control.Left, Control.Top + Control.Height + 1);
      if Assigned(Control.Parent) then begin
         Pnt := Control.Parent.ClientToScreen(Pnt);
         sohoCalculatorForm.Left := Pnt.X;
         sohoCalculatorForm.Top := Pnt.Y;
      end
      else sohoCalculatorForm.Position := poScreenCenter;
    end
    else sohoCalculatorForm.Position := poScreenCenter;
    Result := sohoCalculatorForm.ShowModal = mrOk;
    if Result then FValue := sohoCalculatorForm.Calculator.AsFloat;
  finally
    sohoCalculatorForm.Free;
  end;
end;

end.
