{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{              Written by Igor Chernyshov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SohoLog;

interface
uses Classes;


type

    TsohoLog = class(TComponent)
    private
      FLFile       : TextFile;
      FUseLog      : Boolean;
      FLogFileName : String;
    protected
      procedure   SetLogFileName(const Value : string);
      procedure   SetUseLog(Value : Boolean);
    public
      constructor Create(AOwner : TComponent); override;
      procedure   InLog(const S : String);
      procedure   Trunc_Log(LogSize : Integer);
    published
      property LogFileName : string read FLogFileName write SetLogFileName;
      property UseLog      : boolean read FUseLog write SetUselog default False;
    end;

implementation
uses SysUtils, SoVCLUtils;

//------------------------------------------------------------------------------
// PROTECTED
//------------------------------------------------------------------------------
procedure TSohoLog.SetLogFileName(const Value : string);
Begin
  If FLogFileName = Value then Exit;

  FLogFileName := Value;
  If FLogFileName = ''
    then FUseLog := False
    else begin

      AssignFile(FLFile,FLogFileName);
      If Not(FileExists(FLogFileName)) then Begin
      {$I-}
        Rewrite(FLFile);
        CloseFile(FLFile);
      {$I+}
      If IOResult<>0 Then Begin
         ErrorMsg('   '+FLogFileName);
         FUseLog:=False;
         FLogFileName := '';
       End;
       InLog('***    ***')
       End;
    End; {--- if ---}   
End;

procedure TSohoLog.SetUseLog(Value : Boolean);
Begin
  If FUseLog = Value Then Exit;
  FUseLog := Value;
  If FUseLog then FUseLog := FLogFileName <> '';
End;

//------------------------------------------------------------------------------
// PUBLIC
//------------------------------------------------------------------------------

{----------   TLog ----------}
constructor TSohoLog.Create(AOwner : TComponent);
Begin
 Inherited Create(AOwner);
   FUseLog      := False;
   FLogFileName := '';
End; {--- Create ---}

{----------   S   ----------}
Procedure TSohoLog.InLog(const S : String);
Begin
 if NOT FUseLog then Exit;
 if S='' then Exit;
 Append(FLFile);
 WriteLn(FLFile,DateTimeToStr(Now)+' ; '+S);
 Flush(FLFile);
 CloseFile(FLFile);
End;

{---------- "" LOG- ----------}
Procedure TSohoLog.Trunc_Log(LogSize : Integer);
Var
  j     : Integer;
  F1,F2 : TextFile;
  F     : File of Byte;
  FSize : Integer;
  S     : String;
Begin

  If (NOT(FUseLog)) OR (NOT(FileExists(FLogFileName))) OR (LogSize=0) Then Exit;
  AssignFile(F,FLogFileName);
  Reset(F);
  FSize:=FileSize(F);
  Close(F);

  If (FSize<(LogSize*1024)) Then Exit;

  AssignFile(F1,FLogFileName);
  Reset(F1);
  AssignFile(F2,FLogFileName+'$$$');
  Rewrite(F2);
  Try
    j:=0;
    While j<(FSize-(LogSize*1024)) Do Begin
      Readln(F1,S);
      j:=j+Length(S);
    End; {--- While ---}
    While NOT(EOF(F1)) Do Begin
      ReadLn(F1,S);
      Writeln(F2,S);
    End; {--- While ---}
  Finally
    Flush(F2);
    CloseFile(F1);
    CloseFile(F2);
  End;{--- Try ---}
  DeleteFile(FLogFileName);
  RenameFile(FLogFileName+'$$$',FLogFileName);
End;{--- Trunc_Log ---}
end.
