{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
   Excel  OLE-Automation
}
unit SohoOLE;

{$I SOHOLIB.INC}

interface
uses ComObj, Graphics;

const
   AutoCloseExcel    : boolean = true;
   {   Excel    }
   ShowExcelOnCreate : boolean = false;
   {   Excel    ? }

var

   MsExcel : Variant;
   {    Excel }

function  OpenExcel  : boolean;
{  Excel   ,     }
function  CloseExcel : boolean;
{  Excel }
function  ShowExcel  : boolean;
{  Excel  }
function  HideExcel  : boolean;
{  Excel }
function  ExcelOpened : boolean;
{    -    }
function  ReOpenExcel : boolean;
{    Excel }
function  ExcelVersion : string;
{   Excel }
function ExcelVersionIs70 : boolean;
{ true,    Excel' 7- }
function  ExcelRectToStr (Left, Top, Right, Bottom : LongInt) : string;
{      Excel   , 
   Left, Top, Right, Bottom.
 example: ExcelRectToStr(1,1,5,2) = 'A1:E2' }
procedure SetCellsFont   (Sheet : Variant; const Rect, FontName : string; FontSize : integer;
                          FontColor : TColor; FontStyle : TFontStyles);
{       Rect }
procedure SetCellsBackground  (Sheet : Variant; const Rect : string; Color : TColor);
{     }


implementation
uses SysUtils, SoVCLUtils, SoExlCst, SoCmnCns {$IFDEF Delphi6}, Variants{$ENDIF};

var ExcelWasRunned : boolean;
    Version        : string;


function ExcelVersionIs70 : boolean;
begin
  Result := Copy(ExcelVersion,1,1)='7';
end;

function  ExcelVersion : string;
begin
  if not ExcelOpened then ReOpenExcel;
  Result := Version;
end;

function ExcelRectToStr (Left, Top, Right, Bottom : LongInt) : string;
begin
   if Top=0 then
     Result := chr(Left+ord('@'))+':'+chr(Right+ord('@'))
   else
     if Left = 0 then
        Result := IntToStr(Top)+':'+IntToStr(Bottom)
     else
        Result := chr(Left+ord('@'))+IntToStr(Top)+':'+
                 chr(Right+ord('@'))+IntToStr(Bottom);
end;

procedure SetCellsFont  (Sheet : Variant; const Rect, FontName : string; FontSize : integer;
                         FontColor : TColor; FontStyle : TFontStyles);
begin
   if ExcelVersionIs70 then begin
     if FontName <> '' then Sheet.Range(Rect).Font.Name := FontName;
     if FontSize <> -1 then Sheet.Range(Rect).Font.Size := FontSize;
     Sheet.Range(Rect).Font.Italic := fsItalic in FontStyle;
     Sheet.Range(Rect).Font.Bold := fsBold in FontStyle;
     Sheet.Range(Rect).Font.StrikeOut := fsStrikeOut in FontStyle;
     Sheet.Range(Rect).Font.Color := ColorToRGB(FontColor);
   end
   else begin
     if FontName <> '' then Sheet.Range[Rect].Font.Name := FontName;
     if FontSize <> -1 then Sheet.Range[Rect].Font.Size := FontSize;
     Sheet.Range[Rect].Font.Italic := fsItalic in FontStyle;
     Sheet.Range[Rect].Font.Bold := fsBold in FontStyle;
     if fsStrikeOut in FontStyle then
        Sheet.Range[Rect].Font.Underline := xlUnderlineStyleSingle
     else
        Sheet.Range[Rect].Font.Underline := xlUnderlineStyleNone;
     // Sheet.Range[Rect].Font.StrikeOut := fsStrikeOut in FontStyle;
     Sheet.Range[Rect].Font.Color := ColorToRGB(FontColor);
   end;
end;

procedure SetCellsBackground  (Sheet : Variant; const Rect : string; Color : TColor);
begin
   if ExcelVersionIs70 then
     Sheet.Range(Rect).Interior.Color := ColorToRGB(Color)
   else
     Sheet.Range[Rect].Interior.Color := ColorToRGB(Color)
end;

function  ExcelOpened : boolean;
var Tmp : string;
begin
   Result := false;
   try
     if not ExcelWasRunned then exit;
     Tmp := MsExcel.Caption;
     Result := true;
   except
     MsExcel := UnAssigned;
   end;
end;

function  ShowExcel  : boolean;
begin
  Result := false;
  if ExcelOpened then begin
    try
      MsExcel.Visible := true;
      Result := true;
    except
    end;
  end;
end;

function  HideExcel  : boolean;
begin
  Result := false;
  if ExcelOpened then begin
    MsExcel.Visible := false;
    Result := true;
  end;
end;

function  ReOpenExcel : boolean;
begin
  try
   MsExcel := CreateOLEObject('Excel.Application');
   MsExcel.Visible := ShowExcelOnCreate;
   Result := true;
   ExcelWasRunned := true;
   Version := MsExcel.Version;
  except Result := false;
    Version := sohoOLEUnknown;
    ErrorMsg(sohoOLEExcelCreatingError);
  end;
end;

function  OpenExcel  : boolean;
begin
   Result := ShowExcel;
   if Result then exit;
   Result := ReOpenExcel;
end;

function  CloseExcel : boolean;
begin
   Result := true;
   if not ExcelOpened then exit;
   try
     MsExcel.WorkBooks.Close;
     MsExcel.Quit;
     MsExcel := UnAssigned;
     ExcelWasRunned := false;
   except
     Result := false;
   end;
end;

initialization
   MsExcel        := UnAssigned;
   ExcelWasRunned := false;
   Version        := sohoOLEUnknown;
finalization
   if AutoCloseExcel then CloseExcel;
   MsExcel       := UnAssigned;
end.
