{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{           Partial copyright (c) AllexSoft             }
{           Written by Alexander Govyadinov,            }
{                   Victor Mylcin,                      }
{                   Serg Vostrikov                      }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SohoPswU;

{$I SOHOLIB.INC}

interface

uses SysUtils, Windows, Messages, Classes, Graphics, Controls,
     Forms, StdCtrls, Buttons, IniFiles, SoUnit, DB,
     DBTables, SoSDlgs, SohoBtns, SoCtrls, SoTools, SoFrmStorage;

type
  TPasswordF = class(TForm)
    PasswordE: TsohoEdit;
    NameC: TsohoComboBox;
    Query1: TQuery;
    WDirectoryDialog1: TsohoDirectoryDialog;
    NameL: TsohoLabel;
    PasswordL: TsohoLabel;
    OkB: TsohoBitBtn;
    CancelB: TsohoBitBtn;
    DiskB: TsohoBitBtn;
    PasswordIni: TsohoFormStorage;
    procedure BitBtn3Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
  private
    { Private declarations }
  public
    ReadRight,WriteRight,ActionRight:SetOfByte;
    PasswordValid : Boolean;
    DataBaseName  : string;
    UserID        : Longint;
    DataLevel     : LongInt;
    ShortName     : string;
    function PassWordIsValid(S:String):Boolean;
    { Public declarations }
  end;

var  PasswordF: TPasswordF;

function CalculatePassWord (var DataSet; S:String):Longint;

implementation
uses SoVCLUtils, SoDBCns, SoCmnCns;

{$R *.DFM}
var Ini : TIniFile;

procedure TPasswordF.BitBtn3Click(Sender: TObject);
begin
   with WDirectoryDialog1 do begin
     Directory:=Ini.ReadString('Password','Directory',ExtractFilePath(Paramstr(0)));
     WDirectoryDialog1.Execute;
   end;
   Ini.WriteString('Password','Directory',WDirectoryDialog1.Directory);
end;

procedure TPasswordF.FormCreate(Sender: TObject);
var UserName : PChar;
    BufferSize : DWord; {D4}
begin
  {Fill Names List}
  Ini := TIniFile.Create(ExtractFilePath(ParamStr(0))+'Soho.ini');
  WDirectoryDialog1.Directory:=Ini.ReadString('Password','Directory','');
  with NameC do begin
    Items.Clear;
    Ini.ReadSection('UserList',Items);
    Items.Insert(0, sohoPasswordGuest);
    {        " " }
    GetMem(UserName, 254);
    BufferSize := 254;
    if GetUserName(UserName, BufferSize) then Items.Insert(0,StrPas(UserName));
    FreeMem(UserName, 254);
    ItemIndex:=0;
  end;

  Caption := sohoPasswordTitle;
  NameL.Caption := sohoPasswordUser;
  PasswordL.Caption := sohoPasswordPassword;
  NameC.Text := sohoPasswordGuest;

  OkB.Caption := SOKButton;
  CancelB.Caption := SCancelButton;
  DiskB.Caption := sohoPasswordDisk;
end;

function StringSum(s : string):LongInt;
var i : Longint;
begin
   Result := 14142135;
   if Length(s)>0 then
      for i := 0 to Length(s) do Result := Result+(i+1)*ord(S[i+1]);
end;

function CalculatePassWord(var DataSet;S:String):Longint;
const WE0 = 16111963;
var WE : Extended;
    WS : string;
begin
   with TDataSet(DataSet) do begin
      {ID}        WE := FieldByName('ID').AsInteger*Pi+WE0;
      {Action}    WS := FieldByName('Action').AsString;
                  WE := WE-ln(10)*StringSum(WS);
      {DataRead}  WS := FieldByName('ReadR').AsString;
                  WE := WE/(StringSum(WS));
      {DataWrite} WS := FieldByName('WriteR').AsString;
                  WE := WE+StringSum(WS)/exp(1);
      {Name}      WE := WE+StringSum(FieldByName('UserName').AsString);
      {String}    WE := LN(ABS(WE))*StringSum(S);
      if WE=0 then WE := 999999;
      while WE<MaxLongInt/2 do WE := WE*2;
      while WE>MaxLongInt do WE := WE/2;
      Result := trunc(WE);
   end;
end;

function TPasswordF.PassWordIsValid(S:String):Boolean;
var PassWord : Longint;
begin
   PassWord := CalculatePassword(Query1,S);
   Result   := PassWord=Query1.FieldByName('PassWordR').AsInteger;
end;

function Modified(s: string): string;
var i : byte;
begin
   for i := 1 to length(s) do s[i] := chr(not ord(s[i]));
   Result := S;
end;

procedure TPasswordF.BitBtn1Click(Sender: TObject);
const SuperWizard='';
begin
   {Control of Guest and Supervisor rights}
   if (AnsiUpperCase(NameC.Text) = 'GUEST') or
      (AnsiUpperCase(NameC.Text) = '') then begin
      PassWordValid := true;
      ActionRight   := [0..10];
      ReadRight     := ActionRight;
      WriteRight    := ReadRight;
      UserID        := 0;
      DataLevel     := 0;
      DataBaseName  := WDirectoryDialog1.Directory;
      exit;
   end;

   if AnsiUpperCase(NameC.Text) = 'SUPERVISOR' then begin
      PassWordValid := PassWordE.Text = Modified(SuperWizard);
      ActionRight   := [0..255];
      ReadRight     := ActionRight;
      WriteRight    := ReadRight;
      UserID        := MaxLongInt;
      DataLevel     := 0;
      DataBaseName  := WDirectoryDialog1.Directory;
      if PassWordValid then exit;
   end;

   {Control of DataBase, Read user, Validate Password}
   If not FileExists(WDirectoryDialog1.Directory+'\SoPswrd.db') then begin
      InfoMsg(sohoPswUPasswordError);
      ModalResult := 0;
      exit;
   end;

   with Query1 do begin
      DataBaseName := WDirectoryDialog1.Directory;
      Open;
      First;
      PassWordValid := false;
      while not EOF do begin
         PassWordValid := (FieldByName('UserName').AsString=NameC.Text)
                           and PasswordIsValid(PasswordE.Text);
         if PassWordValid then break;
         Next;
      end;
      try
       ActionRight := StrToSet(FieldByName('Action').AsString);
       ReadRight   := StrToSet(FieldByName('ReadR').AsString);
       WriteRight  := StrToSet(FieldByName('WriteR').AsString);
       UserID      := FieldByName('ID').AsInteger;
       DataLevel   := FieldByName('DataLevel').AsInteger;
       ShortName   := FieldByName('ShortName').AsString;
      except ErrorMsg(sohoPswUWrongUserDate);
             Application.Terminate;
      end;
      Close;
   end;
   DataBaseName := Query1.DataBaseName;
   if not PasswordValid then begin
      ErrorMsg(sohoPswUWrongPassword);
      ModalResult := 0;
      exit;
   end;

   Ini.WriteString('UserList',NameC.Text,DateTimeToStr(Now));
   Ini.Free;
end;

end.



