{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
unit SohoSpin;

interface

{$I SOHOLIB.INC}

uses Windows, ComCtrls, Controls, ExtCtrls, Classes, Graphics, Messages, Forms,
     StdCtrls, Menus, SysUtils, SoCtrls;

type

{$IFDEF CBUILDER}
  TValueType = (vtInt, vtFloat);
{$ELSE}
  {$IFDEF Delphi4_or_5}
    {$DEFINE USE_64INT}
    TValueType = (vtInteger, vtFloat, vtInt64);
  {$ELSE}
    TValueType = (vtInteger, vtFloat);
  {$ENDIF}
{$ENDIF}

  { TsohoSpinEdit }

  TsohoSpinEdit = class(TsohoFlatEdit)
  private
    FMinValue: Extended;
    FMaxValue: Extended;
    FIncrement: Extended;
    FDecimal: Byte;
    FChanging: Boolean;
    FEditorEnabled: Boolean;
    FValueType: TValueType;
    FArrowKeys: Boolean;
    FUpDown: TCustomUpDown;

    procedure UpDownClick(Sender: TObject; Button: TUDBtnType);
    function GetValue: Extended;
    function CheckValue(NewValue: Extended): Extended;
    function GetAsInteger: Longint;
    function IsIncrementStored: Boolean;
    function IsMaxStored: Boolean;
    function IsMinStored: Boolean;
    function IsValueStored: Boolean;
    procedure SetAsInteger(NewValue: Longint);
    procedure SetValue(NewValue: Extended);
    procedure SetValueType(NewType: TValueType);
    procedure SetDecimal(NewValue: Byte);
    function GetButtonWidth: Integer;
    procedure RecreateButton;
    procedure ResizeButton;
    procedure SetEditRect;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMPaste(var Message: TWMPaste); message WM_PASTE;
    procedure WMCut(var Message: TWMCut); message WM_CUT;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
{$IFDEF Delphi4_or_5}
    procedure CMBiDiModeChanged(var Message: TMessage); message CM_BIDIMODECHANGED;
{$ENDIF}
{$IFDEF USE_64INT}
    function GetAsInt64: Int64;
    procedure SetAsInt64(const Value: Int64);
{$ENDIF}
  protected
    function GetDisplayText: string; override;
    procedure Change; override;
    procedure RecalcNonClient(var R: TRect); override;
    function IsValidChar(Key: Char): Boolean; virtual;
    procedure UpClick(Sender: TObject); virtual;
    procedure DownClick(Sender: TObject); virtual;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
  public
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property AsInteger: Longint read GetAsInteger write SetAsInteger default 0;
{$IFDEF USE_64INT}
    property AsInt64: Int64 read GetAsInt64 write SetAsInt64 default 0;
{$ENDIF}
    property Text;
  published
    property Alignment default taLeftJustify;
    property ArrowKeys: Boolean read FArrowKeys write FArrowKeys default True;
    property Decimal: Byte read FDecimal write SetDecimal default 2;
    property DisplayFormat;
    property EditorEnabled: Boolean read FEditorEnabled write FEditorEnabled default True;
    property Increment: Extended read FIncrement write FIncrement stored IsIncrementStored;
    property MaxValue: Extended read FMaxValue write FMaxValue stored IsMaxStored;
    property MinValue: Extended read FMinValue write FMinValue stored IsMinStored;
{$IFDEF CBuilder}
    property ValueType: TValueType read FValueType write SetValueType default vtInt;
{$ELSE}
    property ValueType: TValueType read FValueType write SetValueType default vtInteger;
{$ENDIF}
    property Value: Extended read GetValue write SetValue stored IsValueStored;
    {  ,         }
    property LightLabel;
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
{$IFDEF Delphi4_or_5}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
{$ENDIF}
{$IFNDEF VER90}
    property ImeMode;
    property ImeName;
{$ENDIF}
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
{$IFDEF Delphi4_or_5}
    property OnEndDock;
    property OnStartDock;
{$ENDIF}
    {       ,  , 
        ,    .   
             }
    property OnGetDisplayText;
  end;

implementation

uses CommCtrl;

type
  TsohoUpDown = class(TCustomUpDown)
  private
    FChanging: Boolean;
    procedure ScrollMessage(var Message: TWMVScroll);
    procedure WMHScroll(var Message: TWMHScroll); message CN_HSCROLL;
    procedure WMVScroll(var Message: TWMVScroll); message CN_VSCROLL;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property OnClick;
  end;

constructor TsohoUpDown.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Orientation := udVertical;
  Min := -1;
  Max := 1;
  Position := 0;
end;

procedure TsohoUpDown.ScrollMessage(var Message: TWMVScroll);
begin
  if Message.ScrollCode = SB_THUMBPOSITION then begin
    if not FChanging then begin
      FChanging := True;
      try
        if Message.Pos > 0 then Click(btNext)
        else if Message.Pos < 0 then Click(btPrev);
        if HandleAllocated then
          SendMessage(Handle, UDM_SETPOS, 0, 0);
      finally
        FChanging := False;
      end;
    end;
  end;
end;

procedure TsohoUpDown.WMHScroll(var Message: TWMHScroll);
begin
  ScrollMessage(TWMVScroll(Message));
end;

procedure TsohoUpDown.WMVScroll(var Message: TWMVScroll);
begin
  ScrollMessage(Message);
end;

{ TsohoSpinEdit }
procedure TsohoSpinEdit.RecalcNonClient(var R: TRect);
begin
  InflateRect (R, -1, -1);
end;

constructor TsohoSpinEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Text := '0';
  ControlStyle := ControlStyle - [csSetCaption];
  FIncrement := 1.0;
  FDecimal := 2;
  FEditorEnabled := True;
  FArrowKeys := True;
  ParentCtl3D := False;
  RecreateButton;
end;

destructor TsohoSpinEdit.Destroy;
begin
  FUpDown := nil;
  inherited Destroy;
end;

procedure TsohoSpinEdit.RecreateButton;
begin
  FUpDown.Free;
  FUpDown := nil;
  FUpDown := TsohoUpDown.Create(Self);
  with TsohoUpDown(FUpDown) do begin
    Visible := True;
    SetBounds(0, 14, GetSystemMetrics(SM_CXVSCROLL), Self.Height - 16);
{$IFDEF Delphi4_or_5}
    if (BiDiMode = bdRightToLeft) then Align := alLeft else
{$ENDIF}
    Parent := Self;
    OnClick := UpDownClick;
  end;
end;

procedure TsohoSpinEdit.UpDownClick(Sender: TObject; Button: TUDBtnType);
begin
  if TabStop and CanFocus then SetFocus;
  case Button of
    btNext: UpClick(Sender);
    btPrev: DownClick(Sender);
  end;
end;

function TsohoSpinEdit.GetButtonWidth: Integer;
begin
  if FUpDown <> nil then Result := FUpDown.Width
  else Result := 0;
end;

procedure TsohoSpinEdit.ResizeButton;
begin
  if FUpDown <> nil then begin
    FUpDown.Width := GetSystemMetrics(SM_CXVSCROLL);
    FUpDown.Height := Height - 4;
    FUpDown.Top := 1; // 1
    FUpDown.Left := Width - FUpDown.Width - 3;
  end;
  RedrawBorder(0);
end;

procedure TsohoSpinEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  if ArrowKeys and (Key in [VK_UP, VK_DOWN]) then begin
    if Key = VK_UP then UpClick(Self)
    else if Key = VK_DOWN then DownClick(Self);
    Key := 0;
  end;
end;

procedure TsohoSpinEdit.Change;
begin
  if not FChanging then inherited Change;
end;

procedure TsohoSpinEdit.KeyPress(var Key: Char);
begin
  if not IsValidChar(Key) then begin
    Key := #0;
    MessageBeep(0)
  end;
  inherited;
end;

function TsohoSpinEdit.IsValidChar(Key: Char): Boolean;
var
  ValidChars: set of Char;
begin
  ValidChars := ['+', '-', '0'..'9'];
  if ValueType = vtFloat then
    ValidChars := ValidChars + [DecimalSeparator];
  Result := (Key in ValidChars) or (Key < #32);
  if not FEditorEnabled and Result and ((Key >= #32) or
    (Key = Char(VK_BACK)) or (Key = Char(VK_DELETE))) then Result := False;
end;

procedure TsohoSpinEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or WS_CLIPCHILDREN;
end;

procedure TsohoSpinEdit.CreateWnd;
begin
  inherited CreateWnd;
  SetEditRect;
end;

procedure TsohoSpinEdit.SetEditRect;
var
  Loc: TRect;
begin
  if not Assigned(Parent) then exit; //Wizard
{$IFDEF Delphi4_or_5}
  if (BiDiMode = bdRightToLeft) then
    SetRect(Loc, GetButtonWidth + 1, 0, ClientWidth - 1,
      ClientHeight + 1) else
{$ENDIF Delphi4_or_5}
  SetRect(Loc, 3, 2, ClientWidth - GetButtonWidth - 2, ClientHeight-1); //Wizard
  TextRect := Loc;
  SendMessage(Handle, EM_SETRECTNP, 0, Longint(@Loc));
end;

procedure TsohoSpinEdit.WMSize(var Message: TWMSize);
begin
  inherited;
  ResizeButton;
  SetEditRect;
end;

procedure TsohoSpinEdit.UpClick(Sender: TObject);
var
  OldText: string;
begin
  if ReadOnly then MessageBeep(0)
  else begin
    FChanging := True;
    try
      OldText := inherited Text;
      Value := Value + FIncrement;
    finally
      FChanging := False;
    end;
    if CompareText(inherited Text, OldText) <> 0 then begin
      Modified := True;
      Change;
    end;
  end;
end;

procedure TsohoSpinEdit.DownClick(Sender: TObject);
var
  OldText: string;
begin
  if ReadOnly then MessageBeep(0)
  else begin
    FChanging := True;
    try
      OldText := inherited Text;
      Value := Value - FIncrement;
    finally
      FChanging := False;
    end;
    if CompareText(inherited Text, OldText) <> 0 then begin
      Modified := True;
      Change;
    end;
  end;
end;

{$IFDEF Delphi4_or_5}
procedure TsohoSpinEdit.CMBiDiModeChanged(var Message: TMessage);
begin
  inherited;
  ResizeButton;
  SetEditRect;
end;
{$ENDIF}

procedure TsohoSpinEdit.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if FUpDown <> nil then FUpDown.Enabled := Enabled;
end;

procedure TsohoSpinEdit.WMPaste(var Message: TWMPaste);
begin
  if not FEditorEnabled or ReadOnly then Exit;
  inherited;
end;

procedure TsohoSpinEdit.WMCut(var Message: TWMCut);
begin
  if not FEditorEnabled or ReadOnly then Exit;
  inherited;
end;

procedure TsohoSpinEdit.CMExit(var Message: TCMExit);
begin
  inherited;
  if CheckValue(Value) <> Value then SetValue(Value);
  Invalidate;
end;

function TsohoSpinEdit.GetValue: Extended;
begin
  try
    if ValueType = vtFloat then Result := StrToFloat(Text)
    else Result := StrToIntDef(Text, 0);
  except
    if ValueType = vtFloat then Result := FMinValue
    else Result := Trunc(FMinValue);
  end;
end;

procedure TsohoSpinEdit.SetValue(NewValue: Extended);
begin
  if ValueType = vtFloat then
    Text := FloatToStrF(CheckValue(NewValue), ffFixed, 15, FDecimal)
  else
    Text := IntToStr(Round(CheckValue(NewValue)));
end;

function TsohoSpinEdit.GetAsInteger: Longint;
begin
  Result := Trunc(GetValue);
end;

procedure TsohoSpinEdit.SetAsInteger(NewValue: Longint);
begin
  SetValue(NewValue);
end;

procedure TsohoSpinEdit.SetValueType(NewType: TValueType);
begin
  if FValueType <> NewType then begin
    FValueType := NewType;
    Value := GetValue;
    if FValueType <> vtFloat then begin
      FIncrement := Round(FIncrement);
      if FIncrement = 0 then FIncrement := 1;
    end;
  end;
end;

function TsohoSpinEdit.IsIncrementStored: Boolean;
begin
  Result := FIncrement <> 1.0;
end;

function TsohoSpinEdit.IsMaxStored: Boolean;
begin
  Result := (MaxValue <> 0.0);
end;

function TsohoSpinEdit.IsMinStored: Boolean;
begin
  Result := (MinValue <> 0.0);
end;

function TsohoSpinEdit.IsValueStored: Boolean;
begin
  Result := (GetValue <> 0.0);
end;

procedure TsohoSpinEdit.SetDecimal(NewValue: Byte);
begin
  if FDecimal <> NewValue then begin
    FDecimal := NewValue;
    Value := GetValue;
  end;
end;

function TsohoSpinEdit.CheckValue(NewValue: Extended): Extended;
begin
  Result := NewValue;
  if (FMaxValue <> FMinValue) then begin
    if NewValue < FMinValue then  Result := FMinValue
    else if NewValue > FMaxValue then Result := FMaxValue;
  end;
end;

function TsohoSpinEdit.GetDisplayText: string;
begin
  if DisplayFormat = '' then Result := Text
  else begin
    case FValueType of
      vtFloat: Result := FormatFloat(DisplayFormat, Value);
      else Result := FormatFloat(DisplayFormat, AsInteger);
    end;
  end;
  //  ,    
  DoGetDisplayText(Result);
end;

{$IFDEF USE_64INT}
function TsohoSpinEdit.GetAsInt64: Int64;
begin
  Result := Trunc(GetValue);
end;

procedure TsohoSpinEdit.SetAsInt64(const Value: Int64);
begin
  SetValue(Value);
end;
{$ENDIF}

end.

