{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{
    TsohoTreeView -    
 SohoLib        
}
unit SohoTree;

{$I SOHOLIB.INC}

interface

uses SysUtils, Windows, Messages, Classes, Controls, DB, DBTables,
     SoDBGrid, SoCtmTre, ComCtrls;

type
  
  TsohoTreeView = class(TsohoCustomTreeView)
  {  TsohoTreeView -  ,    :
    TsohoEnhTreeView ( GroupTreeView)  TsohoDBGrid ( DBGrid).  
         -     .
    ,       .   
      Id (ftInteger) -   , Hash (ftInteger) - 
        , IdGrp (ftInteger) -   
    ,     , Name -  .    
          TsohoTreeView.     
    ,       OrderField.   
      : Id, Hash, IdParent (ftInteger) -   
     , Name -  .     ,  
      .      design-time,  
    Component Editor',   TsohoTreeView.   TsohoTreeView 
     TDataSet',         ,   -  
    .       GroupsDataSet  ItemsDataSet.  ,
           GroupsTable  ItemsTable (,
    ItemsTable := "items.db").     TsohoTreeView 
     ,         
     OnNewItem  OnItemEdit.     
          TsohoTVContainer.
     :     ,    
     'Id'      !  ,  
    TsohoTreeView,      .  ,
         ,     
     Hash,  TsohoTreeView     :
        Drag&Drop   , 
        ,   .  Ctrl+Ins 
    Shift-Ins   ""   .   
        ,      
          . TsohoTreeView  
       ( Folder)      ,
       TreeView,    ,    
    IncludeSubTree.
  }
  private
    { Private declarations }
    OperationQuery: TQuery;
    FDataBaseName : string;
    FDefAnswer: integer; // 0 - yestoall, 1 - notoall, 2 - not defined
    FCanMany: boolean; //      
  protected
    { Protected declarations }
    function CreateGroupsTree: boolean;override;
    function  RenumberRecords: boolean;override;
    function  DeleteOneRecord(ItemID: Longint): boolean;override;
    procedure MoveGroup (NewParentId, NewHash, Id : LongInt);override;
    function  TreeViewGetGroupsIds(const aFixedGrpName, aGroupTable: string;
              var Ids: TsohoDBGridResults): boolean; override;
  public
    function QuietAddGroup(var NewName: string): boolean; override;
    {          }
    function QuietEditGroup(var NewName: string): boolean; override;
    {          }
    function DeleteGroup: boolean; override;
    {   }
    function MoveItem: boolean; override;
    {   }
    function DeleteItem: boolean; override;
    {   }
    function GetNewItemHash(GrpId: Longint): Longint;override;
    {     Hash    }
    function AddNewItem(GrpId, ItemID, ItemHash: Longint): boolean;override;
    {     .   
      Id, IdGrp, Hash }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DataBaseName : string read FDataBaseName write FDataBaseName;
    {     }
  end;

function GroupsIdsToStr(const KeyField, GrpField, aFixedGrpName,
  GroupTable, DataBaseName: string): string;

implementation
uses SohoWrds, SoAppUtils, SoUnit, SoCtmRgs, SoDBCns, SoVCLUtils,
     Forms, SoDialogs;

function GetGroupsIds(
  const KeyField,
        aGroupNameField,
        aParentIdField,
        aFixedGrpName, aGroupTable,
        aDataBaseName: string;
  var Ids: TsohoDBGridResults): boolean;
var Query: TQuery;
  AddedCount      : Integer;
  CurId, CurParent: Longint;
begin
  Result := False;
  Query := TQuery.Create(nil);
  PushCursor(crHourGlass);
  with Query, SQL do begin
    DataBaseName := aDataBaseName;
    if ExtractFileExt(aGroupTable)<>'' then
       Add('select * from "' + aGroupTable + '"')
    else
       Add('select * from ' + aGroupTable);
    Add('order by ' + aParentIdField + ', ' + KeyField);
    try
      Open;
    except
      on E: Exception do begin
        Free;
        exit;
      end;
    end;
  end;
  with Query do begin
    First;
    while (not EOF) and (FieldByName(aGroupNameField).AsString <> aFixedGrpName) do Next;
    if FieldByName(aGroupNameField).AsString = aFixedGrpName then
      Ids.Add(FieldByName(KeyField).AsInteger)
    else begin
      Query.Free;
      RestoreCursor;
      exit;
    end;
    AddedCount := -1;
    while AddedCount <> 0 do begin
      First;
      AddedCount := 0;
      while not EOF do begin
        CurId := FieldByName(KeyField).AsInteger;
        CurParent := FieldByName(aParentIdField).AsInteger;
        if (Ids.IndexOf(CurParent) <> -1) and
          (Ids.IndexOf(CurId) = -1) then begin
          Ids.Add(CurId);
          Inc(AddedCount);
        end;
        Next;
      end;
    end;
    Free;
  end;
  Result := True;
  RestoreCursor;
end;

function GroupsIdsToStr(const KeyField, GrpField, aFixedGrpName,
  GroupTable, DataBaseName: string): string;
var Ids: TsohoDBGridResults;
  index: Longint;
begin
  Ids := TsohoDBGridResults.Create;
  Result := '('+GrpField+' = 0)';
  if GetGroupsIds(KeyField, 'Name', 'IdParent', aFixedGrpName, GroupTable, DataBaseName, Ids) then
    for index := 0 to pred(Ids.Count) do begin
      Result := Result + '('+GrpField+'=' + IntToStr(Ids.Items[index]) + ')';
      if index <> pred(Ids.Count) then Result := Result + ' or ';
    end;
  Ids.Free;
end;

{ TsohoTreeView }
function  TsohoTreeView.TreeViewGetGroupsIds(const aFixedGrpName, aGroupTable: string;
              var Ids: TsohoDBGridResults): boolean;
begin
  Result := GetGroupsIds(KeyField,
    NameGroup, NameParentId, aFixedGrpName, aGroupTable,
    FDataBaseName, Ids);
end;

function TsohoTreeView.QuietAddGroup(var NewName: string): boolean;
var ParentNode, LastChild: TTreeNode;
    LastChildHash, NewGroupHash,
    NewGroupId, ParentId: Longint;
  Query               : TQuery;
  Allow               : boolean;
  F1, F2, F3          : string;
  aOnNewCust          : TsohoTreeCustomGroup;
  aOnNew              : TsohoTreeOnNewGroup;
  TmpLeaf             : PTreeLeaf;
  FieldsList          : TStringList;
  ValuesList          : TStringList;

  procedure CreateInsertQuery;
  var
    i : integer;
  begin
    Query := TQuery.Create(Self);
    Query.DataBaseName := DataBaseName;
    with Query, SQL do begin
      if ExtractFileExt(GroupsTable)<>'' then
        Add('Insert into "' + GroupsTable + '"')
      else
        Add('Insert into ' + GroupsTable);
      Add('('+KeyField+', '+HashField+', '+NameParentID+', Name');
      if FieldsList <> nil then
        for i := 0 to pred(FieldsList.Count) do Add(', '+FieldsList[i]);
      Add(')');

      NewGroupId := SoUnit.GetNewId(GetNewGroupIdPreffix);
      Add('Values (' + IntToStr(NewGroupId) + ', ' + IntToStr(NewGroupHash) + ',');
      ParentId := GetIdByNode(ParentNode);
      Add(IntToStr(ParentId) + ',"' + NewName + '"');
      if ValuesList <> nil then
        for i := 0 to pred(ValuesList.Count) do Add(', '+ValuesList[i]);
      Add(')');

    end;
  end;

begin
  Result := False;
  FieldsList := nil;
  ValuesList := nil;
  WordToWordForm(GroupsName, F1, F2, F3, GroupsGenderIndex);
  with GroupTreeView do begin
    ParentNode := Selected;
    LastChild := ParentNode.GetLastChild;
    if LastChild = nil then NewGroupHash := 0
    else begin
      TmpLeaf := GetLeafRecord(LastChild);
      if Assigned(TmpLeaf) then LastChildHash := TmpLeaf^.ItemHash
      else LastChildHash := 0;
      NewGroupHash := GetHash(LastChildHash, MaxLongint);
    end;
  end;
  {  Hash    }
  try
    CreateInsertQuery;
    try
      Allow := True;
      aOnNewCust := OnCustomNewGrp;
      aOnNew     := OnNewGrp;

      if Assigned(aOnNewCust) then aOnNewCust(Self, NewGroupId,
        NewGroupHash, ParentId, NewName, Allow)
      else
        if Assigned(aOnNew) then begin
          FieldsList := TStringList.Create;
          ValuesList := TStringList.Create;
          aOnNew(Self, SohoTreeNode(NewName, NewGroupId), FieldsList, ValuesList, Allow);
          CreateInsertQuery;
          FieldsList.Free;
          ValuesList.Free;
        end;
      if Allow then begin
        if not Assigned(aOnNewCust) then begin
          PushCursor(crHourGlass);
          Query.ExecSQL;
        end;
        {     }
        if AddTreeItem(ParentNode, NewGroupId, ParentId, NewGroupHash,
          NewName) then
          with GroupTreeView do begin
             ParentNode.Expand(false);
            Selected := GetNodeByName(NewName);
          end;
        {   OutLine }
        Result := True;
      end;
      if Allow and Assigned(aOnNewCust) then RefreshTreeView;
      Changed;
    except
      // RestoreCursor;
      on E: Exception do
        if not HandleExceptions(E) then
          ErrorMsg(Format(sohoTreeGroupDontCreate, [Name, F1]));
    end;
  finally
    Query.Free;
    RestoreCursor;
  end;
end;

function TsohoTreeView.MoveItem: boolean;
var Allow: boolean;
  index: Integer;

  procedure DoMoveItem(aItemId, aNewGrpId, aNewHash: Longint);
  var aOnMove : TsohoTreeOnMoveItem;
  begin
    OperationQuery.DataBaseName := DataBaseName;
    with OperationQuery, OperationQuery.SQL do begin
      Allow := True;
      aOnMove := OnItemMove;
      if Assigned(aOnMove) then
        aOnMove(Self, FMoveItem.OldGrpId, aNewGrpId, aItemId, aNewHash, Allow);
      if not Allow then exit;
      Clear;
      if ExtractFileExt(ItemsTable) <> '' then
        Add('update "' + ItemsTable + '"')
      else
        Add('update ' + ItemsTable);
      Add('set '+NameGroupID+'=' + IntToStr(aNewGrpId) + ', '+
          HashField+'=' + IntToStr(aNewHash));
      Add('where '+KeyField+'=' + IntToStr(aItemId));
      try
        try
          PushCursor(crHourGlass);
          ExecSQL;
          Result := True;
          Changed;
        except
          // RestoreCursor;
          on E: Exception do
            if not HandleExceptions(E) then
              ErrorMsg(Format(sohoTreeErrorOnItemMove, [Name]));
        end;
      finally
        RestoreCursor;
      end;
    end;
  end;

begin
  { Event on drag and drop }
  Result := False;
  if not (trMoveItem in Options) then exit;
  if ItemsTable = '' then begin
    ErrorMsg(Format(sohoTreeItemsTableNotDefined,[Name]));
    exit;
  end;
  if DBGrid.Results.Count = 0 then DoMoveItem(FMoveItem.ID, FMoveItem.NewGrpId,
    FMoveItem.NewHash)
  else
    for index := 0 to pred(DBGrid.Results.Count) do begin
      FMoveItem.NewHash := GetHash(FMoveItem.NewHash, RightHash);
      DoMoveItem(DBGrid.Results.Items[index], FMoveItem.NewGrpId,
                 FMoveItem.NewHash);
    end;
  DBGrid.Results.Clear;
  RefreshItems;
end;

function TsohoTreeView.QuietEditGroup(var NewName: string): boolean;
var
  aGroupId,
  aGroupHash,
  aParentId  : Longint;
  Query      : TQuery;
  Allow      : boolean;
  CurNode    : TTreeNode;

  procedure CreateUpdateQuery;
  var TmpLeaf: PTreeLeaf;
  begin
    Query := TQuery.Create(Self);
    Query.DataBaseName := DataBaseName;
    with Query, SQL do begin
      if ExtractFileExt(GroupsTable)<>'' then
        Add('Update "' + GroupsTable + '"')
      else
        Add('Update ' + GroupsTable);
      Add('Set Name="' + NewName + '"');
      TmpLeaf := GetLeafRecord(CurNode);
      if Assigned(TmpLeaf) then begin
        aGroupId := TmpLeaf^.ItemID;
        aGroupHash := TmpLeaf^.ItemHash;
        aParentId := TmpLeaf^.ParentId;
      end
      else begin
        aGroupId := 0;
        aGroupHash := 0;
        aParentId := 0;
      end;
      Add('Where '+KeyField+'=' + IntToStr(aGroupId));
    end;
  end;

var aOnCustEd : TsohoTreeCustomGroup;
    aOnEdit   : TsohoTreeOnEditGroup;
begin
  Result := False;
  CurNode  := GroupTreeView.Selected;
  try
    try
      CreateUpdateQuery;
      Allow := True;
      aOnCustEd := OnCustomEditGrp;
      aOnEdit := OnGrpEdit;
      if Assigned(aOnCustEd) then aOnCustEd(Self, aGroupId,
        aGroupHash, aParentId, NewName, Allow)
      else if Assigned(aOnEdit) then aOnEdit(Self,
        SohoTreeNode(
          CurNode.Text,
          aGroupId), Allow);
      if Allow then begin
        if not Assigned(aOnCustEd) then begin
          PushCursor(crHourGlass);
          Query.ExecSQL;
        end;
        CurNode.Text := NewName;
        Result := True;
        if Allow and Assigned(aOnCustEd) then RefreshTreeView;
        Changed;
      end;
    except
      on E:Exception do
        if not HandleExceptions(E) then
          ErrorMsg(Format(sohoTreeGroupRenameError, [Name]));
    end;
  finally
    Query.Free;
    RestoreCursor;
  end;
end;

function TsohoTreeView.DeleteGroup: boolean;
var JustOneDelete: boolean;
  WasChecked: boolean;

  function DeleteSubTree(Node: TTreeNode; GrpId: Longint): boolean;
  var GrpName: string;
    TopNode, CurNode: TTreeNode;
    DeleteThatGroup: boolean;
    aOnDel: TsohoTreeOnDeleteGroup;
    Confirm: TModalResult;
  begin
    GrpName := Node.Text;
    aOnDel := OnGrpDelete;
    case FDefAnswer of
      0: Result := true;
      1: begin
        Result := false;
        exit;
      end;
      else begin
        if Assigned(aOnDel) then aOnDel(Self, GrpId, Result)
        else begin
          if Node.Count > 0 then begin
            Confirm := MessageDlg(Format(sohoTreeDeleteGroup, [GrpName]),
               mtConfirmation, [mbYes, mbYesToAll, mbNo, mbNoToAll], 0);
            if Confirm = mrYesToAll then FDefAnswer := 0;
            if Confirm = mrNoToAll then FDefAnswer := 1;
            Result := Confirm in [mrYes, mrYesToAll];
          end
          else Result := YesNoMsg(Format(sohoTreeDeleteGroup, [GrpName]));
        end;
        if not Result then exit;
      end;
    end;
    TopNode := Node.GetFirstChild;
    // BottomNode := Node.GetLastChild;
    {  ,  ...  :( }
    { ,  ,        
          ,     . ,    
             :) }
    DeleteThatGroup := True;
    if Assigned(TopNode) then begin
      CurNode := TopNode;
      while CurNode <> nil do begin
        if not DeleteSubTree(CurNode, GetIdByNode(CurNode)) then
          DeleteThatGroup := False
        else JustOneDelete := True;
        CurNode := Node.GetNextChild(CurNode);
      end;
    end;
    {     -   ,
           IdGrp  }
    try
      ItemsDataSet.DisableControls;
      GroupTreeView.Selected := Node;
      OperationQuery.DatabaseName := DataBaseName;
      with OperationQuery, OperationQuery.SQL do begin
        Clear;
        if ExtractFileExt(ItemsTable) <> '' then
          Add('delete from "' + ItemsTable + '"')
        else
          Add('delete from ' + ItemsTable);
        Add('Where '+KeyField+'=:Id');
      end;
      FCanMany := ItemsDataSet.RecordCount > 1;
      with ItemsDataSet do begin
        Refresh;
        {    }
        while not EOF do begin
          if not DeleteOneRecord(ItemsDataSet.FieldByName(KeyField).AsInteger)
            then DeleteThatGroup := False
          else JustOneDelete := True;
          Next;
        end;
      end;
      {     }
      if DeleteThatGroup then
        with OperationQuery, OperationQuery.SQL do begin
          Clear;
          if ExtractFileExt(GroupsTable)<>'' then
             Add('delete from "' + GroupsTable + '"')
          else
             Add('delete from ' + GroupsTable);
          Add('Where '+KeyField+'=' + IntToStr(GrpId));
          try
            ExecSQL;
            JustOneDelete := True;
          except
            on E: Exception do
              if not HandleExceptions(E) then
                ErrorMsg(Format(sohoTreeGroupDeleteError, [Name, GrpName]));
          end;
        end;
    finally
      ItemsDataSet.EnableControls;
      Result := DeleteThatGroup;
    end;
  end;

var aAfterDel : TNotifyEvent;
begin
  FDefAnswer := 2;
  Result := False;
  JustOneDelete := False;
  if not (trDeleteGroup in Options) then exit;
  WasChecked := ChildCheckBox.Checked;
  ChildCheckBox.Checked := False;
  Result := DeleteSubTree(GroupTreeView.Selected,
    GetIdByNode(GroupTreeView.Selected));
  {     ,   }
  ChildCheckBox.Checked := WasChecked;
  if not JustOneDelete then exit;
  aAfterDel := AfterGrpDelete;
  if Assigned(aAfterDel) then aAfterDel(Self);
  {    -   !}
  TreeViewButtonsClick(actRefresh);
  Changed;
end;

function TsohoTreeView.AddNewItem(GrpId, ItemID, ItemHash: Longint): boolean;
begin
  PushCursor(crHourGlass);
  Result := False;
  OperationQuery.DataBaseName := DataBaseName;
  with OperationQuery, OperationQuery.SQL do begin
    Clear;
    if ExtractFileExt(ItemsTable) <> '' then
      Add('insert into "' + ItemsTable + '" ('+KeyField+', '+NameGroupID+', '+HashField+')')
    else
      Add('insert into ' + ItemsTable + ' ('+KeyField+', '+NameGroupID+', '+HashField+')');
    Add('values (' + IntToStr(ItemID) + ', ' + IntToStr(GrpId) +
      ', ' + IntToStr(ItemHash) + ')');
    try
      ExecSQL;
      Result := True;
    except
      on E: Exception do
        if not HandleExceptions(E) then
          ErrorMsg(Format(sohoTreeItemAddError, [Name]));
    end;
  end;
  RestoreCursor;
end;

function TsohoTreeView.DeleteOneRecord(ItemID: Longint): boolean;
var aOnDel : TsohoTreeOnDeleteItem;
    Confirm: TModalResult;
begin
  case FDefAnswer of
    0: Result := true;
    1: begin
      Result := false;
      exit;
    end;
    else begin
      Result := false;
      aOnDel := OnItemDelete;
      if Assigned(aOnDel) then aOnDel(Self, ItemID, Result)
      else begin
        if FCanMany then begin
          Confirm := MessageDlg(sohoTreeDeleteCurrentItem,
             mtConfirmation, [mbYes, mbYesToAll, mbNo, mbNoToAll], 0);
          if Confirm = mrYesToAll then FDefAnswer := 0;
          if Confirm = mrNoToAll then FDefAnswer := 1;
          Result := Confirm in [mrYes, mrYesToAll];
        end
        else Result := YesNoMsg(sohoTreeDeleteCurrentItem);
      end;
    end;
  end;
  if not Result then exit;
  try
    with OperationQuery do begin
      Params[0].AsInteger := ItemID;
      try
        PushCursor(crHourGlass);
        ExecSQL;
        {         
                ! }
        Results.Remove(ItemID);
        Result := True;
      except
        on E: Exception do
          if not HandleExceptions(E) then
            ErrorMsg(Format(sohoTreeDeleteItemError, [name]));
      end;
    end;
  finally
    RestoreCursor;
  end;
end;

function TsohoTreeView.DeleteItem: boolean;
var ID, index: Longint;
begin
  Result := False;
  if not (trDeleteItem in Options) then exit;
  if (ItemsDataSet = nil) or not (ItemsDataSet.Active) then exit;
  if ItemsDataSet.FieldByName(KeyField).IsNull then exit;
  ID := ItemsDataSet.FieldByName(KeyField).AsInteger;
  OperationQuery.DataBaseName := DataBaseName;
  with OperationQuery, OperationQuery.SQL do begin
    Clear;
    if ExtractFileExt(ItemsTable) <> '' then
      Add('delete from "' + ItemsTable + '"')
    else
      Add('delete from ' + ItemsTable);
    Add('Where '+KeyField+'=:Id');
  end;
  FCanMany := Results.Count <> 0;
  FDefAnswer := 2;
  if Results.Count <> 0 then begin
    Result := False;
    {    ,      
           }
    index := 0;
    while index < Results.Count do begin
      if DeleteOneRecord(Results.Items[index]) then Result := True
      else Inc(index);
    end;
  end
  else Result := DeleteOneRecord(ID);
  {  -   ,    }
  if Result then begin
    RefreshItems;
    Changed;
  end;
end;

constructor TsohoTreeView.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  OperationQuery := TQuery.Create(Self);
  FDataBaseName := '';
end;

function TsohoTreeView.RenumberRecords: boolean;
var index: Longint;
  Number: Longint;
  FMany : boolean;
begin
  {    }
  Result := True;
  FMany := ChildCheckBox.Checked;
  ChildCheckBox.Checked := False;
  Number := 0;
  if SaveFile <> '' then ; //  !
  GroupTreeView.FullExpand;
  OperationQuery.DataBaseName := DataBaseName;

  for Index := 0 to pred(GroupTreeView.Items.Count) do begin

    GroupTreeView.Selected := GroupTreeView.Items[Index];
    ItemsDataSet.First;
    while not ItemsDataSet.EOF do begin
      Inc(Number);
      with OperationQuery, SQL do begin
        Clear;
        if ExtractFileExt(ItemsTable) <> '' then
          Add('update "' + ItemsTable + '" as Item')
        else
          Add('update ' + ItemsTable + ' as Item');
        Add('set Item."' + OrderField + '"=' + IntToStr(Number));
        Add('where Item.'+KeyField+'=' + ItemsDataSet.FieldByName(KeyField).AsString);
        ExecSQL;
      end;
      ItemsDataSet.Next;
    end;
  end;

  RefreshItems;
  try
    if SaveFile <> '' then ;
  except
  end;
  ChildCheckBox.Checked := FMany;
end;

destructor TsohoTreeView.Destroy;
begin
  OperationQuery.Free;
  inherited Destroy;
end;

procedure TsohoTreeView.MoveGroup (NewParentId, NewHash, Id : LongInt);
begin
  OperationQuery.DataBaseName := DataBaseName;
  with OperationQuery do begin
   Close;
   SQL.Clear;
   if ExtractFileExt(GroupsTable)<>'' then
     SQL.Add('Update "' + GroupsTable + '"')
   else
     SQL.Add('Update ' + GroupsTable);
   SQL.Add('Set '+HashField+'=' + IntToStr(NewHash) + ',');
   SQL.Add('    '+NameParentID+' =' + IntToStr(NewParentId));
   SQL.Add('Where '+KeyField+'=' + IntToStr(Id));
   try
    PushCursor(crHourGlass);
    ExecSQL;
   except
    on E: Exception do
      if not HandleExceptions(E) then
        ErrorMsg(Format(sohoTreeGroupMoveError,[Name]));
   end;
  end;
  RestoreCursor;
end;

function TsohoTreeView.GetNewItemHash;
var Temp: TQuery;
begin
  Result := MaxLongint;
  if ItemsTable = '' then begin
    ErrorMsg(Format(sohoTreeHashGetError,[Name]));
    exit;
  end;
  Temp := TQuery.Create(Self);
  Temp.DataBaseName := DataBaseName;
  with Temp, Temp.SQL do begin
    Clear;
    if ExtractFileExt(ItemsTable) <> '' then
      Add('select MAX('+HashField+') from "' + ItemsTable + '"')
    else
      Add('select MAX('+HashField+') from ' + ItemsTable);
    Add('where ' + NameGroupID + '=' + IntToStr(GrpId));
    try
      Open;
      if EOF then Result := 0
      else Result := GetHash(Fields[0].AsInteger, MaxLongint);
      { Result = Fields[0].AsInteger  MaxLongInt, 
               }
      Close;
      Temp.Free;
    except
      on E: Exception do begin
        if not HandleExceptions(E) then
         ErrorMsg(Format(sohoTreeLastHashGetError,[Name]));
        Temp.Free;
      end;
    end;
  end;
end;

function TsohoTreeView.CreateGroupsTree: boolean;
begin
  if (FDataBaseName = '') and (SingleRegister<>nil) then
    FDataBaseName := SingleRegister.PathToData;
  Result := inherited CreateGroupsTree;
end;

end.

