{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Igor Chernyshov              }
{                                                       }
{             Devrace SohoLib Pro Components            }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: soholib@devrace.com               }
{                                                       }
{*******************************************************}
{$I SOHOLIB.INC}
unit sohoMultiLng;
{        
.
}
interface

uses
  Classes, Controls, SysUtils, Forms,
  SoRTTI;

type
{        SohoMultiLNGCns -
    
}

  { ,     :
      laNone               - ,     ,
                                LangID   
                             LoadLanguage
      laOnFormCreate       -   OnFormCreate
      laBeforeFormActivate -   OnFormActivate
      laAfterFormActivate  -   OnFormActivate
  }
  TsohoMultiLNGActivate = (laNone, laOnFormCreate,
                           laBeforeFormActivate,laAfterFormActivate);
//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
  TsohoMultiLNG = class(TComponent)
  private
    FLangFile  : TFileName;             //    ""
    FLangID    : Integer;               //  
    FLangName  : string;                //  .
    FVarName   : TStringList;           //    ( )
    FVarValue  : TStringList;           //    ( )
    FLanguages : TStrings;              //  
    FActivate  : TsohoMultiLNGActivate; // ,    
    OldFormCreate : TNotifyEvent;
    //     fs : TFileStream.
    //     (word),    > 0  
    //     
    function StringFromStream(fs : TFileStream) : string;
    //  - 
    function GetLanguageCount : Integer;
    //    OnFormCreate/OnFormActivate   
    //  OldFormCreate
    procedure SetNewEvent;
    //      OnFormCreate/OnFormActivate
    procedure RestoreOldEvent;
  protected
    //   
    procedure SetLangFile(Value : TFileName);
    //   
    procedure SetLangID(Value : Integer);
    //   
    procedure SetLangName(Value : string);
    //    /   
    //   
    procedure DoFormCreate(Sender: TObject);
    //   FActivate
    procedure SetActivate(Value : TsohoMultiLNGActivate);
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    {     LangID      ,
      Form.  Form = nil,       
        ,    .
     ,    ,  :
    "_._."}
    procedure   LoadLanguage(Form : TComponent; LangID : Integer);
    //      
    //    ,   = ''
    function GetVariable(const Name : string) : string;
    //    LangID
    function GetLanguageName(LangID : Integer) : string;
    property LanguageCount : Integer read GetLanguageCount;
  published
    {  -  }
    property LangFile : TFileName read FLangFile  write SetLangFile;
    { ()    .   
       run-time     , 
      - ,   TsohoMultiLNG.  LangFile  
      }
     property LangID : Integer read FLangID write SetLangID default -1;
     {  . Read-only.  ,  
     LangID}
     property LangName : string read FLangName write SetLangName stored False;
     {,     (. TsohoMultiLNGActivate)}
     property Activate : TsohoMultiLNGActivate read FActivate write SetActivate default laNone;
  end;

implementation
Uses SohoMultiLNGCns, SoVCLUtils, SoSTRUtils, SoAppUtils;
// -----------------------------------------------------------------------------
// PRIVATE
// -----------------------------------------------------------------------------
function TsohoMultiLNG.StringFromStream(fs : TFileStream) : string;
var
  w : word;
  p : PChar;
begin
  Result := '';
  //   
  fs.Read(w, SizeOf(W));
  if w > 0 then begin
    GetMem(P,W+1);
    FillChar(P^,W,#0);
    fs.read(P^,W+1);
    Result := String(P);
    FreeMem(P);
  end; {--- IF ---}
end;

function TsohoMultiLNG.GetLanguageCount : Integer;
begin
  Result := FLanguages.Count;
end;

procedure TsohoMultiLNG.SetNewEvent;
begin
  if (not (csDesigning in ComponentState)) and
     (Owner <> nil) and
     (Owner is TForm) then begin
       if FActivate = laOnFormCreate then begin
         OldFormCreate := (Owner as TForm).OnCreate;
         (Owner as TForm).OnCreate := DoFormCreate;
       end; {--- if ---}
       if (FActivate = laBeforeFormActivate) or
          (FActivate = laAfterFormActivate)
       then begin
         OldFormCreate := (Owner as TForm).OnActivate;
         (Owner as TForm).OnActivate := DoFormCreate;
       end; {--- if ---}
  end; {--- if ---}
end;

procedure TsohoMultiLNG.RestoreOldEvent;
begin
  if (Owner <> nil) and
     (Owner is TForm) then begin
       if FActivate = laOnFormCreate then begin
         (Owner as TForm).OnCreate := OldFormCreate;
       end; {--- if ---}
       if (FActivate = laBeforeFormActivate) or
          (FActivate = laAfterFormActivate)
       then begin
         (Owner as TForm).OnActivate := OldFormCreate;
       end; {--- if ---}
  end; {--- if ---}
end;
// -----------------------------------------------------------------------------
// PROTECTED
// -----------------------------------------------------------------------------
procedure TsohoMultiLNG.SetLangFile(Value : TFileName);
var
  Fs       : TFileStream;
  s        : string;
  LngCount : word;
  Buffer   : Array[1..64] of Byte;
  i        : Integer;
begin
  if Value = '' then begin
    FLangFile := Value;
    FLanguages.Clear;
    FLangID   := -1;
    FLangName := '';
    Exit;
  end; {--- if ---}

  if Value <> FLangFile then begin
    try
      Fs := TFileStream.Create(Value,fmOpenRead);
    except
      raise;
      Exit;
    end; {--- try/except ---}

    try
      //     -  - 
      s := StringFromStream(fs);
      if s <> 'SohoMultiLNG' then begin
        ErrorMsg(Format(sohoMultiLNG_NotLNGFile,[FLangFile]));
        Exit;
      end; {--- if ---}
      //   
      fs.Read(LngCount, SizeOF(LngCount));
      FLanguages.Clear;
      for i := 0 to pred(LngCount) do begin
        // 
        S := StringFromStream(fs);
        FLanguages.Add(S);
        // 
        S := StringFromStream(fs);
        // . 
        fs.ReadBuffer(Buffer, SizeOf(Buffer));
      end; {--- for ---}
    finally
      fs.Free;
    end; {--- try/finally ---}
    FLangFile := Value;
  end; {--- if ---}
end; {--- SetLangFile ---}

procedure TsohoMultiLNG.SetLangID(Value : Integer);
begin
  if FLangFile = '' then Exit;
  FLangID := Value;
  if (FLangID >= LanguageCount) then FLangID := LanguageCount-1;
  if (FLangID < 0) then begin
    FLangID   := -1;
    FLangName := '';
    Exit;
  end; {--- if ---}
  FLangName := FLanguages[FLangID];
  if not (csDesigning in ComponentState) and
     not (csLoading in ComponentState)
     then LoadLanguage(Owner,FLangID);
end;

procedure TsohoMultiLNG.SetLangName(Value : string);
begin
  LangID := FLanguages.IndexOf(Value);
end;

procedure TsohoMultiLNG.DoFormCreate(Sender: TObject);
begin
  if (FActivate = laBeforeFormActivate) then LoadLanguage(Owner,FLangID);

  //   
  if assigned(OldFormCreate) then OldFormCreate(Sender);

  if (FActivate = laAfterFormActivate) or
     (FActivate = laOnFormCreate) then LoadLanguage(Owner,FLangID);
end;

procedure TsohoMultiLNG.SetActivate(Value : TsohoMultiLNGActivate);
begin
  if Value = FActivate then Exit;
  //     
  RestoreOldEvent;
  //   FActivate
  FActivate := Value;
  //    
  SetNewEvent;
end;
// -----------------------------------------------------------------------------
// PUBLIC
// -----------------------------------------------------------------------------
constructor TsohoMultiLNG.Create(AOwner: TComponent);
begin
  inherited;
  FLangFile  := '';
  FLangID    := -1;
  FLangName  := '';
  FVarName   := TStringList.Create;
  FVarValue  := TStringList.Create;
  FLanguages := TStringList.Create;
end; {--- Create ---}

destructor TsohoMultiLNG.Destroy;
begin
  FVarName.Free;
  FVarValue.Free;
  FLanguages.Free;
  RestoreOldEvent;
  inherited;
end;

procedure TsohoMultiLNG.LoadLanguage(Form : TComponent; LangID : Integer);
var
  Fs       : TFileStream;
  s        : string;
  LngCount : word;
  tsl      : TStringList;
  Propsl   : TStringList;
  i        : Integer;
  Buffer   : Array[1..64] of Byte;
  DataID   : byte;
  DataType : byte;
  BaseForm : TComponent;
  BaseComp : TComponent;
  FormName : string;
  CompName : string;
  PropName : string;
  VarCount : Integer;
begin
  if FLangFile = '' then Exit;
  if LangID < 0 then Exit;
  FVarName.Clear;
  FVarValue.Clear;
  try
    Fs := TFileStream.Create(FLangFile,fmOpenRead);
  except
    raise;
    Exit;
  end; {--- try/except ---}
  tsl    := TStringList.Create;
  Propsl := TStringList.Create;
  try
    //     -  - 
    s := StringFromStream(fs);
    if s <> 'SohoMultiLNG' then begin
      ErrorMsg(Format(sohoMultiLNG_NotLNGFile,[FLangFile]));
      Exit;
    end; {--- if ---}
    //   
    fs.Read(LngCount, SizeOF(LngCount));
    for i := 0 to pred(LngCount) do begin
      // 
      S := StringFromStream(fs);
      // 
      S := StringFromStream(fs);
      // . 
      fs.ReadBuffer(Buffer, SizeOf(Buffer));
    end; {--- for ---}

    //   
    fs.Read(VarCount, SizeOf(VarCount));
    i := 0;
    while (i < VarCount) do begin
      //  
      fs.Read(DataID, SizeOf(DataID));
      fs.Read(DataType, SizeOf(DataType));
      //  
      S := StringFromStream(fs);
      case DataID of
        //   
        0: begin
             //   
             tsl.Clear;
             SplitString(S,'.',tsl);
             FormName := tsl[0];
             CompName := tsl[1];
             PropName := tsl[2];

             //   
             tsl.Clear;
             tsl.Text := StringFromStream(fs);

             // BaseForm := Form;
             //    
             BaseForm := GetScreenFormByName(FormName);
             if BaseForm <> nil then begin
               //    
               BaseComp := BaseForm.FindComponent(CompName);
               //      
               if BaseComp <> nil then
                 case DataType of
                   //   - String
                   0: SetStringProperty(BaseComp,PropName,
                                        ReplaceStr(tsl[LangID],
                                        sohoMultiLNG_ReplaceCRLF,
                                        sohoMultiLNG_CRLF)
                                        );
                   //   - TStrings
                   1: begin
                        PropSL.Text :=  ReplaceStr(tsl[LangID],
                                        sohoMultiLNG_ReplaceCRLF,
                                        sohoMultiLNG_CRLF
                                        );
                        SetClassProperty(BaseComp, PropName, PropSL);
                      end; {--- 1 ---}
                 end; {--- case ---}
             end; {--- if ---}
           end; {--- 0 ---}

        //  
        1: begin
             FVarName.Add(S);
             //   
             tsl.Clear;
             tsl.Text := StringFromStream(fs);
             FVarValue.Add(tsl[LangID]);
           end; {--- 1 ---}
      end; {--- case ---}
      inc(i);
    end; {--- while ---}
  finally
    fs.Free;
    tsl.Free;
    Propsl.Free;
  end; {--- try/finally---}
end;

function TsohoMultiLNG.GetVariable(const Name : string) : string;
var
  i : Integer;
begin
  Result := '';
  i := FVarName.IndexOf(Name);
  if i <> -1 then Result := ReplaceStr(FvarValue[i],
                                        sohoMultiLNG_ReplaceCRLF,
                                        sohoMultiLNG_CRLF
                                        );
end;

function TsohoMultiLNG.GetLanguageName(LangID : Integer) : string;
begin
  Result := '';
  if LangID > LanguageCount Then Exit;
  Result := FLanguages[LangID];
end;

end.
